@@ -15,7 +15,7 @@
 
     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 
     As a special exception, permission is given to link this program
     with any edition of Qt, and distribute the resulting executable,
@@ -36,6 +36,7 @@ static KCmdLineOptions akregator_options[] =
 	{ "addfeed <url>", I18N_NOOP( "Add a feed with the given URL" ), 0},
 	{ "g", 0, 0 },
 	{ "group <groupname>", I18N_NOOP( "When adding feeds, place them in this group" ), I18N_NOOP("Imported") },
+    { "hide-mainwindow", I18N_NOOP( "Hide main window on startup" ), 0},
 	KCmdLineLastOption
 };
 
