@@ -2,6 +2,7 @@
     This file is part of Akregator.
 
     Copyright (C) 2004 Stanislav Karchebny <Stanislav.Karchebny@kdemail.net>
+                  2005 Frank Osterfeld <frank.osterfeld at kdemail.net>
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -15,17 +16,17 @@
 
     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 
     As a special exception, permission is given to link this program
     with any edition of Qt, and distribute the resulting executable,
     without including the source code for Qt in the source distribution.
 */
-
+#include <dcopclient.h>
 #include <kaboutdata.h>
 #include <kaction.h>
 #include <kactionclasses.h>
-#include <kactioncollection.h> 
+#include <kactioncollection.h>
 #include <kapplication.h>
 #include <kconfig.h>
 #include <kconfigdialog.h>
@@ -35,10 +36,15 @@
 #include <kinstance.h>
 #include <kmainwindow.h>
 #include <kmessagebox.h>
+#include <knotifyclient.h>
 #include <knotifydialog.h>
 #include <kpopupmenu.h>
+#include <kservice.h>
 #include <kstandarddirs.h>
 #include <kstdaction.h>
+#include <ktempfile.h>
+#include <ktrader.h>
+#include <kio/netaccess.h>
 #include <kparts/browserinterface.h>
 #include <kparts/genericfactory.h>
 #include <kparts/partmanager.h>
@@ -51,20 +57,29 @@
 #include <private/qucomextra_p.h>
 
 #include "aboutdata.h"
+#include "actionmanagerimpl.h"
 #include "akregator_part.h"
 #include "akregator_view.h"
 #include "akregatorconfig.h"
-#include "akregator.h"
-#include "fetchtransaction.h"
+#include "articlefilter.h"
+#include "articleinterceptor.h"
+#include "configdialog.h"
+#include "fetchqueue.h"
 #include "frame.h"
-#include "myarticle.h"
+#include "article.h"
+#include "kernel.h"
 #include "notificationmanager.h"
 #include "pageviewer.h"
-#include "settings_appearance.h"
-#include "settings_archive.h"
-#include "settings_browser.h"
-#include "settings_general.h"
+#include "plugin.h"
+#include "pluginmanager.h"
+#include "storage.h"
+#include "storagefactory.h"
+#include "storagefactorydummyimpl.h"
+#include "storagefactoryregistry.h"
+#include "speechclient.h"
 #include "trayicon.h"
+#include "tagset.h"
+#include "tag.h"
 
 namespace Akregator {
 
@@ -82,151 +97,130 @@ void BrowserExtension::saveSettings()
     m_part->saveSettings();
 }
 
-void Part::setupActions()
+class Part::ApplyFiltersInterceptor : public ArticleInterceptor
 {
-    // file menu
-
-    new KAction(i18n("&Import Feeds..."), "", "", this, SLOT(fileImport()), actionCollection(), "file_import");
-    new KAction(i18n("&Export Feeds..."), "", "", this, SLOT(fileExport()), actionCollection(), "file_export");
-        
-    /* --- Feed/Feed Group popup menu */
-    new KAction(i18n("&Open Homepage"), "", "Ctrl+H", m_view, SLOT(slotOpenHomepage()), actionCollection(), "feed_homepage");
-    new KAction(i18n("&Add Feed..."), "bookmark_add", "Insert", m_view, SLOT(slotFeedAdd()), actionCollection(), "feed_add");
-    new KAction(i18n("Ne&w Folder..."), "folder_new", "Shift+Insert", m_view, SLOT(slotFeedAddGroup()), actionCollection(), "feed_add_group");
-    new KAction(i18n("&Delete Feed"), "editdelete", "Alt+Delete", m_view, SLOT(slotFeedRemove()), actionCollection(), "feed_remove");
-    new KAction(i18n("&Edit Feed..."), "edit", "F2", m_view, SLOT(slotFeedModify()), actionCollection(), "feed_modify");
-
-    KActionMenu* vm = new KActionMenu( i18n( "&View Mode" ), actionCollection(), "view_mode" );
-    
-    KRadioAction *ra = new KRadioAction(i18n("&Normal View"), "view_top_bottom", "Ctrl+Shift+1", m_view, SLOT(slotNormalView()), actionCollection(), "normal_view");
-    ra->setExclusiveGroup( "ViewMode" );
-    vm->insert(ra);
-    
-    ra = new KRadioAction(i18n("&Widescreen View"), "view_left_right", "Ctrl+Shift+2", m_view, SLOT(slotWidescreenView()), actionCollection(), "widescreen_view");
-    ra->setExclusiveGroup( "ViewMode" );
-    vm->insert(ra);
-    
-    ra = new KRadioAction(i18n("C&ombined View"), "view_text", "Ctrl+Shift+3", m_view, SLOT(slotCombinedView()), actionCollection(), "combined_view");
-    ra->setExclusiveGroup( "ViewMode" );
-    vm->insert(ra);
-
-    // toolbar / feed menu
-    new KAction(i18n("&Fetch Feed"), "down", "Ctrl+L", m_view, SLOT(slotFetchCurrentFeed()), actionCollection(), "feed_fetch");
-    new KAction(i18n("Fe&tch All Feeds"), "bottom", "Ctrl+Shift+L", m_view, SLOT(slotFetchAllFeeds()), actionCollection(), "feed_fetch_all");
-    new KAction(i18n( "&Abort Fetches" ), "stop", Key_Escape, this, SLOT( slotStop() ), actionCollection(), "feed_stop");
-
-    new KAction(i18n("&Mark Feed as Read"), "goto", "Ctrl+R", m_view, SLOT(slotMarkAllRead()), actionCollection(), "feed_mark_all_as_read");
-    new KAction(i18n("Ma&rk All Feeds as Read"), "goto", "Ctrl+Shift+R", m_view, SLOT(slotMarkAllFeedsRead()), actionCollection(), "feed_mark_all_feeds_as_read");
-
-    // "Go" menu
-
-    new KAction(i18n("&Previous Feed"), "", "P", m_view, SLOT(slotPrevFeed()),actionCollection(), "go_prev_feed");
-    new KAction(i18n("&Next Feed"), "", "N", m_view, SLOT(slotNextFeed()),actionCollection(), "go_next_feed");
-    new KAction(i18n("N&ext Unread Feed"), "", "Alt+Plus", m_view, SLOT(slotNextUnreadFeed()),actionCollection(), "go_next_unread_feed");
-    new KAction(i18n("Prev&ious Unread Feed"), "", "Alt+Minus", m_view, SLOT(slotPrevUnreadFeed()),actionCollection(), "go_prev_unread_feed");
-
-    // Settings menu
-    KToggleAction* sqf = new KToggleAction(i18n("Show Quick Filter"), QString::null, 0, m_view, SLOT(slotToggleShowQuickFilter()), actionCollection(), "show_quick_filter");
-    sqf->setChecked( Settings::showQuickFilter() );
-    KStdAction::configureNotifications(this,SLOT(showKNotifyOptions()), actionCollection()); // options_configure_notifications
-
-    new KAction( i18n("Configure &Akregator..."), "configure", "", this, SLOT(showOptions()), actionCollection(), "akregator_configure_akregator" );
-    //KStdAction::preferences( this, SLOT(showOptions()), actionCollection(), "akregator_configure_akregator" );
-
-    // feed tree navigation
-    new KAction( i18n("Go Up in Tree"), QString::null, "Alt+Up", m_view, SLOT(slotFeedsTreeUp()), actionCollection(), "feedstree_up" );
-    new KAction( i18n("Go Down in Tree"), QString::null, "Alt+Down", m_view, SLOT(slotFeedsTreeDown()), actionCollection(), "feedstree_down" );
-    new KAction( i18n("Go Left in Tree"), QString::null, "Alt+Left", m_view, SLOT(slotFeedsTreeLeft()), actionCollection(), "feedstree_left" );
-    new KAction( i18n("Go Right in Tree"), QString::null, "Alt+Right", m_view, SLOT(slotFeedsTreeRight()), actionCollection(), "feedstree_right" );
-    new KAction( i18n("Go to Top of Tree"), QString::null, "Alt+Home", m_view, SLOT(slotFeedsTreeHome()), actionCollection(), "feedstree_home" );
-    new KAction( i18n("Go to Bottom of Tree"), QString::null, "Alt+End", m_view, SLOT(slotFeedsTreeEnd()), actionCollection(), "feedstree_end" );
-    new KAction( i18n("Move Node Up"), QString::null, "Shift+Alt+Up", m_view, SLOT(slotMoveCurrentNodeUp()), actionCollection(), "feedstree_move_up" );
-    new KAction( i18n("Move Node Down"), QString::null,  "Shift+Alt+Down", m_view, SLOT(slotMoveCurrentNodeDown()), actionCollection(), "feedstree_move_down" );
-    new KAction( i18n("Move Node Left"), QString::null, "Shift+Alt+Left", m_view, SLOT(slotMoveCurrentNodeLeft()), actionCollection(), "feedstree_move_left" );
-    new KAction( i18n("Move Node Right"), QString::null, "Shift+Alt+Right", m_view, SLOT(slotMoveCurrentNodeRight()), actionCollection(), "feedstree_move_right" );
-
-
-    // article list
-    new KAction( i18n("Open Article in Tab"), "tab_new", "Shift+Return", m_view, SLOT(slotOpenCurrentArticle()), actionCollection(), "article_open" );
-    new KAction( i18n("Open Article in Background Tab"), QString::null, "tab_new", m_view, SLOT(slotOpenCurrentArticleBackgroundTab()), actionCollection(), "article_open_background_tab" );
-    new KAction( i18n("Open Article in External Browser"), "window_new", "Ctrl+Shift+Return", m_view, SLOT(slotOpenCurrentArticleExternal()), actionCollection(), "article_open_external" );
-    new KAction( i18n("&Previous Article"), QString::null, "Left", m_view, SLOT(slotPreviousArticle()), actionCollection(), "go_previous_article" );
-    new KAction( i18n("&Next Article"), QString::null, "Right", m_view, SLOT(slotNextArticle()), actionCollection(), "go_next_article" );
-    new KAction(i18n("Pre&vious Unread Article"), "", Key_Minus, m_view, SLOT(slotPrevUnreadArticle()),actionCollection(), "go_prev_unread_article");
-    new KAction(i18n("Ne&xt Unread Article"), "", Key_Plus, m_view, SLOT(slotNextUnreadArticle()),actionCollection(), "go_next_unread_article");
-    KToggleAction* tkf = new KToggleAction(i18n("&Keep Article"), "flag", "Ctrl+K", m_view, SLOT(slotArticleToggleKeepFlag()), actionCollection(), "article_toggle_keep");
-    tkf->setChecked(false);
-
-    new KAction(i18n("&Delete Article"), "editdelete", "Delete", m_view, SLOT(slotArticleDelete()), actionCollection(), "article_delete");
-    
-
-    KActionMenu* statusMenu = new KActionMenu ( i18n( "&Mark Article" ),
-                                    actionCollection(), "article_set_status" );
-
-    statusMenu->insert(new KAction(KGuiItem(i18n("Mark Article as &Unread"), "",
-                       i18n("Mark selected article as unread")),
-    "Ctrl+U", m_view, SLOT(slotSetSelectedArticleUnread()),
-    actionCollection(), "article_set_status_unread"));
-
-    statusMenu->insert(new KAction(KGuiItem(i18n("Mark Article as &New"), "",
-                        i18n("Mark selected article as new")),
-    "Ctrl+N", m_view, SLOT(slotSetSelectedArticleNew()),
-    actionCollection(), "article_set_status_new" ));
-}
+    public:
+    virtual void processArticle(Article& article)
+    {
+        Filters::ArticleFilterList list = Kernel::self()->articleFilterList();
+        for (Filters::ArticleFilterList::ConstIterator it = list.begin(); it != list.end(); ++it)
+            (*it).applyTo(article);
+    }
+};
 
 Part::Part( QWidget *parentWidget, const char * /*widgetName*/,
                               QObject *parent, const char *name, const QStringList& )
-    : DCOPObject("AkregatorIface"), MyBasePart(parent, name), m_shuttingDown(false), m_parentWidget(parentWidget)
+    : DCOPObject("AkregatorIface")
+       , MyBasePart(parent, name)
+       , m_standardListLoaded(false)
+       , m_shuttingDown(false)
+       , m_mergedPart(0)
+       , m_backedUpList(false)
+       , m_view(0)
+       , m_storage(0)
+       
 {
-    m_mergedPart = 0;
-    m_backedUpList = false;
     // we need an instance
     setInstance( AkregatorFactory::instance() );
 
+    // start knotifyclient if not already started. makes it work for people who doesn't use full kde, according to kmail devels
+    KNotifyClient::startDaemon();
+
     m_standardFeedList = KGlobal::dirs()->saveLocation("data", "akregator/data") + "/feeds.opml";
 
-    m_standardListLoaded = false;
-    m_loading = false;
+    m_tagSetPath = KGlobal::dirs()->saveLocation("data", "akregator/data") + "/tagset.xml";
+
+    Backend::StorageFactoryDummyImpl* dummyFactory = new Backend::StorageFactoryDummyImpl();
+    Backend::StorageFactoryRegistry::self()->registerFactory(dummyFactory, dummyFactory->key());
+    loadPlugins(); // FIXME: also unload them!
+
+    m_storage = 0;
+    Backend::StorageFactory* factory = Backend::StorageFactoryRegistry::self()->getFactory(Settings::archiveBackend());
+    if (factory != 0)
+        m_storage = factory->createStorage(QStringList());
+
+    if (!m_storage) // Houston, we have a problem
+    {
+        m_storage = Backend::StorageFactoryRegistry::self()->getFactory("dummy")->createStorage(QStringList());
+
+        KMessageBox::error(parentWidget, i18n("Unable to load storage backend plugin \"%1\". No feeds are archived.").arg(Settings::archiveBackend()), i18n("Plugin error") );
+    }
+
+    Filters::ArticleFilterList list;
+    list.readConfig(Settings::self()->config());
+    Kernel::self()->setArticleFilterList(list);
+
+    m_applyFiltersInterceptor = new ApplyFiltersInterceptor();
+    ArticleInterceptorManager::self()->addInterceptor(m_applyFiltersInterceptor);
+
+    m_storage->open(true);
+    Kernel::self()->setStorage(m_storage);
+    Backend::Storage::setInstance(m_storage); // TODO: kill this one
+
+    loadTagSet(m_tagSetPath);
+
+    m_actionManager = new ActionManagerImpl(this);
+    ActionManager::setInstance(m_actionManager);
+
+    m_view = new Akregator::View(this, parentWidget, m_actionManager, "akregator_view");
+    m_actionManager->initView(m_view);
+    m_actionManager->setTagSet(Kernel::self()->tagSet());
 
-    m_view = new Akregator::View(this, parentWidget, "akregator_view");
     m_extension = new BrowserExtension(this, "ak_extension");
 
+    connect(m_view, SIGNAL(setWindowCaption(const QString&)), this, SIGNAL(setWindowCaption(const QString&)));
+    connect(m_view, SIGNAL(setStatusBarText(const QString&)), this, SIGNAL(setStatusBarText(const QString&)));
+    connect(m_view, SIGNAL(setProgress(int)), m_extension, SIGNAL(loadingProgress(int)));
+    connect(m_view, SIGNAL(signalCanceled(const QString&)), this, SIGNAL(canceled(const QString&)));
+    connect(m_view, SIGNAL(signalStarted(KIO::Job*)), this, SIGNAL(started(KIO::Job*)));
+    connect(m_view, SIGNAL(signalCompleted()), this, SIGNAL(completed()));
+
     // notify the part that this is our internal widget
     setWidget(m_view);
 
-    setupActions();
-    
-    m_trayIcon = new TrayIcon( getMainWindow() );
-    connect(m_trayIcon, SIGNAL(showPart()), this, SIGNAL(showPart()));
+    TrayIcon* trayIcon = new TrayIcon( getMainWindow() );
+    TrayIcon::setInstance(trayIcon);
+    m_actionManager->initTrayIcon(trayIcon);
+
+    connect(trayIcon, SIGNAL(showPart()), this, SIGNAL(showPart()));
 
     if ( isTrayIconEnabled() )
     {
-        m_trayIcon->show();
-        NotificationManager::self()->setWidget(m_trayIcon, instance());
+        trayIcon->show();
+        NotificationManager::self()->setWidget(trayIcon, instance());
     }
     else
         NotificationManager::self()->setWidget(getMainWindow(), instance());
 
-    connect( m_trayIcon, SIGNAL(quitSelected()),
+    connect( trayIcon, SIGNAL(quitSelected()),
             kapp, SLOT(quit())) ;
 
-    KPopupMenu* traypop = m_trayIcon->contextMenu();
-    
-    action("feed_fetch_all")->plug(traypop, 1);
-    action("akregator_configure_akregator")->plug(traypop, 2);
-    
-    connect( m_view, SIGNAL(signalUnreadCountChanged(int)), m_trayIcon, SLOT(slotSetUnread(int)) );
-    
+    connect( m_view, SIGNAL(signalUnreadCountChanged(int)), trayIcon, SLOT(slotSetUnread(int)) );
+
     connect(kapp, SIGNAL(shutDown()), this, SLOT(slotOnShutdown()));
 
     m_autosaveTimer = new QTimer(this);
     connect(m_autosaveTimer, SIGNAL(timeout()), this, SLOT(slotSaveFeedList()));
     m_autosaveTimer->start(5*60*1000); // 5 minutes
-    
-    // set our XML-UI resource file
+
     setXMLFile("akregator_part.rc", true);
-    
+
     initFonts();
+
+    RSS::FileRetriever::setUserAgent(QString("Akregator/%1; librss/remnants").arg(AKREGATOR_VERSION));
+}
+
+void Part::loadPlugins()
+{
+    // "[X-KDE-akregator-plugintype] == 'storage'"
+    KTrader::OfferList offers = PluginManager::query();
+
+    for( KTrader::OfferList::ConstIterator it = offers.begin(), end = offers.end(); it != end; ++it )
+    {
+        Akregator::Plugin* plugin = PluginManager::createFromService(*it);
+        if (plugin)
+            plugin->init();
+    }
 }
 
 void Part::slotOnShutdown()
@@ -235,15 +229,20 @@ void Part::slotOnShutdown()
     m_autosaveTimer->stop();
     saveSettings();
     slotSaveFeedList();
+    saveTagSet(m_tagSetPath);
     m_view->slotOnShutdown();
+    //delete m_view;
+    delete m_storage;
+    m_storage = 0;
+    //delete m_actionManager;
 }
 
 void Part::slotSettingsChanged()
 {
-    NotificationManager::self()->setWidget(isTrayIconEnabled() ? m_trayIcon : getMainWindow(), instance());
+    NotificationManager::self()->setWidget(isTrayIconEnabled() ? TrayIcon::getInstance() : getMainWindow(), instance());
 
     RSS::FileRetriever::setUseCache(Settings::useHTMLCache());
-    
+
     QStringList fonts;
     fonts.append(Settings::standardFont());
     fonts.append(Settings::fixedFont());
@@ -257,107 +256,47 @@ void Part::slotSettingsChanged()
     if (Settings::minimumFontSize() > Settings::mediumFontSize())
         Settings::setMediumFontSize(Settings::minimumFontSize());
     saveSettings();
+    m_view->slotSettingsChanged();
     emit signalSettingsChanged();
 }
 void Part::saveSettings()
 {
+    Kernel::self()->articleFilterList().writeConfig(Settings::self()->config());
     m_view->saveSettings();
 }
 
 Part::~Part()
 {
+    kdDebug() << "Part::~Part() enter" << endl;
     if (!m_shuttingDown)
         slotOnShutdown();
-}
-
-void Part::setCaption(const QString &text)
-{
-    emit setWindowCaption(text);
-}
-
-void Part::setStatusBar(const QString &text)
-{
-    emit setStatusBarText(text);
-}
-
-void Part::setProgress(int percent)
-{
-    emit m_extension->loadingProgress(percent);
-}
-
-void Part::setStarted(KParts::Part* part)
-{
-    if (part == this)
-        actionCollection()->action("feed_stop")->setEnabled(true);
-    
-    emit started(0);
-}
-
-void Part::setCompleted(KParts::Part* part)
-{
-    if (part == this)
-        actionCollection()->action("feed_stop")->setEnabled(false);
-    
-    emit completed(0);
-}
-
-void Part::setCanceled(KParts::Part* part, const QString &s)
-{
-    if (part == this)
-        actionCollection()->action("feed_stop")->setEnabled(false);
-    
-    emit canceled(s);
+    kdDebug() << "Part::~Part(): leaving" << endl;
+    ArticleInterceptorManager::self()->removeInterceptor(m_applyFiltersInterceptor);
+    delete m_applyFiltersInterceptor;
 }
 
 void Part::readProperties(KConfig* config)
 {
     m_backedUpList = false;
     openStandardFeedList();
-    
+
     if(m_view)
         m_view->readProperties(config);
 }
 
 void Part::saveProperties(KConfig* config)
 {
-    if(m_view)
+    if (m_view)
+    {
+        slotSaveFeedList();
         m_view->saveProperties(config);
+    }
 }
 
-/****************************************************************************/
-/* LOAD                                                                     */
-/****************************************************************************/
-
 bool Part::openURL(const KURL& url)
 {
-    // stop whatever we're doing before opening a new feed list
-    if (m_loading)
-    {
-        m_view->endOperation();
-        m_delayURL=url;
-        QTimer::singleShot(1000, this, SLOT(openURLDelayed()));
-        return true;
-    }
-    else if (m_view->transaction()->isRunning())
-    {
-        m_view->endOperation();
-        m_view->transaction()->stop();
-        m_delayURL=url;
-        QTimer::singleShot(1000, this, SLOT(openURLDelayed()));
-        return true;
-    }
-    else
-    {
-        m_file = url.path();
-        bool ret = openFile();
-        return ret;
-    }
-}
-
-void Part::openURLDelayed()
-{
-    m_file = m_delayURL.path();
-    openFile();
+    m_file = url.path();
+    return openFile();
 }
 
 void Part::openStandardFeedList()
@@ -395,6 +334,11 @@ QDomDocument Part::createDefaultFeedList()
     ak.setAttribute("xmlUrl","http://akregator.sf.net/rss2.php");
     mainFolder.appendChild(ak);
 
+    QDomElement akb = doc.createElement( "outline" );
+    akb.setAttribute("text",i18n("Akregator Blog"));
+    akb.setAttribute("xmlUrl","http://akregator.pwsp.net/blog/?feed=rss2");
+    mainFolder.appendChild(akb);
+
     QDomElement dot = doc.createElement( "outline" );
     dot.setAttribute("text",i18n("KDE Dot News"));
     dot.setAttribute("xmlUrl","http://www.kde.org/dotkdeorg.rdf");
@@ -420,14 +364,22 @@ QDomDocument Part::createDefaultFeedList()
 
 bool Part::openFile()
 {
+    emit setStatusBarText(i18n("Opening Feed List...") );
+
     QString str;
     // m_file is always local so we can use QFile on it
     QFile file(m_file);
-    if (!file.exists())
+
+    bool fileExists = file.exists();
+    QString listBackup = m_storage->restoreFeedList();
+     
+    QDomDocument doc;
+
+    if (!fileExists)
     {
-        m_view->loadFeeds(createDefaultFeedList());
+        doc = createDefaultFeedList();
     }
-    else
+    else 
     {
         if (file.open(IO_ReadOnly))
         {
@@ -437,76 +389,54 @@ bool Part::openFile()
             str = stream.read();
             file.close();
         }
-        else
-        {
-            KMessageBox::error(m_view, i18n("Could not read standard feed list (%1). A default feed list will be used.").arg(m_file), i18n("Read Error") );
-            return false;
-        }
-
-        setStatusBar( i18n("Opening Feed List...") );
-
-        QDomDocument doc;
 
         if (!doc.setContent(str))
         {
-            QString backup = m_file + "-backup." +  QString::number(QDateTime::currentDateTime().toTime_t());
 
-            copyFile(backup);
+            if (file.size() > 0) // don't backup empty files 
+            {
+                QString backup = m_file + "-backup." +  QString::number(QDateTime::currentDateTime().toTime_t());
+        
+                copyFile(backup);
+        
+                KMessageBox::error(m_view, i18n("<qt>The standard feed list is corrupted (invalid XML). A backup was created:<p><b>%2</b></p></qt>").arg(backup), i18n("XML Parsing Error") );
+            }
 
-            KMessageBox::error(m_view, i18n("<qt>The standard feed list is corrupted (invalid XML). A backup was created:<p><b>%2</b></p></qt>").arg(backup), i18n("XML Parsing Error") );
-
-            doc = createDefaultFeedList();
+            if (!doc.setContent(listBackup))
+                doc = createDefaultFeedList();
         }
+    }
 
-        if (!m_view->loadFeeds(doc))
+    if (!m_view->loadFeeds(doc))
+    {
+        if (file.size() > 0) // don't backup empty files 
         {
             QString backup = m_file + "-backup." +  QString::number(QDateTime::currentDateTime().toTime_t());
             copyFile(backup);
 
             KMessageBox::error(m_view, i18n("<qt>The standard feed list is corrupted (no valid OPML). A backup was created:<p><b>%2</b></p></qt>").arg(backup), i18n("OPML Parsing Error") );
-            m_view->loadFeeds(createDefaultFeedList());
         }
-        
-        setStatusBar(QString::null);
+        m_view->loadFeeds(createDefaultFeedList());
     }
+
+    emit setStatusBarText(QString::null);
     
+
     if( Settings::markAllFeedsReadOnStartup() )
         m_view->slotMarkAllFeedsRead();
 
     if (Settings::fetchOnStartup())
-    	m_view->slotFetchAllFeeds();
-    
+            m_view->slotFetchAllFeeds();
+
     return true;
 }
 
-bool Part::closeURL()
-{
-    m_view->endOperation();
-    setStatusBar(QString::null);
-
-    if (m_loading)
-    {
-        m_loading = false;
-        kdDebug() << "closeURL: stop loading" << endl;
-        return true;
-    }
-    else if (m_view->transaction()->isRunning())
-    {
-        m_view->transaction()->stop();
-        kdDebug() << "closeURL: stop transaction" << endl;
-        return true;
-    }
-
-   return MyBasePart::closeURL();
-}
-
-
 void Part::slotSaveFeedList()
 {
     // don't save to the standard feed list, when it wasn't completely loaded before
     if (!m_standardListLoaded)
         return;
-    
+
     // the first time we overwrite the feed list, we create a backup
     if (!m_backedUpList)
     {
@@ -516,7 +446,10 @@ void Part::slotSaveFeedList()
             m_backedUpList = true;
     }
 
-    QFile file(m_file);    
+    QString xmlStr = m_view->feedListToOPML().toString();
+    m_storage->storeFeedList(xmlStr);
+
+    QFile file(m_file);
     if (file.open(IO_WriteOnly) == false)
     {
         //FIXME: allow to save the feedlist into different location -tpr 20041118
@@ -531,7 +464,7 @@ void Part::slotSaveFeedList()
     // Write OPML data file.
     // Archive data files are saved elsewhere.
 
-    stream << m_view->feedListToOPML().toString();
+    stream << xmlStr << endl;
 
     file.close();
 }
@@ -541,16 +474,6 @@ bool Part::isTrayIconEnabled() const
     return Settings::showTrayIcon();
 }
 
-TrayIcon* Part::trayIcon()
-{
-    return m_trayIcon;
-}
-
-QPixmap Part::takeTrayIconScreenshot() const
-{
-    return m_trayIcon->takeScreenshot();
-}
-
 bool Part::mergePart(KParts::Part* part)
 {
     if (part != m_mergedPart)
@@ -564,7 +487,7 @@ bool Part::mergePart(KParts::Part* part)
             factory()->removeClient(m_mergedPart);
         if (part)
             factory()->addClient(part);
-    
+
         m_mergedPart = part;
     }
     return true;
@@ -597,103 +520,182 @@ QWidget* Part::getMainWindow()
     return 0;
 }
 
-
-void Part::importFile(const QString& fileName)
+void Part::loadTagSet(const QString& path)
 {
-    QFile file(fileName);
-    if (file.open(IO_ReadOnly) == false)
-        return;
-
-    // Read OPML feeds list and build QDom tree.
     QDomDocument doc;
-    if (!doc.setContent(file.readAll())) {
-        kdDebug() << "Failed to build DOM tree, is " << fileName << " valid XML?" << endl;
-        return;
+
+    QFile file(path);
+    if (file.open(IO_ReadOnly))
+    {
+        doc.setContent(file.readAll());
+        file.close();
+    }
+    // if we can't load the tagset from the xml file, check for the backup in the backend
+    if (doc.isNull())
+    {
+        doc.setContent(m_storage->restoreTagSet());
     }
 
-    m_view->importFeeds(doc);
+    if (!doc.isNull())
+    {
+        Kernel::self()->tagSet()->readFromXML(doc);
+    }
+    else
+    {
+        Kernel::self()->tagSet()->insert(Tag("http://akregator.sf.net/tags/Interesting", i18n("Interesting")));
+    }
 }
 
-void Part::exportFile(const QString& fileName)
+void Part::saveTagSet(const QString& path)
 {
-   // TODO we could KIO here instead of QFile
-    QFile file(fileName);
-    if ( file.exists() )
+    QString xmlStr = Kernel::self()->tagSet()->toXML().toString();
 
-        if ( KMessageBox::questionYesNo(m_view,
-          i18n("The file %1 already exists; do you want to overwrite it?").arg(fileName),
-        i18n("Export"),
-        i18n("Overwrite"),
-        i18n("Cancel")) == KMessageBox::No )
+    m_storage->storeTagSet(xmlStr);
+    
+    QFile file(path);
+    
+    if ( file.open(IO_WriteOnly) )
+    {
+
+        QTextStream stream(&file);
+        stream.setEncoding(QTextStream::UnicodeUTF8);
+        stream << xmlStr << "\n";
+        file.close();
+    }
+}
+
+void Part::importFile(const KURL& url)
+{
+    QString filename;
+
+    bool isRemote = false;
+
+    if (url.isLocalFile())
+        filename = url.path();
+    else
+    {
+        isRemote = true;
+
+        if (!KIO::NetAccess::download(url, filename, m_view) )
+        {
+            KMessageBox::error(m_view, KIO::NetAccess::lastErrorString() );
+            return;
+        }
+    }
+
+    QFile file(filename);
+    if (file.open(IO_ReadOnly))
+    {
+        // Read OPML feeds list and build QDom tree.
+        QDomDocument doc;
+        if (doc.setContent(file.readAll()))
+            m_view->importFeeds(doc);
+        else
+            KMessageBox::error(m_view, i18n("Could not import the file %1 (no valid OPML)").arg(filename), i18n("OPML Parsing Error") );
+    }
+    else
+        KMessageBox::error(m_view, i18n("The file %1 could not be read, check if it exists or if it is readable for the current user.").arg(filename), i18n("Read Error"));
+
+    if (isRemote)
+        KIO::NetAccess::removeTempFile(filename);
+}
+
+void Part::exportFile(const KURL& url)
+{
+    if (url.isLocalFile())
+    {
+        QFile file(url.path());
+
+        if ( file.exists() &&
+                KMessageBox::questionYesNo(m_view,
+            i18n("The file %1 already exists; do you want to overwrite it?").arg(file.name()),
+            i18n("Export"),
+            i18n("Overwrite"),
+            KStdGuiItem::cancel()) == KMessageBox::No )
             return;
 
-    if ( !file.open(IO_WriteOnly) )
-    {
-        KMessageBox::error(m_view, i18n("Access denied: cannot write to file %1").arg(fileName), i18n("Write error") );
-        return;
+        if ( !file.open(IO_WriteOnly) )
+        {
+            KMessageBox::error(m_view, i18n("Access denied: cannot write to file %1").arg(file.name()), i18n("Write Error") );
+            return;
+        }
+
+        QTextStream stream(&file);
+        stream.setEncoding(QTextStream::UnicodeUTF8);
+
+        stream << m_view->feedListToOPML().toString() << "\n";
+        file.close();
     }
-
-    // use QTextStream to dump the text to the file
-    QTextStream stream(&file);
-    stream.setEncoding(QTextStream::UnicodeUTF8);
-
-    QDomDocument doc = m_view->feedListToOPML();
-    
-    stream << doc.toString();
-    file.close();
-}
-
-/****************************************************************************/
-/* SLOTS                                                                    */
-/****************************************************************************/
-
-void Part::fileOpen()
-{
-    // this slot is called whenever the File->Open menu is selected,
-    // the Open shortcut is pressed (usually CTRL+O) or the Open toolbar
-    // button is clicked
-    QString file_name = KFileDialog::getOpenFileName( QString::null,
-                        "*.opml *.xml|" + i18n("OPML Outlines (*.opml, *.xml)")
-                        +"\n*|" + i18n("All Files") );
-
-    if (file_name.isEmpty() == false)
-        openURL(file_name);
-}
-/*
-bool Akregator::Part::fileSaveAs()
-{
-    // this slot is called whenever the File->Save As menu is selected,
-    QString file_name = KFileDialog::getSaveFileName( QString::null,
-                        "*.opml *.xml|" + i18n("OPML Outlines (*.opml, *.xml)")
-                        +"\n*|" + i18n("All Files") );
-    if (file_name.isEmpty() == false)
+    else
     {
-        kdDebug() << "SaveAs called! Shouldn't ever happen!" << endl;
-        //saveAs(file_name);
-        return true;
+        KTempFile tmpfile;
+        tmpfile.setAutoDelete(true);
+
+        QTextStream stream(tmpfile.file());
+        stream.setEncoding(QTextStream::UnicodeUTF8);
+
+        stream << m_view->feedListToOPML().toString() << "\n";
+        tmpfile.close();
+
+        if (!KIO::NetAccess::upload(tmpfile.name(), url, m_view))
+            KMessageBox::error(m_view, KIO::NetAccess::lastErrorString() );
     }
-    return false;
 }
-*/
 
 void Part::fileImport()
 {
-    QString file_name = KFileDialog::getOpenFileName( QString::null,
+    KURL url = KFileDialog::getOpenURL( QString::null,
                         "*.opml *.xml|" + i18n("OPML Outlines (*.opml, *.xml)")
                         +"\n*|" + i18n("All Files") );
 
-    if (file_name.isEmpty() == false)
-        importFile(file_name);
+    if (!url.isEmpty())
+        importFile(url);
 }
 
-void Part::fileExport()
+    void Part::fileExport()
 {
-    QString file_name = KFileDialog::getSaveFileName( QString::null,
+    KURL url= KFileDialog::getSaveURL( QString::null,
                         "*.opml *.xml|" + i18n("OPML Outlines (*.opml, *.xml)")
                         +"\n*|" + i18n("All Files") );
 
-    if ( !file_name.isEmpty() )
-        exportFile(file_name);
+    if ( !url.isEmpty() )
+        exportFile(url);
+}
+
+void Part::fileGetFeeds()
+{
+    /*GetFeeds *gf = new GetFeeds();
+    gf->show();*/
+     //KNS::DownloadDialog::open("akregator/feeds", i18n("Get New Feeds"));
+}
+
+void Part::fileSendArticle(bool attach)
+{
+    // FIXME: you have to open article to tab to be able to send...
+    QString title, text;
+
+    text = m_view->currentFrame()->part()->url().prettyURL();
+    if(text.isEmpty() || text.isNull())
+        return;
+
+    title = m_view->currentFrame()->title();
+
+    if(attach) {
+        kapp->invokeMailer("",
+                           "",
+                           "",
+                           title,
+                           text,
+                           "",
+                           text);
+    }
+    else {
+        kapp->invokeMailer("",
+                           "",
+                           "",
+                           title,
+                           text);
+    }
 }
 
 void Part::fetchAllFeeds()
@@ -716,10 +718,10 @@ void Part::addFeedsToGroup(const QStringList& urls, const QString& group)
     NotificationManager::self()->slotNotifyFeeds(urls);
 }
 
-
-/****************************************************************************/
-/* STATIC METHODS                                                           */
-/****************************************************************************/
+void Part::addFeed()
+{
+    m_view->slotFeedAdd();
+}
 
 KAboutData *Part::createAboutData()
 {
@@ -738,15 +740,12 @@ void Part::showOptions()
     if ( KConfigDialog::showDialog( "settings" ) )
         return;
 
-    KConfigDialog *dialog = new KConfigDialog( m_view, "settings", Settings::self() );
-    dialog->addPage(new SettingsGeneral(0, "General"), i18n("General"), "package_settings");
-    dialog->addPage(new SettingsArchive(0, "Archive"), i18n("Archive"), "package_settings");
-    dialog->addPage(new SettingsAppearance(0, "Appearance"), i18n("Appearance"), "fonts");
-    dialog->addPage(new SettingsBrowser(0, "Browser"), i18n("Browser"), "package_network");
+    KConfigDialog* dialog = new ConfigDialog( m_view, "settings", Settings::self() );
+
     connect( dialog, SIGNAL(settingsChanged()),
              this, SLOT(slotSettingsChanged()) );
     connect( dialog, SIGNAL(settingsChanged()),
-             m_trayIcon, SLOT(settingsChanged()) );
+             TrayIcon::getInstance(), SLOT(settingsChanged()) );
 
     dialog->show();
 }
@@ -811,7 +810,7 @@ void Part::initFonts()
 
     KConfig konq("konquerorrc", true, false);
     konq.setGroup("HTML Settings");
-            
+
     if (!conf->hasKey("MinimumFontSize"))
     {
         int minfs;
@@ -833,7 +832,7 @@ void Part::initFonts()
         kdDebug() << "Part::initFonts(): set MediumFontSize to " << medfs << endl;
         Settings::setMediumFontSize(medfs);
     }
-    
+
     if (!conf->hasKey("UnderlineLinks"))
     {
         bool underline = true;
@@ -843,7 +842,7 @@ void Part::initFonts()
         kdDebug() << "Part::initFonts(): set UnderlineLinks to " << underline << endl;
         Settings::setUnderlineLinks(underline);
     }
-    
+
 }
 
 bool Part::copyFile(const QString& backup)
@@ -872,7 +871,6 @@ bool Part::copyFile(const QString& backup)
     return false;
 }
 
+
 } // namespace Akregator
 #include "akregator_part.moc"
-
-// vim: set et ts=4 sts=4 sw=4:
