@@ -1,7 +1,7 @@
 /*
  *  daemon.cpp  -  interface with alarm daemon
  *  Program:  kalarm
- *  (C) 2001 - 2004 by David Jarvie <software@astrojar.org.uk>
+ *  Copyright (c) 2001 - 2005 by David Jarvie <software@astrojar.org.uk>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -13,9 +13,9 @@
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
  *  GNU General Public License for more details.
  *
- *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+ *  You should have received a copy of the GNU General Public License along
+ *  with this program; if not, write to the Free Software Foundation, Inc.,
+ *  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  */
 
 #include "kalarm.h"
@@ -108,8 +108,8 @@ void Daemon::createDcopHandler()
 	// the program is still initialising.
 	mRunning = isRunning(false);
 
-	mStatusTimerInterval = Preferences::instance()->daemonTrayCheckInterval();
-	connect(Preferences::instance(), SIGNAL(preferencesChanged()), mInstance, SLOT(slotPreferencesChanged()));
+	mStatusTimerInterval = Preferences::daemonTrayCheckInterval();
+	Preferences::connect(SIGNAL(preferencesChanged()), mInstance, SLOT(slotPreferencesChanged()));
 
 	mStatusTimer = new QTimer(mInstance);
 	connect(mStatusTimer, SIGNAL(timeout()), mInstance, SLOT(timerCheckIfRunning()));
@@ -209,9 +209,8 @@ void Daemon::registrationResult(bool reregister, int result)
 			break;
 		case KAlarmd::NOT_FOUND:
 			kdError(5950) << "Daemon::registrationResult(" << reregister << "): registerApp dcop call: " << kapp->aboutData()->appName() << " not found\n";
-			KMessageBox::error(0, i18n("Alarms will be disabled if you stop %1.\n"
-			                           "(Installation or configuration error: %2 cannot locate %3 executable.)")
-					           .arg(kapp->aboutData()->programName())
+			KMessageBox::error(0, i18n("Alarms will be disabled if you stop KAlarm.\n"
+			                           "(Installation or configuration error: %1 cannot locate %2 executable.)")
 			                           .arg(QString::fromLatin1(DAEMON_APP_NAME))
 			                           .arg(kapp->aboutData()->appName()));
 			break;
@@ -505,7 +504,7 @@ void Daemon::setFastCheck()
 */
 void Daemon::slotPreferencesChanged()
 {
-	int newInterval = Preferences::instance()->daemonTrayCheckInterval();
+	int newInterval = Preferences::daemonTrayCheckInterval();
 	if (newInterval != mStatusTimerInterval)
 	{
 		// Daemon check interval has changed
