@@ -1,7 +1,7 @@
 /*
  *  kalarm.h  -  global header file
  *  Program:  kalarm
- *  (C) 2001 - 2005 by David Jarvie <software@astrojar.org.uk>
+ *  Copyright (c) 2001 - 2005 by David Jarvie <software@astrojar.org.uk>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -15,7 +15,7 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+ *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  */
 
 #ifndef KALARM_H
@@ -26,10 +26,11 @@
 #include <config.h>
 #endif
 
-#define KALARM_VERSION "1.2.9"
+#define KALARM_VERSION "1.3.6"
 
 #include <kdeversion.h>
-extern int  marginKDE2;   // KDE2 compatibility
+
+#define AUTOSTART_BY_KALARMD    // temporary fix for autostart before session restoration
 
 #define OLD_DCOP    // retain DCOP pre-1.2 compatibility
 
