@@ -1,7 +1,7 @@
 /*
  *  kalarmapp.cpp  -  the KAlarm application object
  *  Program:  kalarm
- *  (C) 2001 - 2004 by David Jarvie <software@astrojar.org.uk>
+ *  Copyright (c) 2001 - 2005 by David Jarvie <software@astrojar.org.uk>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -13,9 +13,9 @@
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
  *  GNU General Public License for more details.
  *
- *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+ *  You should have received a copy of the GNU General Public License along
+ *  with this program; if not, write to the Free Software Foundation, Inc.,
+ *  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  */
 
 #include "kalarm.h"
@@ -39,12 +39,13 @@
 #include <ktempfile.h>
 #include <kfileitem.h>
 #include <kstdguiitem.h>
+#include <ktrader.h>
 #include <kstaticdeleter.h>
 #include <kdebug.h>
 
-#include <kalarmd/clientinfo.h>
+#include <libkcal/calformat.h>
 
-#include <libkcal/icalformat.h>
+#include <kalarmd/clientinfo.h>
 
 #include "alarmcalendar.h"
 #include "alarmlistview.h"
@@ -53,6 +54,7 @@
 #include "dcophandler.h"
 #include "functions.h"
 #include "kamail.h"
+#include "karecurrence.h"
 #include "mainwindow.h"
 #include "messagebox.h"
 #include "messagewin.h"
@@ -64,10 +66,9 @@
 
 #include <netwm.h>
 
-int         marginKDE2 = 0;
 
 static bool convWakeTime(const QCString timeParam, QDateTime&, bool& noTime);
-static bool convInterval(QCString timeParam, KAEvent::RecurType&, int& timeInterval, bool allowMonthYear = true);
+static bool convInterval(QCString timeParam, KARecurrence::Type&, int& timeInterval, bool allowMonthYear = true);
 
 /******************************************************************************
 * Find the maximum number of seconds late which a late-cancel alarm is allowed
@@ -103,17 +104,14 @@ KAlarmApp::KAlarmApp()
 	  mProcessingQueue(false),
 	  mCheckingSystemTray(false),
 	  mSessionClosingDown(false),
-	  mRefreshExpiredAlarms(false)
+	  mRefreshExpiredAlarms(false),
+	  mSpeechEnabled(false)
 {
-#if KDE_VERSION < 290
-	marginKDE2 = KDialog::marginHint();
-#endif
-	mCommandProcesses.setAutoDelete(true);
-	Preferences* preferences = Preferences::instance();
-	connect(preferences, SIGNAL(preferencesChanged()), SLOT(slotPreferencesChanged()));
+	Preferences::initialise();
+	Preferences::connect(SIGNAL(preferencesChanged()), this, SLOT(slotPreferencesChanged()));
 	KCal::CalFormat::setApplication(aboutData()->programName(),
 	                          QString::fromLatin1("-//K Desktop Environment//NONSGML KAlarm " KALARM_VERSION "//EN"));
-	KAEvent::setFeb29RecurType();
+	KARecurrence::setDefaultFeb29Type(Preferences::defaultFeb29Type());
 
 	// Check if it's a KDE desktop by comparing the window manager name to "KWin"
 	NETRootInfo nri(qt_xdisplay(), NET::SupportingWMCheck);
@@ -129,21 +127,37 @@ KAlarmApp::KAlarmApp()
 		mNoSystemTray           = config->readBoolEntry(QString::fromLatin1("NoSystemTray"), false);
 		mSavedNoSystemTray      = mNoSystemTray;
 		mOldRunInSystemTray     = wantRunInSystemTray();
-		mDisableAlarmsIfStopped = mOldRunInSystemTray && !mNoSystemTray && preferences->disableAlarmsIfStopped();
-		mStartOfDay             = preferences->startOfDay();
-		if (preferences->hasStartOfDayChanged())
+		mDisableAlarmsIfStopped = mOldRunInSystemTray && !mNoSystemTray && Preferences::disableAlarmsIfStopped();
+		mStartOfDay             = Preferences::startOfDay();
+		if (Preferences::hasStartOfDayChanged())
 			mStartOfDay.setHMS(100,0,0);    // start of day time has changed: flag it as invalid
-		mPrefsExpiredColour   = preferences->expiredColour();
-		mPrefsExpiredKeepDays = preferences->expiredKeepDays();
-		mPrefsShowTime        = preferences->showAlarmTime();
-		mPrefsShowTimeTo      = preferences->showTimeToAlarm();
+		mPrefsExpiredColour   = Preferences::expiredColour();
+		mPrefsExpiredKeepDays = Preferences::expiredKeepDays();
+		mPrefsShowTime        = Preferences::showAlarmTime();
+		mPrefsShowTimeTo      = Preferences::showTimeToAlarm();
 	}
+
+	// Check if the speech synthesis daemon is installed
+	mSpeechEnabled = (KTrader::self()->query("DCOP/Text-to-Speech", "Name == 'KTTSD'").count() > 0);
+	if (!mSpeechEnabled)
+		kdDebug(5950) << "KAlarmApp::KAlarmApp(): speech synthesis disabled (KTTSD not found)" << endl;
+	// Check if KOrganizer is installed
+	QString korg = QString::fromLatin1("korganizer");
+	mKOrganizerEnabled = !locate("exe", korg).isNull()  ||  !KStandardDirs::findExe(korg).isNull();
+	if (!mKOrganizerEnabled)
+		kdDebug(5950) << "KAlarmApp::KAlarmApp(): KOrganizer options disabled (KOrganizer not found)" << endl;
 }
 
 /******************************************************************************
 */
 KAlarmApp::~KAlarmApp()
 {
+	while (!mCommandProcesses.isEmpty())
+	{
+		ProcData* pd = mCommandProcesses.first();
+		mCommandProcesses.pop_front();
+		delete pd;
+	}
 	AlarmCalendar::terminateCalendars();
 }
 
@@ -217,7 +231,7 @@ bool KAlarmApp::restoreSession()
 
 	// Try to display the system tray icon if it is configured to be autostarted,
 	// or if we're in run-in-system-tray mode.
-	if (Preferences::instance()->autostartTrayIcon()
+	if (Preferences::autostartTrayIcon()
 	||  MainWindow::count()  &&  wantRunInSystemTray())
 	{
 		displayTrayIcon(true, trayParent);
@@ -369,6 +383,12 @@ int KAlarmApp::newInstance()
 					if (args->isSet("mail"))
 						USAGE(i18n("%1 incompatible with %2").arg(QString::fromLatin1("--mail")).arg(QString::fromLatin1("--exec")))
 					alMessage = args->getOption("exec");
+					int n = args->count();
+					for (int i = 0;  i < n;  ++i)
+					{
+						alMessage += ' ';
+						alMessage += args->arg(i);
+					}
 					action = KAEvent::COMMAND;
 				}
 				else if (args->isSet("mail"))
@@ -412,9 +432,9 @@ int KAlarmApp::newInstance()
 
 				bool      alarmNoTime = false;
 				QDateTime alarmTime, endTime;
-				QColor    bgColour = Preferences::instance()->defaultBgColour();
-				QColor    fgColour = Preferences::instance()->defaultFgColour();
-				KCal::Recurrence recurrence(0);
+				QColor    bgColour = Preferences::defaultBgColour();
+				QColor    fgColour = Preferences::defaultFgColour();
+				KARecurrence recurrence;
 				int       repeatCount    = 0;
 				int       repeatInterval = 0;
 				if (args->isSet("color"))
@@ -457,8 +477,7 @@ int KAlarmApp::newInstance()
 					if (args->isSet("until"))
 						USAGE(i18n("%1 incompatible with %2").arg(QString::fromLatin1("--until")).arg(QString::fromLatin1("--recurrence")))
 					QCString rule = args->getOption("recurrence");
-					KCal::ICalFormat format;
-					format.fromString(&recurrence, QString::fromLocal8Bit((const char*)rule));
+					recurrence.set(QString::fromLocal8Bit(static_cast<const char*>(rule)));
 				}
 				if (args->isSet("interval"))
 				{
@@ -489,17 +508,17 @@ int KAlarmApp::newInstance()
 
 					// Get the recurrence interval
 					int interval;
-					KAEvent::RecurType recurType;
+					KARecurrence::Type recurType;
 					if (!convInterval(args->getOption("interval"), recurType, interval, !haveRecurrence)
 					||  interval < 0)
 						USAGE(i18n("Invalid %1 parameter").arg(QString::fromLatin1("--interval")))
-					if (alarmNoTime  &&  recurType == KAEvent::MINUTELY)
+					if (alarmNoTime  &&  recurType == KARecurrence::MINUTELY)
 						USAGE(i18n("Invalid %1 parameter for date-only alarm").arg(QString::fromLatin1("--interval")))
 
 					if (haveRecurrence)
 					{
 						// There is a also a recurrence specified, so set up a simple repetition
-						int longestInterval = KAEvent::longestRecurrenceInterval(recurrence);
+						int longestInterval = recurrence.longestInterval();
 						if (count * interval > longestInterval)
 							USAGE(i18n("Invalid %1 and %2 parameters: repetition is longer than %3 interval").arg(QString::fromLatin1("--interval")).arg(QString::fromLatin1("--repeat")).arg(QString::fromLatin1("--recurrence")));
 						repeatCount    = count;
@@ -509,7 +528,7 @@ int KAlarmApp::newInstance()
 					{
 						// There is no other recurrence specified, so convert the repetition
 						// parameters into a KCal::Recurrence
-						KAEvent::setRecurrence(recurrence, recurType, interval, count, DateTime(alarmTime, alarmNoTime), endTime);
+						recurrence.set(recurType, interval, count, DateTime(alarmTime, alarmNoTime), endTime);
 					}
 				}
 				else
@@ -534,6 +553,8 @@ int KAlarmApp::newInstance()
 						USAGE(i18n("%1 incompatible with %2").arg(QString::fromLatin1("--play")).arg(QString::fromLatin1("--play-repeat")))
 					if (args->isSet("beep"))
 						USAGE(i18n("%1 incompatible with %2").arg(QString::fromLatin1("--beep")).arg(QString::fromLatin1(audioRepeat ? "--play-repeat" : "--play")))
+					if (args->isSet("speak"))
+						USAGE(i18n("%1 incompatible with %2").arg(QString::fromLatin1("--speak")).arg(QString::fromLatin1(audioRepeat ? "--play-repeat" : "--play")))
 					audioFile = args->getOption(audioRepeat ? "play-repeat" : "play");
 					if (args->isSet("volume"))
 					{
@@ -546,7 +567,13 @@ int KAlarmApp::newInstance()
 				}
 				else if (args->isSet("volume"))
 					USAGE(i18n("%1 requires %2 or %3").arg(QString::fromLatin1("--volume")).arg(QString::fromLatin1("--play")).arg(QString::fromLatin1("--play-repeat")))
-
+				if (args->isSet("speak"))
+				{
+					if (args->isSet("beep"))
+						USAGE(i18n("%1 incompatible with %2").arg(QString::fromLatin1("--beep")).arg(QString::fromLatin1("--speak")))
+					if (!mSpeechEnabled)
+						USAGE(i18n("%1 requires speech synthesis to be configured using KTTSD").arg(QString::fromLatin1("--speak")))
+				}
 				int reminderMinutes = 0;
 				bool onceOnly = args->isSet("reminder-once");
 				if (args->isSet("reminder")  ||  onceOnly)
@@ -559,19 +586,19 @@ int KAlarmApp::newInstance()
 						USAGE(i18n("%1 incompatible with %2").arg(opt).arg(QString::fromLatin1("--exec")))
 					if (args->isSet("mail"))
 						USAGE(i18n("%1 incompatible with %2").arg(opt).arg(QString::fromLatin1("--mail")))
-					KAEvent::RecurType recur;
+					KARecurrence::Type recurType;
 					QString optval = args->getOption(onceOnly ? "reminder-once" : "reminder");
-					bool ok = convInterval(args->getOption(onceOnly ? "reminder-once" : "reminder"), recur, reminderMinutes);
+					bool ok = convInterval(args->getOption(onceOnly ? "reminder-once" : "reminder"), recurType, reminderMinutes);
 					if (ok)
 					{
-						switch (recur)
+						switch (recurType)
 						{
-							case KAEvent::MINUTELY:
+							case KARecurrence::MINUTELY:
 								if (alarmNoTime)
 									USAGE(i18n("Invalid %1 parameter for date-only alarm").arg(opt))
 								break;
-							case KAEvent::DAILY:     reminderMinutes *= 1440;  break;
-							case KAEvent::WEEKLY:    reminderMinutes *= 7*1440;  break;
+							case KARecurrence::DAILY:     reminderMinutes *= 1440;  break;
+							case KARecurrence::WEEKLY:    reminderMinutes *= 7*1440;  break;
 							default:   ok = false;  break;
 						}
 					}
@@ -582,8 +609,8 @@ int KAlarmApp::newInstance()
 				int lateCancel = 0;
 				if (args->isSet("late-cancel"))
 				{
-					KAEvent::RecurType recur;
-					bool ok = convInterval(args->getOption("late-cancel"), recur, lateCancel, false);
+					KARecurrence::Type recurType;
+					bool ok = convInterval(args->getOption("late-cancel"), recurType, lateCancel, false);
 					if (!ok  ||  lateCancel <= 0)
 						USAGE(i18n("Invalid %1 parameter").arg(QString::fromLatin1("late-cancel")))
 				}
@@ -597,6 +624,10 @@ int KAlarmApp::newInstance()
 					flags |= KAEvent::AUTO_CLOSE;
 				if (args->isSet("beep"))
 					flags |= KAEvent::BEEP;
+				if (args->isSet("speak"))
+					flags |= KAEvent::SPEAK;
+				if (args->isSet("korganizer"))
+					flags |= KAEvent::COPY_KORGANIZER;
 				if (args->isSet("disable"))
 					flags |= KAEvent::DISABLED;
 				if (audioRepeat)
@@ -645,6 +676,8 @@ int KAlarmApp::newInstance()
 					usage += QString::fromLatin1("--disable ");
 				if (args->isSet("from-id"))
 					usage += QString::fromLatin1("--from-id ");
+				if (args->isSet("korganizer"))
+					usage += QString::fromLatin1("--korganizer ");
 				if (args->isSet("late-cancel"))
 					usage += QString::fromLatin1("--late-cancel ");
 				if (args->isSet("login"))
@@ -659,6 +692,8 @@ int KAlarmApp::newInstance()
 					usage += QString::fromLatin1("--reminder ");
 				if (args->isSet("reminder-once"))
 					usage += QString::fromLatin1("--reminder-once ");
+				if (args->isSet("speak"))
+					usage += QString::fromLatin1("--speak ");
 				if (args->isSet("subject"))
 					usage += QString::fromLatin1("--subject ");
 				if (args->isSet("time"))
@@ -737,7 +772,7 @@ void KAlarmApp::quitIf(int exitCode, bool force)
 			if (checkSystemTray())
 				return;
 		}
-		if (mDcopQueue.count()  ||  mCommandProcesses.count())
+		if (!mDcopQueue.isEmpty()  ||  !mCommandProcesses.isEmpty())
 		{
 			// Don't quit yet if there are outstanding actions on the DCOP queue
 			mPendingQuit = true;
@@ -853,7 +888,7 @@ void KAlarmApp::processQueue()
 		KAlarm::resetDaemonIfQueued();
 
 		// Process DCOP calls
-		while (mDcopQueue.count())
+		while (!mDcopQueue.isEmpty())
 		{
 			DcopQEntry& entry = mDcopQueue.first();
 			if (entry.eventId.isEmpty())
@@ -1025,7 +1060,6 @@ MainWindow* KAlarmApp::trayMainWindow() const
 */
 void KAlarmApp::slotPreferencesChanged()
 {
-	Preferences* preferences = Preferences::instance();
 	bool newRunInSysTray = wantRunInSystemTray();
 	if (newRunInSysTray != mOldRunInSystemTray)
 	{
@@ -1044,41 +1078,42 @@ void KAlarmApp::slotPreferencesChanged()
 		--mActiveCount;
 	}
 
-	bool newDisableIfStopped = wantRunInSystemTray() && !mNoSystemTray && Preferences::instance()->disableAlarmsIfStopped();
+	bool newDisableIfStopped = wantRunInSystemTray() && !mNoSystemTray && Preferences::disableAlarmsIfStopped();
 	if (newDisableIfStopped != mDisableAlarmsIfStopped)
 	{
 		mDisableAlarmsIfStopped = newDisableIfStopped;    // N.B. this setting is used by Daemon::reregister()
-		preferences->setQuitWarn(true);   // since mode has changed, re-allow warning messages on Quit
+		Preferences::setQuitWarn(true);   // since mode has changed, re-allow warning messages on Quit
 		Daemon::reregister();           // re-register with the alarm daemon
 	}
 
 	// Change alarm times for date-only alarms if the start of day time has changed
-	if (preferences->startOfDay() != mStartOfDay)
+	if (Preferences::startOfDay() != mStartOfDay)
 		changeStartOfDay();
 
-	KAEvent::setFeb29RecurType();    // in case the date for February 29th recurrences has changed
+	// In case the date for February 29th recurrences has changed
+	KARecurrence::setDefaultFeb29Type(Preferences::defaultFeb29Type());
 
-	if (preferences->showAlarmTime()   != mPrefsShowTime
-	||  preferences->showTimeToAlarm() != mPrefsShowTimeTo)
+	if (Preferences::showAlarmTime()   != mPrefsShowTime
+	||  Preferences::showTimeToAlarm() != mPrefsShowTimeTo)
 	{
 		// The default alarm list time columns selection has changed
 		MainWindow::updateTimeColumns(mPrefsShowTime, mPrefsShowTimeTo);
-		mPrefsShowTime   = preferences->showAlarmTime();
-		mPrefsShowTimeTo = preferences->showTimeToAlarm();
+		mPrefsShowTime   = Preferences::showAlarmTime();
+		mPrefsShowTimeTo = Preferences::showTimeToAlarm();
 	}
 
-	if (preferences->expiredColour() != mPrefsExpiredColour)
+	if (Preferences::expiredColour() != mPrefsExpiredColour)
 	{
 		// The expired alarms text colour has changed
 		mRefreshExpiredAlarms = true;
-		mPrefsExpiredColour = preferences->expiredColour();
+		mPrefsExpiredColour = Preferences::expiredColour();
 	}
 
-	if (preferences->expiredKeepDays() != mPrefsExpiredKeepDays)
+	if (Preferences::expiredKeepDays() != mPrefsExpiredKeepDays)
 	{
 		// How long expired alarms are being kept has changed.
 		// N.B. This also adjusts for any change in start-of-day time.
-		mPrefsExpiredKeepDays = preferences->expiredKeepDays();
+		mPrefsExpiredKeepDays = Preferences::expiredKeepDays();
 		AlarmCalendar::expiredCalendar()->setPurgeDays(mPrefsExpiredKeepDays);
 	}
 
@@ -1094,11 +1129,13 @@ void KAlarmApp::slotPreferencesChanged()
 */
 void KAlarmApp::changeStartOfDay()
 {
+	QTime sod = Preferences::startOfDay();
+	DateTime::setStartOfDay(sod);
 	AlarmCalendar* cal = AlarmCalendar::activeCalendar();
 	if (KAEvent::adjustStartOfDay(cal->events()))
 		cal->save();
-	Preferences::instance()->updateStartOfDayCheck();  // now that calendar is updated, set OK flag in config file
-	mStartOfDay = Preferences::instance()->startOfDay();
+	Preferences::updateStartOfDayCheck();  // now that calendar is updated, set OK flag in config file
+	mStartOfDay = sod;
 }
 
 /******************************************************************************
@@ -1116,7 +1153,7 @@ void KAlarmApp::slotExpiredPurged()
 */
 bool KAlarmApp::wantRunInSystemTray() const
 {
-	return Preferences::instance()->runInSystemTray()  &&  mKDEDesktop;
+	return Preferences::runInSystemTray()  &&  mKDEDesktop;
 }
 
 /******************************************************************************
@@ -1127,7 +1164,7 @@ bool KAlarmApp::wantRunInSystemTray() const
 bool KAlarmApp::scheduleEvent(KAEvent::Action action, const QString& text, const QDateTime& dateTime,
                               int lateCancel, int flags, const QColor& bg, const QColor& fg, const QFont& font,
                               const QString& audioFile, float audioVolume, int reminderMinutes,
-                              const KCal::Recurrence& recurrence, int repeatInterval, int repeatCount,
+                              const KARecurrence& recurrence, int repeatInterval, int repeatCount,
                               const QString& mailFromID, const EmailAddressList& mailAddresses,
                               const QString& mailSubject, const QStringList& mailAttachments)
 {
@@ -1148,8 +1185,8 @@ bool KAlarmApp::scheduleEvent(KAEvent::Action action, const QString& text, const
 		event.setReminder((onceOnly ? -reminderMinutes : reminderMinutes), onceOnly);
 	}
 	if (!audioFile.isEmpty())
-		event.setAudioFile(audioFile, audioVolume);
-	if (mailAddresses.count())
+		event.setAudioFile(audioFile, audioVolume, -1, 0);
+	if (!mailAddresses.isEmpty())
 		event.setEmail(mailFromID, mailAddresses, mailSubject, mailAttachments);
 	event.setRecurrence(recurrence);
 	event.setFirstRecurrence();
@@ -1245,10 +1282,14 @@ bool KAlarmApp::handleEvent(const QString& eventID, EventFunc function)
 					displayAlarmValid   = false;
 					updateCalAndDisplay = false;
 				}
+				// Check if the alarm is due yet.
+				// Just in case it's an invalid time during a daylight savings time
+				// shift, check more carefully if it's today.
 				int secs = alarm.dateTime().secsTo(now);
-				if (secs < 0)
+				if (secs < 0
+				&&  (alarm.date() != now.date() || alarm.time() > now.time()))
 				{
-					// This alarm is not due yet
+					// This alarm is definitely not due yet
 					kdDebug(5950) << "KAlarmApp::handleEvent(): alarm " << alarm.type() << ": not due\n";
 					continue;
 				}
@@ -1291,7 +1332,7 @@ bool KAlarmApp::handleEvent(const QString& eventID, EventFunc function)
 					{
 						// The alarm has no time, so cancel it if its date is too far past
 						int maxlate = alarm.lateCancel() / 1440;    // maximum lateness in days
-						QDateTime limit(alarm.date().addDays(maxlate + 1), Preferences::instance()->startOfDay());
+						QDateTime limit(alarm.date().addDays(maxlate + 1), Preferences::startOfDay());
 						if (now >= limit)
 						{
 							// It's too late to display the scheduled occurrence.
@@ -1305,7 +1346,7 @@ bool KAlarmApp::handleEvent(const QString& eventID, EventFunc function)
 								case KAEvent::RECURRENCE_DATE_TIME:
 								case KAEvent::LAST_RECURRENCE:
 									limit.setDate(next.date().addDays(maxlate + 1));
-									limit.setTime(Preferences::instance()->startOfDay());
+									limit.setTime(Preferences::startOfDay());
 									if (now >= limit)
 									{
 										if (type == KAEvent::LAST_RECURRENCE)
@@ -1534,8 +1575,8 @@ void KAlarmApp::cancelAlarm(KAEvent& event, KAAlarm::Type alarmType, bool update
 	if (alarmType == KAAlarm::MAIN_ALARM  &&  !event.displaying()  &&  event.toBeArchived())
 	{
 		// The event is being deleted. Save it in the expired calendar file first.
-		QString id = event.id();    // save event ID since KAlarm::archiveEvent() changes it
-		KAlarm::archiveEvent(event);
+		QString id = event.id();    // save event ID since KAlarm::addExpiredEvent() changes it
+		KAlarm::addExpiredEvent(event);
 		event.setEventID(id);       // restore event ID
 	}
 	event.removeExpiredAlarm(alarmType);
@@ -1636,7 +1677,7 @@ void* KAlarmApp::execAlarm(KAEvent& event, const KAAlarm& alarm, bool reschedule
 			QStringList errmsgs;
 			if (!KAMail::send(event, errmsgs, (reschedule || allowDefer)))
 				result = 0;
-			if (errmsgs.count())
+			if (!errmsgs.isEmpty())
 			{
 				// Some error occurred, although the email may have been sent successfully
 				if (result)
@@ -1656,19 +1697,20 @@ void* KAlarmApp::execAlarm(KAEvent& event, const KAAlarm& alarm, bool reschedule
 }
 
 /******************************************************************************
-* Execute a shell command specified by an alarm.
+* Execute a shell command line specified by an alarm.
 * If the PRE_ACTION bit of 'flags' is set, the alarm will be executed via
 * execAlarm() once the command completes, the execAlarm() parameters being
 * derived from the remaining bits in 'flags'.
 */
 ShellProcess* KAlarmApp::doShellCommand(const QString& command, const KAEvent& event, const KAAlarm* alarm, int flags)
 {
+	KProcess::Communication comms = KProcess::NoCommunication;
 	QString cmd;
 	QString tmpXtermFile;
 	if (flags & ProcData::EXEC_IN_XTERM)
 	{
 		// Execute the command in a terminal window.
-		cmd = Preferences::instance()->cmdXTermCommand();
+		cmd = Preferences::cmdXTermCommand();
 		cmd.replace("%t", aboutData()->programName());     // set the terminal window title
 		if (cmd.find("%C") >= 0)
 		{
@@ -1711,21 +1753,48 @@ ShellProcess* KAlarmApp::doShellCommand(const QString& command, const KAEvent& e
 		}
 	}
 	else
+	{
 		cmd = command;
+		comms = KProcess::AllOutput;
+	}
 	ShellProcess* proc = new ShellProcess(cmd);
 	connect(proc, SIGNAL(shellExited(ShellProcess*)), SLOT(slotCommandExited(ShellProcess*)));
-	ProcData* pd = new ProcData(proc, new KAEvent(event), (alarm ? new KAAlarm(*alarm) : 0), flags);
+	QGuardedPtr<ShellProcess> logproc = 0;
+	if (comms == KProcess::AllOutput  &&  !event.logFile().isEmpty())
+	{
+		// Output is to be appended to a log file.
+		// Set up a logging process to write the command's output to.
+		connect(proc, SIGNAL(receivedStdout(KProcess*,char*,int)), SLOT(slotCommandOutput(KProcess*,char*,int)));
+		connect(proc, SIGNAL(receivedStderr(KProcess*,char*,int)), SLOT(slotCommandOutput(KProcess*,char*,int)));
+		logproc = new ShellProcess(QString::fromLatin1("cat >>%1").arg(event.logFile()));
+		connect(logproc, SIGNAL(shellExited(ShellProcess*)), SLOT(slotLogProcExited(ShellProcess*)));
+		logproc->start(KProcess::Stdin);
+		QCString heading;
+		if (alarm  &&  alarm->dateTime().isValid())
+		{
+			QString dateTime = alarm->dateTime().isDateOnly()
+			                 ? KGlobal::locale()->formatDate(alarm->dateTime().date(), true)
+			                 : KGlobal::locale()->formatDateTime(alarm->dateTime().dateTime());
+			heading.sprintf("\n******* KAlarm %s *******\n", dateTime.latin1());
+		}
+		else
+			heading = "\n******* KAlarm *******\n";
+		logproc->writeStdin(heading, heading.length()+1);
+	}
+	ProcData* pd = new ProcData(proc, logproc, new KAEvent(event), (alarm ? new KAAlarm(*alarm) : 0), flags);
 	if (flags & ProcData::TEMP_FILE)
 		pd->tempFiles += command;
-	pd->tempFiles += tmpXtermFile;
+	if (!tmpXtermFile.isEmpty())
+		pd->tempFiles += tmpXtermFile;
 	mCommandProcesses.append(pd);
-	if (proc->start())
+	if (proc->start(comms))
 		return proc;
 
 	// Error executing command - report it
 	kdError(5950) << "KAlarmApp::doShellCommand(): command failed to start\n";
 	commandErrorMsg(proc, event, alarm, flags);
-	mCommandProcesses.removeRef(pd);
+	mCommandProcesses.remove(pd);
+	delete pd;
 	return 0;
 }
 
@@ -1758,17 +1827,50 @@ QString KAlarmApp::createTempScriptFile(const QString& command, bool insertShell
 }
 
 /******************************************************************************
-* Called when a command alarm execution completes.
+* Called when an executing command alarm sends output to stdout or stderr.
+*/
+void KAlarmApp::slotCommandOutput(KProcess* proc, char* buffer, int bufflen)
+{
+kdDebug(5950) << "KAlarmApp::slotCommandOutput(): '" << QCString(buffer, bufflen+1) << "'\n";
+	// Find this command in the command list
+	for (QValueList<ProcData*>::Iterator it = mCommandProcesses.begin();  it != mCommandProcesses.end();  ++it)
+	{
+		ProcData* pd = *it;
+		if (pd->process == proc  &&  pd->logProcess)
+		{
+			pd->logProcess->writeStdin(buffer, bufflen);
+			break;
+		}
+	}
+}
+
+/******************************************************************************
+* Called when a logging process completes.
+*/
+void KAlarmApp::slotLogProcExited(ShellProcess* proc)
+{
+	// Because it's held as a guarded pointer in the ProcData structure,
+	// we don't need to set any pointers to zero.
+	delete proc;
+}
+
+/******************************************************************************
+* Called when a command alarm's execution completes.
 */
 void KAlarmApp::slotCommandExited(ShellProcess* proc)
 {
 	kdDebug(5950) << "KAlarmApp::slotCommandExited()\n";
 	// Find this command in the command list
-	for (ProcData* pd = mCommandProcesses.first();  pd;  pd = mCommandProcesses.next())
+	for (QValueList<ProcData*>::Iterator it = mCommandProcesses.begin();  it != mCommandProcesses.end();  ++it)
 	{
+		ProcData* pd = *it;
 		if (pd->process == proc)
 		{
-			// Found the command. Check its exit status.
+			// Found the command
+			if (pd->logProcess)
+				pd->logProcess->stdinExit();   // terminate the logging process
+
+			// Check its exit status
 			if (!proc->normalExit())
 			{
 				QString errmsg = proc->errorMessage();
@@ -1792,12 +1894,14 @@ void KAlarmApp::slotCommandExited(ShellProcess* proc)
 			}
 			if (pd->preAction())
 				execAlarm(*pd->event, *pd->alarm, pd->reschedule(), pd->allowDefer(), true);
-			mCommandProcesses.remove();
+			mCommandProcesses.remove(it);
+			delete pd;
+			break;
 		}
 	}
 
-	// Now that there are no executing shell commands, quit if a quit was queued
-	if (mPendingQuit  &&  !mCommandProcesses.count())
+	// If there are now no executing shell commands, quit if a quit was queued
+	if (mPendingQuit  &&  mCommandProcesses.isEmpty())
 		quitIf(mPendingQuitCode);
 }
 
@@ -1823,10 +1927,14 @@ void KAlarmApp::commandErrorMsg(const ShellProcess* proc, const KAEvent& event,
 void KAlarmApp::commandMessage(ShellProcess* proc, QWidget* parent)
 {
 	// Find this command in the command list
-	for (ProcData* pd = mCommandProcesses.first();  pd;  pd = mCommandProcesses.next())
+	for (QValueList<ProcData*>::Iterator it = mCommandProcesses.begin();  it != mCommandProcesses.end();  ++it)
 	{
+		ProcData* pd = *it;
 		if (pd->process == proc)
+		{
 			pd->messageBoxParent = parent;
+			break;
+		}
 	}
 }
 
@@ -1987,7 +2095,7 @@ static bool convWakeTime(const QCString timeParam, QDateTime& dateTime, bool& no
 *  Convert a time interval command line parameter.
 *  Reply = true if successful.
 */
-static bool convInterval(QCString timeParam, KAEvent::RecurType& recurType, int& timeInterval, bool allowMonthYear)
+static bool convInterval(QCString timeParam, KARecurrence::Type& recurType, int& timeInterval, bool allowMonthYear)
 {
 	// Get the recurrence interval
 	bool ok = true;
@@ -2001,15 +2109,15 @@ static bool convInterval(QCString timeParam, KAEvent::RecurType& recurType, int&
 		case 'Y':
 			if (!allowMonthYear)
 				ok = false;
-			recurType = KAEvent::ANNUAL_DATE;
+			recurType = KARecurrence::ANNUAL_DATE;
 			timeParam = timeParam.left(length - 1);
 			break;
 		case 'W':
-			recurType = KAEvent::WEEKLY;
+			recurType = KARecurrence::WEEKLY;
 			timeParam = timeParam.left(length - 1);
 			break;
 		case 'D':
-			recurType = KAEvent::DAILY;
+			recurType = KARecurrence::DAILY;
 			timeParam = timeParam.left(length - 1);
 			break;
 		case 'M':
@@ -2019,19 +2127,19 @@ static bool convInterval(QCString timeParam, KAEvent::RecurType& recurType, int&
 			{
 				if (!allowMonthYear)
 					ok = false;
-				recurType = KAEvent::MONTHLY_DAY;
+				recurType = KARecurrence::MONTHLY_DAY;
 				timeParam = timeParam.left(length - 1);
 			}
 			else
 			{
-				recurType = KAEvent::MINUTELY;
+				recurType = KARecurrence::MINUTELY;
 				interval = timeParam.left(i).toUInt(&ok) * 60;
 				timeParam = timeParam.mid(i + 1, length - i - 2);
 			}
 			break;
 		}
 		default:       // should be a digit
-			recurType = KAEvent::MINUTELY;
+			recurType = KARecurrence::MINUTELY;
 			break;
 	}
 	if (ok)
@@ -2045,7 +2153,7 @@ static bool convInterval(QCString timeParam, KAEvent::RecurType& recurType, int&
 
 KAlarmApp::ProcData::~ProcData()
 {
-	while (tempFiles.count())
+	while (!tempFiles.isEmpty())
 	{
 		// Delete the temporary file called by the XTerm command
 		QFile f(tempFiles.first());
