@@ -1,7 +1,7 @@
 /*
  *  kalarmapp.h  -  the KAlarm application object
  *  Program:  kalarm
- *  (C) 2001 - 2004 by David Jarvie <software@astrojar.org.uk>
+ *  Copyright (C) 2001 - 2005 by David Jarvie <software@astrojar.org.uk>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -13,14 +13,16 @@
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
  *  GNU General Public License for more details.
  *
- *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+ *  You should have received a copy of the GNU General Public License along
+ *  with this program; if not, write to the Free Software Foundation, Inc.,
+ *  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  */
 
 #ifndef KALARMAPP_H
 #define KALARMAPP_H
 
+/** @file kalarmapp.h - the KAlarm application object */
+
 #include <qguardedptr.h>
 class QTimer;
 class QDateTime;
@@ -40,7 +42,6 @@ class MainWindow;
 class AlarmListView;
 class MessageWin;
 class TrayWindow;
-class Preferences;
 class ShellProcess;
 
 
@@ -55,6 +56,8 @@ class KAlarmApp : public KUniqueApplication
 		bool               KDEDesktop() const              { return mKDEDesktop; }
 		bool               wantRunInSystemTray() const;
 		bool               alarmsDisabledIfStopped() const { return mDisableAlarmsIfStopped; }
+		bool               speechEnabled() const           { return mSpeechEnabled; }
+		bool               korganizerEnabled() const       { return mKOrganizerEnabled; }
 		bool               restoreSession();
 		bool               sessionClosingDown() const      { return mSessionClosingDown; }
 		void               quitIf()                        { quitIf(0); }
@@ -78,7 +81,7 @@ class KAlarmApp : public KUniqueApplication
 		bool               scheduleEvent(KAEvent::Action, const QString& text, const QDateTime&,
 		                                 int lateCancel, int flags, const QColor& bg, const QColor& fg,
 		                                 const QFont&, const QString& audioFile, float audioVolume,
-		                                 int reminderMinutes, const KCal::Recurrence& recurrence,
+		                                 int reminderMinutes, const KARecurrence& recurrence,
 						 int repeatInterval, int repeatCount,
 		                                 const QString& mailFromID = QString::null,
 		                                 const EmailAddressList& mailAddresses = EmailAddressList(),
@@ -96,6 +99,8 @@ class KAlarmApp : public KUniqueApplication
 	private slots:
 		void               quitFatal();
 		void               slotPreferencesChanged();
+		void               slotCommandOutput(KProcess*, char* buffer, int bufflen);
+		void               slotLogProcExited(ShellProcess*);
 		void               slotCommandExited(ShellProcess*);
 		void               slotSystemTrayTimer();
 		void               slotExpiredPurged();
@@ -108,8 +113,8 @@ class KAlarmApp : public KUniqueApplication
 		};
 		struct ProcData
 		{
-			ProcData(ShellProcess* p, KAEvent* e, KAAlarm* a, int f = 0)
-			          : process(p), event(e), alarm(a), messageBoxParent(0), flags(f) { }
+			ProcData(ShellProcess* p, ShellProcess* logp, KAEvent* e, KAAlarm* a, int f = 0)
+			          : process(p), logProcess(logp), event(e), alarm(a), messageBoxParent(0), flags(f) { }
 			~ProcData();
 			enum { PRE_ACTION = 0x01, POST_ACTION = 0x02, RESCHEDULE = 0x04, ALLOW_DEFER = 0x08,
 			       TEMP_FILE = 0x10, EXEC_IN_XTERM = 0x20 };
@@ -119,12 +124,13 @@ class KAlarmApp : public KUniqueApplication
 			bool                 allowDefer() const  { return flags & ALLOW_DEFER; }
 			bool                 tempFile() const    { return flags & TEMP_FILE; }
 			bool                 execInXterm() const { return flags & EXEC_IN_XTERM; }
-			ShellProcess*        process;
-			KAEvent*             event;
-			KAAlarm*             alarm;
-			QGuardedPtr<QWidget> messageBoxParent;
-			QStringList          tempFiles;
-			int                  flags;
+			ShellProcess*             process;
+			QGuardedPtr<ShellProcess> logProcess;
+			KAEvent*                  event;
+			KAAlarm*                  alarm;
+			QGuardedPtr<QWidget>      messageBoxParent;
+			QStringList               tempFiles;
+			int                       flags;
 		};
 		struct DcopQEntry
 		{
@@ -163,7 +169,7 @@ class KAlarmApp : public KUniqueApplication
 		QTime                 mStartOfDay;          // start-of-day time currently in use
 		QColor                mPrefsExpiredColour;  // expired alarms text colour
 		int                   mPrefsExpiredKeepDays;// how long expired alarms are being kept
-		QPtrList<ProcData>    mCommandProcesses;    // currently active command alarm processes
+		QValueList<ProcData*> mCommandProcesses;    // currently active command alarm processes
 		QValueList<DcopQEntry> mDcopQueue;          // DCOP command queue
 		int                   mPendingQuitCode;     // exit code for a pending quit
 		bool                  mPendingQuit;         // quit once the DCOP command and shell command queues have been processed
@@ -176,6 +182,8 @@ class KAlarmApp : public KUniqueApplication
 		bool                  mOldRunInSystemTray;  // running continuously in system tray was selected
 		bool                  mDisableAlarmsIfStopped; // disable alarms whenever KAlarm is not running
 		bool                  mRefreshExpiredAlarms; // need to refresh the expired alarms display
+		bool                  mSpeechEnabled;       // speech synthesis is enabled (kttsd exists)
+		bool                  mKOrganizerEnabled;   // KOrganizer options are enabled (korganizer exists)
 		bool                  mPrefsShowTime;       // Preferences setting for show alarm times in alarm list
 		bool                  mPrefsShowTimeTo;     // Preferences setting for show time-to-alarms in alarm list
 };
