@@ -1,7 +1,7 @@
 /*
  *  mainwindow.cpp  -  main application window
  *  Program:  kalarm
- *  (C) 2001 - 2004 by David Jarvie <software@astrojar.org.uk>
+ *  Copyright (c) 2001 - 2005 by David Jarvie <software@astrojar.org.uk>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -13,9 +13,9 @@
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
  *  GNU General Public License for more details.
  *
- *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ *  You should have received a copy of the GNU General Public License along
+ *  with this program; if not, write to the Free Software Foundation, Inc.,
+ *  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  */
 
 #include "kalarm.h"
@@ -28,6 +28,7 @@
 #include <kpopupmenu.h>
 #include <kaccel.h>
 #include <kaction.h>
+#include <kactionclasses.h>
 #include <kstdaction.h>
 #include <kiconloader.h>
 #include <kmessagebox.h>
@@ -40,11 +41,14 @@
 #include <kaboutdata.h>
 #include <dcopclient.h>
 #include <kdebug.h>
-#include <kstdguiitem.h>
 
 #include <libkdepim/maillistdrag.h>
+#include <libkmime/kmime_content.h>
+#include <libkcal/calendarlocal.h>
+#include <libkcal/icaldrag.h>
 
 #include "alarmcalendar.h"
+#include "alarmevent.h"
 #include "alarmlistview.h"
 #include "alarmtext.h"
 #include "birthdaydlg.h"
@@ -64,14 +68,22 @@
 using namespace KCal;
 
 static const char* UI_FILE = "kalarmui.rc";
+QString   undoText;
+QString   undoTextStripped;
+QString   undoIcon;
+KShortcut undoShortcut;
+QString   redoText;
+QString   redoTextStripped;
+QString   redoIcon;
+KShortcut redoShortcut;
 
 
 /*=============================================================================
 =  Class: MainWindow
 =============================================================================*/
 
-QPtrList<MainWindow> MainWindow::mWindowList;
-TemplateDlg*  MainWindow::mTemplateDlg = 0;
+MainWindow::WindowList   MainWindow::mWindowList;
+TemplateDlg*             MainWindow::mTemplateDlg = 0;
 
 // Collect these widget labels together to ensure consistent wording and
 // translations across different modules.
@@ -102,9 +114,9 @@ MainWindow::MainWindow(bool restored)
 	: MainWindowBase(0, 0, WGroupLeader | WStyle_ContextHelp | WDestructiveClose),
 	  mMinuteTimerActive(false),
 	  mHiddenTrayParent(false),
-	  mShowExpired(Preferences::instance()->showExpiredAlarms()),
-	  mShowTime(Preferences::instance()->showAlarmTime()),
-	  mShowTimeTo(Preferences::instance()->showTimeToAlarm())
+	  mShowExpired(Preferences::showExpiredAlarms()),
+	  mShowTime(Preferences::showAlarmTime()),
+	  mShowTimeTo(Preferences::showTimeToAlarm())
 {
 	kdDebug(5950) << "MainWindow::MainWindow()\n";
 	setAutoSaveSettings(QString::fromLatin1("MainWindow"));    // save window sizes etc.
@@ -148,8 +160,7 @@ MainWindow::MainWindow(bool restored)
 MainWindow::~MainWindow()
 {
 	kdDebug(5950) << "MainWindow::~MainWindow()\n";
-	if (findWindow(this))
-		mWindowList.remove();
+	mWindowList.remove(this);
 	if (theApp()->trayWindow())
 	{
 		if (isTrayParent())
@@ -163,6 +174,9 @@ MainWindow::~MainWindow()
 	MainWindow* main = mainMainWindow();
 	if (main)
 		KAlarm::writeConfigWindowSize("MainWindow", main->size());
+	KToolBar* tb = toolBar();
+	if (tb)
+		tb->saveSettings(KGlobal::config(), "Toolbars");
 	KGlobal::config()->sync();    // save any new window size to disc
 	theApp()->quitIf();
 }
@@ -202,11 +216,13 @@ MainWindow* MainWindow::mainMainWindow()
 	MainWindow* tray = theApp()->trayWindow() ? theApp()->trayWindow()->assocMainWindow() : 0;
 	if (tray  &&  tray->isVisible())
 		return tray;
-	for (MainWindow* w = mWindowList.first();  w;  w = mWindowList.next())
-		if (w->isVisible())
-			return w;
+	for (WindowList::Iterator it = mWindowList.begin();  it != mWindowList.end();  ++it)
+		if ((*it)->isVisible())
+			return *it;
 	if (tray)
 		return tray;
+	if (!mWindowList.count())
+		return 0;
 	return mWindowList.first();
 }
 
@@ -224,7 +240,7 @@ bool MainWindow::isTrayParent() const
 void MainWindow::closeAll()
 {
 	while (mWindowList.first())
-		delete mWindowList.first();
+		delete mWindowList.first();    // N.B. the destructor removes the window from the list
 }
 
 /******************************************************************************
@@ -284,29 +300,48 @@ void MainWindow::hideEvent(QHideEvent* he)
 void MainWindow::initActions()
 {
 	KActionCollection* actions = actionCollection();
-	mActionTemplates      = new KAction(i18n("&Templates..."), 0, this, SLOT(slotTemplates()), actions, "templates");
-	mActionNew            = KAlarm::createNewAlarmAction(i18n("&New..."), this, SLOT(slotNew()), actions, "new");
-#ifdef NEW_FROM_TEMPLATE
-	mActionNewFromTemplate = KAlarm::createNewFromTemplateAction(i18n("New &From Template"), actions, "newFromTempl");
-#endif
-	mActionCreateTemplate = new KAction(i18n("Create Tem&plate..."), 0, this, SLOT(slotNewTemplate()), actions, "createTemplate");
-	mActionCopy           = new KAction(i18n("&Copy..."), "editcopy", Qt::SHIFT+Qt::Key_Insert, this, SLOT(slotCopy()), actions, "copy");
-	mActionModify         = new KAction(i18n("&Edit..."), "edit", Qt::CTRL+Qt::Key_E, this, SLOT(slotModify()), actions, "modify");
-	mActionDelete         = new KAction(i18n("&Delete"), "editdelete", Qt::Key_Delete, this, SLOT(slotDelete()), actions, "delete");
-	mActionUndelete       = new KAction(i18n("Reac&tivate"), 0, Qt::CTRL+Qt::Key_R, this, SLOT(slotUndelete()), actions, "undelete");
-	mActionEnable         = new KAction(QString::null, 0, Qt::CTRL+Qt::Key_B, this, SLOT(slotEnable()), actions, "disable");
-	mActionView           = new KAction(i18n("&View"), "viewmag", Qt::CTRL+Qt::Key_W, this, SLOT(slotView()), actions, "view");
-	mActionShowTime       = new KToggleAction(i18n_a_ShowAlarmTimes(), Qt::CTRL+Qt::Key_M, this, SLOT(slotShowTime()), actions, "showAlarmTimes");
+	mActionTemplates       = new KAction(i18n("&Templates..."), 0, this, SLOT(slotTemplates()), actions, "templates");
+	mActionNew             = KAlarm::createNewAlarmAction(i18n("&New..."), this, SLOT(slotNew()), actions, "new");
+	mActionNewFromTemplate = KAlarm::createNewFromTemplateAction(i18n("New &From Template"), this, SLOT(slotNewFromTemplate(const KAEvent&)), actions, "newFromTempl");
+	mActionCreateTemplate  = new KAction(i18n("Create Tem&plate..."), 0, this, SLOT(slotNewTemplate()), actions, "createTemplate");
+	mActionCopy            = new KAction(i18n("&Copy..."), "editcopy", Qt::SHIFT+Qt::Key_Insert, this, SLOT(slotCopy()), actions, "copy");
+	mActionModify          = new KAction(i18n("&Edit..."), "edit", Qt::CTRL+Qt::Key_E, this, SLOT(slotModify()), actions, "modify");
+	mActionDelete          = new KAction(i18n("&Delete"), "editdelete", Qt::Key_Delete, this, SLOT(slotDelete()), actions, "delete");
+	mActionReactivate      = new KAction(i18n("Reac&tivate"), 0, Qt::CTRL+Qt::Key_R, this, SLOT(slotReactivate()), actions, "undelete");
+	mActionEnable          = new KAction(QString::null, 0, Qt::CTRL+Qt::Key_B, this, SLOT(slotEnable()), actions, "disable");
+	mActionView            = new KAction(i18n("&View"), "viewmag", Qt::CTRL+Qt::Key_W, this, SLOT(slotView()), actions, "view");
+	mActionShowTime        = new KToggleAction(i18n_a_ShowAlarmTimes(), Qt::CTRL+Qt::Key_M, this, SLOT(slotShowTime()), actions, "showAlarmTimes");
 	mActionShowTime->setCheckedState(i18n("Hide &Alarm Times"));
-	mActionShowTimeTo     = new KToggleAction(i18n_o_ShowTimeToAlarms(), Qt::CTRL+Qt::Key_I, this, SLOT(slotShowTimeTo()), actions, "showTimeToAlarms");
+	mActionShowTimeTo      = new KToggleAction(i18n_o_ShowTimeToAlarms(), Qt::CTRL+Qt::Key_I, this, SLOT(slotShowTimeTo()), actions, "showTimeToAlarms");
 	mActionShowTimeTo->setCheckedState(i18n("Hide Time t&o Alarms"));
-	mActionShowExpired    = new KToggleAction(i18n_e_ShowExpiredAlarms(), "history", Qt::CTRL+Qt::Key_P, this, SLOT(slotShowExpired()), actions, "showExpiredAlarms");
+	mActionShowExpired     = new KToggleAction(i18n_e_ShowExpiredAlarms(), "history", Qt::CTRL+Qt::Key_P, this, SLOT(slotShowExpired()), actions, "showExpiredAlarms");
 	mActionShowExpired->setCheckedState(i18n_e_HideExpiredAlarms());
-	mActionToggleTrayIcon = new KToggleAction(i18n("Show in System &Tray"), Qt::CTRL+Qt::Key_Y, this, SLOT(slotToggleTrayIcon()), actions, "showInSystemTray");
+	mActionToggleTrayIcon  = new KToggleAction(i18n("Show in System &Tray"), Qt::CTRL+Qt::Key_Y, this, SLOT(slotToggleTrayIcon()), actions, "showInSystemTray");
 	mActionToggleTrayIcon->setCheckedState(i18n("Hide From System &Tray"));
 	new KAction(i18n("Import &Birthdays..."), 0, this, SLOT(slotBirthdays()), actions, "importBirthdays");
 	new KAction(i18n("&Refresh Alarms"), "reload", 0, this, SLOT(slotResetDaemon()), actions, "refreshAlarms");
 	Daemon::createAlarmEnableAction(actions, "alarmEnable");
+	if (undoText.isNull())
+	{
+		// Get standard texts, etc., for Undo and Redo actions
+		KAction* act = KStdAction::undo(this, 0, actions);
+		undoIcon         = act->icon();
+		undoShortcut     = act->shortcut();
+		undoText         = act->text();
+		undoTextStripped = KAlarm::stripAccel(undoText);
+		delete act;
+		act = KStdAction::redo(this, 0, actions);
+		redoIcon         = act->icon();
+		redoShortcut     = act->shortcut();
+		redoText         = act->text();
+		redoTextStripped = KAlarm::stripAccel(redoText);
+		delete act;
+	}
+	mActionUndo = new KToolBarPopupAction(undoText, undoIcon, undoShortcut, this, SLOT(slotUndo()), actions, "edit_undo");
+	mActionRedo = new KToolBarPopupAction(redoText, redoIcon, redoShortcut, this, SLOT(slotRedo()), actions, "edit_redo");
+	KStdAction::find(mListView, SLOT(slotFind()), actions);
+	mActionFindNext = KStdAction::findNext(mListView, SLOT(slotFindNext()), actions);
+	mActionFindPrev = KStdAction::findPrev(mListView, SLOT(slotFindPrev()), actions);
 	KStdAction::quit(this, SLOT(slotQuit()), actions);
 	KStdAction::keyBindings(this, SLOT(slotConfigureKeys()), actions);
 	KStdAction::configureToolbars(this, SLOT(slotConfigureToolbar()), actions);
@@ -318,7 +353,13 @@ void MainWindow::initActions()
 	mActionsMenu = static_cast<KPopupMenu*>(factory()->container("actions", this));
 	mMenuError = (!mContextMenu  ||  !mActionsMenu);
 	connect(mActionsMenu, SIGNAL(aboutToShow()), SLOT(updateActionsMenu()));
-	connect(Preferences::instance(), SIGNAL(preferencesChanged()), SLOT(updateTrayIconAction()));
+	connect(mActionUndo->popupMenu(), SIGNAL(aboutToShow()), SLOT(slotInitUndoMenu()));
+	connect(mActionUndo->popupMenu(), SIGNAL(activated(int)), SLOT(slotUndoItem(int)));
+	connect(mActionRedo->popupMenu(), SIGNAL(aboutToShow()), SLOT(slotInitRedoMenu()));
+	connect(mActionRedo->popupMenu(), SIGNAL(activated(int)), SLOT(slotRedoItem(int)));
+	connect(Undo::instance(), SIGNAL(changed(const QString&, const QString&)), SLOT(slotUndoStatus(const QString&, const QString&)));
+	connect(mListView, SIGNAL(findActive(bool)), SLOT(slotFindActive(bool)));
+	Preferences::connect(SIGNAL(preferencesChanged()), this, SLOT(updateTrayIconAction()));
 	connect(theApp(), SIGNAL(trayIconToggled()), SLOT(updateTrayIconAction()));
 
 	// Set menu item states
@@ -326,18 +367,27 @@ void MainWindow::initActions()
 	mActionShowTime->setChecked(mShowTime);
 	mActionShowTimeTo->setChecked(mShowTimeTo);
 	mActionShowExpired->setChecked(mShowExpired);
-	if (!Preferences::instance()->expiredKeepDays())
+	if (!Preferences::expiredKeepDays())
 		mActionShowExpired->setEnabled(false);
 	updateTrayIconAction();         // set the correct text for this action
+	mActionUndo->setEnabled(Undo::haveUndo());
+	mActionRedo->setEnabled(Undo::haveRedo());
+	mActionFindNext->setEnabled(false);
+	mActionFindPrev->setEnabled(false);
 
 	mActionCopy->setEnabled(false);
 	mActionModify->setEnabled(false);
 	mActionDelete->setEnabled(false);
-	mActionUndelete->setEnabled(false);
+	mActionReactivate->setEnabled(false);
 	mActionView->setEnabled(false);
 	mActionEnable->setEnabled(false);
 	mActionCreateTemplate->setEnabled(false);
 
+	KToolBar* tb = toolBar();
+	if (tb)
+		tb->applySettings(KGlobal::config(), "Toolbars");
+
+	Undo::emitChanged();     // set the Undo/Redo menu texts
 	Daemon::checkStatus();
 	Daemon::monitoringAlarms();
 }
@@ -347,8 +397,8 @@ void MainWindow::initActions()
 */
 void MainWindow::enableTemplateMenuItem(bool enable)
 {
-	for (MainWindow* w = mWindowList.first();  w;  w = mWindowList.next())
-		w->mActionTemplates->setEnabled(enable);
+	for (WindowList::Iterator it = mWindowList.begin();  it != mWindowList.end();  ++it)
+		(*it)->mActionTemplates->setEnabled(enable);
 }
 
 /******************************************************************************
@@ -357,8 +407,8 @@ void MainWindow::enableTemplateMenuItem(bool enable)
 void MainWindow::refresh()
 {
 	kdDebug(5950) << "MainWindow::refresh()\n";
-	for (MainWindow* w = mWindowList.first();  w;  w = mWindowList.next())
-		w->mListView->refresh();
+	for (WindowList::Iterator it = mWindowList.begin();  it != mWindowList.end();  ++it)
+		(*it)->mListView->refresh();
 }
 
 /******************************************************************************
@@ -369,9 +419,10 @@ void MainWindow::refresh()
 void MainWindow::updateExpired()
 {
 	kdDebug(5950) << "MainWindow::updateExpired()\n";
-	bool enableShowExpired = !!Preferences::instance()->expiredKeepDays();
-	for (MainWindow* w = mWindowList.first();  w;  w = mWindowList.next())
+	bool enableShowExpired = Preferences::expiredKeepDays();
+	for (WindowList::Iterator it = mWindowList.begin();  it != mWindowList.end();  ++it)
 	{
+		MainWindow* w = *it;
 		if (w->mShowExpired)
 		{
 			if (!enableShowExpired)
@@ -392,16 +443,17 @@ void MainWindow::updateExpired()
 void MainWindow::updateTimeColumns(bool oldTime, bool oldTimeTo)
 {
 	kdDebug(5950) << "MainWindow::updateShowAlarmTimes()\n";
-	bool newTime   = Preferences::instance()->showAlarmTime();
-	bool newTimeTo = Preferences::instance()->showTimeToAlarm();
+	bool newTime   = Preferences::showAlarmTime();
+	bool newTimeTo = Preferences::showTimeToAlarm();
 	if (!newTime  &&  !newTimeTo)
 		newTime = true;     // at least one time column must be displayed
 	if (!oldTime  &&  !oldTimeTo)
 		oldTime = true;     // at least one time column must have been displayed
 	if (newTime != oldTime  ||  newTimeTo != oldTimeTo)
 	{
-		for (MainWindow* w = mWindowList.first();  w;  w = mWindowList.next())
+		for (WindowList::Iterator it = mWindowList.begin();  it != mWindowList.end();  ++it)
 		{
+			MainWindow* w = *it;
 			if (w->mShowTime   == oldTime
 			&&  w->mShowTimeTo == oldTimeTo)
 			{
@@ -426,8 +478,9 @@ void MainWindow::setUpdateTimer()
 	// Check whether any windows need to be updated
 	MainWindow* needTimer = 0;
 	MainWindow* timerWindow = 0;
-	for (MainWindow* w = mWindowList.first();  w;  w = mWindowList.next())
+	for (WindowList::Iterator it = mWindowList.begin();  it != mWindowList.end();  ++it)
 	{
+		MainWindow* w = *it;
 		if (w->isVisible()  &&  w->mListView->showingTimeTo())
 			needTimer = w;
 		if (w->mMinuteTimerActive)
@@ -455,9 +508,12 @@ void MainWindow::setUpdateTimer()
 void MainWindow::slotUpdateTimeTo()
 {
 	kdDebug(5950) << "MainWindow::slotUpdateTimeTo()" << endl;
-	for (MainWindow* w = mWindowList.first();  w;  w = mWindowList.next())
+	for (WindowList::Iterator it = mWindowList.begin();  it != mWindowList.end();  ++it)
+	{
+		MainWindow* w = *it;
 		if (w->isVisible()  &&  w->mListView->showingTimeTo())
 			w->mListView->updateTimeToAlarms();
+	}
 }
 
 /******************************************************************************
@@ -484,11 +540,12 @@ void MainWindow::slotNew()
 }
 
 /******************************************************************************
-*  Execute a New Alarm dialog, optionally setting the action and text.
+*  Execute a New Alarm dialog, optionally either presetting it to the supplied
+*  event, or setting the action and text.
 */
-void MainWindow::executeNew(MainWindow* win, KAEvent::Action action, const AlarmText& text)
+void MainWindow::executeNew(MainWindow* win, const KAEvent* evnt, KAEvent::Action action, const AlarmText& text)
 {
-	EditAlarmDlg editDlg(false, i18n("New Alarm"), win, "editDlg");
+	EditAlarmDlg editDlg(false, i18n("New Alarm"), win, "editDlg", evnt);
 	if (!text.isEmpty())
 		editDlg.setAction(action, text);
 	if (editDlg.exec() == QDialog::Accepted)
@@ -497,19 +554,30 @@ void MainWindow::executeNew(MainWindow* win, KAEvent::Action action, const Alarm
 		editDlg.getEvent(event);
 
 		// Add the alarm to the displayed lists and to the calendar file
-		KAlarm::addEvent(event, (win ? win->mListView : 0));
+		if (KAlarm::addEvent(event, (win ? win->mListView : 0)) == KAlarm::UPDATE_KORG_ERR)
+			KAlarm::displayUpdateError(win, KAlarm::KORG_ERR_ADD);
+		Undo::saveAdd(event);
 
-		alarmWarnings(&editDlg, &event);
+		KAlarm::outputAlarmWarnings(&editDlg, &event);
 	}
 }
 
+/******************************************************************************
+*  Called when a template is selected from the New From Template popup menu.
+*  Executes a New Alarm dialog, preset from the selected template.
+*/
+void MainWindow::slotNewFromTemplate(const KAEvent& tmplate)
+{
+	executeNew(this, &tmplate);
+}
+
 /******************************************************************************
 *  Called when the New Template button is clicked to create a new template
 *  based on the currently selected alarm.
 */
 void MainWindow::slotNewTemplate()
 {
-	AlarmListViewItem* item = mListView->singleSelectedItem();
+	AlarmListViewItem* item = mListView->selectedItem();
 	if (item)
 	{
 		KAEvent event = item->event();
@@ -523,22 +591,9 @@ void MainWindow::slotNewTemplate()
 */
 void MainWindow::slotCopy()
 {
-	AlarmListViewItem* item = mListView->singleSelectedItem();
+	AlarmListViewItem* item = mListView->selectedItem();
 	if (item)
-	{
-		KAEvent event = item->event();
-		EditAlarmDlg editDlg(false, i18n("New Alarm"), this, "editDlg", &event);
-		if (editDlg.exec() == QDialog::Accepted)
-		{
-			KAEvent event;
-			editDlg.getEvent(event);
-
-			// Add the alarm to the displayed lists and to the calendar file
-			KAlarm::addEvent(event, mListView);
-
-			alarmWarnings(&editDlg, &event);
-		}
-	}
+		executeNew(this, &item->event());
 }
 
 /******************************************************************************
@@ -547,7 +602,7 @@ void MainWindow::slotCopy()
 */
 void MainWindow::slotModify()
 {
-	AlarmListViewItem* item = mListView->singleSelectedItem();
+	AlarmListViewItem* item = mListView->selectedItem();
 	if (item)
 	{
 		KAEvent event = item->event();
@@ -559,11 +614,18 @@ void MainWindow::slotModify()
 
 			// Update the event in the displays and in the calendar file
 			if (changeDeferral)
+			{
+				// The only change has been to an existing deferral
 				KAlarm::updateEvent(newEvent, mListView, true, false);   // keep the same event ID
+			}
 			else
-				KAlarm::modifyEvent(event, newEvent, mListView);
+			{
+				if (KAlarm::modifyEvent(event, newEvent, mListView) == KAlarm::UPDATE_KORG_ERR)
+					KAlarm::displayUpdateError(this, KAlarm::KORG_ERR_MODIFY);
+			}
+			Undo::saveEdit(event, newEvent);
 
-			alarmWarnings(&editDlg, &newEvent);
+			KAlarm::outputAlarmWarnings(&editDlg, &newEvent);
 		}
 	}
 }
@@ -574,11 +636,12 @@ void MainWindow::slotModify()
 */
 void MainWindow::slotView()
 {
-	AlarmListViewItem* item = mListView->singleSelectedItem();
+	AlarmListViewItem* item = mListView->selectedItem();
 	if (item)
 	{
 		KAEvent event = item->event();
-		EditAlarmDlg editDlg(false, (event.expired() ? i18n("Expired Alarm") : i18n("View Alarm")),
+		EditAlarmDlg editDlg(false, (event.expired() ? i18n("Expired Alarm") + " [" + i18n("read-only") + "]"
+		                                             : i18n("View Alarm")),
 		                     this, "editDlg", &event, true);
 		editDlg.exec();
 	}
@@ -591,7 +654,7 @@ void MainWindow::slotView()
 void MainWindow::slotDelete()
 {
 	QValueList<EventListViewItemBase*> items = mListView->selectedItems();
-	if (Preferences::instance()->confirmAlarmDeletion())
+	if (Preferences::confirmAlarmDeletion())
 	{
 		int n = items.count();
 		if (KMessageBox::warningContinueCancel(this, i18n("Do you really want to delete the selected alarm?",
@@ -602,6 +665,9 @@ void MainWindow::slotDelete()
 		    != KMessageBox::Continue)
 			return;
 	}
+
+	int warnKOrg = 0;
+	QValueList<KAEvent> events;
 	AlarmCalendar::activeCalendar()->startUpdate();    // prevent multiple saves of the calendars until we're finished
 	AlarmCalendar::expiredCalendar()->startUpdate();
 	for (QValueList<EventListViewItemBase*>::Iterator it = items.begin();  it != items.end();  ++it)
@@ -610,33 +676,45 @@ void MainWindow::slotDelete()
 		KAEvent event = item->event();
 
 		// Delete the event from the calendar and displays
-		KAlarm::deleteEvent(event);
+		events.append(event);
+		if (KAlarm::deleteEvent(event) == KAlarm::UPDATE_KORG_ERR)
+			++warnKOrg;
 	}
 	AlarmCalendar::activeCalendar()->endUpdate();      // save the calendars now
 	AlarmCalendar::expiredCalendar()->endUpdate();
+	Undo::saveDeletes(events);
+
+	if (warnKOrg)
+		KAlarm::displayUpdateError(this, KAlarm::KORG_ERR_DELETE, (warnKOrg > 1));
 }
 
 /******************************************************************************
-*  Called when the Undelete button is clicked to reinstate the currently
+*  Called when the Reactivate button is clicked to reinstate the currently
 *  highlighted expired alarms in the list.
 */
-void MainWindow::slotUndelete()
+void MainWindow::slotReactivate()
 {
+	int warnKOrg = 0;
+	QValueList<KAEvent> events;
 	QValueList<EventListViewItemBase*> items = mListView->selectedItems();
 	mListView->clearSelection();
 	AlarmCalendar::activeCalendar()->startUpdate();    // prevent multiple saves of the calendars until we're finished
 	AlarmCalendar::expiredCalendar()->startUpdate();
 	for (QValueList<EventListViewItemBase*>::Iterator it = items.begin();  it != items.end();  ++it)
 	{
+		// Add the alarm to the displayed lists and to the calendar file
 		AlarmListViewItem* item = (AlarmListViewItem*)(*it);
 		KAEvent event = item->event();
-		event.setArchive();    // ensure that it gets re-archived if it is deleted
-
-		// Add the alarm to the displayed lists and to the calendar file
-		KAlarm::undeleteEvent(event, mListView);
+		events.append(event);
+		if (KAlarm::reactivateEvent(event, mListView, true) == KAlarm::UPDATE_KORG_ERR)
+			++warnKOrg;
 	}
 	AlarmCalendar::activeCalendar()->endUpdate();      // save the calendars now
 	AlarmCalendar::expiredCalendar()->endUpdate();
+	Undo::saveReactivates(events);
+
+	if (warnKOrg)
+		KAlarm::displayUpdateError(this, KAlarm::KORG_ERR_ADD, (warnKOrg > 1));
 }
 
 /******************************************************************************
@@ -711,9 +789,16 @@ void MainWindow::slotBirthdays()
 		if (events.count())
 		{
 			mListView->clearSelection();
+			int warnKOrg = 0;
 			for (QValueList<KAEvent>::Iterator ev = events.begin();  ev != events.end();  ++ev)
-				KAlarm::addEvent(*ev, mListView);    // add alarm to the displayed lists and to the calendar file
-			alarmWarnings(&dlg);
+			{
+				// Add alarm to the displayed lists and to the calendar file
+				if (KAlarm::addEvent(*ev, mListView) == KAlarm::UPDATE_KORG_ERR)
+					++warnKOrg;
+			}
+			if (warnKOrg)
+				KAlarm::displayUpdateError(this, KAlarm::KORG_ERR_ADD, (warnKOrg > 1));
+			KAlarm::outputAlarmWarnings(&dlg);
 		}
 	}
 }
@@ -774,6 +859,109 @@ void MainWindow::updateActionsMenu()
 	Daemon::checkStatus();   // update the Alarms Enabled item status
 }
 
+/******************************************************************************
+*  Called when the active status of Find changes.
+*/
+void MainWindow::slotFindActive(bool active)
+{
+	mActionFindNext->setEnabled(active);
+	mActionFindPrev->setEnabled(active);
+}
+
+/******************************************************************************
+*  Called when the Undo action is selected.
+*/
+void MainWindow::slotUndo()
+{
+	Undo::undo(this, KAlarm::stripAccel(mActionUndo->text()));
+}
+
+/******************************************************************************
+*  Called when the Redo action is selected.
+*/
+void MainWindow::slotRedo()
+{
+	Undo::redo(this, KAlarm::stripAccel(mActionRedo->text()));
+}
+
+/******************************************************************************
+*  Called when an Undo item is selected.
+*/
+void MainWindow::slotUndoItem(int id)
+{
+	Undo::undo(id, this, Undo::actionText(Undo::UNDO, id));
+}
+
+/******************************************************************************
+*  Called when a Redo item is selected.
+*/
+void MainWindow::slotRedoItem(int id)
+{
+	Undo::redo(id, this, Undo::actionText(Undo::REDO, id));
+}
+
+/******************************************************************************
+*  Called when the Undo menu is about to show.
+*  Populates the menu.
+*/
+void MainWindow::slotInitUndoMenu()
+{
+	initUndoMenu(mActionUndo->popupMenu(), Undo::UNDO);
+}
+
+/******************************************************************************
+*  Called when the Redo menu is about to show.
+*  Populates the menu.
+*/
+void MainWindow::slotInitRedoMenu()
+{
+	initUndoMenu(mActionRedo->popupMenu(), Undo::REDO);
+}
+
+/******************************************************************************
+*  Populate the undo or redo menu.
+*/
+void MainWindow::initUndoMenu(KPopupMenu* menu, Undo::Type type)
+{
+	menu->clear();
+	const QString& action = (type == Undo::UNDO) ? undoTextStripped : redoTextStripped;
+	QValueList<int> ids = Undo::ids(type);
+	for (QValueList<int>::ConstIterator it = ids.begin();  it != ids.end();  ++it)
+	{
+		int id = *it;
+		menu->insertItem(i18n("Undo [action]: message", "%1 %2: %3")
+		                       .arg(action).arg(Undo::actionText(type, id)).arg(Undo::description(type, id)), id);
+	}
+}
+
+/******************************************************************************
+*  Called when the status of the Undo or Redo list changes.
+*  Change the Undo or Redo text to include the action which would be undone/redone.
+*/
+void MainWindow::slotUndoStatus(const QString& undo, const QString& redo)
+{
+	if (undo.isNull())
+	{
+		mActionUndo->setEnabled(false);
+		mActionUndo->setText(undoText);
+	}
+	else
+	{
+		mActionUndo->setEnabled(true);
+		mActionUndo->setText(QString("%1 %2").arg(undoText).arg(undo));
+	}
+	if (redo.isNull())
+	{
+		mActionRedo->setEnabled(false);
+		mActionRedo->setText(redoText);
+	}
+	else
+	{
+		mActionRedo->setEnabled(true);
+		mActionRedo->setText(QString("%1 %2").arg(redoText).arg(redo));
+	}
+}
+
 /******************************************************************************
 *  Called when the Reset Daemon menu item is selected.
 */
@@ -806,9 +994,20 @@ void MainWindow::slotConfigureToolbar()
 {
 	saveMainWindowSettings(KGlobal::config(), "MainWindow");
 	KEditToolbar dlg(factory());
+	connect(&dlg, SIGNAL(newToolbarConfig()), this, SLOT(slotNewToolbarConfig()));
 	dlg.exec();
 }
 
+/******************************************************************************
+*  Called when OK or Apply is clicked in the Configure Toolbars dialog, to save
+*  the new configuration.
+*/
+void MainWindow::slotNewToolbarConfig()
+{
+	createGUI(UI_FILE);
+	applyMainWindowSettings(KGlobal::config(), "MainWindow");
+}
+
 /******************************************************************************
 *  Called when the Quit menu item is selected.
 */
@@ -849,7 +1048,7 @@ void MainWindow::slotDeletion()
 		mActionModify->setEnabled(false);
 		mActionView->setEnabled(false);
 		mActionDelete->setEnabled(false);
-		mActionUndelete->setEnabled(false);
+		mActionReactivate->setEnabled(false);
 		mActionEnable->setEnabled(false);
 	}
 }
@@ -868,9 +1067,12 @@ void MainWindow::dragEnterEvent(QDragEnterEvent* e)
 */
 void MainWindow::executeDragEnterEvent(QDragEnterEvent* e)
 {
-	e->accept(QTextDrag::canDecode(e)
-	       || KURLDrag::canDecode(e)
-	       || KPIM::MailListDrag::canDecode(e));
+	if (KCal::ICalDrag::canDecode(e))
+		e->accept(!AlarmListView::dragging());   // don't accept "text/calendar" objects from KAlarm
+	else
+		e->accept(QTextDrag::canDecode(e)
+		       || KURLDrag::canDecode(e)
+		       || KPIM::MailListDrag::canDecode(e));
 }
 
 /******************************************************************************
@@ -882,6 +1084,12 @@ void MainWindow::dropEvent(QDropEvent* e)
 	executeDropEvent(this, e);
 }
 
+static QString getMailHeader(const char* header, KMime::Content& content)
+{
+	KMime::Headers::Base* hd = content.getHeaderByType(header);
+	return hd ? hd->asUnicodeString() : QString::null;
+}
+
 /******************************************************************************
 *  Called when an object is dropped on a main or system tray window, to
 *  evaluate the action required and extract the text.
@@ -889,37 +1097,97 @@ void MainWindow::dropEvent(QDropEvent* e)
 void MainWindow::executeDropEvent(MainWindow* win, QDropEvent* e)
 {
 	KAEvent::Action action = KAEvent::MESSAGE;
-	QString text;
-	AlarmText alarmText;
+	QString        text;
+	QByteArray     bytes;
+	AlarmText      alarmText;
 	KPIM::MailList mailList;
-	KURL::List files;
-	if (KURLDrag::decode(e, files)  &&  files.count())
+	KURL::List     files;
+	KCal::CalendarLocal calendar(QString::fromLatin1("UTC"));
+	calendar.setLocalTime();    // default to local time (i.e. no time zone)
+#ifndef NDEBUG
+	QCString fmts;
+	for (int idbg = 0;  e->format(idbg);  ++idbg)
 	{
+		if (idbg) fmts += ", ";
+		fmts += e->format(idbg);
+	}
+	kdDebug(5950) << "MainWindow::executeDropEvent(): " << fmts << endl;
+#endif
+
+	/* The order of the tests below matters, since some dropped objects
+	 * provide more than one mime type.
+	 * Don't change them without careful thought !!
+	 */
+	if (e->provides("message/rfc822")
+	&&  !(bytes = e->encodedData("message/rfc822")).isEmpty())
+	{
+		// Email message(s). Ignore all but the first.
+		kdDebug(5950) << "MainWindow::executeDropEvent(email)" << endl;
+		QCString mails(bytes.data(), bytes.size());
+		KMime::Content content;
+		content.setContent(mails);
+		content.parse();
+		QString body;
+		content.textContent()->decodedText(body, true, true);    // strip trailing newlines & spaces
+		unsigned long sernum = 0;
+		if (e->provides(KPIM::MailListDrag::format())
+		&&  KPIM::MailListDrag::decode(e, mailList)
+		&&  mailList.count())
+		{
+			// Get its KMail serial number to allow the KMail message
+			// to be called up from the alarm message window.
+			sernum = mailList.first().serialNumber();
+		}
+		alarmText.setEmail(getMailHeader("To", content),
+		                   getMailHeader("From", content),
+		                   getMailHeader("Cc", content),
+		                   getMailHeader("Date", content),
+		                   getMailHeader("Subject", content),
+				   body, sernum);
+	}
+	else if (KURLDrag::decode(e, files)  &&  files.count())
+	{
+		kdDebug(5950) << "MainWindow::executeDropEvent(URL)" << endl;
 		action = KAEvent::FILE;
 		alarmText.setText(files.first().prettyURL());
 	}
 	else if (e->provides(KPIM::MailListDrag::format())
-	&&  KPIM::MailListDrag::decode(e, mailList))
+	&&       KPIM::MailListDrag::decode(e, mailList))
 	{
 		// KMail message(s). Ignore all but the first.
+		kdDebug(5950) << "MainWindow::executeDropEvent(KMail_list)" << endl;
 		if (!mailList.count())
 			return;
 		KPIM::MailSummary& summary = mailList.first();
 		QDateTime dt;
 		dt.setTime_t(summary.date());
 		QString body = KAMail::getMailBody(summary.serialNumber());
-		alarmText.setEmail(summary.to(), summary.from(), KGlobal::locale()->formatDateTime(dt),
-		                   summary.subject(), body);
+		alarmText.setEmail(summary.to(), summary.from(), QString::null,
+		                   KGlobal::locale()->formatDateTime(dt), summary.subject(),
+		                   body, summary.serialNumber());
+	}
+	else if (KCal::ICalDrag::decode(e, &calendar))
+	{
+		// iCalendar - ignore all but the first event
+		kdDebug(5950) << "MainWindow::executeDropEvent(iCalendar)" << endl;
+		KCal::Event::List events = calendar.rawEvents();
+		if (!events.isEmpty())
+		{
+			KAEvent ev(*events.first());
+			executeNew(win, &ev);
+		}
+		return;
 	}
 	else if (QTextDrag::decode(e, text))
 	{
+		kdDebug(5950) << "MainWindow::executeDropEvent(text)" << endl;
 		alarmText.setText(text);
 	}
 	else
 		return;
 
 	if (!alarmText.isEmpty())
-		executeNew(win, action, alarmText);
+		executeNew(win, 0, action, alarmText);
 }
 
 /******************************************************************************
@@ -932,7 +1200,7 @@ void MainWindow::slotSelection()
 	QValueList<EventListViewItemBase*> items = mListView->selectedItems();
 	int count = items.count();
 	AlarmListViewItem* item = (AlarmListViewItem*)((count == 1) ? items.first() : 0);
-	bool enableUndelete = true;
+	bool enableReactivate = true;
 	bool enableEnableDisable = true;
 	bool enableEnable = false;
 	bool enableDisable = false;
@@ -940,9 +1208,9 @@ void MainWindow::slotSelection()
 	for (QValueList<EventListViewItemBase*>::Iterator it = items.begin();  it != items.end();  ++it)
 	{
 		const KAEvent& event = ((AlarmListViewItem*)(*it))->event();
-		if (enableUndelete
+		if (enableReactivate
 		&&  (!event.expired()  ||  !event.occursAfter(now, true)))
-			enableUndelete = false;
+			enableReactivate = false;
 		if (enableEnableDisable)
 		{
 			if (event.expired())
@@ -963,7 +1231,7 @@ void MainWindow::slotSelection()
 	mActionModify->setEnabled(item && !mListView->expired(item));
 	mActionView->setEnabled(count == 1);
 	mActionDelete->setEnabled(count);
-	mActionUndelete->setEnabled(count && enableUndelete);
+	mActionReactivate->setEnabled(count && enableReactivate);
 	mActionEnable->setEnabled(enableEnable || enableDisable);
 	if (enableEnable || enableDisable)
 		setEnableText(enableEnable);
@@ -991,7 +1259,7 @@ void MainWindow::slotMouseClicked(int button, QListViewItem* item, const QPoint&
 		mActionModify->setEnabled(false);
 		mActionView->setEnabled(false);
 		mActionDelete->setEnabled(false);
-		mActionUndelete->setEnabled(false);
+		mActionReactivate->setEnabled(false);
 		mActionEnable->setEnabled(false);
 	}
 }
@@ -1023,34 +1291,13 @@ void MainWindow::setEnableText(bool enable)
 	mActionEnable->setText(enable ? i18n("Ena&ble") : i18n("Disa&ble"));
 }
 
-/******************************************************************************
-* Prompt the user to re-enable alarms if they are currently disabled, and if
-* it's an email alarm, warn if no 'From' email address is configured.
-*/
-void MainWindow::alarmWarnings(QWidget* parent, const KAEvent* event)
-{
-	if (event  &&  event->action() == KAEvent::EMAIL
-	&&  Preferences::instance()->emailAddress().isEmpty())
-		KMessageBox::information(parent, i18n("Please set the 'From' email address...",
-		                                      "%1\nPlease set it in the Preferences dialog.").arg(KAMail::i18n_NeedFromEmailAddress()));
-
-	if (!Daemon::monitoringAlarms())
-	{
-		if (KMessageBox::warningYesNo(parent, i18n("Alarms are currently disabled.\nDo you want to enable alarms now?"),
-		                              QString::null, KStdGuiItem::yes(), KStdGuiItem::no(),
-		                              QString::fromLatin1("EditEnableAlarms"))
-		                == KMessageBox::Yes)
-			Daemon::setAlarmsEnabled();
-	}
-}
-
 /******************************************************************************
 *  Display or hide the specified main window.
 *  This should only be called when the application doesn't run in the system tray.
 */
 MainWindow* MainWindow::toggleWindow(MainWindow* win)
 {
-	if (win  &&  findWindow(win))
+	if (win  &&  mWindowList.find(win) != mWindowList.end())
 	{
 		// A window is specified (and it exists)
 		if (win->isVisible())
@@ -1075,14 +1322,3 @@ MainWindow* MainWindow::toggleWindow(MainWindow* win)
 	win->show();
 	return win;
 }
-
-/******************************************************************************
-* Find the specified window in the main window list.
-*/
-bool MainWindow::findWindow(MainWindow* win)
-{
-	for (MainWindow* w = mWindowList.first();  w;  w = mWindowList.next())
-		if (w == win)
-			return true;
-	return false;
-}
