@@ -1,7 +1,7 @@
 /*
  *  traywindow.cpp  -  the KDE system tray applet
  *  Program:  kalarm
- *  (C) 2002 - 2004 by David Jarvie <software@astrojar.org.uk>
+ *  Copyright (c) 2002 - 2005 by David Jarvie <software@astrojar.org.uk>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -13,12 +13,13 @@
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
  *  GNU General Public License for more details.
  *
- *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ *  You should have received a copy of the GNU General Public License along
+ *  with this program; if not, write to the Free Software Foundation, Inc.,
+ *  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  */
 
 #include "kalarm.h"
+
 #include <stdlib.h>
 
 #include <qtooltip.h>
@@ -32,11 +33,13 @@
 #include <kstandarddirs.h>
 #include <kstdaction.h>
 #include <kstdguiitem.h>
+#include <kaccel.h>
 #include <kconfig.h>
 #include <kdebug.h>
 
 #include "alarmcalendar.h"
 #include "alarmlistview.h"
+#include "alarmtext.h"
 #include "daemon.h"
 #include "functions.h"
 #include "kalarmapp.h"
@@ -44,6 +47,7 @@
 #include "messagewin.h"
 #include "prefdlg.h"
 #include "preferences.h"
+#include "templatemenuaction.h"
 #include "traywindow.moc"
 
 
@@ -51,6 +55,7 @@ class TrayTooltip : public QToolTip
 {
 	public:
 		TrayTooltip(QWidget* parent) : QToolTip(parent) { }
+		virtual ~TrayTooltip() {}
 	protected:
 		virtual void maybeTip(const QPoint&);
 };
@@ -76,8 +81,7 @@ TrayWindow::TrayWindow(MainWindow* parent, const char* name)
 	mPixmapEnabled  = loadIcon("kalarm");
 	mPixmapDisabled = loadIcon("kalarm_disabled");
 	if (mPixmapEnabled.isNull() || mPixmapDisabled.isNull())
-		KMessageBox::sorry(this, i18n("Cannot load system tray icon."),
-		                         i18n("KAlarm Error", "%1 Error").arg(kapp->aboutData()->programName()));
+		KMessageBox::sorry(this, i18n("Cannot load system tray icon."));
 	setAcceptDrops(true);         // allow drag-and-drop onto this window
 
 	// Set up the context menu
@@ -86,14 +90,14 @@ TrayWindow::TrayWindow(MainWindow* parent, const char* name)
 	a->plug(contextMenu());
 	connect(a, SIGNAL(switched(bool)), SLOT(setEnabledStatus(bool)));
 	KAlarm::createNewAlarmAction(i18n("&New Alarm..."), this, SLOT(slotNewAlarm()), actcol, "tNew")->plug(contextMenu());
-#ifdef NEW_FROM_TEMPLATE
-	KAlarm::createNewFromTemplateAction(i18n("New Alarm From &Template"), actions, "tNewFromTempl")->plug(contextMenu());
-#endif
+	KAlarm::createNewFromTemplateAction(i18n("New Alarm From &Template"), this, SLOT(slotNewFromTemplate(const KAEvent&)), actcol, "tNewFromTempl")->plug(contextMenu());
 	KStdAction::preferences(this, SLOT(slotPreferences()), actcol)->plug(contextMenu());
 
 	// Replace the default handler for the Quit context menu item
-	actcol->remove(actcol->action(KStdAction::stdName(KStdAction::Quit)));
-	actcol->insert(KStdAction::quit(this, SLOT(slotQuit()), actcol));
+	const char* quitName = KStdAction::name(KStdAction::Quit);
+	actcol->remove(actcol->action(quitName));
+	actcol->accel()->remove(quitName);
+	KStdAction::quit(this, SLOT(slotQuit()), actcol);
 
 	// Set icon to correspond with the alarms enabled menu status
 	Daemon::checkStatus();
@@ -129,6 +133,14 @@ void TrayWindow::slotNewAlarm()
 	MainWindow::executeNew();
 }
 
+/******************************************************************************
+*  Called when the "New Alarm" menu item is selected to edit a new alarm.
+*/
+void TrayWindow::slotNewFromTemplate(const KAEvent& event)
+{
+	MainWindow::executeNew(event);
+}
+
 /******************************************************************************
 *  Called when the "Configure KAlarm" menu item is selected.
 */
@@ -214,9 +226,8 @@ void TrayWindow::dropEvent(QDropEvent* e)
 void TrayWindow::tooltipAlarmText(QString& text) const
 {
 	KAEvent event;
-	Preferences* preferences = Preferences::instance();
-	const QString& prefix = preferences->tooltipTimeToPrefix();
-	int maxCount = preferences->tooltipAlarmCount();
+	const QString& prefix = Preferences::tooltipTimeToPrefix();
+	int maxCount = Preferences::tooltipAlarmCount();
 	QDateTime now = QDateTime::currentDateTime();
 
 	// Get today's and tomorrow's alarms, sorted in time order
@@ -242,13 +253,13 @@ void TrayWindow::tooltipAlarmText(QString& text) const
 
 			// The alarm is due today, or early tomorrow
 			bool space = false;
-			if (preferences->showTooltipAlarmTime())
+			if (Preferences::showTooltipAlarmTime())
 			{
 				item.text += KGlobal::locale()->formatTime(item.dateTime.time());
 				item.text += ' ';
 				space = true;
 			}
-			if (preferences->showTooltipTimeToAlarm())
+			if (Preferences::showTooltipTimeToAlarm())
 			{
 				int mins = (now.secsTo(item.dateTime) + 59) / 60;
 				if (mins < 0)
@@ -256,7 +267,7 @@ void TrayWindow::tooltipAlarmText(QString& text) const
 				char minutes[3] = "00";
 				minutes[0] = (mins%60) / 10 + '0';
 				minutes[1] = (mins%60) % 10 + '0';
-				if (preferences->showTooltipAlarmTime())
+				if (Preferences::showTooltipAlarmTime())
 					item.text += i18n("prefix + hours:minutes", "(%1%2:%3)").arg(prefix).arg(mins/60).arg(minutes);
 				else
 					item.text += i18n("prefix + hours:minutes", "%1%2:%3").arg(prefix).arg(mins/60).arg(minutes);
@@ -265,7 +276,7 @@ void TrayWindow::tooltipAlarmText(QString& text) const
 			}
 			if (space)
 				item.text += ' ';
-			item.text += AlarmListViewItem::alarmText(event, false);
+			item.text += AlarmText::summary(event);
 
 			// Insert the item into the list in time-sorted order
 			for (iit = items.begin();  iit != items.end();  ++iit)
@@ -346,7 +357,7 @@ void TrayTooltip::maybeTip(const QPoint&)
 	else
 		text = i18n("%1 - disabled").arg(kapp->aboutData()->programName());
 	kdDebug(5950) << "TrayTooltip::maybeTip(): " << text << endl;
-	if (Preferences::instance()->tooltipAlarmCount())
+	if (Preferences::tooltipAlarmCount())
 		parent->tooltipAlarmText(text);
 	tip(parent->rect(), text);
 }
