@@ -30,6 +30,7 @@
 #include "task.h"
 #include "taskview.h"
 #include "timekard.h"
+#include "taskviewwhatsthis.h"
 
 #define T_LINESIZE 1023
 #define HIDDEN_COLUMN -10
@@ -41,9 +42,6 @@ TaskView::TaskView(QWidget *parent, const char *name, const QString &icsfile ):K
   _preferences = Preferences::instance( icsfile );
   _storage = KarmStorage::instance();
 
-  connect(this, SIGNAL( doubleClicked( QListViewItem * )),
-          this, SLOT( changeTimer( QListViewItem * )));
-
   connect( this, SIGNAL( expanded( QListViewItem * ) ),
            this, SLOT( itemStateChanged( QListViewItem * ) ) );
   connect( this, SIGNAL( collapsed( QListViewItem * ) ),
@@ -110,6 +108,7 @@ TaskView::TaskView(QWidget *parent, const char *name, const QString &icsfile ):K
            this, SLOT( startTimerFor(Task*) ));
   connect( _desktopTracker, SIGNAL( leftActiveDesktop( Task* ) ),
            this, SLOT( stopTimerFor(Task*) ));
+  new TaskViewWhatsThis( this );
 }
 
 KarmStorage* TaskView::storage()
@@ -117,6 +116,53 @@ KarmStorage* TaskView::storage()
   return _storage;
 }
 
+void TaskView::contentsMousePressEvent ( QMouseEvent * e )
+{
+  kdDebug(5970) << "entering contentsMousePressEvent" << endl;
+  KListView::contentsMousePressEvent(e);
+  Task* task = current_item();
+  // This checks that there has been a click onto an item,  
+  // not into an empty part of the KListView.
+  if ( task != 0 &&  // zero can happen if there is no task
+        e->pos().y() >= current_item()->itemPos() &&
+        e->pos().y() < current_item()->itemPos()+current_item()->height() )
+  { 
+    // see if the click was on the completed icon
+    int leftborder = treeStepSize() * ( task->depth() + ( rootIsDecorated() ? 1 : 0)) + itemMargin();
+    if ((leftborder < e->x()) && (e->x() < 19 + leftborder ))
+    {
+      if ( task->isComplete() ) task->setPercentComplete( 0, _storage );
+      else task->setPercentComplete( 100, _storage );
+    }
+    emit updateButtons();
+  }
+}
+
+void TaskView::contentsMouseDoubleClickEvent ( QMouseEvent * e )
+// if the user double-clicks onto a tasks, he says "I am now working exclusively
+// on that task". That means, on a doubleclick, we check if it occurs on an item
+// not in the blank space, if yes, stop all other tasks and start the new timer.
+{
+  kdDebug(5970) << "entering contentsMouseDoubleClickEvent" << endl;
+  KListView::contentsMouseDoubleClickEvent(e);
+  
+  Task *task = current_item();
+
+  if ( task != 0 )  // current_item() exists
+  {
+    if ( e->pos().y() >= task->itemPos() &&  // doubleclick was onto current_item()
+          e->pos().y() < task->itemPos()+task->height() )
+    {
+      if ( activeTasks.findRef(task) == -1 )  // task is active
+      {
+        stopAllTimers();
+        startCurrentTimer();
+      }
+      else stopCurrentTimer();
+    }
+  }
+}
+
 TaskView::~TaskView()
 {
   _preferences->save();
@@ -137,10 +183,12 @@ Task* TaskView::item_at_index(int i)
   return static_cast<Task*>(itemAtIndex(i));
 }
 
-void TaskView::load()
+void TaskView::load( QString fileName )
 {
+  // if the program is used as an embedded plugin for konqueror, there may be a need
+  // to load from a file without touching the preferences.
   _isloading = true;
-  QString err = _storage->load(this, _preferences);
+  QString err = _storage->load(this, _preferences, fileName);
 
   if (!err.isEmpty())
   {
@@ -160,6 +208,7 @@ void TaskView::load()
   setCurrentItem(first_child());
   _desktopTracker->startTracking();
   _isloading = false;
+  refresh();
 }
 
 void TaskView::restoreItemState( QListViewItem *item )
@@ -183,21 +232,11 @@ void TaskView::itemStateChanged( QListViewItem *item )
   if( _preferences ) _preferences->writeEntry( t->uid(), t->isOpen() );
 }
 
-void TaskView::deleteItemState( QListViewItem *item )
-{
-  if ( !item ) return;
-  Task *t = (Task *)item;
-  kdDebug(5970) << "TaskView:deleteItemState()" 
-    << " uid=" << t->uid() << endl;
-  if( _preferences ) _preferences->deleteEntry( t->uid() );
-}
-
 void TaskView::closeStorage() { _storage->closeStorage( this ); }
 
 void TaskView::iCalFileModified(ResourceCalendar *rc)
 {
   kdDebug(5970) << "entering iCalFileModified" << endl;
-  stopAllTimers();
   kdDebug(5970) << rc->infoText() << endl;
   rc->dump();
   _storage->buildTaskView(rc,this);
@@ -211,16 +250,7 @@ void TaskView::refresh()
   int i = 0;
   for ( Task* t = item_at_index(i); t; t = item_at_index(++i) )
   {
-    if (!t->isComplete())
-    {
-      t->setOpen(true);
-      t->setEnabled(true);
-    }
-    else
-    {
-      t->setOpen(false);
-      t->setEnabled(false);
-    }
+    t->setPixmapProgress();
   }
   
   // remove root decoration if there is no more children.
@@ -236,6 +266,7 @@ void TaskView::refresh()
   if (!anyChilds) {
     setRootIsDecorated(false);
   }
+  emit updateButtons();
   kdDebug(5970) << "exiting TaskView::refresh()" << endl;
 }
     
@@ -271,19 +302,23 @@ void TaskView::loadFromFlatFile()
   }
 }
 
-void TaskView::importPlanner()
+QString TaskView::importPlanner(QString fileName)
 {
   kdDebug(5970) << "entering importPlanner" << endl;
-  {
-    PlannerParser* handler=new PlannerParser(this);
-    QFile xmlFile(KFileDialog::getOpenFileName(QString::null, QString::null, 0) );
-    QXmlInputSource source( xmlFile );
-    QXmlSimpleReader reader;
-    reader.setContentHandler( handler );
-    reader.parse( source );
-    // we need a refresh
-    refresh();
-  }
+  PlannerParser* handler=new PlannerParser(this);
+  if (fileName.isEmpty()) fileName=KFileDialog::getOpenFileName(QString::null, QString::null, 0);
+  QFile xmlFile( fileName );
+  QXmlInputSource source( xmlFile );
+  QXmlSimpleReader reader;
+  reader.setContentHandler( handler );
+  reader.parse( source );
+  refresh();
+  return "";
+}
+
+QString TaskView::report( const ReportCriteria& rc )
+{
+  return _storage->report( this, rc );
 }
 
 void TaskView::exportcsvFile()
@@ -363,11 +398,11 @@ long TaskView::count()
   return n;
 }
 
-void TaskView::startTimerFor(Task* task)
+void TaskView::startTimerFor(Task* task, QDateTime startTime )
 {
   if (task != 0 && activeTasks.findRef(task) == -1) {
     _idleTimeDetector->startIdleDetection();
-    task->setRunning(true, _storage);
+    task->setRunning(true, _storage, startTime);
     activeTasks.append(task);
     emit updateButtons();
     if ( activeTasks.count() == 1 )
@@ -377,6 +412,11 @@ void TaskView::startTimerFor(Task* task)
   }
 }
 
+void TaskView::clearActiveTasks()
+{
+  activeTasks.clear();
+}
+
 void TaskView::stopAllTimers()
 {
   for ( unsigned int i = 0; i < activeTasks.count(); i++ )
@@ -426,24 +466,6 @@ void TaskView::stopCurrentTimer()
   stopTimerFor( current_item());
 }
 
-
-void TaskView::changeTimer(QListViewItem *)
-{
-  Task *task = current_item();
-
-  if ( task != 0 && activeTasks.findRef(task) == -1 )
-  {
-    // Stop all the other timers.
-    for (unsigned int i=0; i<activeTasks.count();i++)
-      (activeTasks.at(i))->setRunning(false, _storage);
-    activeTasks.clear();
-
-    // Start the new timer.
-    startCurrentTimer();
-  }
-  else stopCurrentTimer();
-}
-
 void TaskView::minuteUpdate()
 {
   addTimeToActiveTasks(1, false);
@@ -483,7 +505,7 @@ void TaskView::newTask(QString caption, Task *parent)
     if ( uid.isNull() )
     {
       KMessageBox::error( 0, i18n(
-            "Error storing new task. Your changes were not saved." ) );
+            "Error storing new task. Your changes were not saved. Make sure you can edit your iCalendar file. Also quit all applications using this file and remove any lock file related to its name from ~/.kde/share/apps/kabc/lock/ " ) );
     }
 
     delete dialog;
@@ -491,10 +513,13 @@ void TaskView::newTask(QString caption, Task *parent)
 }
 
 QString TaskView::addTask
-( const QString& taskname, long total, long session, const DesktopList& desktops, Task* parent )
+( const QString& taskname, long total, long session, 
+  const DesktopList& desktops, Task* parent )
 {
   Task *task;
 
+  kdDebug(5970) << "TaskView::addTask: taskname = " << taskname << endl;
+
   if ( parent ) task = new Task( taskname, total, session, desktops, parent );
   else          task = new Task( taskname, total, session, desktops, this );
 
@@ -508,6 +533,8 @@ QString TaskView::addTask
 
     setSelected( task, true );
 
+    task->setPixmapProgress();
+
     save();
   }
   else
@@ -585,6 +612,23 @@ void TaskView::editTask()
 //    task->addComment( comment, _storage );
 //}
 
+void TaskView::reinstateTask(int completion)
+{
+  Task* task = current_item();
+  if (task == 0) {
+    KMessageBox::information(0,i18n("No task selected."));
+    return;
+  }
+
+  if (completion<0) completion=0;
+  if (completion<100)
+  {
+    task->setPercentComplete(completion, _storage);
+    task->setPixmapProgress();
+    save();
+    emit updateButtons();
+  }
+}
 
 void TaskView::deleteTask(bool markingascomplete)
 {
@@ -594,31 +638,33 @@ void TaskView::deleteTask(bool markingascomplete)
     return;
   }
 
-  int response = KMessageBox::Yes;
+  int response = KMessageBox::Continue;
   if (!markingascomplete && _preferences->promptDelete()) {
     if (task->childCount() == 0) {
-      response = KMessageBox::warningYesNo( 0,
+      response = KMessageBox::warningContinueCancel( 0,
           i18n( "Are you sure you want to delete "
           "the task named\n\"%1\" and its entire history?")
           .arg(task->name()),
-          i18n( "Deleting Task"));
+          i18n( "Deleting Task"), KStdGuiItem::del());
     }
     else {
-      response = KMessageBox::warningYesNo( 0,
+      response = KMessageBox::warningContinueCancel( 0,
           i18n( "Are you sure you want to delete the task named"
           "\n\"%1\" and its entire history?\n"
           "NOTE: all its subtasks and their history will also "
           "be deleted.").arg(task->name()),
-          i18n( "Deleting Task"));
+          i18n( "Deleting Task"), KStdGuiItem::del());
     }
   }
 
-  if (response == KMessageBox::Yes)
+  if (response == KMessageBox::Continue)
   {
     if (markingascomplete)
     {
       task->setPercentComplete(100, _storage);
+      task->setPixmapProgress();
       save();
+      emit updateButtons();
 
       // Have to remove after saving, as the save routine only affects tasks
       // that are in the view.  Otherwise, the new percent complete does not
@@ -628,25 +674,15 @@ void TaskView::deleteTask(bool markingascomplete)
     }
     else
     {
+      QString uid=task->uid();
       task->remove(activeTasks, _storage);
       task->removeFromView();
-      deleteItemState( task );
+      if( _preferences ) _preferences->deleteEntry( uid ); // forget if the item was expanded or collapsed
       save();
     }
 
     // remove root decoration if there is no more children.
-    bool anyChilds = false;
-    for(Task* child = first_child();
-              child;
-              child = child->nextSibling()) {
-      if (child->childCount() != 0) {
-        anyChilds = true;
-        break;
-      }
-    }
-    if (!anyChilds) {
-      setRootIsDecorated(false);
-    }
+    refresh();
 
     // Stop idle detection if no more counters are running
     if (activeTasks.count() == 0) {
@@ -743,9 +779,21 @@ void TaskView::markTaskAsComplete()
   deleteTask(markingascomplete);
 }
 
+void TaskView::markTaskAsIncomplete()
+{
+  if (current_item())
+    kdDebug(5970) << "TaskView::markTaskAsComplete: "
+      << current_item()->uid() << endl;
+  else
+    kdDebug(5970) << "TaskView::markTaskAsComplete: null current_item()" << endl;
+
+  reinstateTask(50); // if it has been reopened, assume half-done
+}
+
+
 void TaskView::clipTotals()
 {
-  TimeKard *t = new TimeKard();
+  TimeKard t;
   if (current_item() && current_item()->isRoot())
   {
     int response = KMessageBox::questionYesNo( 0,
@@ -754,28 +802,27 @@ void TaskView::clipTotals()
         i18n("Copy This Task"), i18n("Copy All Tasks") );
     if (response == KMessageBox::Yes) // this task only
     {
-      KApplication::clipboard()->setText(t->totalsAsText(this));
+      KApplication::clipboard()->setText(t.totalsAsText(this));
     }
     else // only task
     {
-      KApplication::clipboard()->setText(t->totalsAsText(this, false));
+      KApplication::clipboard()->setText(t.totalsAsText(this, false));
     }
   }
   else
   {
-    KApplication::clipboard()->setText(t->totalsAsText(this));
+    KApplication::clipboard()->setText(t.totalsAsText(this));
   }
 }
 
 void TaskView::clipHistory()
 {
-
-  PrintDialog *dialog = new PrintDialog();
-  if (dialog->exec()== QDialog::Accepted)
+  PrintDialog dialog;
+  if (dialog.exec()== QDialog::Accepted)
   {
-    TimeKard *t = new TimeKard();
+    TimeKard t;
     KApplication::clipboard()->
-      setText(t->historyAsText(this, dialog->from(), dialog->to()));
+      setText( t.historyAsText(this, dialog.from(), dialog.to(), !dialog.allTasks(), dialog.perWeek(), dialog.totalsOnly() ) );
   }
 }
 
