@@ -16,7 +16,7 @@
  *
  *   You should have received a copy of the GNU General Public License
  *   along with this program; if not, write to the Free Software
- *   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+ *   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
  *   Send comments and bug fixes to s.carstens@gmx.de
  *
@@ -55,7 +55,9 @@ extern "C" {
 #include <kdebug.h>
 #include <kmdcodec.h>
 #include <kurl.h>
+
 #include <kasciistricmp.h>
+#include <kasciistringtools.h>
 
 imapParser::imapParser ()
 {
@@ -69,7 +71,7 @@ imapParser::imapParser ()
 imapParser::~imapParser ()
 {
   delete lastHandled;
-  lastHandled = 0L;
+  lastHandled = 0;
 }
 
 imapCommand *
@@ -184,7 +186,8 @@ static bool sasl_interact( KIO::SlaveBase *slave, KIO::AuthInfo &ai, void *in )
         interact->len = strlen( (const char *) interact->result );
         break;
       default:
-        interact->result = 0; interact->len = 0;
+        interact->result = 0;
+        interact->len = 0;
         break;
     }
     interact++;
@@ -214,7 +217,7 @@ imapParser::clientAuthenticate ( KIO::SlaveBase *slave, KIO::AuthInfo &ai,
     return false;
 
 //  result = sasl_client_new( isSSL ? "imaps" : "imap",
-  result = sasl_client_new( "imap", /* FIXME: with cyrus-imapd, even imaps' digest-uri 
+  result = sasl_client_new( "imap", /* FIXME: with cyrus-imapd, even imaps' digest-uri
                                        must be 'imap'. I don't know if it's good or bad. */
                        aFQDN.latin1(),
                        0, 0, 0, 0, &conn );
@@ -229,11 +232,12 @@ imapParser::clientAuthenticate ( KIO::SlaveBase *slave, KIO::AuthInfo &ai,
     result = sasl_client_start(conn, aAuth.latin1(), &client_interact,
                        hasCapability("SASL-IR") ? &out : 0, &outlen, &mechusing);
 
-    if ( result == SASL_INTERACT )
+    if ( result == SASL_INTERACT ) {
       if ( !sasl_interact( slave, ai, client_interact ) ) {
         sasl_dispose( &conn );
         return false;
       }
+    }
   } while ( result == SASL_INTERACT );
 
   if ( result != SASL_CONTINUE && result != SASL_OK ) {
@@ -249,7 +253,7 @@ imapParser::clientAuthenticate ( KIO::SlaveBase *slave, KIO::AuthInfo &ai,
   tmp.resetRawData( out, outlen );
   // then lets try it
   QString firstCommand = aAuth;
-  if ( !challenge.isEmpty() ) { 
+  if ( !challenge.isEmpty() ) {
     firstCommand += " ";
     firstCommand += QString::fromLatin1( challenge.data(), challenge.size() );
   }
@@ -277,11 +281,12 @@ imapParser::clientAuthenticate ( KIO::SlaveBase *slave, KIO::AuthInfo &ai,
                                   &client_interact,
                                   &out, &outlen);
 
-        if (result == SASL_INTERACT)
+        if (result == SASL_INTERACT) {
           if ( !sasl_interact( slave, ai, client_interact ) ) {
             sasl_dispose( &conn );
             return false;
           }
+        }
       } while ( result == SASL_INTERACT );
 
       if ( result != SASL_CONTINUE && result != SASL_OK ) {
@@ -452,10 +457,6 @@ imapParser::parseUntagged (parseString & result)
           if (qstrncmp(what, "FETCH", what.size()) == 0)
           {
             seenUid = QString::null;
-            if (lastHandled) 
-              lastHandled->clear();
-            else 
-              lastHandled = new imapCache();
             parseFetch (number, result);
           }
           break;
@@ -488,7 +489,7 @@ void
 imapParser::parseResult (QByteArray & result, parseString & rest,
   const QString & command)
 {
-  if (command == "SELECT") 
+  if (command == "SELECT")
     selectInfo.setReadWrite(true);
 
   if (rest[0] == '[')
@@ -631,7 +632,8 @@ imapParser::parseResult (QByteArray & result, parseString & rest,
 
 void imapParser::parseCapability (parseString & result)
 {
-  imapCapabilities = QStringList::split (' ', result.cstr().lower());
+  QCString temp( result.cstr() );
+  imapCapabilities = QStringList::split ( ' ', KPIM::kAsciiToLower( temp.data() ) );
 }
 
 void imapParser::parseFlags (parseString & result)
@@ -649,7 +651,7 @@ void imapParser::parseList (parseString & result)
   result.pos++; // tie off (
 
   this_one.parseAttributes( result );
-  
+
   result.pos++; // tie off )
   skipWS (result);
 
@@ -1363,7 +1365,7 @@ void imapParser::parseFetch (ulong /* value */, parseString & inWords)
 
           if (lastHandled)
             envelope = lastHandled->getHeader ();
-          else 
+          else
             lastHandled = new imapCache();
 
           if (envelope && !envelope->getMessageId ().isEmpty ())
@@ -1392,7 +1394,7 @@ void imapParser::parseFetch (ulong /* value */, parseString & inWords)
         }
         else if (word == "BODY[]" )
         {
-	  // Do the same as with "RFC822"
+          // Do the same as with "RFC822"
           parseLiteralC(inWords, true);
         }
         else if (word == "BODYSTRUCTURE")
@@ -1422,13 +1424,13 @@ void imapParser::parseFetch (ulong /* value */, parseString & inWords)
           mailHeader *envelope = 0;
           if (lastHandled)
             envelope = lastHandled->getHeader ();
-          else 
+          else
             lastHandled = new imapCache();
 
-          if (envelope || seenUid.isEmpty ())
+          if (seenUid.isEmpty ())
           {
-            // we have seen this one already
-            // or don't know where to put it
+            // unknown what to do
+            kdDebug(7116) << "imapParser::parseFetch - UID empty" << endl;
           }
           else
           {
@@ -1550,14 +1552,15 @@ void imapParser::parseNamespace (parseString & result)
     return;
 
   QString delimEmpty;
-  if ( imapNamespaceDelimiter.contains( QString::null ) )
-    delimEmpty = imapNamespaceDelimiter[QString::null];
+  if ( namespaceToDelimiter.contains( QString::null ) )
+    delimEmpty = namespaceToDelimiter[QString::null];
 
-  imapNamespaceDelimiter.clear();
+  namespaceToDelimiter.clear();
   imapNamespaces.clear();
 
   // remember what section we're in (user, other users, shared)
   int ns = -1;
+  bool personalAvailable = false;
   while ( !result.isEmpty() )
   {
     if ( result[0] == '(' )
@@ -1575,15 +1578,20 @@ void imapParser::parseNamespace (parseString & result)
       QCString delim = parseOneWordC( result );
       kdDebug(7116) << "imapParser::parseNamespace ns='" << prefix <<
        "',delim='" << delim << "'" << endl;
-      const QString prefixStr = prefix;
-      QString nsentry = QString::number( ns ) + "=" + prefixStr;
+      if ( ns == 0 )
+      {
+        // at least one personal ns
+        personalAvailable = true;
+      }
+      QString nsentry = QString::number( ns ) + "=" + QString(prefix) +
+        "=" + QString(delim);
       imapNamespaces.append( nsentry );
       if ( prefix.right( 1 ) == delim ) {
         // strip delimiter to get a correct entry for comparisons
         prefix.resize( prefix.length() );
-      }      
-      imapNamespaceDelimiter[prefix] = delim;
-      
+      }
+      namespaceToDelimiter[prefix] = delim;
+
       result.pos++; // tie off )
       skipWS( result );
     } else if ( result[0] == ')' )
@@ -1601,7 +1609,15 @@ void imapParser::parseNamespace (parseString & result)
     }
   }
   if ( !delimEmpty.isEmpty() ) {
-    imapNamespaceDelimiter[QString::null] = delimEmpty;
+    // remember default delimiter
+    namespaceToDelimiter[QString::null] = delimEmpty;
+    if ( !personalAvailable )
+    {
+      // at least one personal ns would be nice
+      kdDebug(7116) << "imapParser::parseNamespace - registering own personal ns" << endl;
+      QString nsentry = "0==" + delimEmpty;
+      imapNamespaces.append( nsentry );
+    }
   }
 }
 
@@ -1715,6 +1731,7 @@ imapParser::parseURL (const KURL & _url, QString & _box, QString & _section,
   QStringList parameters;
 
   _box = _url.path ();
+  kdDebug(7116) << "imapParser::parseURL " << _box << endl;
   int paramStart = _box.find("/;");
   if ( paramStart > -1 )
   {
@@ -1756,7 +1773,8 @@ imapParser::parseURL (const KURL & _url, QString & _box, QString & _section,
     if (!_box.isEmpty () && _box[_box.length () - 1] == '/')
       _box.truncate(_box.length() - 1);
   }
-  kdDebug(7116) << "URL: box= " << _box << ", section= " << _section << ", type= " << _type << ", uid= " << _uid << ", validity= " << _validity << endl;
+  kdDebug(7116) << "URL: box= " << _box << ", section= " << _section << ", type= " 
+    << _type << ", uid= " << _uid << ", validity= " << _validity << ", info= " << _info << endl;
 }
 
 
@@ -1770,8 +1788,8 @@ QCString imapParser::parseLiteralC(parseString & inWords, bool relay, bool stopA
     {
       bool proper;
       ulong runLenSave = runLen + 1;
-      QCString tmpstr;
-      inWords.takeMid(tmpstr, 1, runLen - 1);
+      QCString tmpstr(runLen);
+      inWords.takeMidNoResize(tmpstr, 1, runLen - 1);
       runLen = tmpstr.toULong (&proper);
       inWords.pos += runLenSave;
       if (proper)
@@ -1812,9 +1830,11 @@ QCString imapParser::parseLiteralC(parseString & inWords, bool relay, bool stopA
 // does not know about literals ( {7} literal )
 QCString imapParser::parseOneWordC (parseString & inWords, bool stopAtBracket, int *outLen)
 {
-  QCString retVal;
   uint retValSize = 0;
   uint len = inWords.length();
+  if (len == 0) {
+    return QCString();
+  }
 
   if (len > 0 && inWords[0] == '"')
   {
@@ -1828,31 +1848,10 @@ QCString imapParser::parseOneWordC (parseString & inWords, bool stopAtBracket, i
     }
     if (i < len)
     {
+      QCString retVal(i);
       inWords.pos++;
-      inWords.takeLeft(retVal, i - 1);
+      inWords.takeLeftNoResize(retVal, i - 1);
       len = i - 1;
-#if 0
-      static char *buf = 0L;
-      static int buflen = 0;
-      if (buflen < i) {
-        buflen = i;
-        buf = (char *)realloc(buf, buflen);
-      }
-
-      // If you're keen, you could do this in-place without the buffer
-      int k = 0;
-      for (unsigned int j = 0; j < len; j++) {
-        if (retVal[j] != '\\') {
-          buf[k++] = retVal[j];
-        } else {
-          j++; // skip the next character too
-        }
-      }
-      buf[k] = 0;
-
-      retVal = buf; // deep copy
-      retValSize = k;
-#else
       int offset = 0;
       for (unsigned int j = 0; j <= len; j++) {
         if (retVal[j] == '\\') {
@@ -1863,20 +1862,30 @@ QCString imapParser::parseOneWordC (parseString & inWords, bool stopAtBracket, i
       }
       retVal[len - offset] = 0;
       retValSize = len - offset;
-#endif
       inWords.pos += i;
+      skipWS (inWords);
+      if (outLen) {
+        *outLen = retValSize;
+      }
+      return retVal;
     }
     else
     {
       kdDebug(7116) << "imapParser::parseOneWord - error parsing unmatched \"" << endl;
-      retVal = inWords.cstr();
+      QCString retVal = inWords.cstr();
       retValSize = len;
       inWords.clear();
+      if (outLen) {
+        *outLen = retValSize;
+      }
+      return retVal;
     }
   }
   else
   {
+    // not quoted
     unsigned int i;
+    // search for end
     for (i = 0; i < len; ++i) {
         char ch = inWords[i];
         if (ch <= ' ' || ch == '(' || ch == ')' ||
@@ -1884,28 +1893,21 @@ QCString imapParser::parseOneWordC (parseString & inWords, bool stopAtBracket, i
             break;
     }
 
-    if (i < len)
-    {
-      inWords.takeLeft(retVal, i);
-      retValSize = i;
-      inWords.pos += i;
-    }
-    else
-    {
-      retVal = inWords.cstr();
-      retValSize = len;
-      inWords.clear();
-    }
+    QCString retVal(i+1);
+    inWords.takeLeftNoResize(retVal, i);
+    retValSize = i;
+    inWords.pos += i;
+
     if (retVal == "NIL") {
       retVal.truncate(0);
       retValSize = 0;
     }
+    skipWS (inWords);
+    if (outLen) {
+      *outLen = retValSize;
+    }
+    return retVal;
   }
-  skipWS (inWords);
-  if (outLen) {
-    *outLen = retValSize;
-  }
-  return retVal;
 }
 
 bool imapParser::parseOneNumber (parseString & inWords, ulong & num)
@@ -1942,7 +1944,8 @@ QString imapParser::namespaceForBox( const QString & box )
   QString myNamespace;
   if ( !box.isEmpty() )
   {
-    QValueList<QString> list = imapNamespaceDelimiter.keys();
+    QValueList<QString> list = namespaceToDelimiter.keys();
+    QString cleanPrefix;
     for ( QValueList<QString>::Iterator it = list.begin(); it != list.end(); ++it )
     {
       if ( !(*it).isEmpty() && box.find( *it ) != -1 )
