@@ -18,7 +18,7 @@
  *
  *   You should have received a copy of the GNU General Public License
  *   along with this program; if not, write to the Free Software
- *   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+ *   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
  *   Send comments and bug fixes to s.carstens@gmx.de
  *
@@ -72,11 +72,21 @@ public:
     qmemmove(dest.data(), data.data() + pos, len);
   }
   // Warning: does not check for going past end of "data"
+  void takeLeftNoResize(QCString& dest, uint len) const
+  {
+    qmemmove(dest.data(), data.data() + pos, len);
+  }
+  // Warning: does not check for going past end of "data"
   void takeMid(QCString& dest, uint start, uint len) const
   {
     dest.resize(len + 1);
     qmemmove(dest.data(), data.data() + pos + start, len);
   }
+  // Warning: does not check for going past end of "data"
+  void takeMidNoResize(QCString& dest, uint start, uint len) const
+  {
+    qmemmove(dest.data(), data.data() + pos + start, len);
+  }
   void clear()
   {
     data.resize(0);
@@ -339,7 +349,7 @@ public:
 
   /** parse a literal or word, may require more data */
   QCString parseLiteralC(parseString & inWords, bool relay = false,
-                           bool stopAtBracket = false, int *outlen = 0L);
+                           bool stopAtBracket = false, int *outlen = 0);
   inline QByteArray parseLiteral (parseString & inWords, bool relay = false,
                            bool stopAtBracket = false) {
     int len = 0; // string size
@@ -356,7 +366,7 @@ public:
 
   /** parse one word (maybe quoted) upto next space " ) ] } */
   static QCString parseOneWordC (parseString & inWords,
-    bool stopAtBracket = FALSE, int *len = 0L);
+    bool stopAtBracket = FALSE, int *len = 0);
   static inline QByteArray parseOneWord (parseString & inWords,
     bool stopAtBracket = FALSE) {
     int len = 0; // string size
@@ -466,10 +476,16 @@ protected:
   /** @brief the results from search/acl commands */
   QStringList lastResults;
 
-  /** @brief namespace prefix - delimiter association */
-  QMap<QString, QString> imapNamespaceDelimiter;
+  /** 
+   * @brief namespace prefix - delimiter association
+   * The namespace is cleaned before so that it does not contain the delimiter 
+   */
+  QMap<QString, QString> namespaceToDelimiter;
 
-  /** @brief list of namespace - prefix */
+  /** 
+   * @brief list of namespaces in the form: section=namespace=delimiter
+   * section is 0 (personal), 1 (other users) or 2 (shared)
+   */
   QStringList imapNamespaces;
 
 private:
