@@ -15,8 +15,8 @@
 
     You should have received a copy of the GNU Library General Public License
     along with this library; see the file COPYING.LIB.  If not, write to
-    the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-    Boston, MA 02111-1307, USA.
+    the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+    Boston, MA 02110-1301, USA.
 */
 #ifndef KSYNC_LOCALKONNECTOR_H
 #define KSYNC_LOCALKONNECTOR_H
@@ -55,8 +55,8 @@ class LocalKonnector : public KSync::Konnector
     bool connectDevice();
     bool disconnectDevice();
 
-    /** the state and some informations */
     KSync::KonnectorInfo info() const;
+    virtual QStringList supportedFilterTypes() const;
 
     void setCalendarFile( const QString &f ) { mCalendarFile = f; }
     QString calendarFile() const { return mCalendarFile; }
