@@ -15,7 +15,7 @@
 
     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 
     In addition, as a special exception, the copyright holders give
     permission to link the code of this program with any edition of
@@ -185,6 +185,7 @@ namespace KMail {
     { "Roger Larsson", 0, 0, 0 }, // add name of checked account to status bar message
     { "Jeffrey McGee", 0, 0, 0 }, // fix for bug:64251
     { "Dirk M\303\274ller", 0, 0, 0 }, // KURL() fixes and qt_cast optimizations
+    { "OpenUsability", I18N_NOOP("Usability tests and improvements"), 0, "http://www.openusability.org" },
     { "Mario Teijeiro Otero", 0, 0, 0 }, // various vendor annotations fixes
     { "Simon Perreault", 0, 0, 0 }, // make the composer remember its "Use Fixed Font" setting (bug 49481)
     { "Bernhard Reiter", I18N_NOOP("\xC3\x84gypten and Kroupware project management"),
@@ -203,13 +204,14 @@ namespace KMail {
     { "Wolfgang Westphal", I18N_NOOP("multiple encryption keys per address"),
       "wolfgang.westphal@gmx.de", 0 },
     { "Thorsten Zachmann", I18N_NOOP("POP filters"),
-      "t.zachmann@zagge.de", 0 }
+      "t.zachmann@zagge.de", 0 },
+    { "Thomas Zander", 0, 0, 0 }
   };
 
   AboutData::AboutData()
     : KAboutData( "kmail", I18N_NOOP("KMail"),KMAIL_VERSION,
                   I18N_NOOP("KDE Email Client"), License_GPL,
-                  I18N_NOOP("(c) 1997-2004, The KMail developers"), 0,
+                  I18N_NOOP("(c) 1997-2005, The KMail developers"), 0,
                   "http://kmail.kde.org" )
   {
     using KMail::authors;
