@@ -14,7 +14,7 @@
  *
  *   You should have received a copy of the GNU General Public License
  *   along with this program; if not, write to the Free Software
- *   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+ *   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
  */
 #include <config.h>
@@ -23,7 +23,6 @@
 
 #include <qbuttongroup.h>
 #include <qcheckbox.h>
-#include <klineedit.h>
 #include <qlayout.h>
 #include <qtabwidget.h>
 #include <qradiobutton.h>
@@ -33,6 +32,9 @@
 #include <qwhatsthis.h>
 #include <qhbox.h>
 #include <qcombobox.h>
+#include <qheader.h>
+#include <qtoolbutton.h>
+#include <qgrid.h>
 
 #include <kfiledialog.h>
 #include <klocale.h>
@@ -43,24 +45,23 @@
 #include <kapplication.h>
 #include <kmessagebox.h>
 #include <kprotocolinfo.h>
+#include <kiconloader.h>
+#include <kpopupmenu.h>
 
 #include <netdb.h>
 #include <netinet/in.h>
 
 #include "sieveconfig.h"
-using KMail::SieveConfig;
-using KMail::SieveConfigEditor;
 #include "kmacctmaildir.h"
 #include "kmacctlocal.h"
-#include "kmacctmgr.h"
-#include "kmacctexppop.h"
+#include "accountmanager.h"
+#include "popaccount.h"
 #include "kmacctimap.h"
 #include "kmacctcachedimap.h"
 #include "kmfoldermgr.h"
 #include "kmservertest.h"
 #include "protocols.h"
 #include "folderrequester.h"
-using KMail::FolderRequester;
 #include "kmmainwidget.h"
 #include "kmfolder.h"
 
@@ -75,6 +76,8 @@ using KMail::FolderRequester;
 #define _PATH_MAILDIR "/var/spool/mail"
 #endif
 
+namespace KMail {
+
 class ProcmailRCParser
 {
 public:
@@ -621,9 +624,42 @@ void AccountDialog::makePopAccountPage()
   connect( mPop.leaveOnServerCheck, SIGNAL( clicked() ),
            this, SLOT( slotLeaveOnServerClicked() ) );
   grid->addMultiCellWidget( mPop.leaveOnServerCheck, 6, 6, 0, 1 );
-
+  QHBox *afterDaysBox = new QHBox( page1 );
+  afterDaysBox->setSpacing( KDialog::spacingHint() );
+  mPop.leaveOnServerDaysCheck =
+    new QCheckBox( i18n("Leave messages on the server for"), afterDaysBox );
+  connect( mPop.leaveOnServerDaysCheck, SIGNAL( toggled(bool) ),
+           this, SLOT( slotEnableLeaveOnServerDays(bool)) );
+  mPop.leaveOnServerDaysSpin = new KIntNumInput( afterDaysBox );
+  mPop.leaveOnServerDaysSpin->setRange( 1, 365, 1, false );
+  mPop.leaveOnServerDaysSpin->setSuffix( i18n(" days") );
+  mPop.leaveOnServerDaysSpin->setValue( 1 );
+  afterDaysBox->setStretchFactor( mPop.leaveOnServerDaysSpin, 1 );
+  grid->addMultiCellWidget( afterDaysBox, 7, 7, 0, 1 );
+  QHBox *leaveOnServerCountBox = new QHBox( page1 );
+  leaveOnServerCountBox->setSpacing( KDialog::spacingHint() );
+  mPop.leaveOnServerCountCheck =
+    new QCheckBox( i18n("Keep only the last"), leaveOnServerCountBox );
+  connect( mPop.leaveOnServerCountCheck, SIGNAL( toggled(bool) ),
+           this, SLOT( slotEnableLeaveOnServerCount(bool)) );
+  mPop.leaveOnServerCountSpin = new KIntNumInput( leaveOnServerCountBox );
+  mPop.leaveOnServerCountSpin->setRange( 1, 999999, 1, false );
+  mPop.leaveOnServerCountSpin->setSuffix( i18n(" messages") );
+  mPop.leaveOnServerCountSpin->setValue( 100 );
+  grid->addMultiCellWidget( leaveOnServerCountBox, 8, 8, 0, 1 );
+  QHBox *leaveOnServerSizeBox = new QHBox( page1 );
+  leaveOnServerSizeBox->setSpacing( KDialog::spacingHint() );
+  mPop.leaveOnServerSizeCheck =
+    new QCheckBox( i18n("Keep only the last"), leaveOnServerSizeBox );
+  connect( mPop.leaveOnServerSizeCheck, SIGNAL( toggled(bool) ),
+           this, SLOT( slotEnableLeaveOnServerSize(bool)) );
+  mPop.leaveOnServerSizeSpin = new KIntNumInput( leaveOnServerSizeBox );
+  mPop.leaveOnServerSizeSpin->setRange( 1, 999999, 1, false );
+  mPop.leaveOnServerSizeSpin->setSuffix( i18n(" MB") );
+  mPop.leaveOnServerSizeSpin->setValue( 10 );
+  grid->addMultiCellWidget( leaveOnServerSizeBox, 9, 9, 0, 1 );
 #if 0
-  QHBox* resourceHB = new QHBox( page1 );
+  QHBox *resourceHB = new QHBox( page1 );
   resourceHB->setSpacing( 11 );
   mPop.resourceCheck =
       new QCheckBox( i18n( "Account for semiautomatic resource handling" ), resourceHB );
@@ -645,12 +681,12 @@ void AccountDialog::makePopAccountPage()
                    i18n( "Delete all outdated allocations for the resource represented by this account." ) );
   connect( mPop.resourceClearPastButton, SIGNAL( clicked() ),
            this, SLOT( slotClearPastResourceAllocations() ) );
-  grid->addMultiCellWidget( resourceHB, 7, 7, 0, 2 );
+  grid->addMultiCellWidget( resourceHB, 10, 10, 0, 2 );
 #endif
 
   mPop.includeInCheck =
     new QCheckBox( i18n("Include in man&ual mail check"), page1 );
-  grid->addMultiCellWidget( mPop.includeInCheck, 7, 7, 0, 1 );
+  grid->addMultiCellWidget( mPop.includeInCheck, 10, 10, 0, 1 );
 
   QHBox * hbox = new QHBox( page1 );
   hbox->setSpacing( KDialog::spacingHint() );
@@ -662,7 +698,7 @@ void AccountDialog::makePopAccountPage()
   mPop.filterOnServerSizeSpin->setRange( 1, 10000000, 100, FALSE );
   mPop.filterOnServerSizeSpin->setValue( 50000 );
   mPop.filterOnServerSizeSpin->setSuffix( i18n(" byte") );
-  grid->addMultiCellWidget( hbox, 8, 8, 0, 1 );
+  grid->addMultiCellWidget( hbox, 11, 11, 0, 1 );
   connect( mPop.filterOnServerCheck, SIGNAL(toggled(bool)),
 	   mPop.filterOnServerSizeSpin, SLOT(setEnabled(bool)) );
   connect( mPop.filterOnServerCheck, SIGNAL( clicked() ),
@@ -675,29 +711,29 @@ void AccountDialog::makePopAccountPage()
 
   mPop.intervalCheck =
     new QCheckBox( i18n("Enable &interval mail checking"), page1 );
-  grid->addMultiCellWidget( mPop.intervalCheck, 9, 9, 0, 1 );
+  grid->addMultiCellWidget( mPop.intervalCheck, 12, 12, 0, 1 );
   connect( mPop.intervalCheck, SIGNAL(toggled(bool)),
 	   this, SLOT(slotEnablePopInterval(bool)) );
   mPop.intervalLabel = new QLabel( i18n("Chec&k interval:"), page1 );
-  grid->addWidget( mPop.intervalLabel, 10, 0 );
+  grid->addWidget( mPop.intervalLabel, 13, 0 );
   mPop.intervalSpin = new KIntNumInput( page1 );
   mPop.intervalSpin->setRange( 1, 10000, 1, FALSE );
   mPop.intervalSpin->setSuffix( i18n(" min") );
   mPop.intervalSpin->setValue( 1 );
   mPop.intervalLabel->setBuddy( mPop.intervalSpin );
-  grid->addWidget( mPop.intervalSpin, 10, 1 );
+  grid->addWidget( mPop.intervalSpin, 13, 1 );
 
   label = new QLabel( i18n("Des&tination folder:"), page1 );
-  grid->addWidget( label, 11, 0 );
+  grid->addWidget( label, 14, 0 );
   mPop.folderCombo = new QComboBox( false, page1 );
   label->setBuddy( mPop.folderCombo );
-  grid->addWidget( mPop.folderCombo, 11, 1 );
+  grid->addWidget( mPop.folderCombo, 14, 1 );
 
   label = new QLabel( i18n("Pre-com&mand:"), page1 );
-  grid->addWidget( label, 12, 0 );
+  grid->addWidget( label, 15, 0 );
   mPop.precommand = new KLineEdit( page1 );
   label->setBuddy(mPop.precommand);
-  grid->addWidget( mPop.precommand, 12, 1 );
+  grid->addWidget( mPop.precommand, 15, 1 );
 
   QWidget *page2 = new QWidget( tabWidget );
   tabWidget->addTab( page2, i18n("&Extras") );
@@ -791,8 +827,6 @@ void AccountDialog::makeImapAccountPage( bool connected )
   int row = -1;
   QGridLayout *grid = new QGridLayout( page1, 16, 2, marginHint(), spacingHint() );
   grid->addColSpacing( 1, fontMetrics().maxWidth()*16 );
-//  grid->setRowStretch( 15, 10 );
-//  grid->setColStretch( 1, 10 );
 
   ++row;
   QLabel *label = new QLabel( i18n("Account &name:"), page1 );
@@ -835,12 +869,65 @@ void AccountDialog::makeImapAccountPage( bool connected )
   label->setBuddy( mImap.portEdit );
   grid->addWidget( mImap.portEdit, row, 1 );
 
+  // namespace list
   ++row;
-  label = new QLabel( i18n("Prefix to fol&ders:"), page1 );
-  grid->addWidget( label, row, 0 );
-  mImap.prefixEdit = new KLineEdit( page1 );
-  label->setBuddy( mImap.prefixEdit );
-  grid->addWidget( mImap.prefixEdit, row, 1 );
+  QHBox* box = new QHBox( page1 );
+  label = new QLabel( i18n("Namespaces:"), box );
+  QWhatsThis::add( label, i18n( "Here you see the different namespaces that your IMAP server supports."
+        "Each namespace represents a prefix that separates groups of folders."
+        "Namespaces allow KMail for example to display your personal folders and shared folders in one account." ) );
+  // button to reload
+  QToolButton* button = new QToolButton( box );
+  button->setAutoRaise(true);
+  button->setSizePolicy( QSizePolicy( QSizePolicy::Fixed, QSizePolicy::Fixed ) );
+  button->setFixedSize( 22, 22 );
+  button->setIconSet( 
+      KGlobal::iconLoader()->loadIconSet( "reload", KIcon::Small, 0 ) );
+  connect( button, SIGNAL(clicked()), this, SLOT(slotReloadNamespaces()) );
+  QWhatsThis::add( button, 
+      i18n("Reload the namespaces from the server. This overwrites any changes.") );
+  grid->addWidget( box, row, 0 );
+
+  // grid with label, namespace list and edit button
+  QGrid* listbox = new QGrid( 3, page1 );
+  label = new QLabel( i18n("Personal"), listbox );
+  QWhatsThis::add( label, i18n( "Personal namespaces include your personal folders." ) );
+  mImap.personalNS = new KLineEdit( listbox );
+  mImap.personalNS->setReadOnly( true );
+  mImap.editPNS = new QToolButton( listbox );
+  mImap.editPNS->setIconSet( 
+      KGlobal::iconLoader()->loadIconSet( "edit", KIcon::Small, 0 ) );
+  mImap.editPNS->setAutoRaise( true );
+  mImap.editPNS->setSizePolicy( QSizePolicy( QSizePolicy::Fixed, QSizePolicy::Fixed ) );
+  mImap.editPNS->setFixedSize( 22, 22 );
+  connect( mImap.editPNS, SIGNAL(clicked()), this, SLOT(slotEditPersonalNamespace()) );
+
+  label = new QLabel( i18n("Other Users"), listbox );
+  QWhatsThis::add( label, i18n( "These namespaces include the folders of other users." ) );
+  mImap.otherUsersNS = new KLineEdit( listbox );
+  mImap.otherUsersNS->setReadOnly( true );
+  mImap.editONS = new QToolButton( listbox );
+  mImap.editONS->setIconSet( 
+      KGlobal::iconLoader()->loadIconSet( "edit", KIcon::Small, 0 ) );
+  mImap.editONS->setAutoRaise( true );
+  mImap.editONS->setSizePolicy( QSizePolicy( QSizePolicy::Fixed, QSizePolicy::Fixed ) );
+  mImap.editONS->setFixedSize( 22, 22 );
+  connect( mImap.editONS, SIGNAL(clicked()), this, SLOT(slotEditOtherUsersNamespace()) );
+
+  label = new QLabel( i18n("Shared"), listbox );
+  QWhatsThis::add( label, i18n( "These namespaces include the shared folders." ) );
+  mImap.sharedNS = new KLineEdit( listbox );
+  mImap.sharedNS->setReadOnly( true );
+  mImap.editSNS = new QToolButton( listbox );
+  mImap.editSNS->setIconSet( 
+      KGlobal::iconLoader()->loadIconSet( "edit", KIcon::Small, 0 ) );
+  mImap.editSNS->setAutoRaise( true );
+  mImap.editSNS->setSizePolicy( QSizePolicy( QSizePolicy::Fixed, QSizePolicy::Fixed ) );
+  mImap.editSNS->setFixedSize( 22, 22 );
+  connect( mImap.editSNS, SIGNAL(clicked()), this, SLOT(slotEditSharedNamespace()) );
+
+  label->setBuddy( listbox );
+  grid->addWidget( listbox, row, 1 );
 
   ++row;
   mImap.storePasswordCheck =
@@ -936,7 +1023,7 @@ void AccountDialog::makeImapAccountPage( bool connected )
   mImap.intervalLabel = new QLabel( i18n("Check inter&val:"), page1 );
   grid->addWidget( mImap.intervalLabel, row, 0 );
   mImap.intervalSpin = new KIntNumInput( page1 );
-  mImap.intervalSpin->setRange( 1, 60, 1, FALSE );
+  mImap.intervalSpin->setRange( 1, 10000, 1, FALSE );
   mImap.intervalSpin->setValue( 1 );
   mImap.intervalSpin->setSuffix( i18n( " min" ) );
   mImap.intervalLabel->setBuddy( mImap.intervalSpin );
@@ -1058,7 +1145,7 @@ void AccountDialog::setupSettings()
   }
   else if( accountType == "pop" )
   {
-    KMAcctExpPop &ap = *(KMAcctExpPop*)mAccount;
+    PopAccount &ap = *(PopAccount*)mAccount;
     mPop.nameEdit->setText( mAccount->name() );
     mPop.nameEdit->setFocus();
     mPop.loginEdit->setText( ap.login() );
@@ -1068,6 +1155,18 @@ void AccountDialog::setupSettings()
     mPop.usePipeliningCheck->setChecked( ap.usePipelining() );
     mPop.storePasswordCheck->setChecked( ap.storePasswd() );
     mPop.leaveOnServerCheck->setChecked( ap.leaveOnServer() );
+    mPop.leaveOnServerDaysCheck->setEnabled( ap.leaveOnServer() );
+    mPop.leaveOnServerDaysCheck->setChecked( ap.leaveOnServerDays() >= 1 );
+    mPop.leaveOnServerDaysSpin->setValue( ap.leaveOnServerDays() >= 1 ?
+                                            ap.leaveOnServerDays() : 7 );
+    mPop.leaveOnServerCountCheck->setEnabled( ap.leaveOnServer() );
+    mPop.leaveOnServerCountCheck->setChecked( ap.leaveOnServerCount() >= 1 );
+    mPop.leaveOnServerCountSpin->setValue( ap.leaveOnServerCount() >= 1 ?
+                                            ap.leaveOnServerCount() : 100 );
+    mPop.leaveOnServerSizeCheck->setEnabled( ap.leaveOnServer() );
+    mPop.leaveOnServerSizeCheck->setChecked( ap.leaveOnServerSize() >= 1 );
+    mPop.leaveOnServerSizeSpin->setValue( ap.leaveOnServerSize() >= 1 ?
+                                            ap.leaveOnServerSize() : 10 );
     mPop.filterOnServerCheck->setChecked( ap.filterOnServer() );
     mPop.filterOnServerSizeSpin->setValue( ap.filterOnServerCheckSize() );
     mPop.intervalCheck->setChecked( interval >= 1 );
@@ -1098,6 +1197,12 @@ void AccountDialog::setupSettings()
       mPop.authAPOP->setChecked( TRUE );
     else mPop.authUser->setChecked( TRUE );
 
+    slotEnableLeaveOnServerDays( mPop.leaveOnServerDaysCheck->isEnabled() ?
+                                   ap.leaveOnServerDays() >= 1 : 0);
+    slotEnableLeaveOnServerCount( mPop.leaveOnServerCountCheck->isEnabled() ?
+                                    ap.leaveOnServerCount() >= 1 : 0);
+    slotEnableLeaveOnServerSize( mPop.leaveOnServerSizeCheck->isEnabled() ?
+                                    ap.leaveOnServerSize() >= 1 : 0);
     slotEnablePopInterval( interval >= 1 );
     folderCombo = mPop.folderCombo;
   }
@@ -1110,11 +1215,6 @@ void AccountDialog::setupSettings()
     mImap.passwordEdit->setText( ai.passwd());
     mImap.hostEdit->setText( ai.host() );
     mImap.portEdit->setText( QString("%1").arg( ai.port() ) );
-    QString prefix = ai.prefix();
-    if (!prefix.isEmpty() && prefix[0] == '/') prefix = prefix.mid(1);
-    if (!prefix.isEmpty() && prefix[prefix.length() - 1] == '/')
-      prefix = prefix.left(prefix.length() - 1);
-    mImap.prefixEdit->setText( prefix );
     mImap.autoExpungeCheck->setChecked( ai.autoExpunge() );
     mImap.hiddenFoldersCheck->setChecked( ai.hiddenFolders() );
     mImap.subscribedFoldersCheck->setChecked( ai.onlySubscribedFolders() );
@@ -1166,11 +1266,6 @@ void AccountDialog::setupSettings()
     mImap.passwordEdit->setText( ai.passwd());
     mImap.hostEdit->setText( ai.host() );
     mImap.portEdit->setText( QString("%1").arg( ai.port() ) );
-    QString prefix = ai.prefix();
-    if (!prefix.isEmpty() && prefix[0] == '/') prefix = prefix.mid(1);
-    if (!prefix.isEmpty() && prefix[prefix.length() - 1] == '/')
-      prefix = prefix.left(prefix.length() - 1);
-    mImap.prefixEdit->setText( prefix );
 #if 0
     mImap.resourceCheck->setChecked( ai.resource() );
 #endif
@@ -1232,6 +1327,20 @@ void AccountDialog::setupSettings()
   else // Unknown account type
     return;
 
+  if ( accountType == "imap" || accountType == "cachedimap" )
+  {
+    // settings for imap in general
+    ImapAccountBase &ai = *(ImapAccountBase*)mAccount;
+    // namespaces
+    if ( ( ai.namespaces().isEmpty() || ai.namespaceToDelimiter().isEmpty() ) &&
+         !ai.login().isEmpty() && !ai.passwd().isEmpty() && !ai.host().isEmpty() )
+    {
+      slotReloadNamespaces();
+    } else {
+      slotSetupNamespaces( ai.namespacesWithDelimiter() );
+    }
+  }
+
   if (!folderCombo) return;
 
   KMFolderDir *fdir = (KMFolderDir*)&kmkernel->folderMgr()->dir();
@@ -1273,9 +1382,27 @@ void AccountDialog::setupSettings()
   }
 }
 
-
 void AccountDialog::slotLeaveOnServerClicked()
 {
+  bool state = mPop.leaveOnServerCheck->isChecked();
+  mPop.leaveOnServerDaysCheck->setEnabled( state );
+  mPop.leaveOnServerCountCheck->setEnabled( state );
+  mPop.leaveOnServerSizeCheck->setEnabled( state );
+  if ( state ) {
+    if ( mPop.leaveOnServerDaysCheck->isChecked() ) {
+      slotEnableLeaveOnServerDays( state );
+    }
+    if ( mPop.leaveOnServerCountCheck->isChecked() ) {
+      slotEnableLeaveOnServerCount( state );
+    }
+    if ( mPop.leaveOnServerSizeCheck->isChecked() ) {
+      slotEnableLeaveOnServerSize( state );
+    }
+  } else {
+    slotEnableLeaveOnServerDays( state );
+    slotEnableLeaveOnServerCount( state );
+    slotEnableLeaveOnServerSize( state );
+  }
   if ( !( mCurCapa & UIDL ) && mPop.leaveOnServerCheck->isChecked() ) {
     KMessageBox::information( topLevelWidget(),
                               i18n("The server does not seem to support unique "
@@ -1636,37 +1763,22 @@ void AccountDialog::saveSettings()
 
     mAccount->setFolder( *mFolderList.at(mPop.folderCombo->currentItem()) );
 
-    KMAcctExpPop &epa = *(KMAcctExpPop*)mAccount;
-    epa.setHost( mPop.hostEdit->text().stripWhiteSpace() );
-    epa.setPort( mPop.portEdit->text().toInt() );
-    epa.setLogin( mPop.loginEdit->text().stripWhiteSpace() );
-    epa.setPasswd( mPop.passwordEdit->text(), true );
+    initAccountForConnect();
+    PopAccount &epa = *(PopAccount*)mAccount;
     epa.setUsePipelining( mPop.usePipeliningCheck->isChecked() );
-    epa.setStorePasswd( mPop.storePasswordCheck->isChecked() );
-    epa.setPasswd( mPop.passwordEdit->text(), epa.storePasswd() );
     epa.setLeaveOnServer( mPop.leaveOnServerCheck->isChecked() );
+    epa.setLeaveOnServerDays( mPop.leaveOnServerCheck->isChecked() ?
+                              ( mPop.leaveOnServerDaysCheck->isChecked() ?
+                                mPop.leaveOnServerDaysSpin->value() : -1 ) : 0);
+    epa.setLeaveOnServerCount( mPop.leaveOnServerCheck->isChecked() ?
+                               ( mPop.leaveOnServerCountCheck->isChecked() ?
+                                 mPop.leaveOnServerCountSpin->value() : -1 ) : 0 );
+    epa.setLeaveOnServerSize( mPop.leaveOnServerCheck->isChecked() ?
+                              ( mPop.leaveOnServerSizeCheck->isChecked() ?
+                                mPop.leaveOnServerSizeSpin->value() : -1 ) : 0 );
     epa.setFilterOnServer( mPop.filterOnServerCheck->isChecked() );
     epa.setFilterOnServerCheckSize (mPop.filterOnServerSizeSpin->value() );
     epa.setPrecommand( mPop.precommand->text() );
-    epa.setUseSSL( mPop.encryptionSSL->isChecked() );
-    epa.setUseTLS( mPop.encryptionTLS->isChecked() );
-    if (mPop.authUser->isChecked())
-      epa.setAuth("USER");
-    else if (mPop.authLogin->isChecked())
-      epa.setAuth("LOGIN");
-    else if (mPop.authPlain->isChecked())
-      epa.setAuth("PLAIN");
-    else if (mPop.authCRAM_MD5->isChecked())
-      epa.setAuth("CRAM-MD5");
-    else if (mPop.authDigestMd5->isChecked())
-      epa.setAuth("DIGEST-MD5");
-    else if (mPop.authNTLM->isChecked())
-      epa.setAuth("NTLM");
-    else if (mPop.authGSSAPI->isChecked())
-      epa.setAuth("GSSAPI");
-    else if (mPop.authAPOP->isChecked())
-      epa.setAuth("APOP");
-    else epa.setAuth("AUTO");
   }
   else if( accountType == "imap" )
   {
@@ -1679,20 +1791,13 @@ void AccountDialog::saveSettings()
     mAccount->setCheckExclude( !mImap.includeInCheck->isChecked() );
     mAccount->setFolder( kmkernel->imapFolderMgr()->findById(mAccount->id()) );
 
+    initAccountForConnect();
     KMAcctImap &epa = *(KMAcctImap*)mAccount;
-    epa.setHost( mImap.hostEdit->text().stripWhiteSpace() );
-    epa.setPort( mImap.portEdit->text().toInt() );
-    QString prefix = "/" + mImap.prefixEdit->text();
-    if (prefix[prefix.length() - 1] != '/') prefix += "/";
-    epa.setPrefix( prefix );
-    epa.setLogin( mImap.loginEdit->text().stripWhiteSpace() );
     epa.setAutoExpunge( mImap.autoExpungeCheck->isChecked() );
     epa.setHiddenFolders( mImap.hiddenFoldersCheck->isChecked() );
     epa.setOnlySubscribedFolders( mImap.subscribedFoldersCheck->isChecked() );
     epa.setLoadOnDemand( mImap.loadOnDemandCheck->isChecked() );
     epa.setListOnlyOpenFolders( mImap.listOnlyOpenCheck->isChecked() );
-    epa.setStorePasswd( mImap.storePasswordCheck->isChecked() );
-    epa.setPasswd( mImap.passwordEdit->text(), epa.storePasswd() );
     KMFolder *t = mImap.trashCombo->folder();
     if ( t )
       epa.setTrash( mImap.trashCombo->folder()->idString() );
@@ -1702,23 +1807,6 @@ void AccountDialog::saveSettings()
     epa.setResource( mImap.resourceCheck->isChecked() );
 #endif
     epa.setCheckExclude( !mImap.includeInCheck->isChecked() );
-    epa.setUseSSL( mImap.encryptionSSL->isChecked() );
-    epa.setUseTLS( mImap.encryptionTLS->isChecked() );
-    if (mImap.authCramMd5->isChecked())
-      epa.setAuth("CRAM-MD5");
-    else if (mImap.authDigestMd5->isChecked())
-      epa.setAuth("DIGEST-MD5");
-    else if (mImap.authNTLM->isChecked())
-      epa.setAuth("NTLM");
-    else if (mImap.authGSSAPI->isChecked())
-      epa.setAuth("GSSAPI");
-    else if (mImap.authAnonymous->isChecked())
-      epa.setAuth("ANONYMOUS");
-    else if (mImap.authLogin->isChecked())
-      epa.setAuth("LOGIN");
-    else if (mImap.authPlain->isChecked())
-      epa.setAuth("PLAIN");
-    else epa.setAuth("*");
     if ( mSieveConfigEditor )
       epa.setSieveConfig( mSieveConfigEditor->config() );
   }
@@ -1733,20 +1821,12 @@ void AccountDialog::saveSettings()
     mAccount->setCheckExclude( !mImap.includeInCheck->isChecked() );
     //mAccount->setFolder( NULL );
     mAccount->setFolder( kmkernel->dimapFolderMgr()->findById(mAccount->id()) );
-    kdDebug(5006) << mAccount->name() << endl;
     //kdDebug(5006) << "account for folder " << mAccount->folder()->name() << endl;
 
+    initAccountForConnect();
     KMAcctCachedImap &epa = *(KMAcctCachedImap*)mAccount;
-    epa.setHost( mImap.hostEdit->text().stripWhiteSpace() );
-    epa.setPort( mImap.portEdit->text().toInt() );
-    QString prefix = "/" + mImap.prefixEdit->text();
-    if (prefix[prefix.length() - 1] != '/') prefix += "/";
-    epa.setPrefix( prefix );
-    epa.setLogin( mImap.loginEdit->text().stripWhiteSpace() );
     epa.setHiddenFolders( mImap.hiddenFoldersCheck->isChecked() );
     epa.setOnlySubscribedFolders( mImap.subscribedFoldersCheck->isChecked() );
-    epa.setStorePasswd( mImap.storePasswordCheck->isChecked() );
-    epa.setPasswd( mImap.passwordEdit->text(), epa.storePasswd() );
     KMFolder *t = mImap.trashCombo->folder();
     if ( t )
       epa.setTrash( mImap.trashCombo->folder()->idString() );
@@ -1756,23 +1836,6 @@ void AccountDialog::saveSettings()
     epa.setResource( mImap.resourceCheck->isChecked() );
 #endif
     epa.setCheckExclude( !mImap.includeInCheck->isChecked() );
-    epa.setUseSSL( mImap.encryptionSSL->isChecked() );
-    epa.setUseTLS( mImap.encryptionTLS->isChecked() );
-    if (mImap.authCramMd5->isChecked())
-      epa.setAuth("CRAM-MD5");
-    else if (mImap.authDigestMd5->isChecked())
-      epa.setAuth("DIGEST-MD5");
-    else if (mImap.authNTLM->isChecked())
-      epa.setAuth("NTLM");
-    else if (mImap.authGSSAPI->isChecked())
-      epa.setAuth("GSSAPI");
-    else if (mImap.authAnonymous->isChecked())
-      epa.setAuth("ANONYMOUS");
-    else if (mImap.authLogin->isChecked())
-      epa.setAuth("LOGIN");
-    else if (mImap.authPlain->isChecked())
-      epa.setAuth("PLAIN");
-    else epa.setAuth("*");
     if ( mSieveConfigEditor )
       epa.setSieveConfig( mSieveConfigEditor->config() );
   }
@@ -1805,6 +1868,27 @@ void AccountDialog::saveSettings()
     mAccount->setPrecommand( mMaildir.precommand->text() );
   }
 
+  if ( accountType == "imap" || accountType == "cachedimap" )
+  {
+    // settings for imap in general
+    ImapAccountBase &ai = *(ImapAccountBase*)mAccount;
+    // namespace
+    ImapAccountBase::nsMap map;
+    ImapAccountBase::namespaceDelim delimMap;
+    ImapAccountBase::nsDelimMap::Iterator it;
+    ImapAccountBase::namespaceDelim::Iterator it2;
+    for ( it = mImap.nsMap.begin(); it != mImap.nsMap.end(); ++it ) {
+      QStringList list;
+      for ( it2 = it.data().begin(); it2 != it.data().end(); ++it2 ) {
+        list << it2.key();
+        delimMap[it2.key()] = it2.data();
+      }
+      map[it.key()] = list;
+    }
+    ai.setNamespaces( map );
+    ai.setNamespaceToDelimiter( delimMap );
+  }  
+
   kmkernel->acctMgr()->writeConfig(TRUE);
 
   // get the new account and register the new destination folder
@@ -1869,6 +1953,25 @@ void AccountDialog::slotMaildirChooser()
   directory = dir;
 }
 
+void AccountDialog::slotEnableLeaveOnServerDays( bool state )
+{
+  if ( state && !mPop.leaveOnServerDaysCheck->isEnabled()) return;
+  mPop.leaveOnServerDaysSpin->setEnabled( state );
+}
+
+void AccountDialog::slotEnableLeaveOnServerCount( bool state )
+{
+  if ( state && !mPop.leaveOnServerCountCheck->isEnabled()) return;
+  mPop.leaveOnServerCountSpin->setEnabled( state );
+  return;
+}
+
+void AccountDialog::slotEnableLeaveOnServerSize( bool state )
+{
+  if ( state && !mPop.leaveOnServerSizeCheck->isEnabled()) return;
+  mPop.leaveOnServerSizeSpin->setEnabled( state );
+  return;
+}
 
 void AccountDialog::slotEnablePopInterval( bool state )
 {
@@ -1932,4 +2035,242 @@ void AccountDialog::slotClearPastResourceAllocations()
 }
 #endif
 
+void AccountDialog::slotReloadNamespaces()
+{
+  if ( mAccount->type() == "imap" || mAccount->type() == "cachedimap" )
+  {
+    initAccountForConnect();
+    mImap.personalNS->setText( i18n("Fetching Namespaces...") );
+    mImap.otherUsersNS->setText( QString::null );
+    mImap.sharedNS->setText( QString::null );
+    ImapAccountBase* ai = static_cast<ImapAccountBase*>( mAccount );
+    connect( ai, SIGNAL( namespacesFetched( const ImapAccountBase::nsDelimMap& ) ),
+        this, SLOT( slotSetupNamespaces( const ImapAccountBase::nsDelimMap& ) ) );
+    connect( ai, SIGNAL( connectionResult(int, const QString&) ),
+          this, SLOT( slotConnectionResult(int, const QString&) ) );    
+    ai->getNamespaces();
+  }
+}
+
+void AccountDialog::slotConnectionResult( int errorCode, const QString& )
+{
+  if ( errorCode > 0 ) {
+    ImapAccountBase* ai = static_cast<ImapAccountBase*>( mAccount );
+    disconnect( ai, SIGNAL( namespacesFetched( const ImapAccountBase::nsDelimMap& ) ),
+        this, SLOT( slotSetupNamespaces( const ImapAccountBase::nsDelimMap& ) ) );
+    disconnect( ai, SIGNAL( connectionResult(int, const QString&) ),
+          this, SLOT( slotConnectionResult(int, const QString&) ) );    
+    mImap.personalNS->setText( QString::null );
+  }
+}
+
+void AccountDialog::slotSetupNamespaces( const ImapAccountBase::nsDelimMap& map )
+{
+  disconnect( this, SLOT( slotSetupNamespaces( const ImapAccountBase::nsDelimMap& ) ) );
+  mImap.personalNS->setText( QString::null );
+  mImap.otherUsersNS->setText( QString::null );
+  mImap.sharedNS->setText( QString::null );
+  mImap.nsMap = map;
+
+  ImapAccountBase::namespaceDelim ns = map[ImapAccountBase::PersonalNS];
+  ImapAccountBase::namespaceDelim::ConstIterator it;
+  if ( !ns.isEmpty() ) {
+    mImap.personalNS->setText( namespaceListToString( ns.keys() ) );
+    mImap.editPNS->setEnabled( true );
+  } else {
+    mImap.editPNS->setEnabled( false );
+  }
+  ns = map[ImapAccountBase::OtherUsersNS];
+  if ( !ns.isEmpty() ) {
+    mImap.otherUsersNS->setText( namespaceListToString( ns.keys() ) );
+    mImap.editONS->setEnabled( true );
+  } else {
+    mImap.editONS->setEnabled( false );
+  }
+  ns = map[ImapAccountBase::SharedNS];
+  if ( !ns.isEmpty() ) {
+    mImap.sharedNS->setText( namespaceListToString( ns.keys() ) );
+    mImap.editSNS->setEnabled( true );
+  } else {
+    mImap.editSNS->setEnabled( false );
+  }
+}
+
+const QString AccountDialog::namespaceListToString( const QStringList& list )
+{
+  QStringList myList = list;
+  for ( QStringList::Iterator it = myList.begin(); it != myList.end(); ++it ) {
+    if ( (*it).isEmpty() ) {
+      (*it) = "<" + i18n("Empty") + ">";
+    }
+  }
+  return myList.join(",");
+}
+
+void AccountDialog::initAccountForConnect()
+{
+  QString type = mAccount->type();
+  if ( type == "local" )
+    return;
+
+  NetworkAccount &na = *(NetworkAccount*)mAccount;
+
+  if ( type == "pop" ) {
+    na.setHost( mPop.hostEdit->text().stripWhiteSpace() );
+    na.setPort( mPop.portEdit->text().toInt() );
+    na.setLogin( mPop.loginEdit->text().stripWhiteSpace() );
+    na.setStorePasswd( mPop.storePasswordCheck->isChecked() );
+    na.setPasswd( mPop.passwordEdit->text(), na.storePasswd() );
+    na.setUseSSL( mPop.encryptionSSL->isChecked() );
+    na.setUseTLS( mPop.encryptionTLS->isChecked() );
+    if (mPop.authUser->isChecked())
+      na.setAuth("USER");
+    else if (mPop.authLogin->isChecked())
+      na.setAuth("LOGIN");
+    else if (mPop.authPlain->isChecked())
+      na.setAuth("PLAIN");
+    else if (mPop.authCRAM_MD5->isChecked())
+      na.setAuth("CRAM-MD5");
+    else if (mPop.authDigestMd5->isChecked())
+      na.setAuth("DIGEST-MD5");
+    else if (mPop.authNTLM->isChecked())
+      na.setAuth("NTLM");
+    else if (mPop.authGSSAPI->isChecked())
+      na.setAuth("GSSAPI");
+    else if (mPop.authAPOP->isChecked())
+      na.setAuth("APOP");
+    else na.setAuth("AUTO");    
+  } 
+  else if ( type == "imap" || type == "cachedimap" ) {
+    na.setHost( mImap.hostEdit->text().stripWhiteSpace() );
+    na.setPort( mImap.portEdit->text().toInt() );
+    na.setLogin( mImap.loginEdit->text().stripWhiteSpace() );
+    na.setStorePasswd( mImap.storePasswordCheck->isChecked() );
+    na.setPasswd( mImap.passwordEdit->text(), na.storePasswd() );
+    na.setUseSSL( mImap.encryptionSSL->isChecked() );
+    na.setUseTLS( mImap.encryptionTLS->isChecked() );
+    if (mImap.authCramMd5->isChecked())
+      na.setAuth("CRAM-MD5");
+    else if (mImap.authDigestMd5->isChecked())
+      na.setAuth("DIGEST-MD5");
+    else if (mImap.authNTLM->isChecked())
+      na.setAuth("NTLM");
+    else if (mImap.authGSSAPI->isChecked())
+      na.setAuth("GSSAPI");
+    else if (mImap.authAnonymous->isChecked())
+      na.setAuth("ANONYMOUS");
+    else if (mImap.authLogin->isChecked())
+      na.setAuth("LOGIN");
+    else if (mImap.authPlain->isChecked())
+      na.setAuth("PLAIN");
+    else na.setAuth("*");    
+  }
+}
+
+void AccountDialog::slotEditPersonalNamespace()
+{
+  NamespaceEditDialog dialog( this, ImapAccountBase::PersonalNS, &mImap.nsMap );
+  if ( dialog.exec() == QDialog::Accepted ) {
+    slotSetupNamespaces( mImap.nsMap );
+  }
+}
+
+void AccountDialog::slotEditOtherUsersNamespace()
+{
+  NamespaceEditDialog dialog( this, ImapAccountBase::OtherUsersNS, &mImap.nsMap );
+  if ( dialog.exec() == QDialog::Accepted ) {
+    slotSetupNamespaces( mImap.nsMap );
+  }
+}
+
+void AccountDialog::slotEditSharedNamespace()
+{
+  NamespaceEditDialog dialog( this, ImapAccountBase::SharedNS, &mImap.nsMap );
+  if ( dialog.exec() == QDialog::Accepted ) {
+    slotSetupNamespaces( mImap.nsMap );
+  }
+}
+
+NamespaceLineEdit::NamespaceLineEdit( QWidget* parent )
+  : KLineEdit( parent )
+{
+}
+
+void NamespaceLineEdit::setText( const QString& text )
+{
+  mLastText = text;
+  KLineEdit::setText( text );
+}
+
+NamespaceEditDialog::NamespaceEditDialog( QWidget *parent, 
+    ImapAccountBase::imapNamespace type, ImapAccountBase::nsDelimMap* map )
+  : KDialogBase( parent, "edit_namespace", false, QString::null,
+      Ok|Cancel, Ok, true ), mType( type ), mNamespaceMap( map )
+{
+  QVBox *page = makeVBoxMainWidget();
+
+  QString ns;
+  if ( mType == ImapAccountBase::PersonalNS ) {
+    ns = i18n("Personal");
+  } else if ( mType == ImapAccountBase::OtherUsersNS ) {
+    ns = i18n("Other Users");
+  } else {
+    ns = i18n("Shared");
+  }
+  setCaption( i18n("Edit Namespace '%1'").arg(ns) );
+  QGrid* grid = new QGrid( 2, page );
+
+  mBg = new QButtonGroup( 0 );
+  connect( mBg, SIGNAL( clicked(int) ), this, SLOT( slotRemoveEntry(int) ) );
+  mDelimMap = mNamespaceMap->find( mType ).data();
+  ImapAccountBase::namespaceDelim::Iterator it;
+  for ( it = mDelimMap.begin(); it != mDelimMap.end(); ++it ) {
+    NamespaceLineEdit* edit = new NamespaceLineEdit( grid );
+    edit->setText( it.key() );
+    QToolButton* button = new QToolButton( grid );
+    button->setIconSet( 
+      KGlobal::iconLoader()->loadIconSet( "editdelete", KIcon::Small, 0 ) );
+    button->setAutoRaise( true );
+    button->setSizePolicy( QSizePolicy( QSizePolicy::Fixed, QSizePolicy::Fixed ) );
+    button->setFixedSize( 22, 22 );
+    mLineEditMap[ mBg->insert( button ) ] = edit;
+  }
+}
+
+void NamespaceEditDialog::slotRemoveEntry( int id )
+{
+  if ( mLineEditMap.contains( id ) ) {
+    // delete the lineedit and remove namespace from map
+    NamespaceLineEdit* edit = mLineEditMap[id];
+    mDelimMap.remove( edit->text() );
+    if ( edit->isModified() ) {
+      mDelimMap.remove( edit->lastText() );
+    }
+    mLineEditMap.remove( id );
+    delete edit;
+  }
+  if ( mBg->find( id ) ) {
+    // delete the button
+    delete mBg->find( id );
+  }
+  adjustSize();
+}
+
+void NamespaceEditDialog::slotOk()
+{
+  QMap<int, NamespaceLineEdit*>::Iterator it;
+  for ( it = mLineEditMap.begin(); it != mLineEditMap.end(); ++it ) {
+    NamespaceLineEdit* edit = it.data();
+    if ( edit->isModified() ) {
+      // register delimiter for new namespace
+      mDelimMap[edit->text()] = mDelimMap[edit->lastText()];
+      mDelimMap.remove( edit->lastText() );
+    }
+  }
+  mNamespaceMap->replace( mType, mDelimMap );
+  KDialogBase::slotOk();
+}
+
+} // namespace KMail
+
 #include "accountdialog.moc"
