@@ -1,4 +1,4 @@
-/**
+/*
  * Copyright (c) 2004 Carsten Burghardt <burghardt@kde.org>
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -12,7 +12,7 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+ *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
  *  In addition, as a special exception, the copyright holders give
  *  permission to link the code of this program with any edition of
@@ -27,11 +27,13 @@
  */
 
 #include "folderrequester.h"
+#include "kmfolder.h"
 #include "kmfoldertree.h"
 #include "kmfolderseldlg.h"
 
 #include <kdebug.h>
 #include <klineedit.h>
+#include <kiconloader.h>
 #include <kdialog.h>
 
 #include <qlayout.h>
@@ -40,7 +42,7 @@
 namespace KMail {
 
 FolderRequester::FolderRequester( QWidget *parent, KMFolderTree *tree )
-  : QWidget( parent ), mFolder( 0 ), mFolderTree( tree ), 
+  : QWidget( parent ), mFolder( 0 ), mFolderTree( tree ),
     mMustBeReadWrite( true ), mShowOutbox( true ), mShowImapFolders( true )
 {
   QHBoxLayout * hlay = new QHBoxLayout( this, 0, KDialog::spacingHint() );
@@ -50,10 +52,7 @@ FolderRequester::FolderRequester( QWidget *parent, KMFolderTree *tree )
   edit->setReadOnly( true );
 
   QToolButton* button = new QToolButton( this );
-  button->setAutoRaise(true);
-  button->setSizePolicy( QSizePolicy( QSizePolicy::Fixed, QSizePolicy::Fixed ) );
-  button->setFixedSize( 16, 16 );
-  button->setIconSet( KGlobal::iconLoader()->loadIconSet( "up", KIcon::Small, 0 ) );
+  button->setIconSet( KGlobal::iconLoader()->loadIconSet( "folder", KIcon::Small, 0 ) );
   connect( button, SIGNAL(clicked()), this, SLOT(slotOpenDialog()) );
 
   setSizePolicy( QSizePolicy( QSizePolicy::MinimumExpanding,
@@ -64,7 +63,7 @@ FolderRequester::FolderRequester( QWidget *parent, KMFolderTree *tree )
 //-----------------------------------------------------------------------------
 void FolderRequester::slotOpenDialog()
 {
-  KMFolderSelDlg dlg( this, mFolderTree, i18n("Select Folder"), 
+  KMFolderSelDlg dlg( this, mFolderTree, i18n("Select Folder"),
       mMustBeReadWrite, false );
   dlg.setFlags( mMustBeReadWrite, mShowOutbox, mShowImapFolders );
   dlg.setFolder( mFolder );
@@ -88,8 +87,10 @@ KMFolder * FolderRequester::folder( void ) const
 void FolderRequester::setFolder( KMFolder *folder )
 {
   mFolder = folder;
-  if ( mFolder )
+  if ( mFolder ) {
     edit->setText( mFolder->prettyURL() );
+    mFolderId = mFolder->idString();
+  }
   else if ( !mMustBeReadWrite ) // the Local Folders root node was selected
     edit->setText( i18n("Local Folders") );
   emit folderChanged( folder );
@@ -98,7 +99,14 @@ void FolderRequester::setFolder( KMFolder *folder )
 //-----------------------------------------------------------------------------
 void FolderRequester::setFolder( const QString &idString )
 {
-  setFolder( kmkernel->findFolderById( idString ) );
+  KMFolder *folder = kmkernel->findFolderById( idString );
+  if ( folder )
+    setFolder( folder );
+  else {
+    edit->setText( i18n( "Unknown folder '%1'" ).arg( idString ) );
+    mFolder = 0;
+  }
+  mFolderId = idString;
 }
 
 //-----------------------------------------------------------------------------
