@@ -28,7 +28,7 @@ using KPIM::ProgressManager;
 #include <assert.h>
 
 //-----------------------------------------------------------------------------
-KMAcctLocal::KMAcctLocal(KMAcctMgr* aOwner, const QString& aAccountName, uint id):
+KMAcctLocal::KMAcctLocal(AccountManager* aOwner, const QString& aAccountName, uint id):
   KMAccount(aOwner, aAccountName, id), mHasNewMail( false ),
   mProcessingNewMail( false ), mAddedOk( true ), mNumMsgs( 0 ),
   mMsgsFetched( 0 ), mMailFolder( 0 )
@@ -114,7 +114,8 @@ bool KMAcctLocal::preProcess()
     }
   }
 
-  mMailFolder = new KMFolder( 0, location(), KMFolderTypeMbox );
+  mMailFolder = new KMFolder( 0, location(), KMFolderTypeMbox,
+                              false /* no index */, false /* don't export sernums */ );
   KMFolderMbox* mboxStorage =
     static_cast<KMFolderMbox*>(mMailFolder->storage());
   mboxStorage->setLockType( mLock );
@@ -148,9 +149,7 @@ bool KMAcctLocal::preProcess()
     BroadcastStatus::instance()->setStatusMsg( i18n( "Running precommand failed." ));
     return false;
   }
-
-  mMailFolder->setAutoCreateIndex(FALSE);
-
+  
   const int rc = mMailFolder->open();
   if ( rc != 0 ) {
     QString aStr;
