@@ -1,8 +1,21 @@
-/* Mail Filter Action: a filter action has one method that processes the
- * given mail message.
+/*
+ * kmail: KDE mail client
+ * Copyright (c) 1996-1998 Stefan Taferner <taferner@kde.org>
+ *
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * Author: Stefan Taferner <taferner@kde.org>
- * This code is under GPL
  */
 #ifndef kmfilteraction_h
 #define kmfilteraction_h
@@ -42,7 +55,7 @@ class KTempFile;
 class KMFilterAction
 {
 public:
-  /** Possible return codes of @ref process:
+  /** Possible return codes of process:
 
       @li @p ErrorNeedComplete: Could not process because a
       complete message is needed.
@@ -119,6 +132,10 @@ public:
   /** Return extra arguments as string. Must not contain newlines. */
   virtual const QString argsAsString() const = 0;
 
+  /** Returns a translated string describing this filter for visualization
+      purposes, e.g. in the filter log. */
+  virtual const QString displayString() const = 0;
+
   /** Called from the filter when a folder is removed.  Tests if the
       folder @p aFolder is used and changes to @p aNewFolder in this
       case. Returns TRUE if a change was made.  */
@@ -127,7 +144,7 @@ public:
   /** Static function that creates a filter action of this type. */
   static KMFilterAction* newAction();
 
-  /** Temporarily open folder. Will be closed by the next @ref
+  /** Temporarily open folder. Will be closed by the next 
     KMFilterMgr::cleanup() call.  */
   static int tempOpenFolder(KMFolder* aFolder);
 
@@ -149,12 +166,12 @@ private:
 
 
 /** Abstract base class for KMail's filter actions that need no
-    parameter, e.g. 'bounce'. Creates an (empty) @ref QWidget as
+    parameter, e.g. "Confirm Delivery". Creates an (empty) QWidget as
     parameter widget. A subclass of this must provide at least
     implementations for the following methods:
 
-    @li virtual @ref KMFilterAction::ReturnCodes @ref KMFilterAction::process
-    @li static @ref KMFilterAction::newAction
+    @li virtual KMFilterAction::ReturnCodes KMFilterAction::process
+    @li static KMFilterAction::newAction
 
     @short Abstract base class for filter actions with no parameter.
     @author Marc Mutz <Marc@Mutz.com>, based upon work by Stefan Taferner <taferner@kde.org>
@@ -173,10 +190,15 @@ public:
   virtual void argsFromString(const QString) {};
 
   /** Return extra arguments as string. Must not contain newlines. We
-      return @ref QString::null, because we have no parameter. */
+      return QString::null, because we have no parameter. */
   virtual const QString argsAsString() const { return QString::null; }
+
+  /** Returns a translated string describing this filter for visualization
+      purposes, e.g. in the filter log. */
+  virtual const QString displayString() const;
 };
 
+
 //=========================================================
 //
 // class KMFilterActionWithString
@@ -186,11 +208,11 @@ public:
 
 /** Abstract base class for KMail's filter actions that need a
     free-form parameter, e.g. 'set transport' or 'set reply to'.  Can
-    create a @ref QLineEdit as parameter widget. A subclass of this
+    create a QLineEdit as parameter widget. A subclass of this
     must provide at least implementations for the following methods:
 
-    @li virtual @ref KMFilterAction::ReturnCodes @ref KMFilterAction::process
-    @li static @ref KMFilterAction::newAction
+    @li virtual KMFilterAction::ReturnCodes KMFilterAction::process
+    @li static KMFilterAction::newAction
 
     @short Abstract base class for filter actions with a free-form string as parameter.
     @author Marc Mutz <Marc@Mutz.com>, based upon work by Stefan Taferner <taferner@kde.org>
@@ -233,6 +255,10 @@ public:
   /** Return extra arguments as string. Must not contain newlines. */
   virtual const QString argsAsString() const;
 
+  /** Returns a translated string describing this filter for visualization
+      purposes, e.g. in the filter log. */
+  virtual const QString displayString() const;
+
 protected:
   QString mParameter;
 };
@@ -248,8 +274,8 @@ protected:
     parameter that has a UOID, e.g. "set identity". A subclass of this
     must provide at least implementations for the following methods:
 
-    @li virtual @ref KMFilterAction::ReturnCodes @ref KMFilterAction::process
-    @li static @ref KMFilterAction::newAction
+    @li virtual KMFilterAction::ReturnCodes KMFilterAction::process
+    @li static KMFilterAction::newAction
     @li the *ParamWidget* methods.
 
     @short Abstract base class for filter actions with a free-form string as parameter.
@@ -276,6 +302,10 @@ public:
   /** Return extra arguments as string. Must not contain newlines. */
   virtual const QString argsAsString() const;
 
+  /** Returns a translated string describing this filter for visualization
+      purposes, e.g. in the filter log. */
+  virtual const QString displayString() const;
+
 protected:
   uint mParameter;
 };
@@ -289,14 +319,14 @@ protected:
 
 /** Abstract base class for KMail's filter actions that need a
     parameter which can be chosen from a fixed set, e.g. 'set
-    identity'.  Can create a @ref QComboBox as parameter widget. A
+    identity'.  Can create a QComboBox as parameter widget. A
     subclass of this must provide at least implementations for the
     following methods:
 
-    @li virtual @ref KMFilterAction::ReturnCodes @ref KMFilterAction::process
-    @li static @ref KMFilterAction::newAction
+    @li virtual KMFilterAction::ReturnCodes KMFilterAction::process
+    @li static  KMFilterAction::newAction
 
-    Additionally, it's constructor should populate the @ref
+    Additionally, it's constructor should populate the
     QStringList @p mParameterList with the valid parameter
     strings. The combobox will then contain be populated automatically
     with those strings. The default string will be the first one.
@@ -346,12 +376,12 @@ protected:
 
 
 /** Abstract base class for KMail's filter actions that need a
-    mail folder as parameter, e.g. 'file into folder'. Can
-    create a @ref QComboBox as parameter widget. A subclass of this
+    mail folder as parameter, e.g. 'move into folder'. Can
+    create a QComboBox as parameter widget. A subclass of this
     must provide at least implementations for the following methods:
 
-    @li virtual @ref KMFilterAction::ReturnCodes @ref KMFilterAction::process
-    @li static @ref KMFilterAction::newAction
+    @li virtual KMFilterAction::ReturnCodes KMFilterAction::process
+    @li static KMFilterAction::newAction
 
     @short Abstract base class for filter actions with a mail folder as parameter.
     @author Marc Mutz <Marc@Mutz.com>, based upon work by Stefan Taferner <taferner@kde.org>
@@ -395,6 +425,10 @@ public:
   /** Return extra arguments as string. Must not contain newlines. */
   virtual const QString argsAsString() const;
 
+  /** Returns a translated string describing this filter for visualization
+      purposes, e.g. in the filter log. */
+  virtual const QString displayString() const;
+
   /** Called from the filter when a folder is removed.  Tests if the
       folder @p aFolder is used and changes to @p aNewFolder in this
       case. Returns TRUE if a change was made.  */
@@ -413,13 +447,13 @@ protected:
 
 
 /** Abstract base class for KMail's filter actions that need a mail
-    address as parameter, e.g. 'forward to'. Can create a @ref
+    address as parameter, e.g. 'forward to'. Can create a 
     QComboBox (capable of completion from the address book) as
     parameter widget. A subclass of this must provide at least
     implementations for the following methods:
 
-    @li virtual @ref KMFilterAction::ReturnCodes @ref KMFilterAction::process
-    @li static @ref KMFilterAction::newAction
+    @li virtual KMFilterAction::ReturnCodes KMFilterAction::process
+    @li static KMFilterAction::newAction
 
     @short Abstract base class for filter actions with a mail address as parameter.
     @author Marc Mutz <Marc@Mutz.com>, based upon work by Stefan Taferner <taferner@kde.org>
@@ -459,15 +493,15 @@ public:
 
 
 /** Abstract base class for KMail's filter actions that need a command
-    line as parameter, e.g. 'forward to'. Can create a @ref QLineEdit
+    line as parameter, e.g. 'forward to'. Can create a QLineEdit
     (are there better widgets in the depths of the kdelibs?) as
     parameter widget. A subclass of this must provide at least
     implementations for the following methods:
 
-    @li virtual @ref KMFilterAction::ReturnCodes @ref KMFilterAction::process
-    @li static @ref KMFilterAction::newAction
+    @li virtual KMFilterAction::ReturnCodes KMFilterAction::process
+    @li static KMFilterAction::newAction
 
-    The implementation of @ref KMFilterAction::process should take the
+    The implementation of KMFilterAction::process should take the
     command line specified in mParameter, make all required
     modifications and stream the resulting command line into @p
     mProcess. Then you can start the command with @p mProcess.start().
@@ -513,6 +547,10 @@ public:
   /** Return extra arguments as string. Must not contain newlines. */
   virtual const QString argsAsString() const;
 
+  /** Returns a translated string describing this filter for visualization
+      purposes, e.g. in the filter log. */
+  virtual const QString displayString() const;
+
 protected:
   QString mParameter;
 };
@@ -590,6 +628,10 @@ public:
   /** Return extra arguments as string. Must not contain newlines. */
   virtual const QString argsAsString() const;
 
+  /** Returns a translated string describing this filter for visualization
+      purposes, e.g. in the filter log. */
+  virtual const QString displayString() const;
+
 protected:
   QString mParameter;
 };
@@ -599,7 +641,7 @@ typedef KMFilterAction* (*KMFilterActionNewFunc)(void);
 
 
 //-----------------------------------------------------------------------------
-/** Auxiliary struct to @ref KMFilterActionDict.  */
+/** Auxiliary struct to KMFilterActionDict.  */
 struct KMFilterActionDesc
 {
   QString label, name;
@@ -608,20 +650,20 @@ struct KMFilterActionDesc
 
 /** Dictionary that contains a list of all registered filter actions
     with their creation functions. They are hard-coded into the
-    constructor. If you want to add a new @ref KMFilterAction, make
-    sure you add the details of it in @ref init, too.
+    constructor. If you want to add a new KMFilterAction, make
+    sure you add the details of it in init, too.
 
     You will be able to find a description of a KMFilterAction by
     looking up either it's (english) name or it's (i18n) label:
     <pre>
     KMFilterActionDict dict;
-    // get name of the action with label "file into folder":
-    dict[i18n("file into folder")]->name; // == "transfer"
+    // get name of the action with label "move into folder":
+    dict[i18n("move into folder")]->name; // == "transfer"
     // create one such action:
     KMFilterAction *action = dict["transfer"]->create();
     </pre>
 
-    You can iterate over all known filter actions by using @ref list.
+    You can iterate over all known filter actions by using list.
 
     @short List of known KMFilterAction-types.
     @author Marc Mutz <Marc@Mutz.com>, based on work by Stefan Taferner <taferner@kde.org>
@@ -634,10 +676,10 @@ public:
   KMFilterActionDict();
 
   /** Overloaded member function, provided for convenience. Thin
-      wrapper around @ref QDict::insert and @ref
-      QPtrList::insert. Inserts the resulting @ref KMFilterActionDesc
+      wrapper around QDict::insert and QPtrList::insert.
+      Inserts the resulting KMFilterActionDesc
       thrice: First with the name, then with the label as key into the
-      @ref @QDict, then into the @ref QPtrList. For that, it creates an
+      QDict, then into the QPtrList. For that, it creates an
       instance of the action internally and deletes it again after
       querying it for name and label. */
   void insert(KMFilterActionNewFunc aNewFunc);
@@ -647,7 +689,7 @@ public:
   const QPtrList<KMFilterActionDesc>& list() const { return mList; }
 
 protected:
-  /** Populate the dictionary with all known @ref KMFilterAction
+  /** Populate the dictionary with all known  KMFilterAction
      types. Called automatically from the constructor. */
   virtual void init(void);
 
