@@ -1,7 +1,22 @@
-// -*- mode: C++; c-file-style: "gnu" -*-
-// kmfolder.cpp
-// Author: Stefan Taferner <taferner@alpin.or.at>
-
+/* -*- mode: C++; c-file-style: "gnu" -*-
+ * kmail: KDE mail client
+ * Copyright (c) 1996-1998 Stefan Taferner <taferner@kde.org>
+ *
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
+ *
+ */
 #include <config.h>
 
 #include "kmfolder.h"
@@ -26,18 +41,25 @@
 #include <kshortcut.h>
 #include <kmessagebox.h>
 #include <qfile.h>
+#include <qfileinfo.h>
 
 
 KMFolder::KMFolder( KMFolderDir* aParent, const QString& aFolderName,
-                    KMFolderType aFolderType )
+                             KMFolderType aFolderType, bool withIndex, bool exportedSernums )
   : KMFolderNode( aParent, aFolderName ), mStorage(0),
     mChild( 0 ),
     mIsSystemFolder( false ),
+    mHasIndex( withIndex ),
+    mExportsSernums( exportedSernums ),
     mExpireMessages( false ), mUnreadExpireAge( 28 ),
     mReadExpireAge( 14 ), mUnreadExpireUnits( expireNever ),
     mReadExpireUnits( expireNever ),
     mExpireAction( ExpireDelete ),
-    mUseCustomIcons( false ), mMailingListEnabled( false )
+    mUseCustomIcons( false ), mMailingListEnabled( false ),
+    mAcctList( 0 ),
+    mIdentity( 0 ), // default identity
+    mPutRepliesInSameFolder( false ),
+    mIgnoreNewMail( false )
 {
   if( aFolderType == KMFolderTypeCachedImap )
     mStorage = new KMFolderCachedImap( this, aFolderName.latin1() );
@@ -50,6 +72,24 @@ KMFolder::KMFolder( KMFolderDir* aParent, const QString& aFolderName,
   else
     mStorage = new KMFolderMbox( this, aFolderName.latin1() );
 
+  assert( mStorage );
+
+  QFileInfo dirinfo;
+  dirinfo.setFile( mStorage->location() );
+  if ( !dirinfo.exists() ) {
+    int rc = mStorage->create();
+    QString msg = i18n("<qt>Error while creating file <b>%1</b>:<br>%2</qt>").arg(aFolderName).arg(strerror(rc));
+    if ( rc ) {
+      KMessageBox::information(0, msg);
+    }
+  }
+
+   // trigger from here, since it needs a fully constructed FolderStorage
+  if ( mExportsSernums )
+    mStorage->registerWithMessageDict();
+  if ( !mHasIndex )
+    mStorage->setAutoCreateIndex( false );
+
   if ( aParent ) {
     connect( mStorage, SIGNAL( msgAdded( KMFolder*, Q_UINT32 ) ),
              aParent->manager(), SIGNAL( msgAdded( KMFolder*, Q_UINT32 ) ) );
@@ -59,6 +99,8 @@ KMFolder::KMFolder( KMFolderDir* aParent, const QString& aFolderName,
              parent()->manager(), SIGNAL( msgChanged( KMFolder*, Q_UINT32, int ) ) );
     connect( this, SIGNAL( msgHeaderChanged( KMFolder*,  int ) ),
              parent()->manager(), SIGNAL( msgHeaderChanged( KMFolder*, int ) ) );
+    connect( mStorage, SIGNAL( invalidated( KMFolder* ) ),
+             parent()->manager(), SIGNAL( folderInvalidated( KMFolder* ) ) );
   }
 
   // Resend all mStorage signals
@@ -99,6 +141,8 @@ KMFolder::KMFolder( KMFolderDir* aParent, const QString& aFolderName,
 
 KMFolder::~KMFolder()
 {
+  delete mAcctList;
+  if ( mHasIndex ) mStorage->deregisterFromMessageDict();
   delete mStorage;
 }
 
@@ -219,8 +263,13 @@ KMFolderDir* KMFolder::createChildFolder()
     }
   }
 
-  mChild = new KMFolderDir( this, parent(), childName,
-    (folderType() == KMFolderTypeImap) ? KMImapDir : KMStandardDir);
+  KMFolderDirType newType = KMStandardDir;
+  if( folderType() == KMFolderTypeCachedImap )
+    newType = KMDImapDir;
+  else if( folderType() == KMFolderTypeImap )
+    newType = KMImapDir;
+
+  mChild = new KMFolderDir( this, parent(), childName, newType );
   if( !mChild )
     return 0;
   mChild->reload();
@@ -452,11 +501,6 @@ void KMFolder::markUnreadAsRead()
   mStorage->markUnreadAsRead();
 }
 
-int KMFolder::create( bool imap )
-{
-  return mStorage->create( imap );
-}
-
 void KMFolder::remove()
 {
   mStorage->remove();
@@ -472,16 +516,6 @@ int KMFolder::rename( const QString& newName, KMFolderDir *aParent )
   return mStorage->rename( newName, aParent );
 }
 
-bool KMFolder::autoCreateIndex() const
-{
-  return mStorage->autoCreateIndex();
-}
-
-void KMFolder::setAutoCreateIndex( bool b )
-{
-  mStorage->setAutoCreateIndex( b );
-}
-
 bool KMFolder::dirty() const
 {
   return mStorage->dirty();
@@ -519,7 +553,7 @@ QString KMFolder::label() const
   if ( !mLabel.isEmpty() )
      return mLabel;
   if ( isSystemFolder() )
-     return i18n( name().latin1() );
+     return i18n( name().utf8() );
   return name();
 }
 
@@ -551,16 +585,6 @@ QString KMFolder::mailingListPostAddress() const
   return QString::null;
 }
 
-const char* KMFolder::type() const
-{
-  return mStorage->type();
-}
-
-bool KMFolder::hasAccounts() const
-{
-  return mStorage->hasAccounts();
-}
-
 void KMFolder::setMailingListEnabled( bool enabled )
 {
   mMailingListEnabled = enabled;
@@ -590,7 +614,8 @@ void KMFolder::setWhoField(const QString& aWhoField )
 
 void KMFolder::setUserWhoField( const QString& whoField, bool writeConfig )
 {
-  mUserWhoField = whoField;
+  if ( mUserWhoField == whoField )
+    return;
   if ( whoField.isEmpty() )
   {
     // default setting
@@ -615,13 +640,17 @@ void KMFolder::setUserWhoField( const QString& whoField, bool writeConfig )
   } else if ( whoField == "From" || whoField == "To" )
     // set the whoField according to the user-setting
     mWhoField = whoField;
-  else
+  else {
     // this should not happen...
     kdDebug(5006) << "Illegal setting " << whoField << " for userWhoField!"
                   << endl;
+    return; // don't use the value
+  }
+  mUserWhoField = whoField;
 
   if (writeConfig)
     mStorage->writeConfig();
+  emit viewConfigChanged();
 }
 
 void KMFolder::correctUnreadMsgsCount()
@@ -753,36 +782,6 @@ int KMFolder::writeIndex( bool createEmptyIndex )
   return mStorage->writeIndex( createEmptyIndex );
 }
 
-void KMFolder::fillMsgDict( KMMsgDict* dict )
-{
-  mStorage->fillMsgDict( dict );
-}
-
-int KMFolder::writeMsgDict( KMMsgDict* dict)
-{
-  return mStorage->writeMsgDict( dict );
-}
-
-int KMFolder::touchMsgDict()
-{
-  return mStorage->touchMsgDict();
-}
-
-int KMFolder::appendtoMsgDict( int idx )
-{
-  return mStorage->appendtoMsgDict( idx );
-}
-
-void KMFolder::setRDict( KMMsgDictREntry* rentry )
-{
-  mStorage->setRDict( rentry );
-}
-
-KMMsgDictREntry* KMFolder::rDict() const
-{
-  return mStorage->rDict();
-}
-
 void KMFolder::setStatus( int idx, KMMsgStatus status, bool toggle )
 {
   mStorage->setStatus( idx, status, toggle );
@@ -808,11 +807,6 @@ void KMFolder::removeJobs()
   mStorage->removeJobs();
 }
 
-size_t KMFolder::crlf2lf( char* str, const size_t strLen )
-{
-  return FolderStorage::crlf2lf( str, strLen );
-}
-
 int KMFolder::updateIndex()
 {
   return mStorage->updateIndex();
@@ -847,5 +841,4 @@ void KMFolder::slotContentsTypeChanged( KMail::FolderContentsType type )
   emit iconsChanged();
 }
 
-#
 #include "kmfolder.moc"
