@@ -15,7 +15,7 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+ *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
  *  In addition, as a special exception, the copyright holders give
  *  permission to link the code of this program with any edition of
@@ -35,16 +35,15 @@
 
 #include <errno.h>
 
-#include "kmkernel.h"
 #include "kmfoldercachedimap.h"
 #include "undostack.h"
 #include "kmfoldermgr.h"
-#include "kmmessage.h"
 #include "kmacctcachedimap.h"
-#include "kmacctmgr.h"
+#include "accountmanager.h"
+using KMail::AccountManager;
 #include "kmailicalifaceimpl.h"
 #include "kmfolder.h"
-#include "kmdict.h"
+#include "kmglobal.h"
 #include "acljobs.h"
 #include "broadcaststatus.h"
 using KPIM::BroadcastStatus;
@@ -54,6 +53,10 @@ using KMail::CachedImapJob;
 #include "imapaccountbase.h"
 using KMail::ImapAccountBase;
 #include "listjob.h"
+using KMail::ListJob;
+
+#include "kmfolderseldlg.h"
+#include "kmcommands.h"
 
 #include <kapplication.h>
 #include <kmessagebox.h>
@@ -104,7 +107,7 @@ DImapTroubleShootDialog::DImapTroubleShootDialog( QWidget* parent,
                       "not cause any problems.</p><p>If that is not enough, "
                       "you can try refreshing the IMAP cache. If you do this, "
                       "you will loose all your local changes for this folder "
-                      "and all it's subfolders.</p>" );
+                      "and all its subfolders.</p>" );
   topLayout->addWidget( new QLabel( txt, page ) );
   enableButtonSeparator( true );
 
@@ -147,7 +150,7 @@ KMFolderCachedImap::KMFolderCachedImap( KMFolder* folder, const char* aName )
     mFolderRemoved( false ),
     /*mHoldSyncs( false ),*/ mRecurse( true ),
     mStatusChangedLocally( false ), mAnnotationFolderTypeChanged( false ),
-    mIncidencesForChanged( false )
+    mIncidencesForChanged( false ), mPersonalNamespacesCheckDone( true )
 {
   setUidValidity("");
   readUidCache();
@@ -190,8 +193,8 @@ void KMFolderCachedImap::readConfig()
 
   if ( mAnnotationFolderType != "FROMSERVER" ) {
     mAnnotationFolderType = config->readEntry( "Annotation-FolderType" );
-    // if there is an annotation, it has to XML
-    if ( !mAnnotationFolderType.isEmpty() )
+    // if there is an annotation, it has to be XML
+    if ( !mAnnotationFolderType.isEmpty() && !mAnnotationFolderType.startsWith( "mail" ) )
       kmkernel->iCalIface().setStorageFormat( folder(), KMailICalIfaceImpl::StorageXML );
 //    kdDebug(5006) << ( mImapPath.isEmpty() ? label() : mImapPath )
 //                  << " readConfig: mAnnotationFolderType=" << mAnnotationFolderType << endl;
@@ -207,6 +210,9 @@ void KMFolderCachedImap::readConfig()
 
   mAnnotationFolderTypeChanged = config->readBoolEntry( "AnnotationFolderTypeChanged", false );
   mIncidencesForChanged = config->readBoolEntry( "IncidencesForChanged", false );
+  if ( mImapPath.isEmpty() ) {
+    mImapPathCreation = config->readEntry("ImapPathCreation");
+  }
 }
 
 void KMFolderCachedImap::writeConfig()
@@ -216,6 +222,13 @@ void KMFolderCachedImap::writeConfig()
   configGroup.writeEntry( "NoContent", mNoContent );
   configGroup.writeEntry( "ReadOnly", mReadOnly );
   configGroup.writeEntry( "StatusChangedLocally", mStatusChangedLocally );
+  if ( !mImapPathCreation.isEmpty() ) {
+    if ( mImapPath.isEmpty() ) {
+      configGroup.writeEntry( "ImapPathCreation", mImapPathCreation );
+    } else {
+      configGroup.deleteEntry( "ImapPathCreation" );
+    }
+  }
   writeAnnotationConfig();
   KMFolderMaildir::writeConfig();
 }
@@ -232,6 +245,15 @@ void KMFolderCachedImap::writeAnnotationConfig()
   }
 }
 
+int KMFolderCachedImap::create()
+{
+  int rc = KMFolderMaildir::create();
+  // FIXME why the below? - till
+  readConfig();
+  mUnreadMsgs = -1;
+  return rc;
+}
+
 void KMFolderCachedImap::remove()
 {
   mFolderRemoved = true;
@@ -305,12 +327,14 @@ int KMFolderCachedImap::writeUidCache()
 
 void KMFolderCachedImap::reloadUidMap()
 {
+  kdDebug(5006) << "Reloading Uid Map " << endl;
   uidMap.clear();
   open();
   for( int i = 0; i < count(); ++i ) {
     KMMsgBase *msg = getMsgBase( i );
     if( !msg ) continue;
     ulong uid = msg->UID();
+    //kdDebug(5006) << "Inserting: " << i << " with uid: " << uid << endl;
     uidMap.insert( uid, i );
   }
   close();
@@ -445,9 +469,11 @@ KMMsgBase* KMFolderCachedImap::findByUID( ulong uid )
     KMMsgBase *msg = getMsgBase( *it );
     if( msg && msg->UID() == uid )
       return msg;
+  } else {
+    kdDebug(5006) << "Didn't find uid: " << uid << "in cache!" << endl;
   }
   // Not found by now
-  if( mapReloaded )
+ // if( mapReloaded )
     // Not here then
     return 0;
   // There could be a problem in the maps. Rebuild them and try again
@@ -455,7 +481,9 @@ KMMsgBase* KMFolderCachedImap::findByUID( ulong uid )
   it = uidMap.find( uid );
   if( it != uidMap.end() )
     // Since the uid map is just rebuilt, no need for the sanity check
-    return getMsg( *it );
+    return getMsgBase( *it );
+  else
+    kdDebug(5006) << "Reloaded, but stil didn't find uid: " << uid << endl;
   // Then it's not here
   return 0;
 }
@@ -503,7 +531,7 @@ void KMFolderCachedImap::slotTroubleshoot()
 void KMFolderCachedImap::serverSync( bool recurse )
 {
   if( mSyncState != SYNC_STATE_INITIAL ) {
-    if( KMessageBox::warningYesNo( 0, i18n("Folder %1 is not in initial sync state (state was %2). Do you want to reset it to initial sync state and sync anyway?" ).arg( imapPath() ).arg( mSyncState ) ) == KMessageBox::Yes ) {
+    if( KMessageBox::warningYesNo( 0, i18n("Folder %1 is not in initial sync state (state was %2). Do you want to reset it to initial sync state and sync anyway?" ).arg( imapPath() ).arg( mSyncState ), QString::null, i18n("Reset && Sync"), KStdGuiItem::cancel() ) == KMessageBox::Yes ) {
       mSyncState = SYNC_STATE_INITIAL;
     } else return;
   }
@@ -540,6 +568,7 @@ QString KMFolderCachedImap::state2String( int state ) const
   case SYNC_STATE_UPLOAD_FLAGS:      return "SYNC_STATE_UPLOAD_FLAGS";
   case SYNC_STATE_CREATE_SUBFOLDERS: return "SYNC_STATE_CREATE_SUBFOLDERS";
   case SYNC_STATE_LIST_SUBFOLDERS:   return "SYNC_STATE_LIST_SUBFOLDERS";
+  case SYNC_STATE_LIST_NAMESPACES:   return "SYNC_STATE_LIST_NAMESPACES";
   case SYNC_STATE_LIST_SUBFOLDERS2:  return "SYNC_STATE_LIST_SUBFOLDERS2";
   case SYNC_STATE_DELETE_SUBFOLDERS: return "SYNC_STATE_DELETE_SUBFOLDERS";
   case SYNC_STATE_LIST_MESSAGES:     return "SYNC_STATE_LIST_MESSAGES";
@@ -607,6 +636,7 @@ void KMFolderCachedImap::serverSyncInternal()
   case SYNC_STATE_INITIAL:
   {
     mProgress = 0;
+    foldersForDeletionOnServer.clear();
     newState( mProgress, i18n("Synchronizing"));
 
     open();
@@ -621,7 +651,7 @@ void KMFolderCachedImap::serverSyncInternal()
       // We stop here. We're already in SYNC_STATE_INITIAL for the next time.
       newState( mProgress, i18n( "Error connecting to server %1" ).arg( mAccount->host() ) );
       close();
-      emit folderComplete(this, FALSE);
+      emit folderComplete(this, false);
       break;
     } else if ( cs == ImapAccountBase::Connecting ) {
       mAccount->setAnnotationCheckPassed( false );
@@ -638,7 +668,7 @@ void KMFolderCachedImap::serverSyncInternal()
       // Fall through to next state
     }
   }
-  
+
 
   case SYNC_STATE_GET_USERRIGHTS:
     //kdDebug(5006) << "===== Syncing " << ( mImapPath.isEmpty() ? label() : mImapPath ) << endl;
@@ -653,7 +683,7 @@ void KMFolderCachedImap::serverSyncInternal()
       mAccount->getUserRights( folder(), imapPath() ); // after connecting, due to the INBOX case
       break;
     }
-   
+
   case SYNC_STATE_RENAME_FOLDER:
   {
     mSyncState = SYNC_STATE_CHECK_UIDVALIDITY;
@@ -691,7 +721,7 @@ void KMFolderCachedImap::serverSyncInternal()
     }
     // Else carry on
   case SYNC_STATE_UPLOAD_FLAGS:
-    mSyncState = SYNC_STATE_LIST_SUBFOLDERS;
+    mSyncState = SYNC_STATE_LIST_NAMESPACES;
     if( !noContent() ) {
        // We haven't downloaded messages yet, so we need to build the map.
        if( uidMapDirty )
@@ -708,9 +738,18 @@ void KMFolderCachedImap::serverSyncInternal()
        }
     }
     // Else carry on
+
+  case SYNC_STATE_LIST_NAMESPACES:
+    if ( this == mAccount->rootFolder() ) {
+      listNamespaces();
+      break;
+    }
+    mSyncState = SYNC_STATE_LIST_SUBFOLDERS;
+    // Else carry on
+
   case SYNC_STATE_LIST_SUBFOLDERS:
-    mSyncState = SYNC_STATE_LIST_SUBFOLDERS2;
     newState( mProgress, i18n("Retrieving folderlist"));
+    mSyncState = SYNC_STATE_LIST_SUBFOLDERS2;
     if( !listDirectory() ) {
       mSyncState = SYNC_STATE_INITIAL;
       KMessageBox::error(0, i18n("Error while retrieving the folderlist"));
@@ -731,7 +770,7 @@ void KMFolderCachedImap::serverSyncInternal()
       CachedImapJob* job = new CachedImapJob( foldersForDeletionOnServer,
                                                   CachedImapJob::tDeleteFolders, this );
       connect( job, SIGNAL( result(KMail::FolderJob *) ), this, SLOT( slotIncreaseProgress() ) );
-      connect( job, SIGNAL( finished() ), this, SLOT( serverSyncInternal() ) );
+      connect( job, SIGNAL( finished() ), this, SLOT( slotFolderDeletionOnServerFinished() ) );
       job->start();
       break;
     }
@@ -821,16 +860,16 @@ void KMFolderCachedImap::serverSyncInternal()
          && !imapPath().isEmpty() && imapPath() != "/" ) {
       kdDebug(5006) << "Setting test attribute on folder: "<< folder()->prettyURL() << endl;
       newState( mProgress, i18n("Checking annotation support"));
-    
+
       KURL url = mAccount->getUrl();
       url.setPath( imapPath() );
       KMail::AnnotationList annotations; // to be set
-      
+
       KMail::AnnotationAttribute attr( KOLAB_FOLDERTEST, "value.shared", "true" );
       annotations.append( attr );
-      
+
       kdDebug(5006) << "Setting test attribute to "<< url << endl;
-      KIO::Job* job = AnnotationJobs::multiSetAnnotation( mAccount->slave(), 
+      KIO::Job* job = AnnotationJobs::multiSetAnnotation( mAccount->slave(),
           url, annotations );
       ImapAccountBase::jobData jd( url.url(), folder() );
       jd.cancellable = true; // we can always do so later
@@ -839,7 +878,7 @@ void KMFolderCachedImap::serverSyncInternal()
               SLOT(slotTestAnnotationResult(KIO::Job *)));
       break;
     }
- 
+
   case SYNC_STATE_GET_ANNOTATIONS: {
 #define KOLAB_FOLDERTYPE "/vendor/kolab/folder-type"
 #define KOLAB_INCIDENCESFOR "/vendor/kolab/incidences-for"
@@ -859,7 +898,7 @@ void KMFolderCachedImap::serverSyncInternal()
 
     // First retrieve the annotation, so that we know we have to set it if it's not set.
     // On the other hand, if the user changed the contentstype, there's no need to get first.
-    if ( !noContent() && mAccount->hasAnnotationSupport() && 
+    if ( !noContent() && mAccount->hasAnnotationSupport() &&
         ( kmkernel->iCalIface().isEnabled() || needToGetInitialAnnotations ) ) {
       QStringList annotations; // list of annotations to be fetched
       if ( !mAnnotationFolderTypeChanged || mAnnotationFolderType.isEmpty() )
@@ -973,8 +1012,12 @@ void KMFolderCachedImap::serverSyncInternal()
             // Only sync folders that have been accepted by the server
             if ( !storage->imapPath().isEmpty()
                  // and that were not just deleted from it
-                 && !foldersForDeletionOnServer.contains( storage->imapPath() ) )
+                 && !foldersForDeletionOnServer.contains( storage->imapPath() ) ) {
               mSubfoldersForSync << storage;
+            } else {
+              kdDebug(5006) << "Do not add " << storage->label()
+                << " to synclist" << endl;
+            }
           }
           node = folder()->child()->next();
         }
@@ -1004,7 +1047,7 @@ void KMFolderCachedImap::serverSyncInternal()
         mSyncState = SYNC_STATE_INITIAL;
         mAccount->addUnreadMsgCount( this, countUnread() ); // before closing
         close();
-        emit folderComplete( this, TRUE );
+        emit folderComplete( this, true );
       } else {
         mCurrentSubfolder = mSubfoldersForSync.front();
         mSubfoldersForSync.pop_front();
@@ -1063,18 +1106,46 @@ void KMFolderCachedImap::uploadNewMessages()
 {
   QValueList<unsigned long> newMsgs = findNewMessages();
   if( !newMsgs.isEmpty() ) {
-
-    newState( mProgress, i18n("Uploading messages to server"));
-    CachedImapJob *job = new CachedImapJob( newMsgs, CachedImapJob::tPutMessage, this );
-    connect( job, SIGNAL( progress( unsigned long, unsigned long) ),
-             this, SLOT( slotPutProgress(unsigned long, unsigned long) ) );
-    connect( job, SIGNAL( finished() ), this, SLOT( serverSyncInternal() ) );
-    job->start();
-  } else {
-    newState( mProgress, i18n("No messages to upload to server"));
-
-    serverSyncInternal();
+    if ( mUserRights <= 0 || ( mUserRights & ( KMail::ACLJobs::Insert ) ) ) {
+      newState( mProgress, i18n("Uploading messages to server"));
+      CachedImapJob *job = new CachedImapJob( newMsgs, CachedImapJob::tPutMessage, this );
+      connect( job, SIGNAL( progress( unsigned long, unsigned long) ),
+               this, SLOT( slotPutProgress(unsigned long, unsigned long) ) );
+      connect( job, SIGNAL( finished() ), this, SLOT( serverSyncInternal() ) );
+      job->start();
+      return;
+    } else {
+      const QString msg ( i18n( "<p>There are new messages in this folder, which "
+            "have not been uploaded to the server yet, but you do not seem to "
+            "have sufficient access rights on the folder now to upload them. "
+            "Please contact your administrator to allow upload of new messages "
+            "to you, or move them out of this folder.</p> "
+            "<p>Do you want to move these messages to another folder now?</p>") );
+      if ( KMessageBox::warningYesNo( 0, msg, QString::null, i18n("Move"), i18n("Do Not Move") ) == KMessageBox::Yes ) {
+        KMail::KMFolderSelDlg dlg( kmkernel->getKMMainWidget(),
+            i18n("Move Messages to Folder"), true );
+        if ( dlg.exec() ) {
+          KMFolder* dest = dlg.folder();
+          if ( dest ) {
+            QPtrList<KMMsgBase> msgs;
+            for( int i = 0; i < count(); ++i ) {
+              KMMsgBase *msg = getMsgBase( i );
+              if( !msg ) continue; /* what goes on if getMsg() returns 0? */
+              if ( msg->UID() == 0 )
+                msgs.append( msg );
+            }
+            KMCommand *command = new KMMoveCommand( dest, msgs );
+            connect( command, SIGNAL( completed( KMCommand * ) ),
+                     this, SLOT( serverSyncInternal() ) );
+            command->start();
+            return;
+          }
+        }
+      }
+    }
   }
+  newState( mProgress, i18n("No messages to upload to server"));
+  serverSyncInternal();
 }
 
 /* Progress info during uploadNewMessages */
@@ -1189,7 +1260,9 @@ QValueList<KMFolderCachedImap*> KMFolderCachedImap::findNewFolders()
           assert(0);
         }
         KMFolderCachedImap* folder = static_cast<KMFolderCachedImap*>(static_cast<KMFolder*>(node)->storage());
-        if( folder->imapPath().isEmpty() ) newFolders << folder;
+        if( folder->imapPath().isEmpty() ) {
+          newFolders << folder;
+        }
       }
       node = folder()->child()->next();
     }
@@ -1199,6 +1272,8 @@ QValueList<KMFolderCachedImap*> KMFolderCachedImap::findNewFolders()
 
 bool KMFolderCachedImap::deleteMessages()
 {
+  if ( mUserRights > 0 && !( mUserRights & KMail::ACLJobs::Delete ) )
+    return false;
   /* Delete messages from cache that are gone from the server */
   QPtrList<KMMessage> msgsForDeletion;
 
@@ -1354,30 +1429,38 @@ void KMFolderCachedImap::slotGetMessagesData(KIO::Job * job, const QByteArray &
         }
         uidsOnServer.insert( uid, &v );
       }
+      bool redownload = false;
       if (  uid <= lastUid() ) {
        /*
         * If this message UID is not present locally, then it must
         * have been deleted by the user, so we delete it on the
-        * server also.
+        * server also. If we don't have delete permissions on the server,
+        * re-download the message, it must have vanished by some error, or
+        * while we still thought we were allowed to delete (ACL change).
         *
         * This relies heavily on lastUid() being correct at all times.
         */
         // kdDebug(5006) << "KMFolderCachedImap::slotGetMessagesData() : folder "<<label()<<" already has msg="<<msg->headerField("Subject") << ", UID="<<uid << ", lastUid = " << mLastUid << endl;
         KMMsgBase *existingMessage = findByUID(uid);
+        if( !existingMessage ) {
+          if ( mUserRights <= 0 || ( mUserRights & KMail::ACLJobs::Delete ) ) {
+            // kdDebug(5006) << "message with uid " << uid << " is gone from local cache. Must be deleted on server!!!" << endl;
+            uidsForDeletionOnServer << uid;
+          } else {
+            redownload = true;
+          }
+        } else {
           // if this is a read only folder, ignore status updates from the server
           // since we can't write our status back our local version is what has to
           // be considered correct.
-        if( !existingMessage ) {
-          // kdDebug(5006) << "message with uid " << uid << " is gone from local cache. Must be deleted on server!!!" << endl;
-          uidsForDeletionOnServer << uid;
-        } else {
           if (!mReadOnly) {
             /* The message is OK, update flags */
             KMFolderImap::flagsToStatus( existingMessage, flags );
           }
         }
         // kdDebug(5006) << "message with uid " << uid << " found in the local cache. " << endl;
-      } else {
+      }
+      if ( uid > lastUid() || redownload ) {
         // The message is new since the last sync, but we might have just uploaded it, in which case
         // the uid map already contains it.
         if ( !uidMap.contains( uid ) ) {
@@ -1440,27 +1523,136 @@ void KMFolderCachedImap::setAccount(KMAcctCachedImap *aAccount)
       static_cast<KMFolderCachedImap*>(static_cast<KMFolder*>(node)->storage())->setAccount(aAccount);
 }
 
+void KMFolderCachedImap::listNamespaces()
+{
+  ImapAccountBase::ListType type = ImapAccountBase::List;
+  if ( mAccount->onlySubscribedFolders() )
+    type = ImapAccountBase::ListSubscribed;
+
+  kdDebug(5006) << "listNamespaces " << mNamespacesToList << endl;
+  if ( mNamespacesToList.isEmpty() ) {
+    mSyncState = SYNC_STATE_DELETE_SUBFOLDERS;
+    mPersonalNamespacesCheckDone = true;
+
+    QStringList ns = mAccount->namespaces()[ImapAccountBase::OtherUsersNS];
+    ns += mAccount->namespaces()[ImapAccountBase::SharedNS];
+    mNamespacesToCheck = ns.count();
+    for ( QStringList::Iterator it = ns.begin(); it != ns.end(); ++it )
+    {
+      if ( (*it).isEmpty() ) {
+        // ignore empty listings as they have been listed before
+        --mNamespacesToCheck;
+        continue;
+      }
+      KMail::ListJob* job = new KMail::ListJob( mAccount, type, this, mAccount->addPathToNamespace( *it ) );
+      connect( job, SIGNAL(receivedFolders(const QStringList&, const QStringList&,
+              const QStringList&, const QStringList&, const ImapAccountBase::jobData&)),
+          this, SLOT(slotCheckNamespace(const QStringList&, const QStringList&,
+              const QStringList&, const QStringList&, const ImapAccountBase::jobData&)));
+      job->start();
+    }
+    if ( mNamespacesToCheck == 0 ) {
+      serverSyncInternal();
+    }
+    return;
+  }
+  mPersonalNamespacesCheckDone = false;
+
+  QString ns = mNamespacesToList.front();
+  mNamespacesToList.pop_front();
+
+  mSyncState = SYNC_STATE_LIST_SUBFOLDERS2;
+  newState( mProgress, i18n("Retrieving folders for namespace %1").arg(ns));
+  KMail::ListJob* job = new KMail::ListJob( mAccount, type, this,
+      mAccount->addPathToNamespace( ns ) );
+  job->setNamespace( ns );
+  connect( job, SIGNAL(receivedFolders(const QStringList&, const QStringList&,
+          const QStringList&, const QStringList&, const ImapAccountBase::jobData&)),
+      this, SLOT(slotListResult(const QStringList&, const QStringList&,
+          const QStringList&, const QStringList&, const ImapAccountBase::jobData&)));
+  job->start();
+}
+
+void KMFolderCachedImap::slotCheckNamespace( const QStringList& subfolderNames,
+                                             const QStringList& subfolderPaths,
+                                             const QStringList& subfolderMimeTypes,
+                                             const QStringList& subfolderAttributes,
+                                             const ImapAccountBase::jobData& jobData )
+{
+  Q_UNUSED( subfolderPaths );
+  Q_UNUSED( subfolderMimeTypes );
+  Q_UNUSED( subfolderAttributes );
+  --mNamespacesToCheck;
+  kdDebug(5006) << "slotCheckNamespace " << subfolderNames << ",remain=" <<
+   mNamespacesToCheck << endl;
+
+  // get a correct foldername:
+  // strip / and make sure it does not contain the delimiter
+  QString name = jobData.path.mid( 1, jobData.path.length()-2 );
+  name.remove( mAccount->delimiterForNamespace( name ) );
+  if ( name.isEmpty() ) {
+    // should not happen
+    kdWarning(5006) << "slotCheckNamespace: ignoring empty folder!" << endl;
+    return;
+  }
+
+  folder()->createChildFolder();
+  KMFolderNode *node = 0;
+  for ( node = folder()->child()->first(); node;
+        node = folder()->child()->next())
+  {
+    if ( !node->isDir() && node->name() == name )
+      break;
+  }
+  if ( !subfolderNames.isEmpty() ) {
+    if ( node ) {
+      // folder exists so we have nothing to do - it will be listed later
+      kdDebug(5006) << "found namespace folder " << name << endl;
+    } else
+    {
+      // create folder
+      kdDebug(5006) << "create namespace folder " << name << endl;
+      KMFolder* newFolder = folder()->child()->createFolder( name, false,
+          KMFolderTypeCachedImap );
+      if ( newFolder ) {
+        KMFolderCachedImap *f = static_cast<KMFolderCachedImap*>( newFolder->storage() );
+        f->setImapPath( mAccount->addPathToNamespace( name ) );
+        f->setNoContent( true );
+        f->setAccount( mAccount );
+        f->close();
+        kmkernel->dimapFolderMgr()->contentsChanged();
+      }
+    }
+  } else {
+    if ( node ) {
+      kdDebug(5006) << "delete namespace folder " << name << endl;
+      KMFolder* fld = static_cast<KMFolder*>(node);
+      kmkernel->dimapFolderMgr()->remove( fld );
+    }
+  }
+
+  if ( mNamespacesToCheck == 0 ) {
+    // all namespaces are done so continue with the next step
+    serverSyncInternal();
+  }
+}
 
 // This lists the subfolders on the server
 // and (in slotListResult) takes care of folders that have been removed on the server
-bool KMFolderCachedImap::listDirectory(bool secondStep)
+bool KMFolderCachedImap::listDirectory()
 {
-  mSubfolderState = imapInProgress;
   if( !mAccount->slave() ) { // sync aborted
     resetSyncState();
     emit folderComplete( this, false );
     return false;
   }
-  // reset
-  if ( this == mAccount->rootFolder() )
-    mAccount->setHasInbox( false );
+  mSubfolderState = imapInProgress;
 
   // get the folders
   ImapAccountBase::ListType type = ImapAccountBase::List;
   if ( mAccount->onlySubscribedFolders() )
     type = ImapAccountBase::ListSubscribed;
-  KMail::ListJob* job = new KMail::ListJob( this, mAccount, type, secondStep,
-      false, mAccount->hasInbox() );
+  KMail::ListJob* job = new KMail::ListJob( mAccount, type, this );
   connect( job, SIGNAL(receivedFolders(const QStringList&, const QStringList&,
           const QStringList&, const QStringList&, const ImapAccountBase::jobData&)),
       this, SLOT(slotListResult(const QStringList&, const QStringList&,
@@ -1476,88 +1668,87 @@ void KMFolderCachedImap::slotListResult( const QStringList& folderNames,
                                          const QStringList& folderAttributes,
                                          const ImapAccountBase::jobData& jobData )
 {
-  //kdDebug(5006) << label() << ": folderNames=" << folderNames << " folderPaths=" << folderPaths << " mimeTypes=" << folderMimeTypes << endl;
+  Q_UNUSED( jobData );
+  //kdDebug(5006) << label() << ": folderNames=" << folderNames << " folderPaths="
+  //<< folderPaths << " mimeTypes=" << folderMimeTypes << endl;
   mSubfolderNames = folderNames;
   mSubfolderPaths = folderPaths;
   mSubfolderMimeTypes = folderMimeTypes;
   mSubfolderAttributes = folderAttributes;
 
   mSubfolderState = imapFinished;
-  bool it_inboxOnly = jobData.inboxOnly;
-  // pass it to listDirectory2
-  mCreateInbox = jobData.createInbox;
 
-  if (it_inboxOnly) {
-    // list again only for the INBOX
-    listDirectory(TRUE);
-    return;
-  }
-
-  if ( folder()->isSystemFolder() && mImapPath == "/INBOX/"
-       && mAccount->prefix() == "/INBOX/" )
-  {
-    // do not create folders under INBOX
-    mCreateInbox = false;
-    mSubfolderNames.clear();
-  }
   folder()->createChildFolder();
-  // Find all subfolders present on disk but not on the server
   KMFolderNode *node = folder()->child()->first();
+  bool root = ( this == mAccount->rootFolder() );
+
   QPtrList<KMFolder> toRemove;
-  while (node) {
-    if (!node->isDir() ) {
-      KMFolderCachedImap *f = static_cast<KMFolderCachedImap*>(static_cast<KMFolder*>(node)->storage());
-      if ( mSubfolderNames.findIndex(node->name()) == -1 &&
-          (node->name().upper() != "INBOX" || !mCreateInbox) )
-      {
-        // This subfolder isn't present on the server
-        if( !f->imapPath().isEmpty()  ) {
-          // The folder has an imap path set, so it has been
-          // on the server before. Delete it locally.
-          toRemove.append( f->folder() );
-          kdDebug(5006) << node->name() << " isn't on the server. It has an imapPath -> delete it locally" << endl;
-        } else { // shouldn't happen
-          kdDebug(5006) << node->name() << " isn't on the server, but has no imapPath. ERROR - why didn't createNewFolders create it?" << endl;
+  bool emptyList = ( root && mSubfolderNames.empty() );
+  if ( !emptyList ) {
+    while (node) {
+      if (!node->isDir() ) {
+        KMFolderCachedImap *f = static_cast<KMFolderCachedImap*>(static_cast<KMFolder*>(node)->storage());
+
+        if ( mSubfolderNames.findIndex(node->name()) == -1 ) {
+          QString name = node->name();
+          // as more than one namespace can be listed in the root folder we need to make sure
+          // that the folder is within the current namespace
+          bool isInNamespace = ( jobData.curNamespace.isEmpty() ||
+              jobData.curNamespace == mAccount->namespaceForFolder( f ) );
+          // ignore some cases
+          bool ignore = root && ( f->imapPath() == "/INBOX/" ||
+              mAccount->isNamespaceFolder( name ) || !isInNamespace );
+
+          // This subfolder isn't present on the server
+          if( !f->imapPath().isEmpty() && !ignore  ) {
+            // The folder has an imap path set, so it has been
+            // on the server before. Delete it locally.
+            toRemove.append( f->folder() );
+            kdDebug(5006) << node->name() << " isn't on the server. It has an imapPath -> delete it locally" << endl;
+          }
+        } else { // folder both local and on server
+          //kdDebug(5006) << node->name() << " is on the server." << endl;
         }
-      } else { // folder both local and on server
-        //kdDebug(5006) << node->name() << " is on the server." << endl;
+      } else {
+        //kdDebug(5006) << "skipping dir node:" << node->name() << endl;
       }
-    } else {
-      //kdDebug(5006) << "skipping dir node:" << node->name() << endl;
+      node = folder()->child()->next();
     }
-    node = folder()->child()->next();
   }
 
-  for ( KMFolder* doomed=toRemove.first(); doomed; doomed = toRemove.next() )
+  for ( KMFolder* doomed=toRemove.first(); doomed; doomed = toRemove.next() ) {
     kmkernel->dimapFolderMgr()->remove( doomed );
+  }
 
   mProgress += 5;
   serverSyncInternal();
 }
 
 // This synchronizes the local folders as needed (creation/deletion). No network communication here.
-void KMFolderCachedImap::listDirectory2() {
-  foldersForDeletionOnServer.clear();
+void KMFolderCachedImap::listDirectory2()
+{
   QString path = folder()->path();
   KMFolderCachedImap *f = 0;
   kmkernel->dimapFolderMgr()->quiet(true);
 
-  if (mCreateInbox)
-  {
-    KMFolderNode *node;
+  KMFolderNode *node;
+  bool root = ( this == mAccount->rootFolder() );
+  if ( root && !mAccount->hasInbox() ) {
+    kdDebug(5006) << "check INBOX" << endl;
     // create the INBOX
     for (node = folder()->child()->first(); node; node = folder()->child()->next())
       if (!node->isDir() && node->name() == "INBOX") break;
-    if (node)
+    if (node) {
       f = static_cast<KMFolderCachedImap*>(static_cast<KMFolder*>(node)->storage());
-    else {
+    } else {
       KMFolder* newFolder = folder()->child()->createFolder("INBOX", true, KMFolderTypeCachedImap);
-      if (newFolder)
+      if ( newFolder ) {
         f = static_cast<KMFolderCachedImap*>(newFolder->storage());
+      }
     }
-    f->setAccount(mAccount);
-    f->setImapPath("/INBOX/");
-    f->folder()->setLabel(i18n("inbox"));
+    f->setAccount( mAccount );
+    f->setImapPath( "/INBOX/" );
+    f->folder()->setLabel( i18n("inbox") );
     if (!node) {
       f->close();
       kmkernel->dimapFolderMgr()->contentsChanged();
@@ -1566,16 +1757,20 @@ void KMFolderCachedImap::listDirectory2() {
     mAccount->setHasInbox( true );
   }
 
+  if ( root && !mSubfolderNames.isEmpty() ) {
+    KMFolderCachedImap* parent =
+      findParent( mSubfolderPaths.first(), mSubfolderNames.first() );
+    if ( parent ) {
+      kdDebug(5006) << "KMFolderCachedImap::listDirectory2 - pass listing to "
+        << parent->label() << endl;
+      mSubfolderNames.clear();
+    }
+  }
+
   // Find all subfolders present on server but not on disk
   for (uint i = 0; i < mSubfolderNames.count(); i++) {
 
-    if (mSubfolderNames[i].upper() == "INBOX" &&
-        mSubfolderPaths[i] == "/INBOX/" &&
-        mAccount->hasInbox()) // do not create an additional inbox
-      continue;
-
     // Find the subdir, if already present
-    KMFolderNode *node;
     for (node = folder()->child()->first(); node;
          node = folder()->child()->next())
       if (!node->isDir() && node->name() == mSubfolderNames[i]) break;
@@ -1593,7 +1788,7 @@ void KMFolderCachedImap::listDirectory2() {
       // (could be that the folder was deleted & recreated meanwhile from another client...)
       if ( !locallyDeleted && mAccount->isPreviouslyDeletedFolder( subfolderPath ) ) {
            locallyDeleted = KMessageBox::warningYesNo(
-             0, i18n( "<qt><p>It seems that the folder <b>%1</b> was deleted. Do you want to delete it from the server?</p></qt>" ).arg( mSubfolderNames[i] ) ) == KMessageBox::Yes;
+             0, i18n( "<qt><p>It seems that the folder <b>%1</b> was deleted. Do you want to delete it from the server?</p></qt>" ).arg( mSubfolderNames[i] ), QString::null, KStdGuiItem::del(), KStdGuiItem::cancel() ) == KMessageBox::Yes;
       }
 
       if ( locallyDeleted ) {
@@ -1602,8 +1797,9 @@ void KMFolderCachedImap::listDirectory2() {
       } else {
         kdDebug(5006) << subfolderPath << " is a new folder on the server => create local cache" << endl;
         KMFolder* newFolder = folder()->child()->createFolder(mSubfolderNames[i], false, KMFolderTypeCachedImap);
-        if (newFolder)
+        if ( newFolder ) {
           f = static_cast<KMFolderCachedImap*>(newFolder->storage());
+        }
         if (f) {
           f->close();
           f->setAccount(mAccount);
@@ -1631,9 +1827,42 @@ void KMFolderCachedImap::listDirectory2() {
   }
   kmkernel->dimapFolderMgr()->quiet(false);
   emit listComplete(this);
+  if ( !mPersonalNamespacesCheckDone ) {
+    // we're not done with the namespaces
+    mSyncState = SYNC_STATE_LIST_NAMESPACES;
+  }
   serverSyncInternal();
 }
 
+//-----------------------------------------------------------------------------
+KMFolderCachedImap* KMFolderCachedImap::findParent( const QString& path,
+                                                    const QString& name )
+{
+  QString parent = path.left( path.length() - name.length() - 2 );
+  if ( parent.length() > 1 )
+  {
+    // extract name of the parent
+    parent = parent.right( parent.length() - 1 );
+    if ( parent != label() )
+    {
+      KMFolderNode *node = folder()->child()->first();
+      // look for a better parent
+      while ( node )
+      {
+        if ( node->name() == parent )
+        {
+          KMFolder* fld = static_cast<KMFolder*>(node);
+          KMFolderCachedImap* imapFld =
+            static_cast<KMFolderCachedImap*>( fld->storage() );
+          return imapFld;
+        }
+        node = folder()->child()->next();
+      }
+    }
+  }
+  return 0;
+}
+
 void KMFolderCachedImap::slotSubFolderComplete(KMFolderCachedImap* sub, bool success)
 {
   Q_UNUSED(sub);
@@ -1900,10 +2129,11 @@ void KMFolderCachedImap::slotAnnotationResult(const QString& entry, const QStrin
         if ( type == KMailICalIfaceImpl::annotationForContentsType( contentsType ) ) {
           // Case 3: known content-type on server, get it
           //kdDebug(5006) << mImapPath << ": slotGetAnnotationResult: found known type of annotation" << endl;
-          kmkernel->iCalIface().setStorageFormat( folder(), KMailICalIfaceImpl::StorageXML );
+          if ( contentsType != ContentsTypeMail )
+            kmkernel->iCalIface().setStorageFormat( folder(), KMailICalIfaceImpl::StorageXML );
           mAnnotationFolderType = value;
-          if ( folder()->parent()->owner()->idString() != GlobalSettings::theIMAPResourceFolderParent()
-               && GlobalSettings::theIMAPResourceEnabled()
+          if ( folder()->parent()->owner()->idString() != GlobalSettings::self()->theIMAPResourceFolderParent()
+               && GlobalSettings::self()->theIMAPResourceEnabled()
                && subtype == "default" ) {
             // Truncate subtype if this folder can't be a default resource folder for us,
             // although it apparently is for someone else.
@@ -1958,8 +2188,8 @@ void KMFolderCachedImap::slotGetAnnotationResult( KIO::Job* job )
   if ( annjob->error() ) {
     if ( job->error() == KIO::ERR_UNSUPPORTED_ACTION ) {
       // that's when the imap server doesn't support annotations
-      if ( GlobalSettings::theIMAPResourceStorageFormat() == GlobalSettings::EnumTheIMAPResourceStorageFormat::XML
-           && (uint)GlobalSettings::theIMAPResourceAccount() == mAccount->id() )
+      if ( GlobalSettings::self()->theIMAPResourceStorageFormat() == GlobalSettings::EnumTheIMAPResourceStorageFormat::XML
+           && (uint)GlobalSettings::self()->theIMAPResourceAccount() == mAccount->id() )
 	KMessageBox::error( 0, i18n( "The IMAP server %1 does not have support for IMAP annotations. The XML storage cannot be used on this server; please re-configure KMail differently." ).arg( mAccount->host() ) );
       mAccount->setHasNoAnnotationSupport();
     }
@@ -2035,8 +2265,19 @@ void KMFolderCachedImap::slotUpdateLastUid()
 
 bool KMFolderCachedImap::isMoveable() const
 {
-  return ( hasChildren() == HasNoChildren && 
+  return ( hasChildren() == HasNoChildren &&
       !folder()->isSystemFolder() ) ? true : false;
 }
 
+void KMFolderCachedImap::slotFolderDeletionOnServerFinished()
+{
+  for ( QStringList::const_iterator it = foldersForDeletionOnServer.constBegin();
+      it != foldersForDeletionOnServer.constEnd(); ++it ) {
+    KURL url( mAccount->getUrl() );
+    url.setPath( *it );
+    kmkernel->iCalIface().folderDeletedOnServer( url );
+  }
+  serverSyncInternal();
+}
+
 #include "kmfoldercachedimap.moc"
