@@ -1,6 +1,20 @@
-// -*- mode: C++; c-file-style: "gnu" -*-
-// Author:: Don Sanders <sanders@kde.org>
-// License GPL
+/* -*- mode: C++; c-file-style: "gnu" -*-
+    This file is part of KMail, the KDE mail client.
+    Copyright (c) 2000 Don Sanders <sanders@kde.org>
+
+    KMail is free software; you can redistribute it and/or modify it
+    under the terms of the GNU General Public License, version 2, as
+    published by the Free Software Foundation.
+
+    KMail is distributed in the hope that it will be useful, but
+    WITHOUT ANY WARRANTY; without even the implied warranty of
+    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+    General Public License for more details.
+
+    You should have received a copy of the GNU General Public License
+    along with this program; if not, write to the Free Software
+    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
+*/
 
 #include "kmfolderindex.h"
 #include "kmfolder.h"
@@ -80,8 +94,10 @@ QString KMFolderIndex::indexLocation() const
 {
   QString sLocation(folder()->path());
 
-  if (!sLocation.isEmpty()) sLocation += '/';
-  sLocation += '.';
+  if ( !sLocation.isEmpty() ) {
+    sLocation += '/';
+    sLocation += '.';
+  }
   sLocation += dotEscape(fileName());
   sLocation += ".index";
 
@@ -98,7 +114,7 @@ int KMFolderIndex::updateIndex()
     if (mMsgList.at(i))
       dirty = !mMsgList.at(i)->syncIndexString();
   if (!dirty) { // Update successful
-      touchMsgDict();
+      touchFolderIdsFile();
       return 0;
   }
   return writeIndex();
@@ -187,7 +203,7 @@ int KMFolderIndex::writeIndex( bool createEmptyIndex )
 
   updateIndexStreamPtr();
 
-  writeMsgDict();
+  writeFolderIdsFile();
 
   setDirty( false );
   return 0;
@@ -318,7 +334,7 @@ bool KMFolderIndex::readIndexHeader(int *gv)
 					    "The mail index for '%1' is from an unknown version of KMail (%2).\n"
 					    "This index can be regenerated from your mail folder, but some "
 					    "information, including status flags, may be lost. Do you wish "
-					    "to downgrade your index file?") .arg(name()) .arg(indexVersion) );
+					    "to downgrade your index file?") .arg(name()) .arg(indexVersion), QString::null, i18n("Downgrade"), i18n("Do Not Downgrade") );
       kapp->restoreOverrideCursor();
       if (r == KMessageBox::Yes)
 	  createIndexFromContents();
@@ -380,7 +396,7 @@ bool KMFolderIndex::updateIndexStreamPtr(bool)
     // running, while the clock switches from daylight savings time to normal time
     utime(QFile::encodeName(location()), 0);
     utime(QFile::encodeName(indexLocation()), 0);
-    utime(QFile::encodeName(KMMsgDict::getFolderIdsLocation( folder() )), 0);
+    utime(QFile::encodeName( KMMsgDict::getFolderIdsLocation( *this ) ), 0);
 
   mIndexSwapByteOrder = false;
 #ifdef HAVE_MMAP
@@ -445,11 +461,12 @@ void KMFolderIndex::truncateIndex()
     writeIndex( true );
 }
 
-
-void KMFolderIndex::fillDictFromIndex(KMMsgDict *dict)
+void KMFolderIndex::fillMessageDict()
 {
   open();
-  mMsgList.fillMsgDict(dict);
+  for (unsigned int idx = 0; idx < mMsgList.high(); idx++)
+    if ( mMsgList.at( idx ) )
+      KMMsgDict::mutableInstance()->insert(0, mMsgList.at( idx ), idx);
   close();
 }
 
@@ -461,4 +478,5 @@ KMMsgInfo* KMFolderIndex::setIndexEntry( int idx, KMMessage *msg )
   mMsgList.set( idx, msgInfo );
   return msgInfo;
 }
+
 #include "kmfolderindex.moc"
