@@ -1,22 +1,46 @@
 /* -*- mode: C++; c-file-style: "gnu" -*-
- * Virtual base class for mail folder with .*.index style index
- *
- * Author: Don Sanders <sanders@kde.org>
- * License: GPL
- */
+    This file is part of KMail, the KDE mail client.
+    Copyright (c) 2000 Don Sanders <sanders@kde.org>
+
+    KMail is free software; you can redistribute it and/or modify it
+    under the terms of the GNU General Public License, version 2, as
+    published by the Free Software Foundation.
+
+    KMail is distributed in the hope that it will be useful, but
+    WITHOUT ANY WARRANTY; without even the implied warranty of
+    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+    General Public License for more details.
+
+    You should have received a copy of the GNU General Public License
+    along with this program; if not, write to the Free Software
+    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
+*/
+// Virtual base class for mail folder with .*.index style index
+
 #ifndef kmfolderindex_h
 #define kmfolderindex_h
 
 #include "folderstorage.h"
 #include "kmmsglist.h"
 
+/**
+ * @short A FolderStorage with an index for faster access to often used
+ * message properties..
+ *
+ * This class represents a message store which has an index for providing fast
+ * access to often used message properties, namely those displayed in the list
+ * of messages (KMHeaders).
+ * 
+ * @author Don Sanders <sanders@kde.org>
+ */
+
 class KMFolderIndex: public FolderStorage
 {
   Q_OBJECT
   //TODO:Have to get rid of this friend declaration and add necessary pure
   //virtuals to kmfolder.h so that KMMsgBase::parent() can be a plain KMFolder
   //rather than a KMFolderIndex. Need this for database indices.
-  friend class KMMsgBase;
+  friend class ::KMMsgBase;
 public:
 
   /** This enum indicates the status of the index file. It's returned by
@@ -37,7 +61,6 @@ public:
   virtual KMMsgBase* takeIndexEntry( int idx ) { return mMsgList.take( idx ); }
   virtual KMMsgInfo* setIndexEntry( int idx, KMMessage *msg );
   virtual void clearIndex(bool autoDelete=true, bool syncDict = false);
-  virtual void fillDictFromIndex(KMMsgDict *dict);
   virtual void truncateIndex();
 
   virtual const KMMsgBase* getMsgBase(int idx) const { return mMsgList[idx]; }
@@ -82,6 +105,12 @@ protected:
   */
   virtual IndexStatus indexStatus() = 0;
 
+    /** Inserts messages into the message dictionary by iterating over the
+     * message list. The messages will get new serial numbers. This is only
+     * used on newly appeared folders, where there is no .ids file yet, or
+     * when that has been invalidated. */
+  virtual void fillMessageDict();
+
   /** table of contents file */
   FILE* mIndexStream;
   /** list of index entries or messages */
