@@ -6,14 +6,14 @@
 #include "kmfoldertree.h"
 
 #include "kmfoldermgr.h"
+#include "kmfolder.h"
 #include "kmfolderimap.h"
 #include "kmfoldercachedimap.h"
 #include "kmfolderdia.h"
-#include "kmcomposewin.h"
 #include "kmmainwidget.h"
 #include "kmailicalifaceimpl.h"
-#include "kmacctmgr.h"
-#include "kmkernel.h"
+#include "accountmanager.h"
+using KMail::AccountManager;
 #include "globalsettings.h"
 #include "kmcommands.h"
 #include "foldershortcutdialog.h"
@@ -29,11 +29,13 @@ using namespace KPIM;
 #include <kiconloader.h>
 #include <kmessagebox.h>
 #include <kconfig.h>
+#include <kpopupmenu.h>
 #include <kdebug.h>
 
 #include <qpainter.h>
 #include <qcursor.h>
 #include <qregexp.h>
+#include <qpopupmenu.h>
 
 #include <unistd.h>
 #include <assert.h>
@@ -116,13 +118,16 @@ QPixmap KMFolderTreeItem::normalIcon(int size) const
       case Outbox: icon = "folder_outbox"; break;
       case SentMail: icon = "folder_sent_mail"; break;
       case Trash: icon = "trashcan_empty"; break;
-      case Drafts: icon = "edit";break;
+      case Drafts: icon = "edit"; break;
       default: icon = kmkernel->iCalIface().folderPixmap( type() ); break;
     }
     // non-root search folders
-    if ( protocol() == KMFolderTreeItem::Search) {
+    if ( protocol() == KMFolderTreeItem::Search ) {
       icon = "mail_find";
     }
+    if ( mFolder && mFolder->noContent() ) {
+      icon = "folder_grey";
+    }
   }
 
   if ( icon.isEmpty() )
@@ -159,12 +164,19 @@ QPixmap KMFolderTreeItem::unreadIcon(int size) const
       pm = il->loadIcon( mFolder->normalIconPath(), KIcon::Small, size,
                          KIcon::DefaultState, 0, true );
   }
-  if ( pm.isNull() )
-    pm = il->loadIcon( kmkernel->iCalIface().folderPixmap( type() ),
-                       KIcon::Small, size, KIcon::DefaultState, 0, true );
-  if ( pm.isNull() )
-    pm = il->loadIcon( "folder_open", KIcon::Small, size,
+  if ( pm.isNull() ) {
+    if ( mFolder && mFolder->noContent() ) {
+      pm = il->loadIcon( "folder_grey_open", KIcon::Small, size,
+                         KIcon::DefaultState, 0, true );
+    } else {
+      pm = il->loadIcon( kmkernel->iCalIface().folderPixmap( type() ),
+                         KIcon::Small, size, KIcon::DefaultState, 0, true );
+      if ( pm.isNull() )
+        pm = il->loadIcon( "folder_open", KIcon::Small, size,
                            KIcon::DefaultState, 0, true );
+    }
+  }
+
   return pm;
 }
 
@@ -218,7 +230,8 @@ void KMFolderTreeItem::adjustUnreadCount( int newUnreadCount ) {
   setUnreadCount( newUnreadCount );
 }
 
-void KMFolderTreeItem::slotRepaint() {
+void KMFolderTreeItem::slotIconsChanged()
+{
   kdDebug(5006) << k_funcinfo << endl;
   // this is prone to change, so better check
   if( kmkernel->iCalIface().isResourceFolder( mFolder ) )
@@ -232,6 +245,13 @@ void KMFolderTreeItem::slotRepaint() {
   repaint();
 }
 
+void KMFolderTreeItem::slotNameChanged()
+{
+  setText( 0, mFolder->label() );
+  emit nameChanged( this );
+  repaint();
+}
+
 
 //-----------------------------------------------------------------------------
 bool KMFolderTreeItem::acceptDrag(QDropEvent*) const
@@ -298,6 +318,8 @@ KMFolderTree::KMFolderTree( KMMainWidget *mainWidget, QWidget *parent,
   mMainWidget = mainWidget;
   mReloading = false;
 
+  mUpdateCountTimer= new QTimer( this );
+
   addAcceptableDropMimetype(MailListDrag::format(), false);
 
   int namecol = addColumn( i18n("Folder"), 250 );
@@ -320,6 +342,9 @@ KMFolderTree::KMFolderTree( KMMainWidget *mainWidget, QWidget *parent,
 // connects all needed signals to their slots
 void KMFolderTree::connectSignals()
 {
+  connect( mUpdateCountTimer, SIGNAL(timeout()),
+          this, SLOT(slotUpdateCountTimeout()) );
+
   connect(&mUpdateTimer, SIGNAL(timeout()),
           this, SLOT(delayedUpdate()));
 
@@ -533,15 +558,18 @@ void KMFolderTree::reload(bool openFolders)
       continue;
 
     disconnect(fti->folder(),SIGNAL(iconsChanged()),
-               fti,SLOT(slotRepaint()));
+               fti,SLOT(slotIconsChanged()));
     connect(fti->folder(),SIGNAL(iconsChanged()),
-            fti,SLOT(slotRepaint()));
+            fti,SLOT(slotIconsChanged()));
 
     disconnect(fti->folder(),SIGNAL(nameChanged()),
                fti,SLOT(slotNameChanged()));
     connect(fti->folder(),SIGNAL(nameChanged()),
             fti,SLOT(slotNameChanged()));
 
+    // With the use of slotUpdateCountsDelayed is not necesary
+    // a specific processing for Imap
+#if 0
     if (fti->folder()->folderType() == KMFolderTypeImap) {
       // imap-only
       KMFolderImap *imapFolder =
@@ -550,23 +578,24 @@ void KMFolderTree::reload(bool openFolders)
           this,SLOT(slotUpdateCounts(KMFolderImap*, bool)));
       connect( imapFolder, SIGNAL(folderComplete(KMFolderImap*, bool)),
           this,SLOT(slotUpdateCounts(KMFolderImap*, bool)));
-    } else {
-      // others-only, imap doesn't need this because of the folderComplete-signal
-      // we want to be noticed of changes to update the unread/total columns
-      disconnect(fti->folder(), SIGNAL(msgAdded(KMFolder*,Q_UINT32)),
-          this,SLOT(slotUpdateCounts(KMFolder*)));
-      connect(fti->folder(), SIGNAL(msgAdded(KMFolder*,Q_UINT32)),
-          this,SLOT(slotUpdateCounts(KMFolder*)));
-    }
+    } else {*/
+#endif
+
+    // we want to be noticed of changes to update the unread/total columns
+    disconnect(fti->folder(), SIGNAL(msgAdded(KMFolder*,Q_UINT32)),
+        this,SLOT(slotUpdateCountsDelayed(KMFolder*)));
+    connect(fti->folder(), SIGNAL(msgAdded(KMFolder*,Q_UINT32)),
+        this,SLOT(slotUpdateCountsDelayed(KMFolder*)));
+    //}
 
     disconnect(fti->folder(), SIGNAL(numUnreadMsgsChanged(KMFolder*)),
-               this,SLOT(slotUpdateCounts(KMFolder*)));
+               this,SLOT(slotUpdateCountsDelayed(KMFolder*)));
     connect(fti->folder(), SIGNAL(numUnreadMsgsChanged(KMFolder*)),
-            this,SLOT(slotUpdateCounts(KMFolder*)));
+            this,SLOT(slotUpdateCountsDelayed(KMFolder*)));
     disconnect(fti->folder(), SIGNAL(msgRemoved(KMFolder*)),
-               this,SLOT(slotUpdateCounts(KMFolder*)));
+               this,SLOT(slotUpdateCountsDelayed(KMFolder*)));
     connect(fti->folder(), SIGNAL(msgRemoved(KMFolder*)),
-            this,SLOT(slotUpdateCounts(KMFolder*)));
+            this,SLOT(slotUpdateCountsDelayed(KMFolder*)));
 
     disconnect(fti->folder(), SIGNAL(shortcutChanged(KMFolder*)),
                mMainWidget, SLOT( slotShortcutChanged(KMFolder*)));
@@ -647,12 +676,14 @@ void KMFolderTree::addDirectory( KMFolderDir *fdir, KMFolderTreeItem* parent )
 
       // create new child
       fti = new KMFolderTreeItem( parent, folder->label(), folder );
-      // set folders explicitely to exandable even with unknown child state
-      // this way we can do a listing for IMAP folders when they're expanded
-      if ( folder->storage()->hasChildren() != FolderStorage::HasNoChildren )
+      // set folders explicitely to exandable when they have children
+      // this way we can do a listing for IMAP folders when the user expands them
+      // even when the child folders are not created yet
+      if ( folder->storage()->hasChildren() == FolderStorage::HasChildren ) {
         fti->setExpandable( true );
-      else
+      } else {
         fti->setExpandable( false );
+      }
 
       connect (fti, SIGNAL(iconChanged(KMFolderTreeItem*)),
           this, SIGNAL(iconChanged(KMFolderTreeItem*)));
@@ -770,8 +801,8 @@ void KMFolderTree::nextUnreadFolder(bool confirm)
 //-----------------------------------------------------------------------------
 bool KMFolderTree::checkUnreadFolder (KMFolderTreeItem* fti, bool confirm)
 {
-  if (fti && fti->folder()  &&
-      (fti->folder()->countUnread() > 0)) {
+  if ( fti && fti->folder() && !fti->folder()->ignoreNewMail() &&
+       ( fti->folder()->countUnread() > 0 ) ) {
 
     // Don't change into the trash or outbox folders.
     if (fti->type() == KFolderTreeItem::Trash ||
@@ -793,8 +824,8 @@ bool KMFolderTree::checkUnreadFolder (KMFolderTreeItem* fti, bool confirm)
       if ( KMessageBox::questionYesNo( this,
             i18n( "<qt>Go to the next unread message in folder <b>%1</b>?</qt>" )
             .arg( fti->folder()->label() ),
-            i18n( "Go to the Next Unread Message" ),
-            KStdGuiItem::yes(), KStdGuiItem::no(), // defaults
+            i18n( "Go to Next Unread Message" ),
+            i18n("Go To"), i18n("Do Not Go To"), // defaults
             "AskNextFolder",
             false)
           == KMessageBox::No ) return true;
@@ -940,7 +971,7 @@ void KMFolderTree::slotContextMenuRequested( QListViewItem *lvi,
   if ( (fti->folder() == kmkernel->outboxFolder()) && fti->folder()->count() )
         mMainWidget->action("send_queued")->plug( folderMenu );
   // Mark all as read is supposedly used often, therefor it is first
-  if ( fti->folder() && !fti->folder()->noContent() && fti->folder()->count() )
+  if ( fti->folder() && !fti->folder()->noContent() )
       mMainWidget->action("mark_all_as_read")->plug( folderMenu );
 
   /* Treat the special case of the root and account folders */
@@ -988,9 +1019,6 @@ void KMFolderTree::slotContextMenuRequested( QListViewItem *lvi,
 
       mMainWidget->action("compact")->plug(folderMenu);
 
-      if ( !fti->folder()->isSystemFolder() )
-        mMainWidget->action("delete_folder")->plug(folderMenu);
-
       folderMenu->insertSeparator();
       mMainWidget->action("empty")->plug(folderMenu);
       folderMenu->insertSeparator();
@@ -1099,12 +1127,15 @@ static bool folderHasCreateRights( const KMFolder *folder )
 //-----------------------------------------------------------------------------
 // Create a subfolder.
 // Requires creating the appropriate subdirectory and show a dialog
-void KMFolderTree::addChildFolder()
+void KMFolderTree::addChildFolder( KMFolder *folder, QWidget * parent )
 {
-  KMFolderTreeItem *fti = static_cast<KMFolderTreeItem*>(currentItem());
-  if (!fti)
-    return;
-  KMFolder *aFolder = fti->folder();
+  KMFolder *aFolder = folder;
+  if ( !aFolder ) {
+    KMFolderTreeItem *fti = static_cast<KMFolderTreeItem*>(currentItem());
+    if (!fti)
+      return;
+    aFolder = fti->folder();
+  }
   if (aFolder) {
     if (!aFolder->createChildFolder())
       return;
@@ -1119,7 +1150,10 @@ void KMFolderTree::addChildFolder()
     }
   }
 
-  ( new KMail::NewFolderDialog( this, aFolder ) )->show();
+  if ( parent )
+    ( new KMail::NewFolderDialog( parent, aFolder ) )->exec();
+  else
+    ( new KMail::NewFolderDialog( this, aFolder ) )->show();
   return;
 /*
   KMFolderDir *dir = &(kmkernel->folderMgr()->dir());
@@ -1228,6 +1262,7 @@ void KMFolderTree::cleanupConfigFile()
       if ( folder && kmkernel->iCalIface().hideResourceFolder( folder ) )
         continue; // hidden IMAP resource folder, don't delete info
 
+      //KMessageBox::error( 0, "cleanupConfigFile: Deleting group " + *grpIt );
       config->deleteGroup(*grpIt, TRUE);
       kdDebug(5006) << "Deleting information about folder " << name << endl;
     }
@@ -1347,7 +1382,7 @@ void KMFolderTree::contentsDropEvent( QDropEvent *e )
       } else if ( keybstate & KApplication::ShiftModifier ) {
         emit folderDrop(fti->folder());
       } else {
-        if ( GlobalSettings::showPopupAfterDnD() ) {
+        if ( GlobalSettings::self()->showPopupAfterDnD() ) {
           KPopupMenu *menu = new KPopupMenu( this );
           menu->insertItem( i18n("&Move Here"), DRAG_MOVE, 0 );
           menu->insertItem( SmallIcon("editcopy"), i18n("&Copy Here"), DRAG_COPY, 1 );
@@ -1462,8 +1497,38 @@ void KMFolderTree::slotUpdateCounts(KMFolderImap * folder, bool success)
 }
 
 //-----------------------------------------------------------------------------
+void KMFolderTree::slotUpdateCountsDelayed(KMFolder * folder)
+{
+//  kdDebug(5006) << "KMFolderTree::slotUpdateCountsDelayed()" << endl;
+  if ( !mFolderToUpdateCount.contains( folder->idString() ) )
+  {
+//    kdDebug( 5006 )<< "adding " << folder->idString() << " to updateCountList " << endl;
+    mFolderToUpdateCount.insert( folder->idString(),folder );
+  }
+  if ( !mUpdateCountTimer->isActive() )
+    mUpdateCountTimer->start( 500 );
+}
+
+
+void KMFolderTree::slotUpdateCountTimeout()
+{
+//  kdDebug(5006) << "KMFolderTree::slotUpdateCountTimeout()" << endl;
+
+  QMap<QString,KMFolder*>::iterator it;
+  for ( it= mFolderToUpdateCount.begin();
+      it!=mFolderToUpdateCount.end();
+      ++it )
+  {
+    slotUpdateCounts( it.data() );
+  }
+  mFolderToUpdateCount.clear();
+  mUpdateCountTimer->stop();
+
+}
+
 void KMFolderTree::slotUpdateCounts(KMFolder * folder)
 {
+ // kdDebug(5006) << "KMFolderTree::slotUpdateCounts()" << endl;
   QListViewItem * current;
   if (folder)
     current = indexOfFolder(folder);
@@ -1715,6 +1780,18 @@ void KMFolderTree::folderToPopupMenu( MenuAction action, QObject *receiver,
       item = item->nextSibling();
       continue;
     }
+    if ( action == MoveFolder ) {
+      // FIXME remove in KDE 4
+      // skip all but local folders if a folder is to be moved
+      // because moving of nested folders to IMAP and DIMAP
+      // looses all messages in the subfolders
+      if ( fti->protocol() != KFolderTreeItem::Local
+        && fti->protocol() != KFolderTreeItem::NONE )
+      {
+        item = item->nextSibling();
+        continue;
+      }
+    }
     QString label = fti->text( 0 );
     label.replace( "&","&&" );
     if ( fti->firstChild() )
@@ -1723,10 +1800,10 @@ void KMFolderTree::folderToPopupMenu( MenuAction action, QObject *receiver,
       QPopupMenu* popup = new QPopupMenu( menu, "subMenu" );
       folderToPopupMenu( action, receiver, aMenuToFolder, popup, fti->firstChild() );
       bool subMenu = false;
-      if ( ( action == MoveMessage || action == CopyMessage ) && 
+      if ( ( action == MoveMessage || action == CopyMessage ) &&
            fti->folder() && !fti->folder()->noContent() )
         subMenu = true;
-      if ( action == MoveFolder && ( !fti->folder() || 
+      if ( action == MoveFolder && ( !fti->folder() ||
             ( fti->folder() && !fti->folder()->noChildren() ) ) )
         subMenu = true;
       if ( subMenu )
@@ -1770,7 +1847,7 @@ void KMFolderTree::moveFolder( KMFolder* destination )
   KMFolderDir* parent = &(kmkernel->folderMgr()->dir());
   if ( destination )
     parent = destination->createChildFolder();
-  QString message = 
+  QString message =
     i18n( "<qt>Cannot move folder <b>%1</b> into a subfolder below itself.</qt>" ).
         arg( folder->label() );
 
@@ -1802,7 +1879,7 @@ void KMFolderTree::moveFolder( KMFolder* destination )
     return;
   }
 
-  kdDebug(5006) << "move folder " << currentFolder()->label() << " to " 
+  kdDebug(5006) << "move folder " << currentFolder()->label() << " to "
     << ( destination ? destination->label() : "Local Folders" ) << endl;
   kmkernel->folderMgr()->moveFolder( folder, parent );
 }
