@@ -1,10 +1,23 @@
- /* -*- mode: C++; c-file-style: "gnu" -*-
- * kmail main window
- * Copyright 2002 Don Sanders <sanders@kde.org>
- * Based on the work of Stefan Taferner <taferner@kde.org>
- *
- * License GPL
- */
+/* -*- mode: C++; c-file-style: "gnu" -*-
+    This file is part of KMail, the KDE mail client.
+    Copyright (c) 2002 Don Sanders <sanders@kde.org>
+
+    Based on the work of Stefan Taferner <taferner@kde.org>
+
+    KMail is free software; you can redistribute it and/or modify it
+    under the terms of the GNU General Public License, version 2, as
+    published by the Free Software Foundation.
+
+    KMail is distributed in the hope that it will be useful, but
+    WITHOUT ANY WARRANTY; without even the implied warranty of
+    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+    General Public License for more details.
+
+    You should have received a copy of the GNU General Public License
+    along with this program; if not, write to the Free Software
+    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
+*/
+
 #ifndef __KMMAINWIDGET
 #define __KMMAINWIDGET
 
@@ -17,14 +30,13 @@
 #include "kmkernel.h" // for access to config
 #include <kaction.h>
 
+class QAccel;
 class QVBoxLayout;
 class QSplitter;
-class QTextCodec;
 
 class KActionMenu;
 class KActionCollection;
 class KConfig;
-class KSelectAction;
 class KRadioAction;
 class KToggleAction;
 class KMenuBar;
@@ -33,14 +45,14 @@ class KMFolder;
 class KMFolderDir;
 class KMFolderTree;
 class KMFolderTreeItem;
-class KMHeaders;
 class KMCommand;
 class KMMetaFilterActionCommand;
+class FolderShortcutCommand;
 class KMMessage;
 class KMFolder;
 class KMAccount;
-class KMFldSearch;
 class KMSystemTray;
+class KMHeaders;
 
 template <typename T> class QValueList;
 template <typename T, typename S> class QMap;
@@ -52,17 +64,16 @@ namespace KIO {
 
 namespace KMail {
   class Vacation;
-  class AttachmentStrategy;
-  class HeaderStrategy;
-  class HeaderStyle;
+  class SieveDebugDialog;
   class FolderJob;
   class HeaderListQuickSearch;
+  class SearchWindow;
 }
 
 typedef QMap<int,KMFolder*> KMMenuToFolder;
 
 
-class KMMainWidget : public QWidget
+class KDE_EXPORT KMMainWidget : public QWidget
 {
   Q_OBJECT
 
@@ -105,19 +116,21 @@ public:
   KAction *forwardAction() const { return mForwardAction; }
   KAction *forwardAttachedAction() const { return mForwardAttachedAction; }
   KAction *redirectAction() const { return mRedirectAction; }
-  KAction *bounceAction() const { return mBounceAction; }
   KAction *noQuoteReplyAction() const { return mNoQuoteReplyAction; }
   KActionMenu *filterMenu() const { return mFilterMenu; }
   KAction *printAction() const { return mPrintAction; }
   KAction *trashAction() const { return mTrashAction; }
   KAction *deleteAction() const { return mDeleteAction; }
+  KAction *trashThreadAction() const { return mTrashThreadAction; }
+  KAction *deleteThreadAction() const { return mDeleteThreadAction; }
   KAction *saveAsAction() const { return mSaveAsAction; }
   KAction *editAction() const { return mEditAction; }
   KAction *sendAgainAction() const { return mSendAgainAction; }
-  KAction *applyFiltersAction() const { return mSendAgainAction; }
+  KAction *applyAllFiltersAction() const { return mApplyAllFiltersAction; }
   KAction *findInMessageAction() const { return mFindInMessageAction; }
   KAction *saveAttachmentsAction() const { return mSaveAttachmentsAction; }
   KAction *openAction() const { return mOpenAction; }
+  KAction *viewSourceAction() { return mViewSourceAction; }
 
   KActionMenu *statusMenu()  const{ return mStatusMenu; }
   KActionMenu *threadStatusMenu() const { return mThreadStatusMenu; }
@@ -128,22 +141,30 @@ public:
   KToggleAction *watchThreadAction() const { return mWatchThreadAction; }
   KToggleAction *ignoreThreadAction() const { return mIgnoreThreadAction; }
 
-  // Forwarded to the reader window.
-  KToggleAction *toggleFixFontAction() { return mMsgView->toggleFixFontAction(); }
-  KAction *viewSourceAction() { return mMsgView->viewSourceAction(); }
-
   KMHeaders *headers() const { return mHeaders; }
   void toggleSystemTray();
 
   void updateListFilterAction();
 
-  /// @return a list of all KMMainWidgets. Warning, the list itself can be 0.
-  static QPtrList<KMMainWidget>* mainWidgetList() { return s_mainWidgetList; }
+  /** Returns a list of all KMMainWidgets. Warning, the list itself can be 0.
+   * @return the list of all main widgets, or 0 if it is not yet initialized */
+  static const QValueList<KMMainWidget*>* mainWidgetList() { return s_mainWidgetList; }
 
   KMSystemTray *systray() const;
 
+  /** Checks a shortcut against the actioncollection and returns whether it
+   * is already used and therefor not valid or not. */
+  bool shortcutIsValid( const KShortcut& ) const;
+
+
   void modifyFolder( KMFolderTreeItem* folderItem );
 
+  /**
+   * Enable or disable the global accelerators. This is useful for keyboard
+   * navigation inside child widgets like combo boxes.
+   */
+  void setAccelsEnabled( bool enabled = true );
+
 public slots:
   void slotMoveMsgToFolder( KMFolder *dest);
   void slotTrashMsg();   // move to trash
@@ -190,7 +211,16 @@ public slots:
   /** Clear and create actions for marked filters */
   void clearFilterActions();
   void initializeFilterActions();
- 
+
+  /** Create IMAP-account-related actions if applicable */
+  void initializeIMAPActions() { initializeIMAPActions( true ); }
+
+  /** Create actions for the folder shortcuts. */
+  void initializeFolderShortcutActions();
+
+  /** Add, remove or adjust the folder's shortcut. */
+  void slotShortcutChanged( KMFolder *folder );
+
 signals:
   void messagesTransfered( bool );
   void captionChangeRequest( const QString & caption );
@@ -201,13 +231,8 @@ protected:
   void activatePanners();
   void showMsg(KMReaderWin *win, KMMessage *msg);
   void updateFileMenu();
-  void updateViewMenu();
 
-  KActionCollection * actionCollection() { return mActionCollection; }
-
-  KRadioAction * actionForHeaderStyle( const KMail::HeaderStyle *,
-                                       const KMail::HeaderStrategy * );
-  KRadioAction * actionForAttachmentStrategy( const KMail::AttachmentStrategy * );
+  KActionCollection * actionCollection() const { return mActionCollection; }
 
   /** @return the correct config dialog depending on whether the parent of the mainWidget
    *          is a KPart or a KMMainWindow. When dealing with geometries, use this pointer
@@ -219,17 +244,19 @@ protected slots:
   void slotMailChecked( bool newMail, bool sendOnCheck,
                         const QMap<QString, int> & newInFolder );
   void getAccountMenu();
+  void getTransportMenu();
   void slotHelp();
-  void slotNewMailReader();
   void slotFilter();
   void slotPopFilter();
+  void slotManageSieveScripts();
   void slotAddrBook();
   void slotImport();
   void slotViewChange();
-  void slotAddFolder();
   void slotCompose();
   void slotPostToML();
   void slotModifyFolder();
+  void slotFolderMailingListProperties();
+  void slotFolderShortcutCommand();
   void slotExpireFolder();
   void slotExpireAll();
   void slotInvalidateIMAPFolders();
@@ -238,14 +265,18 @@ protected slots:
   void slotEmptyFolder();
   void slotCompactFolder();
   void slotRefreshFolder();
+  void slotTroubleshootFolder();
   void slotCompactAll();
   void slotOverrideHtml();
+  void slotOverrideHtmlLoadExt();
   void slotOverrideThread();
   void slotToggleSubjectThreading();
   void slotMessageQueuedOrDrafted();
   void slotEditMsg();
   //void slotTrashMsg();   // move to trash
   void slotDeleteMsg( bool confirmDelete = true );  // completely delete message
+  void slotTrashThread();
+  void slotDeleteThread( bool confirmDelete = true );  // completely delete thread
   void slotUndo();
   void slotReadOn();
   void slotSaveMsg();
@@ -257,6 +288,7 @@ protected slots:
   void slotCopyMsg();
   void slotResendMsg();
   void slotEditVacation();
+  void slotDebugSieve();
   void slotStartCertManager();
   void slotStartWatchGnuPG();
   void slotApplyFilters();
@@ -264,43 +296,25 @@ protected slots:
   void slotExpandAllThreads();
   void slotCollapseThread();
   void slotCollapseAllThreads();
+  void slotShowMsgSrc();
   void slotSetMsgStatusNew();
   void slotSetMsgStatusUnread();
   void slotSetMsgStatusRead();
-  void slotSetMsgStatusReplied();
-  void slotSetMsgStatusForwarded();
-  void slotSetMsgStatusQueued();
+  void slotSetMsgStatusTodo();
   void slotSetMsgStatusSent();
   void slotSetMsgStatusFlag();
-  void slotSetMsgStatusSpam();
-  void slotSetMsgStatusHam();
   void slotSetThreadStatusNew();
   void slotSetThreadStatusUnread();
   void slotSetThreadStatusRead();
-  void slotSetThreadStatusReplied();
-  void slotSetThreadStatusForwarded();
-  void slotSetThreadStatusQueued();
-  void slotSetThreadStatusSent();
+  void slotSetThreadStatusTodo();
   void slotSetThreadStatusFlag();
-  void slotSetThreadStatusSpam();
-  void slotSetThreadStatusHam();
   void slotSetThreadStatusWatched();
   void slotSetThreadStatusIgnored();
   void slotToggleUnread();
   void slotToggleTotalColumn();
-  void slotBriefHeaders();
-  void slotFancyHeaders();
-  void slotStandardHeaders();
-  void slotLongHeaders();
-  void slotAllHeaders();
-  void slotIconicAttachments();
-  void slotSmartAttachments();
-  void slotInlineAttachments();
-  void slotHideAttachments();
-  void slotCycleHeaderStyles();
-  void slotCycleAttachmentStrategy();
-  void slotSetEncoding();
   void slotSendQueued();
+  void slotSendQueuedVia( int item );
+  void slotOnlineStatus();
   void slotMsgPopup(KMMessage &msg, const KURL &aUrl, const QPoint&);
   void slotMarkAll();
   void slotMemInfo();
@@ -326,6 +340,7 @@ protected slots:
   void slotPrevUnreadFolder();
 
   /** etc. */
+  void slotDisplayCurrentMessage();
   void slotMsgActivated(KMMessage*);
 
   /** Update the undo action */
@@ -336,7 +351,7 @@ protected slots:
   /** Copy selected messages to folder with corresponding to given menuid */
   virtual void copySelectedToFolder( int menuId );
   /** Update html and threaded messages preferences in Folder menu. */
-  virtual void updateFolderMenu();
+  void updateFolderMenu();
   /** Enable or disable the "mark all as read" action. Needs to happen more
    * often the the other updates and is therefor in its own method. */
   void updateMarkAsReadAction();
@@ -360,7 +375,6 @@ protected slots:
   void slotForwardMsg();
   void slotForwardAttachedMsg();
   void slotRedirectMsg();
-  void slotBounceMsg();
   void slotNoQuoteReplyToMsg();
   void slotSubjectFilter();
   void slotMailingListFilter();
@@ -369,17 +383,32 @@ protected slots:
   void slotPrintMsg();
 
   void slotConfigChanged();
+  /** Remove the shortcut actions associated with a folder. */
+  void slotFolderRemoved( KMFolder *folder );
+
+  /** Show a splash screen for the longer-lasting operation */
+  void slotShowBusySplash();
+  /** Show a message screen explaining that we are currently offline, when
+   * an online folder is selected. */
+  void showOfflinePage();
+
+private:
+  /** Get override character encoding. */
+  QString overrideEncoding() const;
+
+  void initializeIMAPActions( bool setState );
 
 private:
   // Message actions
-  KAction *mTrashAction, *mDeleteAction, *mSaveAsAction, *mEditAction,
-    *mSendAgainAction, *mApplyFiltersAction, *mFindInMessageAction,
-    *mSaveAttachmentsAction, *mOpenAction;
+  KAction *mTrashAction, *mDeleteAction, *mTrashThreadAction,
+    *mDeleteThreadAction, *mSaveAsAction, *mEditAction,
+    *mSendAgainAction, *mApplyAllFiltersAction, *mFindInMessageAction,
+    *mSaveAttachmentsAction, *mOpenAction, *mViewSourceAction;
   // Composition actions
   KAction *mPrintAction, *mReplyAction, *mReplyAllAction, *mReplyAuthorAction,
       *mReplyListAction,
       *mForwardAction, *mForwardAttachedAction, *mRedirectAction,
-      *mBounceAction, *mNoQuoteReplyAction;
+      *mNoQuoteReplyAction;
   KActionMenu *mReplyActionMenu;
   KActionMenu *mForwardActionMenu;
   // Filter actions
@@ -392,20 +421,11 @@ private:
   KAction *mMarkThreadAsNewAction;
   KAction *mMarkThreadAsReadAction;
   KAction *mMarkThreadAsUnreadAction;
-  KToggleAction *mToggleThreadRepliedAction;
-  KToggleAction *mToggleThreadForwardedAction;
-  KToggleAction *mToggleThreadQueuedAction;
-  KToggleAction *mToggleThreadSentAction;
+  KToggleAction *mToggleThreadTodoAction;
   KToggleAction *mToggleThreadFlagAction;
-  KAction *mMarkThreadAsSpamAction;
-  KAction *mMarkThreadAsHamAction;
-  KToggleAction *mToggleRepliedAction;
-  KToggleAction *mToggleForwardedAction;
-  KToggleAction *mToggleQueuedAction;
+  KToggleAction *mToggleTodoAction;
   KToggleAction *mToggleSentAction;
   KToggleAction *mToggleFlagAction;
-  KAction *mMarkAsSpamAction;
-  KAction *mMarkAsHamAction;
 
   KToggleAction *mWatchThreadAction, *mIgnoreThreadAction;
 
@@ -415,7 +435,7 @@ private:
   KToggleAction* mTotalColumnToggle;
 
   KToggleAction *mToggleShowQuickSearchAction;
-private:
+
   KMFolderTree *mFolderTree;
   KMReaderWin  *mMsgView;
   QSplitter    *mPanner1, *mPanner2;
@@ -424,11 +444,8 @@ private:
   KToolBar     *mSearchToolBar;
   KMail::HeaderListQuickSearch *mQuickSearchLine;
   KMFolder     *mFolder;
-  const QTextCodec   *mCodec;
   QPopupMenu   *mViewMenu, *mBodyPartsMenu;
-  KSelectAction *mEncoding;
   KAction       *mlistFilterAction;
-  QCString	mEncodingStr;
   bool		mIntegrated;
   bool          mBeepOnNew;
   bool          mConfirmEmpty;
@@ -437,6 +454,7 @@ private:
   QValueList<int> mPanner1Sep, mPanner2Sep;
   KURL          mUrlCurrent;
   QPopupMenu	*mActMenu;
+  QPopupMenu    *mSendMenu;
   QPopupMenu	*mFileMenu;
 
   bool mLongFolderList;
@@ -444,35 +462,45 @@ private:
   bool		mStartupDone;
   KMMenuToFolder mMenuToFolder;
   int copyId, moveId, htmlId, threadId;
-  bool mHtmlPref, mThreadPref, mFolderHtmlPref, mFolderThreadPref,
+  bool mHtmlPref, mHtmlLoadExtPref, mThreadPref,
+       mFolderHtmlPref, mFolderHtmlLoadExtPref, mFolderThreadPref,
        mFolderThreadSubjPref, mReaderWindowActive, mReaderWindowBelow;
 
 //  QPopupMenu *mMessageMenu;
-  KMFldSearch *mSearchWin;
+  KMail::SearchWindow *mSearchWin;
 
   KAction *mModifyFolderAction, *mRemoveFolderAction, *mExpireFolderAction,
       *mCompactFolderAction, *mRefreshFolderAction, *mEmptyFolderAction,
-      *mMarkAllAsReadAction;
-  KToggleAction *mPreferHtmlAction, *mThreadMessagesAction;
+      *mMarkAllAsReadAction, *mFolderMailingListPropertiesAction,
+      *mFolderShortCutCommandAction, *mTroubleshootFolderAction;
+  KToggleAction *mPreferHtmlAction, *mPreferHtmlLoadExtAction, *mThreadMessagesAction;
   KToggleAction *mThreadBySubjectAction;
   KToggleAction *mFolderAction, *mHeaderAction, *mMimeAction;
 
   QTimer *menutimer;
+  QTimer *mShowBusySplashTimer;
 
   QGuardedPtr<KMail::Vacation> mVacation;
+#if !defined(NDEBUG)
+  QGuardedPtr<KMail::SieveDebugDialog> mSieveDebugDialog;
+#endif
   KActionCollection *mActionCollection;
+  KActionSeparator  *mToolbarActionSeparator;
   QVBoxLayout *mTopLayout;
-  bool mDestructed, mForceJumpToUnread;
+  bool mDestructed, mForceJumpToUnread, mShowingOfflineScreen;
   QPtrList<KAction> mFilterMenuActions;
   QPtrList<KAction> mFilterTBarActions;
   QPtrList<KMMetaFilterActionCommand> mFilterCommands;
+  QDict<FolderShortcutCommand> mFolderShortcutCommands;
   QGuardedPtr <KMail::FolderJob> mJob;
 
   KMSystemTray  *mSystemTray;
   KConfig *mConfig;
   KXMLGUIClient *mGUIClient;
-  
-  static QPtrList<KMMainWidget>* s_mainWidgetList;
+
+  static QValueList<KMMainWidget*>* s_mainWidgetList;
+
+  QAccel *mAccel;
 };
 
 #endif
