@@ -1,6 +1,20 @@
-// kmreadermainwin
-// (c) 2002 Don Sanders <sanders@kde.org>
-// License: GPL
+/*
+    This file is part of KMail, the KDE mail client.
+    Copyright (c) 2002 Don Sanders <sanders@kde.org>
+
+    KMail is free software; you can redistribute it and/or modify it
+    under the terms of the GNU General Public License, version 2, as
+    published by the Free Software Foundation.
+
+    KMail is distributed in the hope that it will be useful, but
+    WITHOUT ANY WARRANTY; without even the implied warranty of
+    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+    General Public License for more details.
+
+    You should have received a copy of the GNU General Public License
+    along with this program; if not, write to the Free Software
+    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
+*/
 //
 // A toplevel KMainWindow derived class for displaying
 // single messages or single message parts.
@@ -58,12 +72,12 @@ KMReaderMainWin::KMReaderMainWin( char *name )
 //-----------------------------------------------------------------------------
 KMReaderMainWin::KMReaderMainWin(KMMessagePart* aMsgPart,
     bool aHTML, const QString& aFileName, const QString& pname,
-    const QTextCodec *codec, char *name )
+    const QString & encoding, char *name )
   : KMail::SecondaryWindow( name ? name : "readerwindow#" ),
     mMsg( 0 )
 {
   mReaderWin = new KMReaderWin( this, this, actionCollection() );
-  mReaderWin->setOverrideCodec( codec );
+  mReaderWin->setOverrideEncoding( encoding );
   mReaderWin->setMsgPart( aMsgPart, aHTML, aFileName, pname );
   initKMReaderMainWin();
 }
@@ -93,9 +107,9 @@ void KMReaderMainWin::setUseFixedFont( bool useFixedFont )
 }
 
 //-----------------------------------------------------------------------------
-void KMReaderMainWin::showMsg( const QTextCodec *codec, KMMessage *msg )
+void KMReaderMainWin::showMsg( const QString & encoding, KMMessage *msg )
 {
-  mReaderWin->setOverrideCodec( codec );
+  mReaderWin->setOverrideEncoding( encoding );
   mReaderWin->setMsg( msg, true );
   setCaption( msg->subject() );
   mMsg = msg;
@@ -106,8 +120,8 @@ void KMReaderMainWin::showMsg( const QTextCodec *codec, KMMessage *msg )
 void KMReaderMainWin::slotPrintMsg()
 {
   KMCommand *command = new KMPrintCommand( this, mReaderWin->message(),
-      mReaderWin->htmlOverride(),  mReaderWin->htmlLoadExtOverride(),
-      mReaderWin->overrideCodec() );
+      mReaderWin->htmlOverride(), mReaderWin->htmlLoadExtOverride(),
+      mReaderWin->isFixedFont(), mReaderWin->overrideEncoding() );
   command->start();
 }
 
@@ -299,7 +313,7 @@ void KMReaderMainWin::slotMsgPopup(KMMessage &aMsg, const KURL &aUrl, const QPoi
   mMsg = &aMsg;
   bool urlMenuAdded=false;
 
-  if (!aUrl.isEmpty()) 
+  if (!aUrl.isEmpty())
   {
     if (aUrl.protocol() == "mailto") {
       // popup on a mailto URL
@@ -315,12 +329,12 @@ void KMReaderMainWin::slotMsgPopup(KMMessage &aMsg, const KURL &aUrl, const QPoi
     } else {
       // popup on a not-mailto URL
       mReaderWin->urlOpenAction()->plug( menu );
+      mReaderWin->addBookmarksAction()->plug( menu );
       mReaderWin->urlSaveAsAction()->plug( menu );
       mReaderWin->copyURLAction()->plug( menu );
-      mReaderWin->addBookmarksAction()->plug( menu );
     }
     urlMenuAdded=true;
-  } 
+  }
   if(mReaderWin && !mReaderWin->copyText().isEmpty()) {
     if ( urlMenuAdded )
       menu->insertSeparator();
@@ -336,10 +350,11 @@ void KMReaderMainWin::slotMsgPopup(KMMessage &aMsg, const KURL &aUrl, const QPoi
       return;
     }
 
-    mReplyActionMenu->plug( menu );
-    mForwardActionMenu->plug( menu );
-
-    menu->insertSeparator();
+    if( !aMsg.parent()->isSent() && !aMsg.parent()->isDrafts() ) {
+      mReplyActionMenu->plug( menu );
+      mForwardActionMenu->plug( menu );
+      menu->insertSeparator();
+    }
 
     QPopupMenu* copyMenu = new QPopupMenu(menu);
     KMMainWidget* mainwin = kmkernel->getKMMainWidget();
