@@ -7,10 +7,11 @@
 
 #include "kmaddrbook.h"
 #include "kmsearchpattern.h"
-#include "kmmessage.h"
-#include "kmmsgindex.h"
 #include "kmmsgdict.h"
 #include "filterlog.h"
+#include "kmkernel.h"
+#include "kmmsgdict.h"
+#include "kmfolder.h"
 using KMail::FilterLog;
 
 #include <libemailfunctions/email.h>
@@ -36,6 +37,32 @@ static const char* funcConfigNames[] =
     "has-attachment", "has-no-attachment"};
 static const int numFuncConfigNames = sizeof funcConfigNames / sizeof *funcConfigNames;
 
+struct _statusNames {
+  const char* name;
+  KMMsgStatus status;
+};
+
+static struct _statusNames statusNames[] = {
+  { "Important", KMMsgStatusFlag },
+  { "New", KMMsgStatusNew },
+  { "Unread", KMMsgStatusUnread | KMMsgStatusNew },
+  { "Read", KMMsgStatusRead },
+  { "Old", KMMsgStatusOld },
+  { "Deleted", KMMsgStatusDeleted },
+  { "Replied", KMMsgStatusReplied },
+  { "Forwarded", KMMsgStatusForwarded },
+  { "Queued", KMMsgStatusQueued },
+  { "Sent", KMMsgStatusSent },
+  { "Watched", KMMsgStatusWatched },
+  { "Ignored", KMMsgStatusIgnored },
+  { "To Do", KMMsgStatusTodo },
+  { "Spam", KMMsgStatusSpam },
+  { "Ham", KMMsgStatusHam },
+  { "Has Attachment", KMMsgStatusHasAttach }
+};
+
+static const int numStatusNames = sizeof statusNames / sizeof ( struct _statusNames );
+
 
 //==================================================
 //
@@ -225,6 +252,8 @@ bool KMSearchRuleString::matches( const DwString & aStr, KMMessage & msg,
   if ( isEmpty() )
     return false;
 
+  bool rc = false;
+
   const DwBoyerMoore * headerField = aHeaderField ? aHeaderField : mBmHeaderField ;
 
   const int headerLen = ( aHeaderLen > -1 ? aHeaderLen : field().length() ) + 2 ; // +1 for ': '
@@ -246,16 +275,18 @@ bool KMSearchRuleString::matches( const DwString & aStr, KMMessage & msg,
     // contain"); note that all negated string functions correspond
     // to an odd value
     if ( start == DwString::npos )
-      return ( ( function() & 1 ) == 1 );
-    start += headerLen;
-    size_t stop = aStr.find( '\n', start );
-    char ch = '\0';
-    while ( stop != DwString::npos && ( ch = aStr.at( stop + 1 ) ) == ' ' || ch == '\t' )
-      stop = aStr.find( '\n', stop + 1 );
-    const int len = stop == DwString::npos ? aStr.length() - start : stop - start ;
-    const QCString codedValue( aStr.data() + start, len + 1 );
-    const QString msgContents = KMMsgBase::decodeRFC2047String( codedValue ).stripWhiteSpace(); // FIXME: This needs to be changed for IDN support.
-    return matchesInternal( msgContents );
+      rc = ( ( function() & 1 ) == 1 );
+    else {
+      start += headerLen;
+      size_t stop = aStr.find( '\n', start );
+      char ch = '\0';
+      while ( stop != DwString::npos && ( ch = aStr.at( stop + 1 ) ) == ' ' || ch == '\t' )
+        stop = aStr.find( '\n', stop + 1 );
+      const int len = stop == DwString::npos ? aStr.length() - start : stop - start ;
+      const QCString codedValue( aStr.data() + start, len + 1 );
+      const QString msgContents = KMMsgBase::decodeRFC2047String( codedValue ).stripWhiteSpace(); // FIXME: This needs to be changed for IDN support.
+      rc = matchesInternal( msgContents );
+    }
   } else if ( field() == "<recipients>" ) {
     static const DwBoyerMoore to("\nTo: ");
     static const DwBoyerMoore cc("\nCc: ");
@@ -265,18 +296,28 @@ bool KMSearchRuleString::matches( const DwString & aStr, KMMessage & msg,
     // of the fields contains "foo"
     if ( ( function() & 1 ) == 0 ) {
       // positive function, e.g. "contains"
-      return ( matches( aStr, msg, &to, 2 ) ||
-               matches( aStr, msg, &cc, 2 ) ||
-               matches( aStr, msg, &bcc, 3 ) );
+      rc = ( matches( aStr, msg, &to, 2 ) ||
+             matches( aStr, msg, &cc, 2 ) ||
+             matches( aStr, msg, &bcc, 3 ) );
     }
     else {
       // negated function, e.g. "does not contain"
-      return ( matches( aStr, msg, &to, 2 ) &&
-               matches( aStr, msg, &cc, 2 ) &&
-               matches( aStr, msg, &bcc, 3 ) );
+      rc = ( matches( aStr, msg, &to, 2 ) &&
+             matches( aStr, msg, &cc, 2 ) &&
+             matches( aStr, msg, &bcc, 3 ) );
     }
   }
-  return false;
+  if ( FilterLog::instance()->isLogging() ) {
+    QString msg = ( rc ? "<font color=#00FF00>1 = </font>"
+                       : "<font color=#FF0000>0 = </font>" );
+    msg += FilterLog::recode( asString() );
+    // only log headers bcause messages and bodies can be pretty large
+// FIXME We have to separate the text which is used for filtering to be able to show it in the log
+//    if ( logContents )
+//      msg += " (<i>" + FilterLog::recode( msgContents ) + "</i>)";
+    FilterLog::instance()->add( msg, FilterLog::ruleResult );
+  }
+  return rc;
 }
 
 bool KMSearchRuleString::matches( const KMMessage * msg ) const
@@ -393,7 +434,7 @@ bool KMSearchRuleString::matchesInternal( const QString & msgContents ) const
       return ( QString::compare( msgContents.lower(), contents().lower() ) >= 0 );
 
   case FuncIsInAddressbook: {
-    KABC::AddressBook *stdAb = KABC::StdAddressBook::self();
+    KABC::AddressBook *stdAb = KABC::StdAddressBook::self( true );
     QStringList addressList =
       KPIM::splitEmailAddrList( msgContents.lower() );
     for( QStringList::ConstIterator it = addressList.begin();
@@ -406,7 +447,7 @@ bool KMSearchRuleString::matchesInternal( const QString & msgContents ) const
   }
 
   case FuncIsNotInAddressbook: {
-    KABC::AddressBook *stdAb = KABC::StdAddressBook::self();
+    KABC::AddressBook *stdAb = KABC::StdAddressBook::self( true );
     QStringList addressList =
       KPIM::splitEmailAddrList( msgContents.lower() );
     for( QStringList::ConstIterator it = addressList.begin();
@@ -421,7 +462,7 @@ bool KMSearchRuleString::matchesInternal( const QString & msgContents ) const
   case FuncIsInCategory: {
     QString category = contents();
     QStringList addressList =  KPIM::splitEmailAddrList( msgContents.lower() );
-    KABC::AddressBook *stdAb = KABC::StdAddressBook::self();
+    KABC::AddressBook *stdAb = KABC::StdAddressBook::self( true );
 
     for( QStringList::ConstIterator it = addressList.begin();
       it != addressList.end(); ++it ) {
@@ -438,7 +479,7 @@ bool KMSearchRuleString::matchesInternal( const QString & msgContents ) const
     case FuncIsNotInCategory: {
       QString category = contents();
       QStringList addressList =  KPIM::splitEmailAddrList( msgContents.lower() );
-      KABC::AddressBook *stdAb = KABC::StdAddressBook::self();
+      KABC::AddressBook *stdAb = KABC::StdAddressBook::self( true );
 
       for( QStringList::ConstIterator it = addressList.begin();
         it != addressList.end(); ++it ) {
@@ -583,43 +624,12 @@ KMSearchRuleStatus::KMSearchRuleStatus( const QCString & field,
 KMMsgStatus KMSearchRuleStatus::statusFromEnglishName(
       const QString & aStatusString )
 {
-  KMMsgStatus status = 0;
-  QString statusString = aStatusString.upper();
-
-  if ( ! statusString.compare( QString("Important").upper() ) )
-    status = KMMsgStatusFlag;
-  if ( ! statusString.compare( QString("New").upper() ) )
-    status = KMMsgStatusNew;
-  if ( ! statusString.compare( QString("Unread").upper() ) )
-    status = KMMsgStatusUnread | KMMsgStatusNew;
-  if ( ! statusString.compare( QString("Read").upper() ) )
-    status = KMMsgStatusRead;
-  if ( ! statusString.compare( QString("Old").upper() ) )
-    status = KMMsgStatusOld;
-  if ( ! statusString.compare( QString("Deleted").upper() ) )
-    status = KMMsgStatusDeleted;
-  if ( ! statusString.compare( QString("Replied").upper() ) )
-    status = KMMsgStatusReplied;
-  if ( ! statusString.compare( QString("Forwarded").upper() ) )
-    status = KMMsgStatusForwarded;
-  if ( ! statusString.compare( QString("Queued").upper() ) )
-    status = KMMsgStatusQueued;
-  if ( ! statusString.compare( QString("Sent").upper() ) )
-    status = KMMsgStatusSent;
-  if ( ! statusString.compare( QString("Watched").upper() ) )
-    status = KMMsgStatusWatched;
-  if ( ! statusString.compare( QString("Ignored").upper() ) )
-    status = KMMsgStatusIgnored;
-  if ( ! statusString.compare( QString("To Do").upper() ) )
-    status = KMMsgStatusTodo;
-  if ( ! statusString.compare( QString("Spam").upper() ) )
-    status = KMMsgStatusSpam;
-  if ( ! statusString.compare( QString("Ham").upper() ) )
-     status = KMMsgStatusHam;
-  if ( ! statusString.compare( QString("Has Attachment").upper() ) )
-     status = KMMsgStatusHasAttach;
-
-  return status;
+  for ( int i=0; i< numStatusNames; i++ ) {
+    if ( !aStatusString.compare( statusNames[i].name ) ) {
+      return statusNames[i].status;
+    }
+  }
+  return KMMsgStatusUnknown;
 }
 
 bool KMSearchRuleStatus::isEmpty() const
@@ -745,7 +755,7 @@ bool KMSearchPattern::matches( Q_UINT32 serNum, bool ignoreBody ) const
   bool res;
   int idx = -1;
   KMFolder *folder = 0;
-  kmkernel->msgDict()->getLocation(serNum, &folder, &idx);
+  KMMsgDict::instance()->getLocation(serNum, &folder, &idx);
   if (!folder || (idx == -1) || (idx >= folder->count())) {
     return false;
   }
