@@ -14,7 +14,7 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
  *
  *  In addition, as a special exception, the copyright holders give
  *  permission to link the code of this program with any edition of
@@ -34,10 +34,10 @@
 
 #include "mailserviceimpl.h"
 
-#include "kmcomposewin.h"
+#include "composer.h"
+
 #include "kmmessage.h"
 #include "kmmsgpart.h"
-#include "kmkernel.h"
 
 #include <dcopobject.h>
 #include <kurl.h>
@@ -72,7 +72,7 @@ bool MailServiceImpl::sendMessage( const QString& from, const QString& to,
   if ( !subject.isEmpty() ) msg->setSubject( subject );
   if ( !body.isEmpty() )    msg->setBody( body.utf8() );
 
-  KMComposeWin *cWin = new KMComposeWin( msg );
+  KMail::Composer * cWin = KMail::makeComposer( msg );
   cWin->setCharset("", TRUE);
 
   for( KURL::List::ConstIterator itr = attachments.begin();
@@ -120,7 +120,7 @@ bool MailServiceImpl::sendMessage( const QString& from, const QString& to,
   part->setBodyEncodedBinary( attachment );
   msg->addBodyPart( part );
 
-  KMComposeWin *cWin = new KMComposeWin( msg );
+  KMail::Composer * cWin = KMail::makeComposer( msg );
   cWin->setCharset("", TRUE);
   return true;
 }
