@@ -27,7 +27,6 @@
 
 #include <klistview.h>
 #include <klocale.h>
-#include <kabc/stdaddressbook.h>
 #include <kabc/resource.h>
 #include <kiconloader.h>
 #include <kdialog.h>
@@ -324,6 +323,13 @@ RecipientsPicker::RecipientsPicker( QWidget *parent )
   buttonLayout->addWidget( closeButton );
   connect( closeButton, SIGNAL( clicked() ), SLOT( close() ) );
 
+  {
+    using namespace KABC;
+    mAddressBook = KABC::StdAddressBook::self( true );
+    connect( mAddressBook, SIGNAL( addressBookChanged( AddressBook * ) ),
+             this, SLOT( insertAddressBook( AddressBook * ) ) );
+  }
+
   initCollections();
 
   mCollectionCombo->setCurrentItem( 0 );
@@ -355,8 +361,21 @@ RecipientsPicker::~RecipientsPicker()
 
 void RecipientsPicker::initCollections()
 {
-  KABC::StdAddressBook *addressbook = KABC::StdAddressBook::self( true );
+  mAllRecipients = new RecipientsCollection;
+  mAllRecipients->setTitle( i18n("All") );
+  insertCollection( mAllRecipients );
 
+  insertDistributionLists();
+
+  insertRecentAddresses();
+
+  mSelectedRecipients = new RecipientsCollection;
+  mSelectedRecipients->setTitle( i18n("Selected Recipients") );
+  insertCollection( mSelectedRecipients );
+}
+
+void RecipientsPicker::insertAddressBook( KABC::AddressBook *addressbook )
+{
   QMap<KABC::Resource *,RecipientsCollection *> collectionMap;
 
   QPtrList<KABC::Resource> resources = addressbook->resources();
@@ -369,9 +388,6 @@ void RecipientsPicker::initCollections()
 
   QMap<QString,RecipientsCollection *> categoryMap;
 
-  mAllRecipients = new RecipientsCollection;
-  mAllRecipients->setTitle( i18n("All") );
-
   KABC::AddressBook::Iterator it;
   for( it = addressbook->begin(); it != addressbook->end(); ++it ) {
     QStringList emails = (*it).emails();
@@ -405,8 +421,6 @@ void RecipientsPicker::initCollections()
     }
   }
 
-  insertCollection( mAllRecipients );
-
   QMap<KABC::Resource *,RecipientsCollection *>::ConstIterator it2;
   for( it2 = collectionMap.begin(); it2 != collectionMap.end(); ++it2 ) {
     insertCollection( *it2 );
@@ -416,14 +430,8 @@ void RecipientsPicker::initCollections()
   for( it3 = categoryMap.begin(); it3 != categoryMap.end(); ++it3 ) {
     insertCollection( *it3 );
   }
-
-  insertDistributionLists();
-
-  insertRecentAddresses();
-
-  mSelectedRecipients = new RecipientsCollection;
-  mSelectedRecipients->setTitle( i18n("Selected Recipients") );
-  insertCollection( mSelectedRecipients );
+  
+  updateList();
 }
 
 void RecipientsPicker::insertDistributionLists()
