@@ -1,7 +1,7 @@
 /*******************************************************************
  KNotes -- Notes for the KDE project
 
- Copyright (c) 1997-2004, The KNotes Developers
+ Copyright (c) 1997-2006, The KNotes Developers
 
  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
@@ -15,10 +15,11 @@
 
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
- Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+ Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *******************************************************************/
 
 #include <qlabel.h>
+#include <qdrawutil.h>
 #include <qsize.h>
 #include <qsizegrip.h>
 #include <qbitmap.h>
@@ -66,6 +67,8 @@
 #include "knotesnetsend.h"
 #include "version.h"
 
+#include "pushpin.xpm"
+
 #include <kwin.h>
 #include <netwm.h>
 
@@ -75,15 +78,14 @@ using namespace KCal;
 
 extern Time qt_x_time;
 
+int KNote::s_ppOffset = 0;
 
 KNote::KNote( QDomDocument buildDoc, Journal *j, QWidget *parent, const char *name )
   : QFrame( parent, name, WStyle_Customize | WStyle_NoBorder | WDestructiveClose ),
-    m_label( 0 ), m_button( 0 ), m_tool( 0 ), m_editor( 0 ), m_config( 0 ),
-    m_journal( j ), m_find( 0 ), m_kwinConf( KSharedConfig::openConfig( "kwinrc", true ) )
+    m_label( 0 ), m_pushpin( 0 ), m_fold( 0 ), m_button( 0 ), m_tool( 0 ), m_editor( 0 ),
+    m_config( 0 ), m_journal( j ), m_find( 0 ),
+    m_kwinConf( KSharedConfig::openConfig( "kwinrc", true ) )
 {
-    // be explicit
-    KWin::setIcons( winId(), kapp->icon(), kapp->miniIcon() );
-
     setAcceptDrops( true );
     actionCollection()->setWidget( this );
 
@@ -146,6 +148,8 @@ KNote::KNote( QDomDocument buildDoc, Journal *j, QWidget *parent, const char *na
 
     // create the note header, button and label...
     m_label = new QLabel( this );
+    m_label->setFrameStyle( NoFrame );
+    m_label->setLineWidth( 0 );
     m_label->installEventFilter( this );  // receive events (for dragging & action menu)
     setName( m_journal->summary() );      // don't worry, no signals are connected at this stage yet
 
@@ -156,6 +160,7 @@ KNote::KNote( QDomDocument buildDoc, Journal *j, QWidget *parent, const char *na
     m_editor = new KNoteEdit( actionCollection(), this );
     m_editor->installEventFilter( this ); // receive events (for modified)
     m_editor->viewport()->installEventFilter( this );
+    connect( m_editor, SIGNAL(contentsMoving( int, int )), this, SLOT(slotUpdateViewport( int, int )));
 
     KXMLGUIBuilder builder( this );
     KXMLGUIFactory factory( &builder, this );
@@ -166,6 +171,7 @@ KNote::KNote( QDomDocument buildDoc, Journal *j, QWidget *parent, const char *na
     m_tool = static_cast<KToolBar*>(factory.container( "note_tool", this ));
     m_tool->setIconSize( 10 );
     m_tool->setFixedHeight( 16 );
+    m_tool->setIconText( KToolBar::IconOnly );
 
     // if there was just a way of making KComboBox adhere the toolbar height...
     QObjectList *list = m_tool->queryList( "KComboBox" );
@@ -200,15 +206,7 @@ KNote::KNote( QDomDocument buildDoc, Journal *j, QWidget *parent, const char *na
     p.drawPolygon( array );
     p.end();
     m_editor->cornerWidget()->setMask( mask );
-
-    // set up the look&feel of the note
-    setMinimumSize( 20, 20 );
-    setFrameStyle( WinPanel | Raised );
-    setLineWidth( 1 );
-
-    m_editor->setMargin( 5 );
-    m_editor->setFrameStyle( NoFrame );
-    m_editor->setBackgroundMode( PaletteBase );
+    m_editor->cornerWidget()->setBackgroundMode( PaletteBase );
 
     // the config file location
     QString configFile = KGlobal::dirs()->saveLocation( "appdata", "notes/" );
@@ -248,21 +246,44 @@ KNote::KNote( QDomDocument buildDoc, Journal *j, QWidget *parent, const char *na
         m_config->writeConfig();
     }
 
+    // set up the look&feel of the note
+    setMinimumSize( 20, 20 );
+    setLineWidth( 1 );
+    setMargin( 0 );
+
+    m_editor->setMargin( 0 );
+    m_editor->setFrameStyle( NoFrame );
+    m_editor->setBackgroundOrigin( WindowOrigin );
+
+    // can be done here since this doesn't pick up changes while KNotes is running anyway
+    bool closeLeft = false;
+    m_kwinConf->setGroup( "Style" );
+    if ( m_kwinConf->readBoolEntry( "CustomButtonPositions" ) )
+        closeLeft = m_kwinConf->readEntry( "ButtonsOnLeft" ).find( 'X' ) > -1;
+
+    QPixmap pushpin_pix;
+    if ( closeLeft )
+        pushpin_pix = QPixmap( QPixmap( pushpin_xpm ).convertToImage().mirror( true, false ) );
+    else
+        pushpin_pix = QPixmap( pushpin_xpm );
+
+    // the pushpin label at the top left or right corner
+    m_pushpin = new QLabel( this );
+    m_pushpin->setScaledContents( true );
+    m_pushpin->setBackgroundMode( NoBackground );
+    m_pushpin->setPixmap( pushpin_pix );
+    m_pushpin->resize( pushpin_pix.size() );
+
+    // fold label at bottom right corner
+    m_fold = new QLabel( this );
+    m_fold->setScaledContents( true );
+    m_fold->setBackgroundMode( NoBackground );
+
     // load the display configuration of the note
     width = m_config->width();
     height = m_config->height();
     resize( width, height );
 
-    if ( m_config->keepAbove() )
-        m_keepAbove->setChecked( true );
-    else if ( m_config->keepBelow() )
-        m_keepBelow->setChecked( true );
-    else
-    {
-        m_keepAbove->setChecked( false );
-        m_keepBelow->setChecked( false );
-    }
-
     // let KWin do the placement if the position is illegal--at least 10 pixels
     // of a note need to be visible
     const QPoint& position = m_config->position();
@@ -293,6 +314,10 @@ KNote::KNote( QDomDocument buildDoc, Journal *j, QWidget *parent, const char *na
     // read configuration settings...
     slotApplyConfig();
 
+    // create the mask for the fold---to be done after slotApplyConfig(),
+    // which calls createFold()
+    m_fold->setMask( QRegion( m_fold->pixmap()->createHeuristicMask() ) );
+
     // if this is a new note put on current desktop - we can't use defaults
     // in KConfig XT since only _changes_ will be stored in the config file
     int desktop = m_config->desktop();
@@ -316,6 +341,23 @@ KNote::KNote( QDomDocument buildDoc, Journal *j, QWidget *parent, const char *na
 
     m_readOnly->setChecked( m_config->readOnly() );
     slotUpdateReadOnly();
+
+    if ( m_config->keepAbove() )
+        m_keepAbove->setChecked( true );
+    else if ( m_config->keepBelow() )
+        m_keepBelow->setChecked( true );
+    else
+    {
+        m_keepAbove->setChecked( false );
+        m_keepBelow->setChecked( false );
+    }
+    slotUpdateKeepAboveBelow();
+
+    // HACK: update the icon color - again after showing the note, to make kicker aware of the new colors
+    KIconEffect effect;
+    QPixmap icon = effect.apply( kapp->icon(), KIconEffect::Colorize, 1, m_config->bgColor(), false );
+    QPixmap miniIcon = effect.apply( kapp->miniIcon(), KIconEffect::Colorize, 1, m_config->bgColor(), false );
+    KWin::setIcons( winId(), icon, miniIcon );
 }
 
 KNote::~KNote()
@@ -329,10 +371,11 @@ KNote::~KNote()
 void KNote::slotKill( bool force )
 {
     if ( !force &&
-         KMessageBox::warningContinueCancel( this,
-            i18n("<qt>Do you really want to delete note <b>%1</b>?</qt>")
-                .arg( m_label->text() ),
-            i18n("Confirm Delete"), KGuiItem( i18n("&Delete"), "editdelete") )
+         KMessageBox::warningContinueCancel( this, 
+             i18n("<qt>Do you really want to delete note <b>%1</b>?</qt>").arg( m_label->text() ),
+             i18n("Confirm Delete"), KGuiItem( i18n("&Delete"), "editdelete" ),
+             "ConfirmDeleteNote"
+         )
          != KMessageBox::Continue )
     {
         return;
@@ -547,6 +590,14 @@ bool KNote::isModified( const QString& app ) const
         return true;
 }
 
+void KNote::setStyle( int style )
+{
+    if ( style == KNotesGlobalConfig::EnumStyle::Plain )
+        s_ppOffset = 0;
+    else
+        s_ppOffset = 12;
+}
+
 
 // ------------------ private slots (menu actions) ------------------ //
 
@@ -574,9 +625,11 @@ void KNote::slotUpdateReadOnly()
     actionCollection()->action( "insert_date" )->setEnabled( !readOnly );
     actionCollection()->action( "delete_note" )->setEnabled( !readOnly );
 
-    // TODO: replace the menu
-    actionCollection()->action( "edit_clear" )->setEnabled( !readOnly );
+    actionCollection()->action( "edit_undo" )->setEnabled( !readOnly && m_editor->isUndoAvailable() );
+    actionCollection()->action( "edit_redo" )->setEnabled( !readOnly && m_editor->isRedoAvailable() );
+    actionCollection()->action( "edit_cut" )->setEnabled( !readOnly && m_editor->hasSelectedText() );
     actionCollection()->action( "edit_paste" )->setEnabled( !readOnly );
+    actionCollection()->action( "edit_clear" )->setEnabled( !readOnly );
 
     updateFocus();
 }
@@ -589,6 +642,7 @@ void KNote::slotClose()
 
     m_editor->clearFocus();
     m_config->setHideNote( true );
+    m_config->setPosition( pos() );
 
     // just hide the note so it's still available from the dock window
     hide();
@@ -746,14 +800,23 @@ void KNote::slotSaveAs()
         return;
 
     QFile file( fileName );
+
+    if ( file.exists() &&
+         KMessageBox::warningContinueCancel( this, i18n("<qt>A file named <b>%1</b> already exists.<br>"
+                           "Are you sure you want to overwrite it?</qt>").arg( QFileInfo(file).fileName() ) )
+         != KMessageBox::Continue )
+    {
+        return;
+    }
+
     if ( file.open( IO_WriteOnly ) )
     {
         QTextStream stream( &file );
         // convert rich text to plain text first
         if ( convert && convert->isChecked() )
-            stream << text();
-        else
             stream << plainText();
+        else
+            stream << text();
     }
 }
 
@@ -843,6 +906,11 @@ void KNote::slotUpdateDesktopActions()
         m_toDesktop->setCurrentItem( wm_client.desktop() + 1 ); // compensate for separator (+1)
 }
 
+void KNote::slotUpdateViewport( int /*x*/, int y )
+{
+    if ( s_ppOffset )
+        updateBackground( y );
+}
 
 // -------------------- private methods -------------------- //
 
@@ -865,17 +933,27 @@ void KNote::setColor( const QColor &fg, const QColor &bg )
     newpalette.setColor( QColorGroup::Base,       bg ); // text background
     newpalette.setColor( QColorGroup::Text,       fg ); // text color
     newpalette.setColor( QColorGroup::Button,     bg );
+    newpalette.setColor( QColorGroup::ButtonText, fg );
+    
+//    newpalette.setColor( QColorGroup::Highlight,  bg );
+//    newpalette.setColor( QColorGroup::HighlightedText, fg );
 
     // the shadow
-    newpalette.setColor( QColorGroup::Midlight, bg.light(110) );
-    newpalette.setColor( QColorGroup::Shadow, bg.dark(116) );  // 132 ?
+    newpalette.setColor( QColorGroup::Midlight, bg.light(150) );
+    newpalette.setColor( QColorGroup::Shadow, bg.dark(116) );
     newpalette.setColor( QColorGroup::Light, bg.light(180) );
-    newpalette.setColor( QColorGroup::Dark, bg.dark(108) );
+    if ( s_ppOffset )
+        newpalette.setColor( QColorGroup::Dark, bg.dark(200) );
+    else
+        newpalette.setColor( QColorGroup::Dark, bg.dark(108) );
     setPalette( newpalette );
 
     // set the text color
     m_editor->setTextColor( fg );
 
+    // set the background color or gradient
+    updateBackground();
+
     // set darker value for the hide button...
     QPalette darker = palette();
     darker.setColor( QColorGroup::Button, bg.dark(116) );
@@ -894,11 +972,27 @@ void KNote::setColor( const QColor &fg, const QColor &bg )
 
     m_editor->setSelectionAttributes( 1, sel, true );
 
-    // to set the color of the title
+    // update the color of the fold
+    createFold();
+
+    // update the color of the title
     updateFocus();
     emit sigColorChanged();
 }
 
+void KNote::createFold()
+{
+    QPixmap fold( 15, 15 );
+    QPainter foldp( &fold );
+    foldp.setPen( Qt::NoPen );
+    foldp.setBrush( palette().active().dark() );
+    QPointArray foldpoints( 3 );
+    foldpoints.putPoints( 0, 3, 0, 0, 14, 0, 0, 14 );
+    foldp.drawPolygon( foldpoints );
+    foldp.end();
+    m_fold->setPixmap( fold );
+}
+
 void KNote::updateLabelAlignment()
 {
     // if the name is too long to fit, left-align it, otherwise center it (#59028)
@@ -931,10 +1025,11 @@ void KNote::updateFocus()
             setGeometry( x(), y(), width(), height() - m_tool->height() );
             updateLayout();     // to update the minimum height
         }
+
+        m_fold->hide();
     }
     else
     {
-        m_label->setBackgroundColor( palette().active().background() );
         m_button->hide();
         m_editor->cornerWidget()->hide();
 
@@ -944,60 +1039,172 @@ void KNote::updateFocus()
             setGeometry( x(), y(), width(), height() - m_tool->height() );
             updateLayout();     // to update the minimum height
         }
+
+        if ( s_ppOffset )
+        {
+            m_label->setBackgroundColor( palette().active().midlight() );
+            m_fold->show();
+        }
+        else
+            m_label->setBackgroundColor( palette().active().background() );
     }
 }
 
+void KNote::updateMask()
+{
+    if ( !s_ppOffset )
+    {
+        clearMask();
+        return;
+    }
+
+    int w = width();
+    int h = height();
+    QRegion reg( 0, s_ppOffset, w, h - s_ppOffset );
+
+    const QBitmap *pushpin_bitmap = m_pushpin->pixmap()->mask();
+    QRegion pushpin_reg( *pushpin_bitmap );
+    m_pushpin->setMask( pushpin_reg );
+    pushpin_reg.translate( m_pushpin->x(), m_pushpin->y() );
+
+    if ( !hasFocus() )
+    {
+        QPointArray foldpoints( 3 );
+        foldpoints.putPoints( 0, 3, w-15, h, w, h-15, w, h );
+        QRegion fold( foldpoints, false );
+        setMask( reg.unite( pushpin_reg ).subtract( fold ) );
+    }
+    else
+        setMask( reg.unite( pushpin_reg ) );
+}
+
+void KNote::updateBackground( int y_offset )
+{
+    if ( !s_ppOffset )
+    {
+        m_editor->setPaper( QBrush( colorGroup().background() ) );
+        return;
+    }
+
+    int w = m_editor->visibleWidth();
+    int h = m_editor->visibleHeight();
+
+    // in case y_offset is not set, calculate y_offset as the content
+    // y-coordinate of the top-left point of the viewport - which is essentially
+    // the vertical scroll amount
+    if ( y_offset == -1 )
+        y_offset = m_editor->contentsY();
+
+    y_offset = y_offset % h;
+
+    QImage grad_img( w, h, 32 );
+    QRgb rgbcol;
+    QColor bg = palette().active().background();
+
+    for ( int i = 0; i < h; ++i )
+    {
+        // if the scrollbar has moved, then adjust the gradient by the amount the
+        // scrollbar moved -- so that the background gradient looks ok when tiled
+
+        // the lightness is calculated as follows:
+        // if i >= y, then lightness = 150 - (i-y)*75/h;
+        // if i < y, then lightness = 150 - (i+h-y)*75/h
+
+        int i_1 = 150 - 75 * ((i - y_offset + h) % h) / h;
+        rgbcol = bg.light( i_1 ).rgb();
+        for ( int j = 0; j < w; ++j )
+            grad_img.setPixel( j, i, rgbcol );
+    }
+
+    // setPaletteBackgroundPixmap makes QTextEdit::color() stop working!!
+    m_editor->setPaper( QBrush( Qt::black, QPixmap( grad_img ) ) );
+}
+
 void KNote::updateLayout()
 {
-    // DAMN, Qt still has no support for widgets with a fixed aspect ratio :-(
-    // So we have to write our own layout manager...
-
     const int headerHeight = m_label->sizeHint().height();
     const int margin = m_editor->margin();
-    static const int border = 2;
     bool closeLeft = false;
 
     m_kwinConf->setGroup( "Style" );
     if ( m_kwinConf->readBoolEntry( "CustomButtonPositions" ) )
         closeLeft = m_kwinConf->readEntry( "ButtonsOnLeft" ).find( 'X' ) > -1;
 
+    if ( s_ppOffset )
+    {
+        if ( !m_editor->paper().pixmap() )  // just changed the style
+            setColor( palette().active().foreground(), palette().active().background() );
+
+        m_pushpin->show();
+        setFrameStyle( Panel | Raised );
+
+        if ( closeLeft )
+            m_pushpin->move( width() - m_pushpin->width(), 0 );
+        else
+            m_pushpin->move( 0, 0 );
+    }
+    else
+    {
+        if ( m_editor->paper().pixmap() )  // just changed the style
+            setColor( palette().active().foreground(), palette().active().background() );
+
+        setFrameStyle( WinPanel | Raised );
+        m_pushpin->hide();
+        m_fold->hide();
+    }
+
     m_button->setGeometry(
-        closeLeft ? frameRect().x() + border
-                  : frameRect().width() - headerHeight - border,
-        frameRect().y() + border,
+        closeLeft ? contentsRect().x() : contentsRect().width() - headerHeight,
+        contentsRect().y() + s_ppOffset,
         headerHeight,
         headerHeight
     );
 
     m_label->setGeometry(
-        frameRect().x() + border, frameRect().y() + border,
-        frameRect().width() - border*2, headerHeight
+        contentsRect().x(), contentsRect().y() + s_ppOffset,
+        contentsRect().width(), headerHeight
     );
 
-    m_editor->setGeometry(
-        contentsRect().x(),
-        contentsRect().y() + headerHeight + border,
-        contentsRect().width(),
-        contentsRect().height() - headerHeight -
-                (m_tool->isHidden() ? 0 : m_tool->height()) - border*2
-    );
+    m_editor->setGeometry( QRect(
+        QPoint( contentsRect().x(),
+                contentsRect().y() + headerHeight + s_ppOffset ),
+        QPoint( contentsRect().right(),
+                contentsRect().bottom() - (m_tool->isHidden() ? 0 : m_tool->height()) )
+    ) );
 
     m_tool->setGeometry(
-        contentsRect().x(), contentsRect().height() - m_tool->height(),
-        contentsRect().width(), m_tool->height()
+        contentsRect().x(),
+        contentsRect().bottom() - m_tool->height() + 1,
+        contentsRect().width(),
+        m_tool->height()
     );
 
+    if ( s_ppOffset )
+        m_fold->move( width() - 15, height() - 15 );
+
     setMinimumSize(
-        m_editor->cornerWidget()->width() + margin*2 + border*2,
-        headerHeight + (m_tool->isHidden() ? 0 : m_tool->height()) +
-                m_editor->cornerWidget()->height() + margin*2 + border*2
+        m_editor->cornerWidget()->width() + margin*2,
+        headerHeight + s_ppOffset + (m_tool->isHidden() ? 0 : m_tool->height()) +
+                m_editor->cornerWidget()->height() + margin*2
     );
 
     updateLabelAlignment();
+    updateMask();
+    updateBackground();
 }
 
 // -------------------- protected methods -------------------- //
 
+void KNote::drawFrame( QPainter *p )
+{
+    QRect r = frameRect();
+    r.setTop( s_ppOffset );
+    if ( s_ppOffset )
+        qDrawShadePanel( p, r, colorGroup(), false, lineWidth() );
+    else
+        qDrawWinPanel( p, r, colorGroup(), false );
+}
+
 void KNote::showEvent( QShowEvent * )
 {
     if ( m_config->hideNote() )
@@ -1006,6 +1213,7 @@ void KNote::showEvent( QShowEvent * )
         slotUpdateKeepAboveBelow();
         slotUpdateShowInTaskbar();
         toDesktop( m_config->desktop() );
+        move( m_config->position() );
         m_config->setHideNote( false );
     }
 }
@@ -1023,11 +1231,15 @@ void KNote::closeEvent( QCloseEvent * )
 
 void KNote::dragEnterEvent( QDragEnterEvent *e )
 {
-    e->accept( KColorDrag::canDecode( e ) );
+    if ( !m_config->readOnly() )
+        e->accept( KColorDrag::canDecode( e ) );
 }
 
 void KNote::dropEvent( QDropEvent *e )
 {
+    if ( m_config->readOnly() )
+        return;
+
     QColor bg;
     if ( KColorDrag::decode( e, bg ) )
     {
@@ -1088,6 +1300,15 @@ bool KNote::eventFilter( QObject *o, QEvent *ev )
             return true;
         }
 
+#if KDE_IS_VERSION( 3, 5, 1 )
+        if ( ev->type() == QEvent::MouseButtonRelease )
+        {
+            NETRootInfo wm_root( qt_xdisplay(), NET::WMMoveResize );
+            wm_root.moveResizeRequest( winId(), e->globalX(), e->globalY(), NET::MoveResizeCancel );
+            return false;
+        }
+#endif
+
         if ( m_menu && ( ev->type() == QEvent::MouseButtonPress )
             && ( e->button() == RightButton ) )
         {
