@@ -17,14 +17,19 @@
  *                                                                             *
  * You should have received a copy of the GNU General Public License           *
  * along with this program; if not, write to the Free Software                 *
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA. *
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA. *
  *                                                                             *
  * As a special exception, permission is given to link this program            *
  * with any edition of Qt, and distribute the resulting executable,            *
  * without including the source code for Qt in the source distribution.        *
  *                                                                             *
  ******************************************************************************/
-
+/**
+ * @file konsolekalendar.cpp
+ * Provides the KonsoleKalendar class definition.
+ * @author Tuukka Pasanen
+ * @author Allen Winter
+ */
 #include <stdio.h>
 #include <stdlib.h>
 #include <iostream>
@@ -35,8 +40,6 @@
 
 #include <kdebug.h>
 #include <klocale.h>
-#include <kstandarddirs.h>
-#include <kmacroexpander.h>
 
 #include <libkcal/calendarlocal.h>
 #include <libkcal/resourcecalendar.h>
@@ -44,6 +47,7 @@
 #include <libkcal/calendar.h>
 #include <libkcal/event.h>
 #include <libkcal/htmlexport.h>
+#include <libkdepim/kpimprefs.h>
 
 #include "konsolekalendar.h"
 #include "konsolekalendaradd.h"
@@ -54,10 +58,9 @@
 using namespace KCal;
 using namespace std;
 
-KonsoleKalendar::KonsoleKalendar(KonsoleKalendarVariables *variables)
+KonsoleKalendar::KonsoleKalendar( KonsoleKalendarVariables *variables )
 {
   m_variables = variables;
-  // m_Calendar =  new ResourceCalendar;
 }
 
 KonsoleKalendar::~KonsoleKalendar()
@@ -76,10 +79,10 @@ bool KonsoleKalendar::importCalendar()
 bool KonsoleKalendar::createCalendar()
 {
   bool status = false;
-  CalendarLocal newCalendar;
+  CalendarLocal newCalendar( KPimPrefs::timezone() );
 
   if ( m_variables->isDryRun() ) {
-    cout << i18n("Create Calendar <Dry Run>: %1").
+    cout << i18n( "Create Calendar <Dry Run>: %1" ).
       arg( m_variables->getCalendarFile() ).local8Bit()
          << endl;
   } else {
@@ -89,7 +92,7 @@ bool KonsoleKalendar::createCalendar()
               << endl;
 
     if ( m_variables->isVerbose() ) {
-      cout << i18n("Create Calendar <Verbose>: %1").
+      cout << i18n( "Create Calendar <Verbose>: %1" ).
         arg( m_variables->getCalendarFile() ).local8Bit()
            << endl;
     }
@@ -107,11 +110,10 @@ bool KonsoleKalendar::showInstance()
   bool status = true;
   QFile f;
   QString title;
-  Event::List *eventList;
   Event *event;
 
   if ( m_variables->isDryRun() ) {
-    cout << i18n("View Events <Dry Run>:").local8Bit()
+    cout << i18n( "View Events <Dry Run>:" ).local8Bit()
          << endl;
     printSpecs();
   } else {
@@ -120,7 +122,6 @@ bool KonsoleKalendar::showInstance()
               << "open export file"
               << endl;
 
-
     if ( m_variables->isExportFile() ) {
       f.setName( m_variables->getExportFile() );
       if ( !f.open( IO_WriteOnly ) ) {
@@ -140,7 +141,7 @@ bool KonsoleKalendar::showInstance()
                 << endl;
 
       if ( m_variables->isVerbose() ) {
-	cout << i18n("View Event <Verbose>:").local8Bit()
+	cout << i18n( "View Event <Verbose>:" ).local8Bit()
              << endl;
 	printSpecs();
       }
@@ -157,15 +158,20 @@ bool KonsoleKalendar::showInstance()
 
 	  Event::List sortedList =
             m_variables->getCalendar()->events( EventSortStartDate );
-
-          QDate dt, firstdate, lastdate;
-          firstdate = sortedList.first()->dtStart().date();
-          lastdate = sortedList.last()->dtStart().date();
-          for ( dt=firstdate;
-                dt<=lastdate && status != false;
-                dt=dt.addDays(1) ) {
-            Event::List events = m_variables->getCalendar()->events( dt, true );
-            status = printEventList( &ts, &events, dt );
+          if( sortedList.count() > 0 )
+          {
+            QDate dt, firstdate, lastdate;
+            firstdate = sortedList.first()->dtStart().date();
+            lastdate = sortedList.last()->dtStart().date();
+            for ( dt = firstdate;
+                  dt <= lastdate && status != false;
+                  dt = dt.addDays(1) ) {
+              Event::List events =
+                m_variables->getCalendar()->events( dt,
+                                                    EventSortStartDate,
+                                                    SortDirectionAscending );
+              status = printEventList( &ts, &events, dt );
+            }
           }
 
 	} else if ( m_variables->isUID() ) {
@@ -187,10 +193,13 @@ bool KonsoleKalendar::showInstance()
           datetime = datetime.addDays( 720 );
 
 	  QDate dt;
-	  for ( dt=m_variables->getStartDateTime().date();
-                dt<=datetime.date() && status != false;
-                dt=dt.addDays(1) ) {
-	    Event::List events = m_variables->getCalendar()->events( dt, true );
+	  for ( dt = m_variables->getStartDateTime().date();
+                dt <= datetime.date() && status != false;
+                dt = dt.addDays(1) ) {
+	    Event::List events =
+              m_variables->getCalendar()->events( dt,
+                                                  EventSortStartDate,
+                                                  SortDirectionAscending );
 	    status = printEventList( &ts, &events, dt );
 
 	    // when we get next event we exit..
@@ -207,26 +216,30 @@ bool KonsoleKalendar::showInstance()
                     << endl;
 
 	  QDate dt;
-	  for ( dt=m_variables->getStartDateTime().date();
-                dt<=m_variables->getEndDateTime().date() && status != false;
-                dt=dt.addDays(1) ) {
-	    Event::List events = m_variables->getCalendar()->events( dt, true );
+	  for ( dt = m_variables->getStartDateTime().date();
+                dt <= m_variables->getEndDateTime().date() && status != false;
+                dt = dt.addDays(1) ) {
+	    Event::List events =
+              m_variables->getCalendar()->events( dt,
+                                                  EventSortStartDate,
+                                                  SortDirectionAscending );
 	    status = printEventList( &ts, &events, dt );
 	  }
 	}
       } else {
 	QDate firstdate, lastdate;
 	if ( m_variables->getAll() ) {
-          // TODO: this is broken since the date on last() may not be last date
-          // (this is the case for me)
 	  kdDebug() << "konsolekalendar.cpp::showInstance() | "
                     << "HTML view all events sorted list"
                     << endl;
-          eventList =
-            new Event::List ( m_variables->getCalendar()->rawEvents() );
-	  firstdate = eventList->first()->dtStart().date();
-	  lastdate = eventList->last()->dtStart().date();
-          delete eventList;
+          // sort the events for this date by start date
+          // in order to determine the date range.
+          Event::List *events =
+            new Event::List ( m_variables->getCalendar()->rawEvents(
+                                EventSortStartDate,
+                                SortDirectionAscending ) );
+	  firstdate = events->first()->dtStart().date();
+	  lastdate = events->last()->dtStart().date();
 	} else if ( m_variables->isUID() ) {
 	  // TODO
 	  kdDebug() << "konsolekalendar.cpp::showInstance() | "
@@ -284,11 +297,7 @@ bool KonsoleKalendar::showInstance()
         htmlSettings.setDateEnd( QDateTime( lastdate ) ) ;
 
         KCal::HtmlExport *Export;
-        if ( !m_variables->isCalendarResources() ) {
-          Export = new HtmlExport( m_variables->getCalendar(), &htmlSettings );
-        } else {
-          Export = new HtmlExport( m_variables->getCalendarResources(), &htmlSettings );
-        }
+        Export = new HtmlExport( m_variables->getCalendar(), &htmlSettings );
 	status = Export->save( &ts );
         delete Export;
       }
@@ -301,7 +310,6 @@ bool KonsoleKalendar::showInstance()
 bool KonsoleKalendar::printEventList( QTextStream *ts,
                                       Event::List *eventList, QDate date )
 {
-
   bool status = true;
 
   if ( eventList->count() ) {
@@ -322,50 +330,11 @@ bool KonsoleKalendar::printEventList( QTextStream *ts,
 
 bool KonsoleKalendar::printEvent( QTextStream *ts, Event *event, QDate dt )
 {
-  /* This has not been finished
-  bool status = false;
-  QString output;
-  QString floating = "location %location, summary %summary\r\n";
-  QString sameday = "time is %starttime, location %location, summary %summary\r\n";
-  QString newday = "date is %startdate, time is %starttime, location %location, summary %summary\r\n";
-
-  QMap<QString, QString> macros;
-
-
-  macros["startdate"] = KGlobal::locale()->formatDate( event->dtStart().date() );
-  macros["enddate"] = KGlobal::locale()->formatDate( event->dtEnd().date() );
-
-  if ( !event->doesFloat() ) {
-   macros["starttime"] = KGlobal::locale()->formatTime( event->dtStart().time() );
-   macros["endtime"] = KGlobal::locale()->formatTime( event->dtEnd().time() );
-  } // if
-
-  macros["summary"] = event->summary();
-  macros["description"] = event->description();
-  macros["location"] = event->location();
-  macros["UID"] = event->uid();
-
-  if ( event->doesFloat() ) {
-        output = KMacroExpander::expandMacros( floating, macros, '%' );
-  } else {
-      if ( dt.daysTo( m_saveDate ) ) {
-        output = KMacroExpander::expandMacros( newday, macros, '%' );
-      } else {
-        output = KMacroExpander::expandMacros( sameday, macros, '%' );
-      }
-  }
-
-  *ts << output ;
-
-  return( status );
-  */
-
   bool status = false;
   bool sameDay = true;
   KonsoleKalendarExports exports;
 
-  if ( event )
-  {
+  if ( event ) {
     switch ( m_variables->getExportType() ) {
 
     case ExportTypeCSV:
@@ -450,10 +419,12 @@ bool KonsoleKalendar::isEvent( QDateTime startdate,
   bool found = false;
 
   Event::List eventList( m_variables->getCalendar()->
-                         rawEventsForDate( startdate.date(), true ) );
-  for ( it =  eventList.begin(); it != eventList.end(); ++it ) {
+                         rawEventsForDate( startdate.date(),
+                                           EventSortStartDate,
+                                           SortDirectionAscending ) );
+  for ( it = eventList.begin(); it != eventList.end(); ++it ) {
     event = *it;
-    if ( event->dtEnd()==enddate && event->summary()==summary ) {
+    if ( event->dtEnd() == enddate && event->summary() == summary ) {
       found = true;
       break;
     }
@@ -463,28 +434,28 @@ bool KonsoleKalendar::isEvent( QDateTime startdate,
 
 void KonsoleKalendar::printSpecs()
 {
-  cout << i18n("  What:  %1").
+  cout << i18n( "  What:  %1" ).
     arg( m_variables->getSummary() ).local8Bit()
        << endl;
 
-  cout << i18n("  Begin: %1").
-    arg( m_variables->getStartDateTime().toString(Qt::TextDate) ).local8Bit()
+  cout << i18n( "  Begin: %1" ).
+    arg( m_variables->getStartDateTime().toString( Qt::TextDate ) ).local8Bit()
        << endl;
 
-  cout << i18n("  End:   %1").
-    arg( m_variables->getEndDateTime().toString(Qt::TextDate) ).local8Bit()
+  cout << i18n( "  End:   %1" ).
+    arg( m_variables->getEndDateTime().toString( Qt::TextDate ) ).local8Bit()
        << endl;
 
   if ( m_variables->getFloating() == true ) {
-    cout << i18n("  No Time Associated with Event").local8Bit()
+    cout << i18n( "  No Time Associated with Event" ).local8Bit()
          << endl;
   }
 
-  cout << i18n("  Desc:  %1").
+  cout << i18n( "  Desc:  %1" ).
     arg( m_variables->getDescription() ).local8Bit()
        << endl;
 
-  cout << i18n("  Location:  %1").
+  cout << i18n( "  Location:  %1" ).
     arg( m_variables->getLocation() ).local8Bit()
        << endl;
 }
