@@ -1,7 +1,7 @@
 /*
    This file is part of the KDE project
    Copyright (C) 2002-2003 Daniel Molkentin <molkentin@kde.org>
-   Copyright (C) 2004 Michael Brade <brade@kde.org>
+   Copyright (C) 2004-2006 Michael Brade <brade@kde.org>
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public
@@ -15,8 +15,8 @@
 
    You should have received a copy of the GNU General Public License
    along with this program; see the file COPYING.  If not, write to
-   the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-   Boston, MA 02111-1307, USA.
+   the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+   Boston, MA 02110-1301, USA.
 */
 
 #include <qpopupmenu.h>
@@ -63,6 +63,8 @@ KNotesPart::KNotesPart( QObject *parent, const char *name )
   mNotesView->setSelectionMode( QIconView::Extended );
   mNotesView->setItemsMovable( false );
   mNotesView->setResizeMode( QIconView::Adjust );
+  mNotesView->setAutoArrange( true );
+  mNotesView->setSorting( true );
 
   connect( mNotesView, SIGNAL( executed( QIconViewItem* ) ),
            this, SLOT( editNote( QIconViewItem* ) ) );
@@ -130,7 +132,7 @@ QString KNotesPart::newNote( const QString& name, const QString& text )
 
 
   // Edit the new note if text is empty
-  if ( text.isNull() ) 
+  if ( text.isNull() )
   {
     if ( !mNoteEditDlg )
       mNoteEditDlg = new KNoteEditDlg( widget() );
@@ -138,24 +140,25 @@ QString KNotesPart::newNote( const QString& name, const QString& text )
     mNoteEditDlg->setTitle( journal->summary() );
     mNoteEditDlg->setText( journal->description() );
 
-    if ( mNoteEditDlg->exec() == QDialog::Accepted ) 
+    if ( mNoteEditDlg->exec() == QDialog::Accepted )
     {
       journal->setSummary( mNoteEditDlg->title() );
       journal->setDescription( mNoteEditDlg->text() );
-    } 
-    else 
+    }
+    else
     {
       delete journal;
       return "";
     }
   }
 
+  mManager->addNewNote( journal );
+  mManager->save();
+
   KNotesIconViewItem *note = mNoteList[ journal->uid() ];
   mNotesView->ensureItemVisible( note );
   mNotesView->setCurrentItem( note );
 
-  mManager->addNewNote( journal );
-  mManager->save();
   return journal->uid();
 }
 
@@ -177,7 +180,8 @@ void KNotesPart::killNote( const QString& id, bool force )
    if ( note && !force && KMessageBox::warningContinueCancelList( mNotesView,
         i18n( "Do you really want to delete this note?" ),
         mNoteList[ id ]->text(), i18n( "Confirm Delete" ),
-        KStdGuiItem::del() ) == KMessageBox::Continue ) {
+        KStdGuiItem::del() ) == KMessageBox::Continue ) 
+   {
      mManager->deleteNote( mNoteList[id]->journal() );
      mManager->save();
    }
@@ -271,9 +275,9 @@ void KNotesPart::popupRMB( QIconViewItem *item, const QPoint& pos )
 {
   QPopupMenu *contextMenu = NULL;
 
-  if ( item ) 
+  if ( item )
     contextMenu = static_cast<QPopupMenu *>( factory()->container( "note_context", this ) );
-  else 
+  else
     contextMenu = static_cast<QPopupMenu *>( factory()->container( "notepart_context", this ) );
 
   if ( !contextMenu )
@@ -312,7 +316,7 @@ void KNotesPart::createNote( KCal::Journal *journal )
 
   property = journal->customProperty( "KNotes", "RichText" );
   if ( property.isNull() )
-    journal->setCustomProperty( "KNotes", "RichText", "false" );
+    journal->setCustomProperty( "KNotes", "RichText", "true" );
 
   mNoteList.insert( journal->uid(), new KNotesIconViewItem( mNotesView, journal ) );
 }
@@ -328,11 +332,13 @@ void KNotesPart::editNote( QIconViewItem *item )
     mNoteEditDlg = new KNoteEditDlg( widget() );
 
   KCal::Journal *journal = static_cast<KNotesIconViewItem *>( item )->journal();
+
+  mNoteEditDlg->setRichText( journal->customProperty( "KNotes", "RichText" ) == "true" );
   mNoteEditDlg->setTitle( journal->summary() );
   mNoteEditDlg->setText( journal->description() );
 
   if ( mNoteEditDlg->exec() == QDialog::Accepted ) {
-    journal->setSummary( mNoteEditDlg->title() );
+    item->setText( mNoteEditDlg->title() );
     journal->setDescription( mNoteEditDlg->text() );
     mManager->save();
   }
