@@ -14,7 +14,7 @@
 
     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 
     As a special exception, permission is given to link this program
     with any edition of Qt, and distribute the resulting executable,
@@ -37,12 +37,14 @@
 #include <libkcal/todo.h>
 #include <libkdepim/kpimprefs.h>
 
-#include <korganizer/stdcalendar.h>
+#include "korganizeriface_stub.h"
 
 #include "core.h"
 #include "plugin.h"
 #include "todoplugin.h"
 
+#include "korganizer/stdcalendar.h"
+
 #include "todosummarywidget.h"
 
 TodoSummaryWidget::TodoSummaryWidget( TodoPlugin *plugin,
@@ -69,6 +71,10 @@ TodoSummaryWidget::TodoSummaryWidget( TodoPlugin *plugin,
   updateView();
 }
 
+TodoSummaryWidget::~TodoSummaryWidget()
+{
+}
+
 void TodoSummaryWidget::updateView()
 {
   mLabels.setAutoDelete( true );
@@ -144,6 +150,7 @@ void TodoSummaryWidget::updateView()
         sSummary = todo->relatedTo()->summary() + ":" + todo->summary();
       }
       KURLLabel *urlLabel = new KURLLabel( todo->uid(), sSummary, this );
+      urlLabel->installEventFilter( this );
       urlLabel->setTextFormat( Qt::RichText );
       mLayout->addWidget( urlLabel, counter, 2 );
       mLabels.append( urlLabel );
@@ -167,8 +174,8 @@ void TodoSummaryWidget::updateView()
 
   if ( counter == 0 ) {
     QLabel *noTodos = new QLabel( i18n( "No to-dos pending" ), this );
-    noTodos->setAlignment( AlignRight );
-    mLayout->addWidget( noTodos, 0, 2 );
+    noTodos->setAlignment( AlignHCenter | AlignVCenter );
+    mLayout->addWidget( noTodos, 0, 1 );
     mLabels.append( noTodos );
   }
 
@@ -176,15 +183,29 @@ void TodoSummaryWidget::updateView()
     label->show();
 }
 
-void TodoSummaryWidget::selectEvent( const QString & )
+void TodoSummaryWidget::selectEvent( const QString &uid )
 {
-  mPlugin->core()->selectPlugin( "kontact_todoplugin" );
-  mPlugin->interface()->showTodoView();
+  mPlugin->core()->selectPlugin( "kontact_todoplugin" );//ensure loaded
+  KOrganizerIface_stub iface( "korganizer", "KOrganizerIface" );
+  iface.editIncidence( uid );
+}
+
+bool TodoSummaryWidget::eventFilter( QObject *obj, QEvent* e )
+{
+  if ( obj->inherits( "KURLLabel" ) ) {
+    KURLLabel* label = static_cast<KURLLabel*>( obj );
+    if ( e->type() == QEvent::Enter )
+      emit message( i18n( "Edit To-do: \"%1\"" ).arg( label->text() ) );
+    if ( e->type() == QEvent::Leave )
+      emit message( QString::null );
+  }
+
+  return Kontact::Summary::eventFilter( obj, e );
 }
 
 QStringList TodoSummaryWidget::configModules() const
 {
-  return QStringList( "kcmkorgsummary.desktop" );
+  return QStringList( "kcmtodosummary.desktop" );
 }
 
 #include "todosummarywidget.moc"
