@@ -14,7 +14,7 @@
 
     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 
     As a special exception, permission is given to link this program
     with any edition of Qt, and distribute the resulting executable,
@@ -55,42 +55,53 @@ extern "C"
   }
 }
 
-class NewsEditDialog : public KDialogBase
+NewsEditDialog::NewsEditDialog( const QString& title, const QString& url, QWidget *parent )
+  : KDialogBase( Plain, i18n( "New News Feed" ), Ok | Cancel,
+                 Ok, parent, 0, true, true )
 {
-  public:
-    NewsEditDialog( const QString& title, const QString& url, QWidget *parent )
-      : KDialogBase( Plain, i18n( "New News Feed" ), Ok | Cancel,
-                     Ok, parent, 0, true, true )
-    {
-      QWidget *page = plainPage();
-      QGridLayout *layout = new QGridLayout( page, 2, 3, marginHint(),
-                                             spacingHint() );
+  QWidget *page = plainPage();
+  QGridLayout *layout = new QGridLayout( page, 2, 3, marginHint(),
+                                         spacingHint() );
 
-      QLabel *label = new QLabel( i18n( "Name:" ), page );
-      layout->addWidget( label, 0, 0 );
+  QLabel *label = new QLabel( i18n( "Name:" ), page );
+  layout->addWidget( label, 0, 0 );
 
-      mTitle = new QLineEdit( page );
-      label->setBuddy( mTitle );
-      layout->addMultiCellWidget( mTitle, 0, 0, 1, 2 );
+  mTitle = new QLineEdit( page );
+  label->setBuddy( mTitle );
+  layout->addMultiCellWidget( mTitle, 0, 0, 1, 2 );
 
-      label = new QLabel( i18n( "URL:" ), page );
-      layout->addWidget( label, 1, 0 );
+  label = new QLabel( i18n( "URL:" ), page );
+  layout->addWidget( label, 1, 0 );
 
-      mURL = new QLineEdit( page );
-      label->setBuddy( mURL );
-      layout->addMultiCellWidget( mURL, 1, 1, 1, 2 );
+  mURL = new QLineEdit( page );
+  label->setBuddy( mURL );
+  layout->addMultiCellWidget( mURL, 1, 1, 1, 2 );
 
-      mTitle->setText( title );
-      mURL->setText( url );
-    }
+  mTitle->setText( title );
+  mURL->setText( url );
 
-    QString title() const { return mTitle->text(); }
-    QString url() const { return mURL->text(); }
+  connect( mTitle, SIGNAL( textChanged( const QString& ) ),
+           this, SLOT( modified() ) );
+  connect( mURL, SIGNAL( textChanged( const QString& ) ),
+           this, SLOT( modified() ) );
 
-  private:
-    QLineEdit *mTitle;
-    QLineEdit *mURL;
-};
+  modified();
+}
+
+void NewsEditDialog::modified()
+{
+  enableButton( KDialogBase::Ok, !title().isEmpty() && !url().isEmpty() );
+}
+
+QString NewsEditDialog::title() const
+{
+  return mTitle->text();
+}
+
+QString NewsEditDialog::url() const
+{
+  return mURL->text();
+}
 
 class NewsItem : public QListViewItem
 {
@@ -284,7 +295,13 @@ void KCMKontactKNT::scanNews()
   QStringList urls = service.call( "list()" );
 
   for ( uint i = 0; i < urls.count(); ++i )
-    new NewsItem( mSelectedNews, mFeedMap[ urls[ i ] ], urls[ i ], false );
+  {
+    QString url = urls[ i ];
+    QString feedName = mFeedMap[ url ];
+    if ( feedName.isEmpty() )
+      feedName = url;
+    new NewsItem( mSelectedNews, feedName, url, false );
+  }
 }
 
 void KCMKontactKNT::selectedChanged( QListViewItem *item )
