@@ -14,7 +14,7 @@
 
     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 
     As a special exception, permission is given to link this program
     with any edition of Qt, and distribute the resulting executable,
@@ -43,7 +43,7 @@
 
 SummaryWidget::SummaryWidget( QWidget *parent, const char *name )
   : Kontact::Summary( parent, name ),
-    DCOPObject( "NewsTickerPlugin" ), mLayout( 0 )
+    DCOPObject( "NewsTickerPlugin" ), mLayout( 0 ), mFeedCounter( 0 )
 {
   QVBoxLayout *vlay = new QVBoxLayout( this, 3, 3 );
 
@@ -72,6 +72,8 @@ SummaryWidget::SummaryWidget( QWidget *parent, const char *name )
 
   readConfig();
 
+  connectDCOPSignal( 0, 0, "documentUpdateError(DCOPRef,int)", "documentUpdateError(DCOPRef, int)", false );
+
   if ( dcopAvailable )
     initDocuments();
 
@@ -79,7 +81,7 @@ SummaryWidget::SummaryWidget( QWidget *parent, const char *name )
   connectDCOPSignal( 0, 0, "removed(QString)", "documentRemoved(QString)", false );
 }
 
-int SummaryWidget::summaryHight() const
+int SummaryWidget::summaryHeight() const
 {
   return ( mFeeds.count() == 0 ? 1 : mFeeds.count() );
 }
@@ -134,10 +136,11 @@ void SummaryWidget::initDocuments()
     feedRef.call( "pixmap()" ).get( feed.logo );
     mFeeds.append( feed );
 
+    disconnectDCOPSignal( "rssservice", feedRef.obj(), "documentUpdated(DCOPRef)", 0 );
     connectDCOPSignal( "rssservice", feedRef.obj(), "documentUpdated(DCOPRef)",
                        "documentUpdated(DCOPRef)", false );
 
-    qApp->processEvents( QEventLoop::ExcludeUserInput | 
+    qApp->processEvents( QEventLoop::ExcludeUserInput |
                          QEventLoop::ExcludeSocketNotifiers );
   }
 
@@ -157,7 +160,6 @@ void SummaryWidget::updateDocuments()
 
 void SummaryWidget::documentUpdated( DCOPRef feedRef )
 {
-  static uint feedCounter = 0;
   ArticleMap map;
 
   int numArticles = feedRef.call( "count()" );
@@ -165,7 +167,7 @@ void SummaryWidget::documentUpdated( DCOPRef feedRef )
     DCOPRef artRef = feedRef.call( "article(int)", i );
     QString title, url;
 
-    qApp->processEvents( QEventLoop::ExcludeUserInput | 
+    qApp->processEvents( QEventLoop::ExcludeUserInput |
                          QEventLoop::ExcludeSocketNotifiers );
 
     artRef.call( "title()" ).get( title );
@@ -187,9 +189,9 @@ void SummaryWidget::documentUpdated( DCOPRef feedRef )
         feedRef.call( "pixmap()" ).get( (*it).logo );
     }
 
-  feedCounter++;
-  if ( feedCounter == mFeeds.count() ) {
-    feedCounter = 0;
+  mFeedCounter++;
+  if ( mFeedCounter == mFeeds.count() ) {
+    mFeedCounter = 0;
     updateView();
   }
 }
@@ -260,11 +262,34 @@ void SummaryWidget::updateView()
     label->show();
 }
 
+void SummaryWidget::documentUpdateError( DCOPRef feedRef, int errorCode )
+{
+  kdDebug() << " error while updating document, error code: " << errorCode << endl;
+  FeedList::Iterator it;
+  for ( it = mFeeds.begin(); it != mFeeds.end(); ++it ) {
+    if ( (*it).ref.obj() == feedRef.obj() ) {
+      mFeeds.remove( it );
+      break;
+    }
+  }
+
+  if ( mFeedCounter == mFeeds.count() ) {
+    mFeedCounter = 0;
+    updateView();
+  }
+
+}
+
 QStringList SummaryWidget::configModules() const
 {
   return "kcmkontactknt.desktop";
 }
 
+void SummaryWidget::updateSummary( bool )
+{
+  updateDocuments();
+}
+
 void SummaryWidget::rmbMenu( const QString& url )
 {
   QPopupMenu menu;
