@@ -15,7 +15,7 @@
 
     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 
     As a special exception, permission is given to link this program
     with any edition of Qt, and distribute the resulting executable,
@@ -30,18 +30,18 @@
 
 class QLabel;
 class KActiveLabel;
-class KPushButton;
 class QCheckBox;
 class QGridLayout;
 class KLineEdit;
 class KTextEdit;
 class KTimeEdit;
+class QButton;
 namespace KOrg {
 class IncidenceChangerBase;
 }
 using namespace KOrg;
 namespace KCal {
-  class Calendar; 
+  class Calendar;
   class Journal;
 }
 using namespace KCal;
@@ -50,10 +50,10 @@ class JournalEntry : public QWidget {
     Q_OBJECT
   public:
     typedef ListBase<JournalEntry> List;
-    
+
     JournalEntry( Journal* j, QWidget *parent );
     virtual ~JournalEntry();
-    
+
     void setJournal(Journal *);
     Journal *journal() const { return mJournal; }
 
@@ -68,37 +68,42 @@ class JournalEntry : public QWidget {
   protected slots:
     void setDirty();
     void deleteItem();
+    void editItem();
+    void timeCheckBoxToggled(bool on);
   public slots:
     void setIncidenceChanger( IncidenceChangerBase *changer ) { mChanger = changer; }
     void setDate(const QDate &);
     void flushEntry();
-    
+
   signals:
     void deleteIncidence( Incidence * );
+    void editIncidence( Incidence * );
 
-  protected:    
+  protected:
     void clearFields();
     bool eventFilter( QObject *o, QEvent *e );
 
     void writeJournal();
-    
+
   private:
     void writeJournalPrivate( Journal *j );
 
     Journal *mJournal;
     QDate mDate;
     bool mReadOnly;
-    
+
     QLabel *mTitleLabel;
     KLineEdit *mTitleEdit;
     KTextEdit *mEditor;
     QCheckBox *mTimeCheck;
     KTimeEdit *mTimeEdit;
-    KPushButton *mDeleteButton;
-    
+    QButton *mDeleteButton;
+    QButton *mEditButton;
+
     QGridLayout *mLayout;
 
     bool mDirty;
+    bool mWriteInProgress;
     IncidenceChangerBase *mChanger;
 };
 
@@ -107,10 +112,10 @@ class JournalDateEntry : public QVBox {
     Q_OBJECT
   public:
     typedef ListBase<JournalDateEntry> List;
-    
+
     JournalDateEntry( Calendar *, QWidget *parent );
     virtual ~JournalDateEntry();
-    
+
     void addJournal( Journal * );
     Journal::List journals() const;
 
@@ -119,19 +124,21 @@ class JournalDateEntry : public QVBox {
 
     void clear();
 
-    
+
   signals:
     void setIncidenceChangerSignal( IncidenceChangerBase *changer );
     void setDateSignal( const QDate & );
     void flushEntries();
+    void editIncidence( Incidence * );
     void deleteIncidence( Incidence * );
-    
+    void newJournal( const QDate & );
+
   public slots:
-    void newJournal();
+    void emitNewJournal();
     void setIncidenceChanger( IncidenceChangerBase *changer );
     void journalEdited( Journal* );
     void journalDeleted( Journal* );
-  
+
   private:
     Calendar *mCalendar;
     QDate mDate;
