@@ -16,7 +16,7 @@
 
     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 
     As a special exception, permission is given to link this program
     with any edition of Qt, and distribute the resulting executable,
@@ -54,6 +54,7 @@
 #include "koglobals.h"
 #include "koincidencetooltip.h"
 #include "koeventpopupmenu.h"
+#include "kohelper.h"
 
 #include "komonthview.h"
 #include "komonthview.moc"
@@ -189,7 +190,7 @@ void KNoScrollListBox::resizeEvent( QResizeEvent *e )
   QListBox::resizeEvent( e );
 }
 
-MonthViewItem::MonthViewItem( Incidence *incidence, const QDateTime &qd, 
+MonthViewItem::MonthViewItem( Incidence *incidence, const QDateTime &qd,
                               const QString & s ) : QListBoxItem()
 {
   setText( s );
@@ -221,8 +222,8 @@ void MonthViewItem::paint( QPainter *p )
 
   QColor bgColor = palette().color( QPalette::Normal,
             sel ? QColorGroup::Highlight : QColorGroup::Background );
-  int offset=0;        
-  if ( KOPrefs::instance()->monthViewUsesResourceColor() && 
+  int offset=0;
+  if ( KOPrefs::instance()->monthViewUsesResourceColor() &&
     mResourceColor.isValid() ) {
     p->setBackgroundColor( mResourceColor );
     p->eraseRect( 0, 0, listBox()->maxItemWidth(), height( listBox() ) );
@@ -325,10 +326,8 @@ MonthViewCell::MonthViewCell( KOMonthView *parent)
   connect( mItemList, SIGNAL( rightButtonPressed( QListBoxItem *,
                                                   const QPoint &) ),
            SLOT( contextMenu( QListBoxItem * ) ) );
-  connect( mItemList, SIGNAL( highlighted( QListBoxItem *) ),
-           SLOT( selection( QListBoxItem * ) ) );
   connect( mItemList, SIGNAL( clicked( QListBoxItem * ) ),
-           SLOT( cellClicked( QListBoxItem * ) ) );
+           SLOT( select() ) );
 }
 
 void MonthViewCell::setDate( const QDate &date )
@@ -337,11 +336,7 @@ void MonthViewCell::setDate( const QDate &date )
 
   mDate = date;
 
-  // show current day with a thicker frame
-  if ( mDate == QDate::currentDate() )
-    mItemList->setLineWidth( 3 );
-  else
-    mItemList->setLineWidth( 1 );
+  setFrameWidth();
 
   QString text;
   if ( KOGlobals::self()->calendarSystem()->day( date ) == 1 ) {
@@ -364,6 +359,15 @@ QDate MonthViewCell::date() const
   return mDate;
 }
 
+void MonthViewCell::setFrameWidth()
+{
+  // show current day with a thicker frame
+  if ( mDate == QDate::currentDate() )
+    mItemList->setLineWidth( 3 );
+  else
+    mItemList->setLineWidth( 1 );
+}
+
 void MonthViewCell::setPrimary( bool primary )
 {
   mPrimary = primary;
@@ -393,17 +397,15 @@ void MonthViewCell::setHoliday( bool holiday )
   }
 }
 
-void MonthViewCell::setHoliday( const QString &holiday )
+void MonthViewCell::setHolidayString( const QString &holiday )
 {
   mHolidayString = holiday;
-
-  if ( !holiday.isEmpty() ) {
-    setHoliday( true );
-  }
 }
 
 void MonthViewCell::updateCell()
 {
+  setFrameWidth();
+
   if ( mDate == QDate::currentDate() ) {
     setPalette( mTodayPalette );
 
@@ -431,11 +433,11 @@ void MonthViewCell::updateCell()
   }
 }
 
-class MonthViewCell::CreateItemVisitor : 
+class MonthViewCell::CreateItemVisitor :
       public IncidenceBase::Visitor
 {
   public:
-    CreateItemVisitor() : mItem(0) {}
+    CreateItemVisitor() : mItem(0) { emails = KOPrefs::instance()->allEmails(); }
 
     bool act( IncidenceBase *incidence, QDate date, QPalette stdPal )
     {
@@ -445,6 +447,7 @@ class MonthViewCell::CreateItemVisitor :
       return incidence->accept( *this );
     }
     MonthViewItem *item() const { return mItem; }
+    QStringList emails;
 
   protected:
     bool visit( Event *event ) {
@@ -485,7 +488,7 @@ class MonthViewCell::CreateItemVisitor :
         mItem->setPalette( mStandardPalette );
       }
 
-      Attendee *me = event->attendeeByMails( KOPrefs::instance()->allEmails() );
+      Attendee *me = event->attendeeByMails( emails );
       if ( me != 0 ) {
         mItem->setReply( me->status() == Attendee::NeedsAction && me->RSVP() );
       } else
@@ -494,7 +497,7 @@ class MonthViewCell::CreateItemVisitor :
     }
     bool visit( Todo *todo ) {
       QString text;
-      if ( !KOPrefs::instance()->showAllDayTodo() ) 
+      if ( !KOPrefs::instance()->showAllDayTodo() )
         return false;
       QDateTime dt( mDate );
       if ( todo->hasDueDate() && !todo->doesFloat() ) {
@@ -505,6 +508,7 @@ class MonthViewCell::CreateItemVisitor :
       text += todo->summary();
 
       mItem = new MonthViewItem( todo, dt, text );
+// FIXME: This breaks with recurring multi-day events!
       if ( todo->doesRecur() ) {
         mDate < todo->dtDue().date() ?
         mItem->setTodoDone( true ) : mItem->setTodo( true );
@@ -521,33 +525,26 @@ class MonthViewCell::CreateItemVisitor :
 };
 
 
-void MonthViewCell::addIncidence( Incidence *incidence )
+void MonthViewCell::addIncidence( Incidence *incidence, CreateItemVisitor& v)
 {
-  CreateItemVisitor v;
-  
-  QColor resourceColor=KOPrefs::instance()->mEventColor;
-  
-  CalendarResources *calendarRsc = dynamic_cast<CalendarResources*>( mCalendar );
-  if ( calendarRsc ) {
-    ResourceCalendar *rscCalendar = calendarRsc->resource( incidence );
-    resourceColor= *KOPrefs::instance()->resourceColor( rscCalendar->identifier() );
-  }else{
-    kdDebug(5850) << "MonthViewCell::addIncidence and mCalendar is not a CalendarResources" <<endl;
-  }
-  
   if ( v.act( incidence, mDate, mStandardPalette ) ) {
     MonthViewItem *item = v.item();
     if ( item ) {
       item->setAlarm( incidence->isAlarmEnabled() );
-      item->setRecur( incidence->doesRecur() );
-      item->setResourceColor(resourceColor);
+      item->setRecur( incidence->recurrenceType() );
+
+      QColor resourceColor = KOHelper::resourceColor( mCalendar, incidence );
+      if ( !resourceColor.isValid() )
+        resourceColor = KOPrefs::instance()->mEventColor;
+      item->setResourceColor( resourceColor );
+
       // FIXME: Find the correct position (time-wise) to insert the item.
       //        Currently, the items are displayed in "random" order instead of
       //        chronologically sorted.
       uint i = 0;
       int pos = -1;
       QDateTime dt( item->incidenceDateTime() );
-      
+
       while ( i < mItemList->count() && pos<0 ) {
         QListBoxItem *item = mItemList->item( i );
         MonthViewItem *mvitem = dynamic_cast<MonthViewItem*>( item );
@@ -650,9 +647,26 @@ QDate MonthViewCell::selectedIncidenceDate()
   return item->incidenceDateTime().date();
 }
 
+void MonthViewCell::select()
+{
+  // setSelectedCall will deselect currently selected cells
+  mMonthView->setSelectedCell( this );
+
+  if( KOPrefs::instance()->enableMonthScroll() )
+    enableScrollBars( true );
+
+  // don't mess up the cell when it represents today
+  if( mDate != QDate::currentDate() ) {
+    mItemList->setFrameStyle( QFrame::Sunken | QFrame::Panel );
+    mItemList->setLineWidth( 3 );
+  }
+}
+
 void MonthViewCell::deselect()
 {
   mItemList->clearSelection();
+  mItemList->setFrameStyle( QFrame::Plain | QFrame::Panel );
+  setFrameWidth();
 
   enableScrollBars( false );
 }
@@ -664,6 +678,8 @@ void MonthViewCell::resizeEvent ( QResizeEvent * )
 
 void MonthViewCell::defaultAction( QListBoxItem *item )
 {
+  select();
+
   if ( !item ) {
     emit newEventSignal( date() );
   } else {
@@ -673,14 +689,10 @@ void MonthViewCell::defaultAction( QListBoxItem *item )
   }
 }
 
-void MonthViewCell::cellClicked( QListBoxItem * )
-{
-  if( KOPrefs::instance()->enableMonthScroll() ) enableScrollBars( true );
-}
-
 void MonthViewCell::contextMenu( QListBoxItem *item )
 {
-  mMonthView->setSelectedCell( this );
+  select();
+
   if ( item ) {
     MonthViewItem *eventItem = static_cast<MonthViewItem *>( item );
     Incidence *incidence = eventItem->incidence();
@@ -691,12 +703,6 @@ void MonthViewCell::contextMenu( QListBoxItem *item )
   }
 }
 
-void MonthViewCell::selection( QListBoxItem *item )
-{
-  if ( !item ) return;
-
-  mMonthView->setSelectedCell( this );
-}
 
 KOMonthView::KOMonthView( Calendar *calendar, QWidget *parent, const char *name )
     : KOEventView( calendar, parent, name ),
@@ -707,11 +713,24 @@ KOMonthView::KOMonthView( Calendar *calendar, QWidget *parent, const char *name
 
   QGridLayout *dayLayout = new QGridLayout( this );
 
+  QFont bfont = font();
+  bfont.setBold( true );
+
+  QFont mfont = bfont;
+  mfont.setPointSize( 20 );
+
+  // month name on top
+  mLabel = new QLabel( this );
+  mLabel->setFont( mfont );
+  mLabel->setAlignment( AlignCenter );
+  mLabel->setLineWidth( 0 );
+  mLabel->setFrameStyle( QFrame::Plain );
+
+  dayLayout->addMultiCellWidget( mLabel, 0, 0, 0, mDaysPerWeek );
+
   // create the day of the week labels (Sun, Mon, etc) and add them to
   // the layout.
   mDayLabels.resize( mDaysPerWeek );
-  QFont bfont = font();
-  bfont.setBold( true );
   int i;
   for( i = 0; i < mDaysPerWeek; i++ ) {
     QLabel *label = new QLabel( this );
@@ -722,7 +741,7 @@ KOMonthView::KOMonthView( Calendar *calendar, QWidget *parent, const char *name
 
     mDayLabels.insert( i, label );
 
-    dayLayout->addWidget( label, 0, i );
+    dayLayout->addWidget( label, 1, i );
     dayLayout->addColSpacing( i, 10 );
     dayLayout->setColStretch( i, 1 );
   }
@@ -735,14 +754,14 @@ KOMonthView::KOMonthView( Calendar *calendar, QWidget *parent, const char *name
       MonthViewCell *cell = new MonthViewCell( this );
       cell->setCalendar(calendar);
       mCells.insert( row * mDaysPerWeek + col, cell );
-      dayLayout->addWidget( cell, row + 1, col );
+      dayLayout->addWidget( cell, row + 2, col );
 
       connect( cell, SIGNAL( defaultAction( Incidence * ) ),
                SLOT( defaultAction( Incidence * ) ) );
       connect( cell, SIGNAL( newEventSignal( const QDate & ) ),
                SIGNAL( newEventSignal( const QDate & ) ) );
     }
-    dayLayout->setRowStretch( row + 1, 1 );
+    dayLayout->setRowStretch( row + 2, 1 );
   }
 
   mEventContextMenu = eventPopup();
@@ -839,33 +858,39 @@ void KOMonthView::showDates( const QDate &start, const QDate & )
 {
 //  kdDebug(5850) << "KOMonthView::showDates(): " << start.toString() << endl;
 
-  mStartDate = start;
+  const KCalendarSystem *calSys = KOGlobals::self()->calendarSystem();
 
+  // show first day of month on top for readability issues
+  mStartDate = start.addDays( -start.day() + 1 );
   // correct begin of week
   int weekdayCol=( mStartDate.dayOfWeek() + 7 - mWeekStartDay ) % 7;
   mStartDate = mStartDate.addDays( -weekdayCol );
 
+  mLabel->setText( i18n( "monthname year", "%1 %2" )
+                   .arg( calSys->monthName( start ) )
+                   .arg( calSys->year( start ) ) );
+
   bool primary = false;
   uint i;
   for( i = 0; i < mCells.size(); ++i ) {
     QDate date = mStartDate.addDays( i );
-    if ( KOGlobals::self()->calendarSystem()->day( date ) == 1 ) {
+    if ( calSys->day( date ) == 1 ) {
       primary = !primary;
     }
+
     mCells[i]->setDate( date );
+    if( date == start )
+      mCells[i]->select();
 
     mCells[i]->setPrimary( primary );
 
-    if ( KOGlobals::self()->calendarSystem()->dayOfWeek( date ) ==
-         KOGlobals::self()->calendarSystem()->weekDayOfPray() ) {
-      mCells[i]->setHoliday( true );
-    } else {
-      mCells[i]->setHoliday( false );
-    }
+    bool isHoliday = calSys->dayOfWeek( date ) == calSys->weekDayOfPray()
+                  || !KOGlobals::self()->isWorkDay( date );
+    mCells[i]->setHoliday( isHoliday );
 
     // add holiday, if present
     QString hstring( KOGlobals::self()->holiday( date ) );
-    mCells[i]->setHoliday( hstring );
+    mCells[i]->setHolidayString( hstring );
   }
 
   updateView();
@@ -896,7 +921,7 @@ class KOMonthView::GetDateVisitor : public IncidenceBase::Visitor
       if ( todo->hasDueDate() ) {
         mDate = todo->dtDue().date();
         return true;
-      } else 
+      } else
         return false;
     }
     bool visit( Journal *journal ) {
@@ -907,13 +932,13 @@ class KOMonthView::GetDateVisitor : public IncidenceBase::Visitor
     QDate mDate;
 };
 
-void KOMonthView::changeIncidenceDisplayAdded( Incidence *incidence )
+void KOMonthView::changeIncidenceDisplayAdded( Incidence *incidence, MonthViewCell::CreateItemVisitor& v)
 {
   MonthViewCell *mvc;
   Event *event = 0;
   Todo *todo = 0;
   QDate date;
-  
+
   // FIXME: use a visitor here
   if ( incidence->type() == "Event" ) {
     event = static_cast<Event *>( incidence );
@@ -926,33 +951,35 @@ void KOMonthView::changeIncidenceDisplayAdded( Incidence *incidence )
   }
 
   if ( incidence->doesRecur() ) {
+// FIXME: This breaks with recurring multi-day events!
      for ( uint i = 0; i < mCells.count(); i++ ) {
        if ( incidence->recursOn( mCells[i]->date() ) ) {
-         mCells[i]->addIncidence( incidence );
+         mCells[i]->addIncidence( incidence, v );
        }
      }
   } else if ( event ) {
       for ( QDateTime _date = date;
             _date <= event->dtEnd(); _date = _date.addDays( 1 ) ) {
         mvc = lookupCellByDate( _date.date() );
-        if ( mvc ) mvc->addIncidence( event );
+        if ( mvc ) mvc->addIncidence( event, v );
       }
     } else if ( todo ) {
         mvc = lookupCellByDate( date );
-        if ( mvc ) mvc->addIncidence( todo );
+        if ( mvc ) mvc->addIncidence( todo, v );
       }
 }
 
 void KOMonthView::changeIncidenceDisplay( Incidence *incidence, int action )
 {
+  MonthViewCell::CreateItemVisitor v;
   switch ( action ) {
     case KOGlobals::INCIDENCEADDED:
-      changeIncidenceDisplayAdded( incidence );
+      changeIncidenceDisplayAdded( incidence, v );
       break;
     case KOGlobals::INCIDENCEEDITED:
       for( uint i = 0; i < mCells.count(); i++ )
         mCells[i]->removeIncidence( incidence );
-      changeIncidenceDisplayAdded( incidence );
+      changeIncidenceDisplayAdded( incidence, v );
       break;
     case KOGlobals::INCIDENCEDELETED:
       for( uint i = 0; i < mCells.count(); i++ )
@@ -972,8 +999,9 @@ void KOMonthView::updateView()
   Incidence::List incidences = calendar()->incidences();
   Incidence::List::ConstIterator it;
 
+  MonthViewCell::CreateItemVisitor v;
   for ( it = incidences.begin(); it != incidences.end(); ++it )
-    changeIncidenceDisplayAdded( *it );
+    changeIncidenceDisplayAdded( *it, v );
 
   processSelectionChange();
 }
