@@ -16,7 +16,7 @@
 
     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 
     As a special exception, permission is given to link this program
     with any edition of Qt, and distribute the resulting executable,
@@ -127,6 +127,7 @@ class MonthViewCell : public QWidget
 {
     Q_OBJECT
   public:
+    class CreateItemVisitor;
     MonthViewCell( KOMonthView * );
 
     /** Sets the date of the cell */
@@ -134,18 +135,33 @@ class MonthViewCell : public QWidget
     /** @return Date of cell */
     QDate date() const;
 
-    void setPrimary( bool );
+    /**
+      Set this cell as primary if @p primary is true. A primary cell belongs
+      to the current month. A non-primary cell belongs to the month before or
+      after the current month.
+      @param primary If true, the cell will be set as primary. Else it will be
+      set as non-primary.
+    */
+    void setPrimary( bool primary );
+    /**
+      @return True if this cell is primary, else false.
+    */
     bool isPrimary() const;
 
     /** Make this cell show as a holiday */
     void setHoliday( bool );
-    void setHoliday( const QString & );
+    /**
+      Sets the holiday name to this cell. This will not call
+      setHoliday( true ).
+      @param name The name of the holiday.
+    */
+    void setHolidayString( const QString &name );
 
     void updateCell();
     /** Adds an incidence to the cell.
         Sets the right text and icons for this incidence.
     */
-    void addIncidence( Incidence * );
+    void addIncidence( Incidence *, MonthViewCell::CreateItemVisitor& );
     /** Removes an incidence from the cell.
         @return True if successful, false if deletion failed
        (e.g. when given incidence doesn't exist in the cell.
@@ -164,19 +180,25 @@ class MonthViewCell : public QWidget
     void setCalendar( Calendar*cal ) { mCalendar = cal; }
   signals:
     void defaultAction( Incidence * );
-    void newEventSignal( const QDate & );
+    /**
+      Notify the view manager that we want to create a new event, so an editor
+      will pop up.
+      @param date The date of the event we want create.
+    */
+    void newEventSignal( const QDate &date );
+
+  public slots:
+    void select();
 
   protected:
+    void setFrameWidth();
     void resizeEvent( QResizeEvent * );
 
   protected slots:
     void defaultAction( QListBoxItem * );
     void contextMenu( QListBoxItem * );
-    void selection( QListBoxItem * );
-    void cellClicked( QListBoxItem * );
 
   private:
-    class CreateItemVisitor;
     KOMonthView *mMonthView;
   // We need the calendar for paint the ResourceColor
     Calendar *mCalendar;
@@ -231,7 +253,7 @@ class KOMonthView: public KOEventView
     virtual void showIncidences( const Incidence::List &incidenceList );
 
     void changeIncidenceDisplay(Incidence *, int);
-    void changeIncidenceDisplayAdded(Incidence *);
+    void changeIncidenceDisplayAdded(Incidence *, MonthViewCell::CreateItemVisitor&);
 
     void clearSelection();
 
@@ -272,6 +294,7 @@ class KOMonthView: public KOEventView
     MonthViewCell *mSelectedCell;
 
     KOEventPopupMenu *mEventContextMenu;
+    QLabel *mLabel;
 };
 
 #endif
