@@ -16,7 +16,7 @@
 
     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 
     As a special exception, permission is given to link this program
     with any edition of Qt, and distribute the resulting executable,
@@ -39,7 +39,6 @@
 #include <libkcal/calendarlocal.h>
 #include <libkcal/calendarresources.h>
 #include <libkcal/resourcecalendar.h>
-#include <libkcal/resourcelocal.h>
 
 #include <kpopupmenu.h>
 #include <kinstance.h>
@@ -100,7 +99,6 @@ KOrganizerPart::KOrganizerPart( QWidget *parentWidget, const char *widgetName,
     setHasDocument( true );
   }
 
-  mBrowserExtension = new KOrganizerBrowserExtension( this );
   mStatusBarExtension = new KParts::StatusBarExtension( this );
 
   setInstance( KOrganizerFactory::instance() );
@@ -124,9 +122,6 @@ KOrganizerPart::KOrganizerPart( QWidget *parentWidget, const char *widgetName,
 
   setXMLFile( "korganizer_part.rc" );
   mActionManager->loadParts();
-  // If korganizer is run as part inside kontact, the alarmdaemon
-  // is not started by KOrganizerApp, so we have to start it here.
-  KOGlobals::self()->alarmClient()->startDaemon();
   setTitle();
 }
 
@@ -242,15 +237,4 @@ void KOrganizerPart::setTitle()
   emit setWindowCaption( title );*/
 }
 
-KOrganizerBrowserExtension::KOrganizerBrowserExtension(KOrganizerPart *parent) :
-  KParts::BrowserExtension(parent, "KOrganizerBrowserExtension")
-{
-}
-
-KOrganizerBrowserExtension::~KOrganizerBrowserExtension()
-{
-}
-
-using namespace KParts;
-
 #include "korganizer_part.moc"
