@@ -13,7 +13,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
 #include "kio_subjects.h"
@@ -49,7 +49,6 @@ KIO_Subjects::~KIO_Subjects( )
 	delete _jobs;
 	delete _kurl;
 	delete _metadata;
-	delete _protocol;
 	_protocol = 0;
 }
 
@@ -59,8 +58,7 @@ void KIO_Subjects::doReadSubjects( KKioDrop *drop )
 	QValueList<KKioDrop::FileInfo>::ConstIterator end_it = drop->_mailurls->end();
 	
 	_kio = drop;
-	delete _protocol;
-	_protocol = _kio->_protocol->clone();
+	_protocol = _kio->_protocol;
 	*_kurl = *_kio->_kurl;
 	*_metadata = *_kio->_metadata;
 
@@ -104,6 +102,7 @@ void KIO_Subjects::getConnection( )
 		if( ! ( _slave = KIO::Scheduler::getConnectedSlave( kurl, metadata ) ) )
 		{
 			kdWarning() << i18n( "Not able to open a kio-slave for %1." ).arg( _protocol->configName() );
+			_kio->emitShowPassivePopup( i18n( "Not able to open a kio-slave for %1." ).arg( _protocol->configName() ) );
 			_valid = false;
 			_kio->emitValidChanged();
 			_slave = 0;
