@@ -273,7 +273,10 @@ void KornSubjectsDlg::prepareStep2Subjects( KMailDrop *drop )
 	connect( drop, SIGNAL( readSubjectsProgress( int ) ), _subjects->progress, SLOT( setProgress( int ) ) );
 	connect( _subjects->progress, SIGNAL( cancelPressed() ), drop, SLOT( readSubjectsCanceled() ) );
 	
-	_subjects->it->current()->readSubjects( 0 );
+	if( _subjects->it->current()->canReadSubjects() )
+		_subjects->it->current()->readSubjects( 0 );
+	else
+		subjectsReady( true );
 }
 
 void KornSubjectsDlg::removeStep2Subjects( KMailDrop *drop )
@@ -415,7 +418,7 @@ void KornSubjectsDlg::deleteMessage()
 	QString confirmation = i18n(	"Do you really want to delete %n message?",
 					"Do you really want to delete %n messages?", _delete->messages->count() );
 	
-	if( KMessageBox::questionYesNo( this, confirmation, i18n( "Confirmation" ) ) != KMessageBox::Yes )
+	if( KMessageBox::questionYesNo( this, confirmation, i18n( "Confirmation" ), KStdGuiItem::del(), KStdGuiItem::cancel() ) != KMessageBox::Yes )
 	{
 		deleteDeleteStruct();
 		return; //Not excepted
