@@ -20,8 +20,8 @@
 **
 ** You should have received a copy of the GNU General Public License
 ** along with this program in a file called COPYING; if not, write to
-** the Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,
-** MA 02111-1307, USA.
+** the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
+** MA 02110-1301, USA.
 */
 
 /*
@@ -214,7 +214,9 @@ void GenericDBWidget::slotSelected(const QString &dbname)
 //			if (!(pilotRec->isDeleted()) )
 			{
 				PilotListViewItem*item=new PilotListViewItem(fRecordList,
-					QString::number(currentRecord), QString::number(pilotRec->getLen()),  QString::number(pilotRec->id()), QString::null,
+					QString::number(currentRecord), QString::number(pilotRec->size()),
+					QString::number(pilotRec->id()),
+					QString::null,
 					pilotRec->id(), pilotRec);
 				item->setNumericCol(0, true);
 				item->setNumericCol(1, true);
@@ -310,7 +312,7 @@ void GenericDBWidget::slotAddRecord()
 	FUNCTIONSETUP;
 	PilotRecord*rec=new PilotRecord(0L, 0, 0, 0, 0);
 	PilotListViewItem*item=new PilotListViewItem(fRecordList,
-		QString::number(-1), QString::number(rec->getLen()),
+		QString::number(-1), QString::number(rec->size()),
 		QString::number(rec->id()), QString::null,
 		rec->id(), rec);
 	if (slotEditRecord(item))
@@ -335,7 +337,7 @@ bool GenericDBWidget::slotEditRecord(QListViewItem*item)
 		DBRecordEditor*dlg=new DBRecordEditor(rec, nr, this);
 		if (dlg->exec())
 		{
-			currRecItem->setText(1, QString::number(rec->getLen()));
+			currRecItem->setText(1, QString::number(rec->size()));
 			currRecItem->setText(2, QString::number(rec->id()));
 			fRecordList->triggerUpdate();
 			writeRecord(rec);
@@ -360,7 +362,7 @@ void GenericDBWidget::slotDeleteRecord()
 {
 	FUNCTIONSETUP;
 	PilotListViewItem*currRecItem=dynamic_cast<PilotListViewItem*>(fRecordList->selectedItem());
-	if (currRecItem && (KMessageBox::questionYesNo(this, i18n("<qt>Do you really want to delete the selected record? This cannot be undone.<br><br>Delete record?<qt>"), i18n("Deleting Record"))==KMessageBox::Yes) )
+	if (currRecItem && (KMessageBox::questionYesNo(this, i18n("<qt>Do you really want to delete the selected record? This cannot be undone.<br><br>Delete record?<qt>"), i18n("Deleting Record"), KStdGuiItem::del(), KStdGuiItem::cancel())==KMessageBox::Yes) )
 	{
 		PilotRecord*rec=(PilotRecord*)currRecItem->rec();
 		rec->setDeleted();
