@@ -19,8 +19,8 @@
 **
 ** You should have received a copy of the GNU Lesser General Public License
 ** along with this program in a file called COPYING; if not, write to
-** the Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,
-** MA 02111-1307, USA.
+** the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
+** MA 02110-1301, USA.
 */
 
 /*
@@ -29,7 +29,7 @@
 
 
 static const char *options_id =
-	"$Id: options.cc 380870 2005-01-21 17:40:23Z adridg $";
+	"$Id: options.cc 449050 2005-08-14 05:03:22Z vanrijn $";
 
 #include "options.h"
 
@@ -90,6 +90,7 @@ struct tm writeTm(const QDateTime &dt)
   t.tm_wday = 0; // unimplemented
   t.tm_yday = 0; // unimplemented
   t.tm_isdst = 0; // unimplemented
+  t.tm_zone = 0; // unimplemented
 
   t.tm_year = dt.date().year() - 1900;
   t.tm_mon = dt.date().month() - 1;
@@ -127,14 +128,14 @@ KPilotDepthCount::KPilotDepthCount(int area, int level, const char *s) :
 	fLevel(level),
 	fName(s)
 {
-	if (debug_level>=fLevel) 
-	{ 
+	if (debug_level>=fLevel)
+	{
 #ifdef DEBUG_CERR
-		DEBUGKPILOT 
+		DEBUGKPILOT
 #else
 		debug(area)
 #endif
-		<< indent() << ">" << name() << endl; 
+		<< indent() << ">" << name() << endl;
 	}
 	depth++;
 }
