@@ -22,8 +22,8 @@
 **
 ** You should have received a copy of the GNU Lesser General Public License
 ** along with this program in a file called COPYING; if not, write to
-** the Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,
-** MA 02111-1307, USA.
+** the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
+** MA 02110-1301, USA.
 */
 
 /*
@@ -48,6 +48,8 @@
 # endif
 #endif
 
+#include "pilotLinkVersion.h"
+
 #include <pi-dlp.h>
 
 
@@ -230,17 +232,18 @@ public:
 	/** Returns the length of the (whole) AppInfo block. */
 	PI_SIZE_T length() const { return fLen; } ;
 
-	/** Search for the given category @param name in the list
-	* of categories; returns the category number. If @param unknownIsUnfiled
+	/** Search for the given category @p name in the list
+	* of categories; returns the category number. If @p unknownIsUnfiled
 	* is true, then map unknown categories to Unfiled instead of returning
 	* an error number.
 	*
-	* @return >=0          is a specific category based on the text ->
+	* @return >=0   is a specific category based on the text-to-
 	*               category number mapping defined by the Pilot,
-	*  @return -1         means unknown category selected when
-	*               @param unknownIsUnfiled is true.
-	*  @return  0         == Unfiled means unknown category selected when
-	*               @param unknownIsUnfiled is false.
+	*               where 0 is always the 'unfiled' category.
+	*  @return -1   means unknown category selected when
+	*               @p unknownIsUnfiled is false.
+	*  @return  0   == Unfiled means unknown category selected when
+	*               @p unknownIsUnfiled is true.
 	*
 	*/
 	static int findCategory(const QString &name, bool unknownIsUnfiled, struct CategoryAppInfo *info);
@@ -251,6 +254,11 @@ public:
 	/** For debugging, display all the category names */
 	void dump() const;
 
+	/** For debugging, display category names for the given AppInfo
+	* structure. Called by dump().
+	*/
+	static void dumpCategories(const struct CategoryAppInfo &info);
+
 	/** Gets a single category name. Returns QString::null if there is no
 	* such category number @p i . */
 	QString category(unsigned int i);
@@ -259,7 +267,7 @@ public:
 	* on failure, e.g. the index @p i was out of range or the category name
 	* was invalid. Category names that are too long are truncated to 15 characters.
 	*/
-	bool setCategory(unsigned int i, const QString &s);
+	bool setCategoryName(unsigned int i, const QString &s);
 
 private:
 	struct CategoryAppInfo *fC;
@@ -267,18 +275,28 @@ private:
 	bool fOwn;
 } ;
 
+/** A template class for reading and interpreting AppInfo blocks;
+* the idea is that it handles all the boilerplate code for reading
+* the app block, converting it to the right kind, and then unpacking
+* it. Template parameters are the type (struct, from pilot-link probably)
+* of the interpreted appinfo, and the pack and unpack functions for it
+* (again, from pilot-link).
+*/
 template <typename appinfo,
 	int(*unpack)(appinfo *, unsigned char *, PI_SIZE_T),
 	int(*pack)(appinfo *, unsigned char *, PI_SIZE_T)>
 class PilotAppInfo : public PilotAppInfoBase
 {
 public:
+	/** Constructor. Read the appinfo from database @p d and
+	* interpret it.
+	*/
 	PilotAppInfo(PilotDatabase *d) : PilotAppInfoBase()
 	{
-		FUNCTIONSETUP;
 		int appLen = MAX_APPINFO_SIZE;
 		unsigned char buffer[MAX_APPINFO_SIZE];
 
+		memset(&fInfo,0,sizeof(fInfo));
 		if (d && d->isDBOpen())
 		{
 			appLen = d->readAppBlock(buffer,appLen);
@@ -288,9 +306,12 @@ public:
 		init(&fInfo.category,appLen);
 	} ;
 
+	/** Write this appinfo block to the database @p d; returns
+	* the number of bytes written or -1 on failure. This
+	* function is robust when called with a NULL database @p d.
+	*/
 	int write(PilotDatabase *d)
 	{
-		FUNCTIONSETUP;
 		unsigned char buffer[MAX_APPINFO_SIZE];
 		if (!d || !d->isDBOpen())
 		{
@@ -304,11 +325,109 @@ public:
 		return appLen;
 	} ;
 
+	/** Returns a (correctly typed) pointer to the interpreted
+	* appinfo block.
+	*/
 	appinfo *info() { return &fInfo; } ;
 
 protected:
 	appinfo fInfo;
 } ;
 
+/** A template class for reading and interpreting a database. This removes
+* the need for a lot of boilerplate code that does the conversions.
+* Parameters are two interpretation classes: one for the KDE side of
+* things (e.g. Event) and one that interprets the Pilot's records into
+* a more sensible structure (e.g. PilotDatebookEntry). The mapping from
+* the KDE type to the Pilot type and vice-versa is done by the mapper
+* class's convert() functions.
+*
+* To interpret a database as pilot-link interpretations (e.g. as
+* PilotDatebookEntry records, not as Events) use the NullMapper class
+* below in combination with a template instantiation with kdetype==pilottype.
+*
+* The database interpreter intentionally has an interface similar to
+* that of a PilotDatabase, but it isn't one.
+*/
+template <class kdetype, class pilottype, class mapper>
+class DatabaseInterpreter
+{
+private:
+	/** Interpret a PilotRecord as an object of type kdetype. */
+	kdetype *interpret(PilotRecord *r)
+	{
+		// NULL records return NULL kde objects.
+		if (!r) return 0;
+		// Interpret the binary blob as a pilot-link object.
+		pilottype *a = new pilottype(r);
+		// The record is now obsolete.
+		delete r;
+		// Interpretation failed.
+		if (!a) { return 0; }
+		// Now convert to KDE type.
+		kdetype *t = mapper::convert(a);
+		// The NULL mapper just returns the pointer a, so we
+		// need to check if anything has changed before deleting.
+		if ( (void *)t != (void *)a )
+		{
+			delete a;
+		}
+		return t;
+	}
+public:
+	/** Constructor. Interpret the database @p d. */
+	DatabaseInterpreter(PilotDatabase *d) : fDB(d) { } ;
+
+	/** Reads a record from database by @p id */
+	kdetype *readRecordById(recordid_t id)
+	{
+		return interpret(fDB->readRecordById(id));
+	}
+
+	/** Reads a record from database with index @p index */
+	kdetype *readRecordByIndex(int index)
+	{
+		return interpret(fDB->readRecordByIndex(index));
+	}
+
+	/** Reads the next record from database in category @p category */
+	kdetype *readNextRecInCategory(int category)
+	{
+		return interpret(fDB->readNextRecInCategory(category));
+	}
+
+	/**
+	* Reads the next record from database that has the dirty flag set.
+	* If @p ind is non-NULL, *ind is set to the index of the current
+	* record (i.e. before the record pointer moves to the next
+	* modified record).
+	*/
+	kdetype *readNextModifiedRec(int *ind=NULL)
+	{
+		return interpret(fDB->readNextModifiedRec(ind));
+	}
+
+
+	/** Retrieve the database pointer; this is useful to just pass
+	* around DatabaseInterpreter objects as if they are databases,
+	* and then perform DB operations on the database it wraps.
+	*/
+	PilotDatabase *db() const { return fDB; }
+
+protected:
+	PilotDatabase *fDB;
+} ;
+
+/** NULL mapper class; the conversions here don't @em do anything,
+* so you can use this when you only need 1 conversion step (from
+* PilotRecord to PilotDatebookEntry, for instance) instead of 2.
+*/
+template <class T>
+class NullMapper
+{
+public:
+	/** NULL Conversion function. */
+	static T *convert(T *t) { return t; }
+} ;
 
 #endif
