@@ -18,8 +18,8 @@
 **
 ** You should have received a copy of the GNU Lesser General Public License
 ** along with this program in a file called COPYING; if not, write to
-** the Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,
-** MA 02111-1307, USA.
+** the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
+** MA 02110-1301, USA.
 */
 
 /*
@@ -39,8 +39,27 @@
 #include "pilotMemo.h"
 
 
+/* Return values for the various check* functions. They
+   return OK if all is OK; ERR is for generic errors.
+   ERR_NO_EXIST is returned if something (usually a database
+   or other file) doesn't exist that should. The latter
+   error might be ignored.
+*/
+#define OK           (0)
+#define ERR          (1)
+#define ERR_NO_EXIST (2)
+
+
+/* Data about the records in a database. The id field is
+   interpreted specially for the first and last entries.
+*/
 typedef struct { int id,size; } recordInfo;
 
+/* Use END in the last recordInfo struct describing a database
+   to indicate you expect the database to end there. Use NO_EXIST
+   as the ID in the first struct to indicate that the database
+   is expected _not_ to exist.
+*/
 #define NO_EXIST   (-2)
 #define END        (-1)
 
@@ -93,18 +112,18 @@ int checkDatabase(const char *path, recordInfo *info)
 		if ( info[0].id == NO_EXIST )
 		{
 			kdDebug() << "This was expected" << endl;
-			return 0;
+			return OK;
 		}
 		else
 		{
-			return 1;
+			return ERR_NO_EXIST;
 		}
 	}
 
 	if ( info[0].id == NO_EXIST )
 	{
 		kdDebug() << "Database not expected" << endl;
-		return 1;
+		return ERR;
 	}
 
 	int fail = 0;
@@ -112,13 +131,13 @@ int checkDatabase(const char *path, recordInfo *info)
 	PilotRecord *r;
 	while( (r = db.readRecordByIndex(index) ) )
 	{
-		kdDebug() << "[" << index << "] id=" << r->id() << " size=" << r->getLen() << endl;
+		kdDebug() << "[" << index << "] id=" << r->id() << " size=" << r->size() << endl;
 		if ( ((recordid_t)info[index].id) != r->id() )
 		{
 			kdDebug() << "* Bad ID (expected" << r->id() << ")" << endl;
 			fail++;
 		}
-		else if ( info[index].size != r->getLen() )
+		else if ( info[index].size != r->size() )
 		{
 			kdDebug() << "* Bad size (expected " << info[index].size << ")" << endl;
 			fail++;
@@ -134,22 +153,70 @@ int checkDatabase(const char *path, recordInfo *info)
 	if (fail)
 	{
 		kdDebug() << "* " << fail << " failures." << endl;
-		return 1;
+		return ERR;
 	}
-	return 0;
+	return OK;
 }
 
-
-struct { const char *path; recordInfo *info; } tests[] = {
-	{ "/tmp/nonexistant/nonexistent", nonexistent },
-	{ "/tmp/Aesop", aesop },
-	{ 0L, 0L }
+const char *categoryNames[4] =
+{
+	"aardvarks",
+	"toolongToBeaCategoryName",
+	"personal",
+	"impersonal"
 } ;
 
+QStringList listCategories()
+{
+	QStringList cats;
+	PilotLocalDatabase *l = new PilotLocalDatabase(SOURCE "/data/MemoDB");
+	PilotMemoInfo *m = new PilotMemoInfo(l);
+
+	if (!l->isDBOpen()) return cats;
+
+	cats.append(CSL1("Unfiled"));
+	m->dump();
+
+	for (int i=0; i<20; i++)
+	{
+		PilotRecord *r = l->readRecordByIndex(i);
+		kdDebug() << "Read record " << (void *)r << " with id=" << r->id() << endl;
+		if (!r) break;
+	}
+
+	for (int i=0; i<4; i++)
+	{
+		QString s = m->category(i);
+		kdDebug() << "Category " << i << ": " << (s.isEmpty() ? CSL1("<empty>") : s) << endl;
+		cats.append(s);
+/*
+		if (i<((sizeof(categoryNames) / sizeof(categoryNames[0]))))
+			m->setCategoryName(i,QString::fromLatin1(categoryNames[i]));
+*/
+	}
+
+	m->write(l);
+
+	delete m;
+	delete l;
+
+	return cats;
+}
+
+int checkCategories()
+{
+	QStringList l = listCategories();
+	QStringList m = listCategories();
+
+	if (l.isEmpty() || m.isEmpty()) return ERR;
+	if (l!=m) return ERR;
+	return OK;
+}
+
 int checkMemo()
 {
 	PilotLocalDatabase *l = new PilotLocalDatabase(SOURCE "/data/MemoDB");
-	if (!l->isDBOpen()) return 1;
+	if (!l->isDBOpen()) return ERR_NO_EXIST;
 
 	PilotMemoInfo *m = new PilotMemoInfo(l);
 	m->dump();
@@ -158,23 +225,23 @@ int checkMemo()
 	if (c != CSL1("Business"))
 	{
 		kdDebug() << "* Category 1 is not 'Business' but " << c << endl;
-		return 1;
+		return ERR;
 	}
 
-	m->setCategory(2,CSL1("Aardvark"));
+	m->setCategoryName(2,CSL1("Aardvark"));
 	m->write(l);
 
 	c = m->category(2);
 	if (c != CSL1("Aardvark"))
 	{
 		kdDebug() << "* Category 2 is not 'Aardvark' but " << c << endl;
-		return 1;
+		return ERR;
 	}
 
 
 	delete m;
 	delete l;
-	return 0;
+	return OK;
 }
 
 static const KCmdLineOptions options[] =
@@ -203,12 +270,33 @@ int main(int argc, char **argv)
 	debug_level=4;
 #endif
 
+	Q_UNUSED(argc);
+	Q_UNUSED(argv);
+
+#define ALLOW_NO_EXIST (1)
+	static struct { const char *path; recordInfo *info; int flags; } 
+		tests[] =
+	{
+		{ "/tmp/nonexistant/nonexistent", nonexistent,0 },
+		{ "/tmp/Aesop", aesop, ALLOW_NO_EXIST },
+		{ 0L, 0L, 0 }
+	} ;
+
 	while ( tests[i].path )
 	{
 		kdDebug() << "*** Test " << i << endl;
-		if ( checkDatabase( tests[i].path, tests[i].info ) )
+		int ret = checkDatabase( tests[i].path, tests[i].info );
+		if ( ret )
 		{
-			r++;
+			if ( (ret==ERR_NO_EXIST) && 
+				(tests[i].flags & ALLOW_NO_EXIST) )
+			{
+				kdDebug() << "* Test database doesn't exist, ignored." << endl;
+			}
+			else
+			{
+				r++;
+			}
 		}
 		i++;
 	}
