@@ -15,14 +15,26 @@
 
     You should have received a copy of the GNU Library General Public License
     along with this library; see the file COPYING.LIB.  If not, write to
-    the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-    Boston, MA 02111-1307, USA.
+    the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+    Boston, MA 02110-1301, USA.
 */
 
 #include "kcal_resourcegroupware.h"
 #include "kcal_resourcegroupwareconfig.h"
 
+#include <kglobal.h>
+#include <klocale.h>
+
 using namespace KCal;
 
-K_EXPORT_COMPONENT_FACTORY(kcal_groupware, KRES::PluginFactory< ResourceGroupware,
-                                    ResourceGroupwareConfig > );
+typedef KRES::PluginFactory<ResourceGroupware, ResourceGroupwareConfig> GroupwareFactory;
+
+extern "C"
+{
+  void *init_kcal_groupware()
+  {
+    KGlobal::locale()->insertCatalogue( "libkcal" );
+    KGlobal::locale()->insertCatalogue( "kres_groupware" );
+    return new ResourceGroupware;
+  }
+}
