@@ -15,8 +15,8 @@
 
     You should have received a copy of the GNU Library General Public License
     along with this library; see the file COPYING.LIB.  If not, write to
-    the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-    Boston, MA 02111-1307, USA.
+    the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+    Boston, MA 02110-1301, USA.
 
     In addition, as a special exception, the copyright holders give
     permission to link the code of this program with any edition of
@@ -34,6 +34,8 @@
 #include "resourcekolab.h"
 
 #include <kabc/addressee.h>
+#include <kabc/stdaddressbook.h>
+#include <libkdepim/distributionlist.h>
 #include <kio/netaccess.h>
 #include <kdebug.h>
 #include <qfile.h>
@@ -543,90 +545,207 @@ void Contact::saveAddressAttributes( QDomElement& element ) const
   }
 }
 
+
+void Kolab::Contact::loadDistrListMember( const QDomElement& element )
+{
+  Member member;
+  for ( QDomNode n = element.firstChild(); !n.isNull(); n = n.nextSibling() ) {
+   if ( n.isElement() ) {
+      QDomElement e = n.toElement();
+      QString tagName = e.tagName();
+      if ( tagName == "display-name" )
+        member.displayName = e.text();
+      else if ( tagName == "smtp-address" )
+        member.email = e.text();
+    }
+  }
+  mDistrListMembers.append( member );
+}
+
+void Contact::saveDistrListMembers( QDomElement& element ) const
+{
+  QValueList<Member>::ConstIterator it = mDistrListMembers.begin();
+  for( ; it != mDistrListMembers.end(); ++it ) {
+    QDomElement e = element.ownerDocument().createElement( "member" );
+    element.appendChild( e );
+    const Member& m = *it;
+    writeString( e, "display-name", m.displayName );
+    writeString( e, "smtp-address", m.email );
+  }
+}
+
 bool Contact::loadAttribute( QDomElement& element )
 {
-  QString tagName = element.tagName();
-
-  if ( tagName == "name" )
-    return loadNameAttribute( element );
-  else if ( tagName == "role" )
-    setRole( element.text() );
-  else if ( tagName == "free-busy-url" )
-    setFreeBusyUrl( element.text() );
-  else if ( tagName == "organization" )
-    setOrganization( element.text() );
-  else if ( tagName == "web-page" )
-    setWebPage( element.text() );
-  else if ( tagName == "im-address" )
-    setIMAddress( element.text() );
-  else if ( tagName == "department" )
-    setDepartment( element.text() );
-  else if ( tagName == "office-location" )
-    setOfficeLocation( element.text() );
-  else if ( tagName == "profession" )
-    setProfession( element.text() );
-  else if ( tagName == "job-title" )
-    // see saveAttributes: <job-title> is mapped to the Role field
-    setRole( element.text() );
-  else if ( tagName == "manager-name" )
-    setManagerName( element.text() );
-  else if ( tagName == "assistant" )
-    setAssistant( element.text() );
-  else if ( tagName == "nick-name" )
-    setNickName( element.text() );
-  else if ( tagName == "spouse-name" )
-    setSpouseName( element.text() );
-  else if ( tagName == "birthday" ) {
-    if ( !element.text().isEmpty() )
-      setBirthday( stringToDate( element.text() ) );
-  }
-  else if ( tagName == "anniversary" ) {
-    if ( !element.text().isEmpty() )
-      setAnniversary( stringToDate( element.text() ) );
-  }
-  else if ( tagName == "picture" ) {
-    mPictureAttachmentName = element.text();
-  }
-  else if ( tagName == "x-logo" ) {
-    mLogoAttachmentName = element.text();
-  }
-  else if ( tagName == "x-sound" ) {
-    mSoundAttachmentName = element.text();
-  }
-  else if ( tagName == "children" )
-    setChildren( element.text() );
-  else if ( tagName == "gender" )
-    setGender( element.text() );
-  else if ( tagName == "language" )
-    setLanguage( element.text() );
-  else if ( tagName == "phone" )
-    return loadPhoneAttribute( element );
-  else if ( tagName == "email" ) {
-    Email email;
-    if ( loadEmailAttribute( element, email ) ) {
-      addEmail( email );
+  const QString tagName = element.tagName();
+  switch ( tagName[0].latin1() ) {
+  case 'a':
+    if ( tagName == "address" )
+      return loadAddressAttribute( element );
+    if ( tagName == "assistant" ) {
+      setAssistant( element.text() );
       return true;
-    } else
-      return false;
-  } else if ( tagName == "address" )
-    return loadAddressAttribute( element );
-  else if ( tagName == "preferred-address" )
-    setPreferredAddress( element.text() );
-  else if ( tagName == "latitude" ) {
-    setLatitude( element.text().toFloat() );
-    mHasGeo = true;
+    }
+    if ( tagName == "anniversary" ) {
+      if ( !element.text().isEmpty() )
+        setAnniversary( stringToDate( element.text() ) );
+      return true;
+    }
+    break;
+  case 'b':
+    if ( tagName == "birthday" ) {
+      if ( !element.text().isEmpty() )
+        setBirthday( stringToDate( element.text() ) );
+      return true;
+    }
+    break;
+  case 'c':
+    if ( tagName == "children" ) {
+      setChildren( element.text() );
+      return true;
+    }
+    break;
+  case 'd':
+    if ( tagName == "department" ) {
+      setDepartment( element.text() );
+      return true;
+    }
+    if ( mIsDistributionList && tagName == "display-name" ) {
+      setFullName( element.text() );
+      return true;
+    }
+    break;
+  case 'e':
+    if ( tagName == "email" ) {
+      Email email;
+      if ( loadEmailAttribute( element, email ) ) {
+        addEmail( email );
+        return true;
+      } else
+        return false;
+    }
+    break;
+  case 'f':
+    if ( tagName == "free-busy-url" ) {
+      setFreeBusyUrl( element.text() );
+      return true;
+    }
+    break;
+  case 'g':
+    if ( tagName == "gender" ) {
+      setGender( element.text() );
+      return true;
+    }
+    break;
+  case 'i':
+    if ( tagName == "im-address" ) {
+      setIMAddress( element.text() );
+      return true;
+    }
+    break;
+  case 'j':
+    if ( tagName == "job-title" ) {
+      // see saveAttributes: <job-title> is mapped to the Role field
+      setRole( element.text() );
+      return true;
+    }
+    break;
+  case 'l':
+    if ( tagName == "language" ) {
+      setLanguage( element.text() );
+      return true;
+    }
+    if ( tagName == "latitude" ) {
+      setLatitude( element.text().toFloat() );
+      mHasGeo = true;
+      return true;
+    }
+    if ( tagName == "longitude" ) {
+      setLongitude( element.text().toFloat() );
+      mHasGeo = true;
+    }
+    break;
+  case 'm':
+    if ( tagName == "manager-name" ) {
+      setManagerName( element.text() );
+      return true;
+    }
+    if ( mIsDistributionList && tagName == "member" ) {
+      loadDistrListMember( element );
+      return true;
+    }
+    break;
+  case 'n':
+    if ( tagName == "name" )
+      return loadNameAttribute( element );
+    if ( tagName == "nick-name" ) {
+      setNickName( element.text() );
+      return true;
+    }
+    break;
+  case 'o':
+    if ( tagName == "organization" ) {
+      setOrganization( element.text() );
+      return true;
+    }
+    if ( tagName == "office-location" ) {
+      setOfficeLocation( element.text() );
+      return true;
+    }
+    break;
+  case 'p':
+    if ( tagName == "profession" ) {
+      setProfession( element.text() );
+      return true;
+    }
+    if ( tagName == "picture" ) {
+      mPictureAttachmentName = element.text();
+      return true;
+    }
+    if ( tagName == "phone" ) {
+      return loadPhoneAttribute( element );
+      return true;
+    }
+    if ( tagName == "preferred-address" ) {
+      setPreferredAddress( element.text() );
+      return true;
+    }
+    break;
+  case 'r':
+    if ( tagName == "role" ) {
+      setRole( element.text() );
+      return true;
+    }
+    break;
+  case 's':
+    if ( tagName == "spouse-name" ) {
+      setSpouseName( element.text() );
+      return true;
+    }
+    break;
+  case 'x':
+    if ( tagName == "x-logo" ) {
+      mLogoAttachmentName = element.text();
+      return true;
+    }
+    if ( tagName == "x-sound" ) {
+      mSoundAttachmentName = element.text();
+      return true;
+    }
+    if ( tagName == "x-custom" ) {
+      loadCustomAttributes( element );
+      return true;
+    }
+    break;
+  case 'w':
+    if ( tagName == "web-page" ) {
+      setWebPage( element.text() );
+      return true;
+    }
+    break;
+  default:
+    break;
   }
-  else if ( tagName == "longitude" ) {
-    setLongitude( element.text().toFloat() );
-    mHasGeo = true;
-  }
-  else if ( tagName == "x-custom" )
-    loadCustomAttributes( element );
-  else
-    return KolabBase::loadAttribute( element );
-
-  // We handled this
-  return true;
+  return KolabBase::loadAttribute( element );
 }
 
 bool Contact::saveAttributes( QDomElement& element ) const
@@ -634,41 +753,46 @@ bool Contact::saveAttributes( QDomElement& element ) const
   // Save the base class elements
   KolabBase::saveAttributes( element );
 
-  saveNameAttribute( element );
-  writeString( element, "free-busy-url", freeBusyUrl() );
-  writeString( element, "organization", organization() );
-  writeString( element, "web-page", webPage() );
-  writeString( element, "im-address", imAddress() );
-  writeString( element, "department", department() );
-  writeString( element, "office-location", officeLocation() );
-  writeString( element, "profession", profession() );
-  // <role> is gone; jobTitle() is not shown in the addresseeeditor,
-  // so let's bind <job-title> to role()
-  //writeString( element, "role", role() );
-  //writeString( element, "job-title", jobTitle() );
-  writeString( element, "job-title", role() );
-  writeString( element, "manager-name", managerName() );
-  writeString( element, "assistant", assistant() );
-  writeString( element, "nick-name", nickName() );
-  writeString( element, "spouse-name", spouseName() );
-  writeString( element, "birthday", dateToString( birthday() ) );
-  writeString( element, "anniversary", dateToString( anniversary() ) );
-  if ( !picture().isNull() )
-    writeString( element, "picture", mPictureAttachmentName );
-  if ( !logo().isNull() )
-    writeString( element, "x-logo", mLogoAttachmentName );
-  if ( !sound().isNull() )
-    writeString( element, "x-sound", mSoundAttachmentName );
-  writeString( element, "children", children() );
-  writeString( element, "gender", gender() );
-  writeString( element, "language", language() );
-  savePhoneAttributes( element );
-  saveEmailAttributes( element );
-  saveAddressAttributes( element );
-  writeString( element, "preferred-address", preferredAddress() );
-  if ( mHasGeo ) {
-    writeString( element, "latitude", QString::number( latitude(), 'g', DBL_DIG ) );
-    writeString( element, "longitude", QString::number( longitude(), 'g', DBL_DIG ) );
+  if ( mIsDistributionList ) {
+    writeString( element, "display-name", fullName() );
+    saveDistrListMembers( element );
+  } else {
+    saveNameAttribute( element );
+    writeString( element, "free-busy-url", freeBusyUrl() );
+    writeString( element, "organization", organization() );
+    writeString( element, "web-page", webPage() );
+    writeString( element, "im-address", imAddress() );
+    writeString( element, "department", department() );
+    writeString( element, "office-location", officeLocation() );
+    writeString( element, "profession", profession() );
+    // <role> is gone; jobTitle() is not shown in the addresseeeditor,
+    // so let's bind <job-title> to role()
+    //writeString( element, "role", role() );
+    //writeString( element, "job-title", jobTitle() );
+    writeString( element, "job-title", role() );
+    writeString( element, "manager-name", managerName() );
+    writeString( element, "assistant", assistant() );
+    writeString( element, "nick-name", nickName() );
+    writeString( element, "spouse-name", spouseName() );
+    writeString( element, "birthday", dateToString( birthday() ) );
+    writeString( element, "anniversary", dateToString( anniversary() ) );
+    if ( !picture().isNull() )
+      writeString( element, "picture", mPictureAttachmentName );
+    if ( !logo().isNull() )
+      writeString( element, "x-logo", mLogoAttachmentName );
+    if ( !sound().isNull() )
+      writeString( element, "x-sound", mSoundAttachmentName );
+    writeString( element, "children", children() );
+    writeString( element, "gender", gender() );
+    writeString( element, "language", language() );
+    savePhoneAttributes( element );
+    saveEmailAttributes( element );
+    saveAddressAttributes( element );
+    writeString( element, "preferred-address", preferredAddress() );
+    if ( mHasGeo ) {
+      writeString( element, "latitude", QString::number( latitude(), 'g', DBL_DIG ) );
+      writeString( element, "longitude", QString::number( longitude(), 'g', DBL_DIG ) );
+    }
   }
   saveCustomAttributes( element );
 
@@ -679,12 +803,14 @@ bool Contact::loadXML( const QDomDocument& document )
 {
   QDomElement top = document.documentElement();
 
-  if ( top.tagName() != "contact" ) {
-    qWarning( "XML error: Top tag was %s instead of the expected contact",
+  mIsDistributionList = top.tagName() == "distribution-list";
+  if ( top.tagName() != "contact" && !mIsDistributionList ) {
+    qWarning( "XML error: Top tag was %s instead of the expected contact or distribution-list",
               top.tagName().ascii() );
     return false;
   }
 
+
   for ( QDomNode n = top.firstChild(); !n.isNull(); n = n.nextSibling() ) {
     if ( n.isComment() )
       continue;
@@ -709,7 +835,8 @@ bool Contact::loadXML( const QDomDocument& document )
 QString Contact::saveXML() const
 {
   QDomDocument document = domTree();
-  QDomElement element = document.createElement( "contact" );
+  QDomElement element = document.createElement(
+    mIsDistributionList ? "distribution-list" : "contact" );
   element.setAttribute( "version", "1.0" );
   saveAttributes( element );
   document.appendChild( element );
@@ -843,6 +970,7 @@ static const char* s_knownCustomFields[] = {
   "X-AssistantsName",
   "X-SpousesName",
   "X-Anniversary",
+  "DistributionList",
   0
 };
 
@@ -851,6 +979,23 @@ void Contact::setFields( const KABC::Addressee* addressee )
 {
   KolabBase::setFields( addressee );
 
+  mIsDistributionList = KPIM::DistributionList::isDistributionList( *addressee );
+  if ( mIsDistributionList ) {
+    // Hopefully all resources are available during saving, so we can look up
+    // in the addressbook to get name+email from the UID.
+    KPIM::DistributionList distrList( *addressee );
+    const KPIM::DistributionList::Entry::List entries = distrList.entries( KABC::StdAddressBook::self() );
+    KPIM::DistributionList::Entry::List::ConstIterator it = entries.begin();
+    for ( ; it != entries.end() ; ++it ) {
+      Member m;
+      m.displayName = (*it).addressee.formattedName();
+      m.email = (*it).email;
+      if ( m.email.isEmpty() )
+        m.email = (*it).addressee.preferredEmail();
+      mDistrListMembers.append( m );
+    }
+  }
+
   setGivenName( addressee->givenName() );
   setMiddleNames( addressee->additionalName() );
   setLastName( addressee->familyName() );
@@ -973,9 +1118,8 @@ void Contact::setFields( const KABC::Addressee* addressee )
   // So it doesn't matter much if we don't have them in the XML.
   // mailer, timezone, productId, sortString, agent, rfc2426 name()
 
-  // TODO: Things KAddressBook can't handle:
+  // Things KAddressBook can't handle, so they are saved as unhandled tags:
   // initials, children, gender, language
-  // Well, so that's part of the "unhandled" tag thing.
 
   // TODO: Free/Busy URL. This is done rather awkward in KAddressBook -
   // it stores it in a local file through a korganizer file :-(
@@ -987,10 +1131,23 @@ void Contact::saveTo( KABC::Addressee* addressee )
   // TODO: This needs the same set of TODOs as the setFields method
   KolabBase::saveTo( addressee );
 
+  if ( mIsDistributionList ) {
+    KPIM::DistributionList distrList( *addressee );
+    distrList.setName( fullName() );
+    QValueList<Member>::ConstIterator mit = mDistrListMembers.begin();
+    for ( ; mit != mDistrListMembers.end(); ++mit ) {
+      distrList.insertEntry( (*mit).displayName, (*mit).email );
+    }
+    addressee->insertCustom( "KADDRESSBOOK", "DistributionList", distrList.custom( "KADDRESSBOOK", "DistributionList" ) );
+    Q_ASSERT( KPIM::DistributionList::isDistributionList( *addressee ) );
+  }
+
   addressee->setGivenName( givenName() );
   addressee->setAdditionalName( middleNames() );
   addressee->setFamilyName( lastName() );
   addressee->setFormattedName( fullName() );
+  if ( mIsDistributionList )
+    addressee->setName( fullName() );
   addressee->setPrefix( prefix() );
   addressee->setSuffix( suffix() );
   addressee->setOrganization( organization() );
