@@ -15,8 +15,8 @@
 
     You should have received a copy of the GNU Library General Public License
     along with this library; see the file COPYING.LIB.  If not, write to
-    the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-    Boston, MA 02111-1307, USA.
+    the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+    Boston, MA 02110-1301, USA.
 */
 
 #include "email.h"
@@ -115,8 +115,6 @@ KPIM::EmailParseResult splitAddressInternal( const QCString& address,
   if ( address.isEmpty() )
     return KPIM::AddressEmpty;
 
-  QCString result;
-
   // The following is a primitive parser for a mailbox-list (cf. RFC 2822).
   // The purpose is to extract a displayable string from the mailboxes.
   // Comments in the addr-spec are not handled. No error checking is done.
@@ -320,6 +318,14 @@ KPIM::EmailParseResult KPIM::isValidEmailAddress( const QString& aStr )
             commentLevel = 1;
           }
           break;
+        case '[' :
+          if ( !inQuotedString ) {
+            return InvalidDisplayName;
+          }
+        case ']' :
+          if ( !inQuotedString ) {
+            return InvalidDisplayName;
+          }
         case ':' :
           if ( !inQuotedString ) {
             return DisallowedChar;
@@ -331,8 +337,9 @@ KPIM::EmailParseResult KPIM::isValidEmailAddress( const QString& aStr )
           break;
         case '\\' : // quoted character
           ++index; // skip the '\'
-          if ( ++index > strlen )
+          if (( index + 1 )> strlen ) {
             return UnexpectedEnd;
+          }
           break;
         case ',' :
           if ( !inQuotedString )
@@ -352,15 +359,14 @@ KPIM::EmailParseResult KPIM::isValidEmailAddress( const QString& aStr )
               return MissingLocalPart;
             } else if( index == strlen-1 ) {
               return MissingDomainPart;
-              break;
-              }
-            } else if ( inQuotedString ) {
-              --atCount;
-              if ( atCount == 1 ) {
-                tooManyAtsFlag = false;
-              }
             }
-            break;
+          } else if ( inQuotedString ) {
+            --atCount;
+            if ( atCount == 1 ) {
+              tooManyAtsFlag = false;
+            }
+          }
+          break;
       }
       break;
     }
@@ -375,9 +381,10 @@ KPIM::EmailParseResult KPIM::isValidEmailAddress( const QString& aStr )
           break;
         case '\\' : // quoted character
           ++index; // skip the '\'
-          if ( ++index > strlen )
+          if (( index + 1 )> strlen ) {
             return UnexpectedEnd;
-            break;
+          }
+          break;
         }
         break;
     }
@@ -407,14 +414,22 @@ KPIM::EmailParseResult KPIM::isValidEmailAddress( const QString& aStr )
           break;
         case '\\' : // quoted character
           ++index; // skip the '\'
-          if ( ++index > strlen )
+          if (( index + 1 )> strlen ) {
             return UnexpectedEnd;
-          break;
           }
+          break;
+        }
         break;
       }
     }
   }
+
+  if ( atCount == 0 && !inQuotedString )
+    return TooFewAts;
+
+  if ( inQuotedString )
+    return UnbalancedQuote;
+
   if ( context == InComment )
     return UnbalancedParens;
 
@@ -478,6 +493,9 @@ QString KPIM::emailParseResultToString( EmailParseResult errorCode )
     case DisallowedChar :
       return i18n("The email address you entered is not valid because it "
                   "contains an illegal character.");
+    case InvalidDisplayName :
+      return i18n("The email address you have entered is not valid because it "
+                  "contains an invalid displayname.");
   }
   return i18n("Unknown problem with email address");
 }
@@ -485,7 +503,7 @@ QString KPIM::emailParseResultToString( EmailParseResult errorCode )
 //-----------------------------------------------------------------------------
 bool KPIM::isValidSimpleEmailAddress( const QString& aStr )
 {
-  // If we are passed an empty string bail right away no need to process further·
+  // If we are passed an empty string bail right away no need to process further
   // and waste resources
   if ( aStr.isEmpty() ) {
     return false;
@@ -619,7 +637,7 @@ bool KPIM::getNameAndMail(const QString& aStr, QString& name, QString& mail)
     ++i;
   }
 
-  if( !iAd ){
+  if ( !iAd ) {
     // We suppose the user is typing the string manually and just
     // has not finished typing the mail address part.
     // So we take everything that's left of the '<' as name and the rest as mail
@@ -634,8 +652,7 @@ bool KPIM::getNameAndMail(const QString& aStr, QString& name, QString& mail)
     if ( mail.endsWith( ">" ) )
       mail.truncate( mail.length() - 1 );
 
-  }else{
-
+  } else {
     // Loop backwards until we find the start of the string
     // or a ',' that is outside of a comment
     //          and outside of quoted text before the leading '<'.
@@ -643,12 +660,12 @@ bool KPIM::getNameAndMail(const QString& aStr, QString& name, QString& mail)
     bInQuotesOutsideOfEmail = false;
     for( i = iAd-1; 0 <= i; --i ) {
       c = aStr[i];
-      if( bInComment ){
-        if( '(' == c ){
+      if( bInComment ) {
+        if( '(' == c ) {
           if( !name.isEmpty() )
             name.prepend( ' ' );
           bInComment = false;
-        }else{
+        } else {
           name.prepend( c ); // all comment stuff is part of the name
         }
       }else if( bInQuotesOutsideOfEmail ){
@@ -697,14 +714,24 @@ bool KPIM::getNameAndMail(const QString& aStr, QString& name, QString& mail)
     //          and outside of quoted text behind the trailing '>'.
     bInComment = false;
     bInQuotesOutsideOfEmail = false;
+    int parenthesesNesting = 0;
     for( i = iAd+1; len > i; ++i ) {
       c = aStr[i];
       if( bInComment ){
         if( ')' == c ){
-          if( !name.isEmpty() )
-            name.append( ' ' );
-          bInComment = false;
-        }else{
+          if ( --parenthesesNesting == 0 ) {
+            bInComment = false;
+            if( !name.isEmpty() )
+              name.append( ' ' );
+          } else {
+            // nested ")", add it
+            name.append( ')' ); // name can't be empty here
+          }
+        } else {
+          if( '(' == c ) {
+            // nested "("
+            ++parenthesesNesting;
+          }
           name.append( c ); // all comment stuff is part of the name
         }
       }else if( bInQuotesOutsideOfEmail ){
@@ -730,7 +757,8 @@ bool KPIM::getNameAndMail(const QString& aStr, QString& name, QString& mail)
             case '(':
               if( !name.isEmpty() )
                 name.append( ' ' );
-              bInComment = true;
+              if ( ++parenthesesNesting > 0 )
+                bInComment = true;
               break;
             default:
               if( ' ' != c )
