@@ -1,5 +1,6 @@
 /*
     This file is part of libkcal.
+
     Copyright (c) 2001 Cornelius Schumacher <schumacher@kde.org>
 
     This library is free software; you can redistribute it and/or
@@ -14,8 +15,8 @@
 
     You should have received a copy of the GNU Library General Public License
     along with this library; see the file COPYING.LIB.  If not, write to
-    the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-    Boston, MA 02111-1307, USA.
+    the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+    Boston, MA 02110-1301, USA.
 */
 
 #include <kglobal.h>
@@ -78,10 +79,16 @@ QDateTime Event::dtEnd() const
   if (hasDuration()) return dtStart().addSecs(duration());
 
   kdDebug(5800) << "Warning! Event '" << summary()
-            << "' does have neither end date nor duration." << endl;
+            << "' has neither end date nor duration." << endl;
   return dtStart();
 }
 
+QDate Event::dateEnd() const
+{
+  if ( doesFloat() ) return dtEnd().date();
+  else return dtEnd().addSecs(-1).date();
+}
+
 QString Event::dtEndTimeStr() const
 {
   return KGlobal::locale()->formatTime(dtEnd().time());
@@ -109,7 +116,13 @@ bool Event::hasEndDate() const
 
 bool Event::isMultiDay() const
 {
-  bool multi = !(dtStart().date() == dtEnd().date());
+  // End date is non inclusive, so subtract 1 second...
+  QDateTime start( dtStart() );
+  QDateTime end( dtEnd() );
+  if ( ! doesFloat() ) {
+    end = end.addSecs(-1);
+  }
+  bool multi = ( start.date() != end.date() && start <= end );
   return multi;
 }
 
