@@ -15,8 +15,8 @@
 
     You should have received a copy of the GNU Library General Public License
     along with this library; see the file COPYING.LIB.  If not, write to
-    the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-    Boston, MA 02111-1307, USA.
+    the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+    Boston, MA 02110-1301, USA.
 */
 
 #include <qdatastream.h>
@@ -42,9 +42,10 @@
 using namespace KCal;
 
 ResourceCached::ResourceCached( const KConfig* config )
-  : ResourceCalendar( config ), mReloadPolicy( ReloadNever ),
-    mReloadInterval( 10 ), mReloaded( false ), mSavePolicy( SaveNever ),
-    mSaveInterval( 10 ), mIdMapper( "kcal/uidmaps/" )
+  : ResourceCalendar( config ), mCalendar( QString::fromLatin1( "UTC" ) ),
+    mReloadPolicy( ReloadNever ),  mReloadInterval( 10 ), mReloaded( false ),
+    mSavePolicy( SaveNever ), mSaveInterval( 10 ),
+    mIdMapper( "kcal/uidmaps/" )
 {
   connect( &mReloadTimer, SIGNAL( timeout() ), SLOT( slotReload() ) );
   connect( &mSaveTimer, SIGNAL( timeout() ), SLOT( slotSave() ) );
@@ -153,11 +154,11 @@ bool ResourceCached::addEvent(Event *event)
 }
 
 // probably not really efficient, but...it works for now.
-void ResourceCached::deleteEvent( Event *event )
+bool ResourceCached::deleteEvent( Event *event )
 {
   kdDebug(5800) << "ResourceCached::deleteEvent" << endl;
 
-  mCalendar.deleteEvent( event );
+  return mCalendar.deleteEvent( event );
 }
 
 
@@ -166,14 +167,15 @@ Event *ResourceCached::event( const QString &uid )
   return mCalendar.event( uid );
 }
 
-Event::List ResourceCached::rawEventsForDate( const QDate &qd, bool sorted )
+Event::List ResourceCached::rawEventsForDate( const QDate &qd,
+                                              EventSortField sortField,
+                                              SortDirection sortDirection )
 {
-  Event::List list = mCalendar.rawEventsForDate( qd, sorted );
+  Event::List list = mCalendar.rawEventsForDate( qd, sortField, sortDirection );
 
   return list;
 }
 
-
 Event::List ResourceCached::rawEvents( const QDate &start, const QDate &end,
                                        bool inclusive )
 {
@@ -195,14 +197,14 @@ bool ResourceCached::addTodo( Todo *todo )
   return mCalendar.addTodo( todo );
 }
 
-void ResourceCached::deleteTodo( Todo *todo )
+bool ResourceCached::deleteTodo( Todo *todo )
 {
-  mCalendar.deleteTodo( todo );
+  return mCalendar.deleteTodo( todo );
 }
 
-void ResourceCached::deleteJournal( Journal *journal )
+bool ResourceCached::deleteJournal( Journal *journal )
 {
-  mCalendar.deleteJournal( journal );
+  return mCalendar.deleteJournal( journal );
 }
 
 
@@ -282,6 +284,13 @@ void ResourceCached::loadCache()
 
   if ( KStandardDirs::exists( cacheFile() ) ) {
     mCalendar.load( cacheFile() );
+    if ( readOnly() ) {
+      Incidence::List incidences( rawIncidences() );
+      Incidence::List::Iterator it;
+      for ( it = incidences.begin(); it != incidences.end(); ++it ) {
+        (*it)->setReadOnly( true );
+      }
+    }
   }
 }
 
@@ -307,7 +316,7 @@ void ResourceCached::clearCache()
 
 void ResourceCached::cleanUpEventCache( const Event::List &eventList )
 {
-  CalendarLocal calendar;
+  CalendarLocal calendar ( QString::fromLatin1( "UTC" ) );
 
   if ( KStandardDirs::exists( cacheFile() ) )
     calendar.load( cacheFile() );
@@ -336,7 +345,7 @@ void ResourceCached::cleanUpEventCache( const Event::List &eventList )
 
 void ResourceCached::cleanUpTodoCache( const Todo::List &todoList )
 {
-  CalendarLocal calendar;
+  CalendarLocal calendar ( QString::fromLatin1( "UTC" ) );
 
   if ( KStandardDirs::exists( cacheFile() ) )
     calendar.load( cacheFile() );
@@ -381,7 +390,7 @@ QString ResourceCached::changesCacheFile( const QString &type ) const
 
 void ResourceCached::saveChangesCache( const QMap<Incidence*, bool> &map, const QString &type )
 {
-  CalendarLocal calendar;
+  CalendarLocal calendar ( QString::fromLatin1( "UTC" ) );
 
   bool isEmpty = true;
   QMap<Incidence *,bool>::ConstIterator it;
@@ -409,7 +418,7 @@ void ResourceCached::saveChangesCache()
 
 void ResourceCached::loadChangesCache( QMap<Incidence*, bool> &map, const QString &type )
 {
-  CalendarLocal calendar;
+  CalendarLocal calendar ( QString::fromLatin1( "UTC" ) );
 
   if ( KStandardDirs::exists( changesCacheFile( type ) ) )
     calendar.load( changesCacheFile( type ) );
