@@ -3,8 +3,8 @@
 // Contents:   Definitions for DwDateTime
 // Maintainer: Doug Sauder <dwsauder@fwb.gulf.net>
 // WWW:        http://www.fwb.gulf.net/~dwsauder/mimepp.html
-// $Revision: 375317 $
-// $Date: 2005-01-03 22:25:34 +0100 (Mo, 03 Jan 2005) $
+// $Revision: 494852 $
+// $Date: 2006-01-06 13:12:33 +0100 (Fr, 06 Jan 2006) $
 //
 // Copyright (c) 1996, 1997 Douglas W. Sauder
 // All rights reserved.
@@ -45,6 +45,7 @@ static char lMonth[12][4]
        "Jul", "Aug", "Sep", "Oct", "Nov", "Dec"};
 
 extern "C" int ParseRfc822Date(const char *str, struct tm *tms, int *z);
+extern "C" int ParseDate(const char *str, struct tm *tms, int *z);
 static DwInt32 ymd_to_jdnl(int year, int mon, int day, int julian);
 static void jdnl_to_ymd(DwInt32 jdn, int *year, int *mon, int *day, int julian);
 static DwUint32 my_inv_gmtime(struct tm* ptms);
@@ -282,6 +283,8 @@ void DwDateTime::Parse()
     struct tm tms;
     int zone;
     int err = ParseRfc822Date(str, &tms, &zone);
+    if ( err == -1 )  // try another format
+        err = ParseDate(str, &tms, &zone);
     if (!err) {
         mYear   = tms.tm_year + 1900;
         mMonth  = tms.tm_mon+1;
